package org.processmining.dataquality;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Set;

import javax.swing.table.DefaultTableModel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.ejml.simple.SimpleMatrix;
import org.processmining.dataquality.Reasons.Reason;
import org.processmining.dataquality.Reasons.ReasonMapping;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;

public class DetectionModel {

	//	Map<String, Integer> multiGranularSameEvent;//same event has multiple granularities
	//	Map<String, Integer> multiGranularDiffEvent;//diff events have multiple granularities
	Set<String> allActs;

	Map<String, Integer> actOccCount;//act occurrence count

	static Map<String, Integer> matIndexMap = new HashMap<String, Integer>();
	static SimpleMatrix dfMat = new SimpleMatrix(0, 0); //DF matrix
	static SimpleMatrix efMat = new SimpleMatrix(0, 0); //EF matrix
	static SimpleMatrix coMat = new SimpleMatrix(0, 0); //CO matrix
	static SimpleMatrix dpMat = new SimpleMatrix(0, 0); //DP matrix
	static SimpleMatrix epMat = new SimpleMatrix(0, 0); //EP matrix

	Map<Set<String>, Integer> sameSetTimeStamps; //set of acts having the same timestamp
	Map<Set<String>, Integer> multiGranWindow; //set of acts in a window having the diff granularity
	Map<String, Integer> emptyTime; //event has empty timestamp

	//record event granularity
	List<Map<String, Integer>> granularity; //month,day,hour,min,sec
	//	Map<String, Integer> monthLevelGran;
	//	Map<String, Integer> dayLevelGran;
	//	Map<String, Integer> hourLevelGran;
	//	Map<String, Integer> minLevelGran;
	//	Map<String, Integer> secLevelGran;

	Map<String, Map<String, Integer>> dfMap;//DF Map
	Map<String, Map<String, Integer>> efMap;//EF Map

	List<Reason> reasons = new ArrayList<Reason>();
	boolean hideWhiteList;
	List<Reason> whiteListedReasons = new ArrayList<Reason>();



	DefaultTableModel metricsTableModel;



	//each map is a value of act -> time,weight
	List<Map<String, Map<Double, Double>>> actTimeBins = new ArrayList<Map<String, Map<Double, Double>>>(); //month of year, date of month, day of week, hour of day, min of day

	//TODO user sets preferred granularity - else choose the max as base granularity

	//	LinkedList<XLog> changedLogs = new LinkedList<XLog>();		only for repair

	public DetectionModel(Set<String> acts) {
		allActs = new HashSet<String>(acts);
		dfMat = new SimpleMatrix(0, 0); //DF matrix
		efMat = new SimpleMatrix(0, 0); //EF matrix
		coMat = new SimpleMatrix(0, 0); //CO matrix
		dpMat = new SimpleMatrix(0, 0); //DP matrix
		epMat = new SimpleMatrix(0, 0); //EP matrix

		emptyTime = new HashMap<String, Integer>();

		sameSetTimeStamps = new HashMap<Set<String>, Integer>();
		multiGranWindow = new HashMap<Set<String>, Integer>();
		//		multiGranularDiffEvent = new HashMap<String, Integer>();
		actOccCount = new HashMap<String, Integer>();

		matIndexMap = new HashMap<String, Integer>();
		granularity = new ArrayList<Map<String, Integer>>();
		for (int i = 0; i < 5; i++) {
			granularity.add(new HashMap<String, Integer>()); //month,day,hour,min,sec
			actTimeBins.add(new HashMap<String, Map<Double, Double>>());
		}

		metricsTableModel = new DefaultTableModel();
		metricsTableModel.addColumn("Case ID");
		metricsTableModel.addColumn("Activity");
		metricsTableModel.addColumn("TimeStamp");

		hideWhiteList = true;
		initMats(acts);

	}

	public Map<Set<String>, Integer> getSameSetTimeStamps() {
		return sameSetTimeStamps;
	}

	public void setSameSetTimeStamps(Queue<XEvent> evts) {
		Set<String> acts = new HashSet<String>();

		for (XEvent e : evts)
			acts.add(XConceptExtension.instance().extractName(e));

		sameSetTimeStamps.put(acts, sameSetTimeStamps.containsKey(acts) ? sameSetTimeStamps.get(acts) + 1 : 1);
	}

	public Map<String, Integer> getEmptyTime() {
		return emptyTime;
	}

	public void addEmptyTime(XEvent evt) {
		String act = XConceptExtension.instance().extractName(evt);
		emptyTime.put(act, emptyTime.containsKey(act) ? emptyTime.get(act) + 1 : 1);
	}

	public void updateGranularity(XEvent evt) {
		String act = XConceptExtension.instance().extractName(evt);
		Date d = XTimeExtension.instance().extractTimestamp(evt);
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(d);
		if (calendar.get(Calendar.SECOND) != 0) {
			granularity.get(4).put(act, granularity.get(4).containsKey(act) ? granularity.get(4).get(act) + 1 : 1);
			//			secGran.add(act);
		} else if (calendar.get(Calendar.MINUTE) != 0) {
			granularity.get(3).put(act, granularity.get(3).containsKey(act) ? granularity.get(3).get(act) + 1 : 1);
			//			minGran.add(act);
		} else if (calendar.get(Calendar.HOUR) != 0) {
			granularity.get(2).put(act, granularity.get(2).containsKey(act) ? granularity.get(2).get(act) + 1 : 1);
			//			hourGran.add(act);
		} else if (calendar.get(Calendar.DAY_OF_WEEK) != 0) {
			granularity.get(1).put(act, granularity.get(1).containsKey(act) ? granularity.get(1).get(act) + 1 : 1);
			//			dayGran.add(act);
		} else if (calendar.get(Calendar.MONTH) != 0) {
			granularity.get(0).put(act, granularity.get(0).containsKey(act) ? granularity.get(0).get(act) + 1 : 1);
			//			monthGran.add(act);
		}
	}

	public void setMultiGranTimeStamps(Queue<XEvent> evts) {
		Set<String> acts = new HashSet<String>();

		for (XEvent e : evts)
			acts.add(XConceptExtension.instance().extractName(e));

		multiGranWindow.put(acts, multiGranWindow.containsKey(acts) ? multiGranWindow.get(acts) + 1 : 1);
	}

	public static void initMats(Set<String> acts) {
		int i = 0;

		dfMat.reshape(acts.size(), acts.size());
		efMat.reshape(acts.size(), acts.size());
		epMat.reshape(acts.size(), acts.size());
		dpMat.reshape(acts.size(), acts.size());
		coMat.reshape(acts.size(), acts.size());
		for (String act : acts) {
			matIndexMap.put(act, i++);
		}
	}

	public DefaultTableModel getMetricsTableModel() {
		return metricsTableModel;
	}

	public void setMetricsTableModel(DefaultTableModel metricsTableModel) {
		this.metricsTableModel = metricsTableModel;
	}

	public void detect(XLog log, int k) {

		//		XLogInfo info = ipn.getLogInfo().info;
		//		Date endDate = info.getLogTimeBoundaries().getEndDate();
		//		Date startDate = info.getLogTimeBoundaries().getStartDate();
		DetectionModel dm = this;
		LinkedList<XEvent> queueMultiGran = new LinkedList<XEvent>();
		LinkedList<XEvent> queueSameTime = new LinkedList<XEvent>();

		for (XTrace trace : log) {
			Set<XEvent> prevEvents = new HashSet<XEvent>();
			XEvent prevEvt = null;

			for (XEvent event : trace) {

				String act = XConceptExtension.instance().extractName(event);

				actOccCount.put(act, actOccCount.containsKey(act) ? actOccCount.get(act) + 1 : 1);
				Date time = XTimeExtension.instance().extractTimestamp(event);

				//check if timestamp is empty 
				if (time == null)
					dm.addEmptyTime(event);

				//update granularity
				dm.updateGranularity(event);

				//check for multigranular timestamps in the queue - compare curr ele with last ele of queue
				if (!queueMultiGran.isEmpty()
						&& DetectionUtils.getDateGranularity(XTimeExtension.instance().extractTimestamp(
								queueMultiGran.peek())) != DetectionUtils.getDateGranularity(time)) {
					if (queueMultiGran.size() > 1) //if already has multi gran issues, then remove the prior version and add this one as this is bigger
						dm.removeMultiGranTimeStamps(queueMultiGran);
					queueMultiGran.add(event);
					dm.setMultiGranTimeStamps(queueMultiGran);
				} else {
					queueMultiGran.clear();
					queueMultiGran.add(event);
				}
				//check if same timestamp values in the queue
				if (!queueSameTime.isEmpty()
						&& time == XTimeExtension.instance().extractTimestamp(queueSameTime.peek())) {
					if (queueSameTime.size() > 1)
						dm.removeSameTimeStamps(queueSameTime);
					queueSameTime.add(event);
					dm.setSameSetTimeStamps(queueSameTime);
				} else {
					queueSameTime.clear();
					queueSameTime.add(event);
				}

				//update relations				
				if (prevEvt != null) {
					DetectionUtils.updateDirectRelations(prevEvt, event, dm.dfMat, dm.dpMat, dm.matIndexMap);
					DetectionUtils.updateEventualAndCoRelations(prevEvents, event, dm.efMat, dm.epMat, dm.coMat,
							dm.matIndexMap);
				}
				prevEvt = event;
				prevEvents.add(event);

				//update actTimeBins - for statistical test
				updateActTimeBins(event);

				//add to metrics model
				dm.getMetricsTableModel().addRow(
						(new Object[] { XConceptExtension.instance().extractName(trace),
								XConceptExtension.instance().extractName(event), time }));

			}
		}

		//find reasons using the info above
		//empty timestamps
		for (String s : emptyTime.keySet())
			reasons.add(new Reason(s, emptyTime.get(s), 0, ReasonMapping.Reasons.NoTimeStampValue));

		//same timestamp
		for (Set<String> acts : sameSetTimeStamps.keySet())
			reasons.add(new Reason(new ArrayList<String>(acts), sameSetTimeStamps.get(acts), 0,
					ReasonMapping.Reasons.SameTimeStampValues));

		//multi gran for a seq of events 
		for (Set<String> acts : multiGranWindow.keySet())
			reasons.add(new Reason(new ArrayList<String>(acts), multiGranWindow.get(acts), 0,
					ReasonMapping.Reasons.MixedGranularityOfDifferentEventsSequence));

		//analyze same event issues
		for (String s : allActs) {
			int max = -1;
			int sum = 0;
			for (int i = 0; i < 5; i++) {
				int val = granularity.get(i).containsKey(s) ? granularity.get(i).get(s) : 0;
				if (val > max)
					max = val;
				sum += val;
			}
			Set<String> acts = new HashSet<String>();
			acts.add(s);
			if (max != sum)
				reasons.add(new Reason(new ArrayList<String>(acts), (sum - max), 0,
						ReasonMapping.Reasons.MixedGranularityOfSameEvent));
		}

		int[] allGran = new int[5];//month,day,hr,min,sec
		for (String s : allActs) {
			for (int i = 0; i < 5; i++)
				allGran[i] += granularity.get(i).containsKey(s) ? granularity.get(i).get(s) : 0;
		}
		int max = -1;
		int sum = 0;
		for (int i = 0; i < 5; i++) {
			if (allGran[i] > max)
				max = allGran[i];
			sum += allGran[i];
		}
		Set<String> actsFromMax = new HashSet<String>();
		for (int i = 0; i < 5; i++) {
			if (allGran[i] == max)
				actsFromMax.addAll(granularity.get(i).keySet());
		}
		Set<String> acts = new HashSet<String>(allActs);
		acts.removeAll(actsFromMax);
		if (max != sum)
			reasons.add(new Reason(new ArrayList<String>(acts), (sum - max), 0,
					ReasonMapping.Reasons.MixedGranularityOfDifferentEvents));

		MatrixUtils.printMatrix(dfMat);
		//analyze df,dp,ef,ep,co matrices
		for (int r = 0; r < dfMat.numRows(); r++) {
			int thresholdVal = valsAboveThreshold(dfMat, r);
			//find cols which do not meet the threshold
			Set<Integer> colsBelowThreshold = new HashSet<Integer>();
			for (int c = 0; c < dfMat.numCols(); c++) {
				if (dfMat.get(r, c) != 0 && dfMat.get(r, c) < thresholdVal)
					colsBelowThreshold.add(c);
			}

			Set<Integer> problematicCols = new HashSet<Integer>();

			//use reverse matrix (precedes) to find the threshold for problematic cols
			for (int r2 : colsBelowThreshold) {
				thresholdVal = valsAboveThreshold(dpMat, r2);
				if (dpMat.get(r2, r) < thresholdVal) {
					List<String> acts2 = new ArrayList<String>();
					for (String s : matIndexMap.keySet())
						if (matIndexMap.get(s) == r)
							acts2.add(s);
					for (String s : matIndexMap.keySet())
						if (matIndexMap.get(s) == r2)
							acts2.add(s);
					reasons.add(new Reason(acts2, (int) dpMat.get(r2, r), 0,
							ReasonMapping.Reasons.DirectlyRelationsIssues));
				}
			}
		}

		//analyze ef/ep
		for (int r = 0; r < efMat.numRows(); r++) {
			int thresholdVal = valsAboveThreshold(efMat, r);
			//find cols which do not meet the threshold
			Set<Integer> colsBelowThreshold = new HashSet<Integer>();
			for (int c = 0; c < efMat.numCols(); c++) {
				if (efMat.get(r, c) != 0 && efMat.get(r, c) < thresholdVal)
					colsBelowThreshold.add(c);
			}

			Set<Integer> problematicCols = new HashSet<Integer>();

			//use reverse matrix (precedes) to find the threshold for problematic cols
			for (int r2 : colsBelowThreshold) {
				thresholdVal = valsAboveThreshold(epMat, r2);
				if (epMat.get(r2, r) < thresholdVal) {
					List<String> acts2 = new ArrayList<String>();
					for (String s : matIndexMap.keySet())
						if (matIndexMap.get(s) == r)
							acts2.add(s);
					for (String s : matIndexMap.keySet())
						if (matIndexMap.get(s) == r2)
							acts2.add(s);
					reasons.add(new Reason(acts2, (int) epMat.get(r2, r), 0,
							ReasonMapping.Reasons.EventualRelationsIssues));
				}
			}

		}

		//perform statistical tests based on five values - pairwise - act vs rest of the acts
		doStatisticalTests();

		//remove reasons which have been whitelisted
		if (hideWhiteList)
			removeWhiteListsFromRepair();

	}

	private void removeWhiteListsFromRepair() {
		Set<Reason> reasonsToRemove = new HashSet<Reason>();
		for (Reason r : whiteListedReasons) {
			for (Reason r2 : reasons) {
				if (r.getActivities().equals(r2.getActivities()) && r.getReason().equals(r2.getReason()))
					reasonsToRemove.add(r2);
			}
		}
		reasons.remove(reasonsToRemove);
	}

	private void doStatisticalTests() {
		for (int i = 0; i < 5; i++) {//month of year, date of month, day of week, hour of day, min of day
			for (String act : actTimeBins.get(i).keySet()) {
				//create act -sample1 data
				double[] d1 = new double[actTimeBins.get(i).get(act).size()];
				double[] w1 = new double[actTimeBins.get(i).get(act).size()];

				int j = -1;
				for (double key : actTimeBins.get(i).get(act).keySet()) {
					j++;
					d1[j] = key;
					w1[j] = actTimeBins.get(i).get(act).get(key);
				}

				//calc the other sample excluding act
				Map<Double, Double> aggMap = new HashMap<Double, Double>();
				for (String act2 : actTimeBins.get(i).keySet())
					if (!act.equals(act2))
						for (double key : actTimeBins.get(i).get(act2).keySet())
							aggMap.put(key, aggMap.containsKey(key) ? aggMap.get(key)
									+ actTimeBins.get(i).get(act2).get(key) : actTimeBins.get(i).get(act2).get(key));

				double[] d2 = new double[aggMap.size()];
				double[] w2 = new double[aggMap.size()];
				j = -1;
				for (double key : aggMap.keySet()) {
					j++;
					d2[j] = key;
					w2[j] = aggMap.get(key);
				}

//				ComparisonData data1 = new ComparisonData(d1, w1, 1);
//				ComparisonData data2 = new ComparisonData(d2, w2, 1);
//				KuiperComparisonAlgorithm s = new KuiperComparisonAlgorithm();
//				IComparisonResult result = s.compare(data1, data2, "rejectionLevel=0.01"); //rejectionLevel=0.05
//				result.nDof();
//				if (!result.isMatching()) {
//					List<String> actsTemp = new ArrayList<String>();
//					actsTemp.add(act);
//					switch (i) {
//						case 0 :
//							reasons.add(new Reason(actsTemp, 99, 99, Reasons.StatsRelatedDayOfMonth));
//							break;
//						case 1 :
//							reasons.add(new Reason(actsTemp, 99, 99, Reasons.StatsRelatedDayOfMonth));
//							break;
//						case 2 :
//							reasons.add(new Reason(actsTemp, 99, 99, Reasons.StatsRelatedDayOfWeek));
//							break;
//						case 3 :
//							reasons.add(new Reason(actsTemp, 99, 99, Reasons.StatsRelatedHour));
//							break;
//						case 4 :
//							reasons.add(new Reason(actsTemp, 99, 99, Reasons.StatsRelatedMin));
//							break;
//					}
//					//					reasons.add(new Reason(actsTemp, 99, 99, Reasons.StatsRelated));
//				}

			}
		}

	}

	private void updateActTimeBins(XEvent evt) {
		String act = XConceptExtension.instance().extractName(evt);
		Date d = XTimeExtension.instance().extractTimestamp(evt);

		Calendar c = Calendar.getInstance();

		c.setTime(d);
		//month of year, date of month, day of week, hour of day, min of day
		for (int i = 0; i < 5; i++) {
			Map<Double, Double> map = actTimeBins.get(i).containsKey(act) ? actTimeBins.get(i).get(act)
					: new HashMap<Double, Double>();
			switch (i) {
				case 0 :
					map.put((double) c.get(c.MONTH),
							map.containsKey((double) c.get(c.MONTH)) ? map.get((double) c.get(c.MONTH)) + 1 : 1);
					break;
				case 1 :
					map.put((double) c.get(c.DAY_OF_MONTH),
							map.containsKey((double) c.get(c.DAY_OF_MONTH)) ? map.get((double) c.get(c.DAY_OF_MONTH)) + 1
									: 1);
					break;
				case 2 :
					map.put((double) c.get(c.DAY_OF_WEEK),
							map.containsKey((double) c.get(c.DAY_OF_WEEK)) ? map.get((double) c.get(c.DAY_OF_WEEK)) + 1
									: 1);
					break;
				case 3 :
					map.put((double) c.get(c.HOUR_OF_DAY),
							map.containsKey((double) c.get(c.HOUR_OF_DAY)) ? map.get((double) c.get(c.HOUR_OF_DAY)) + 1
									: 1);
					break;
				case 4 :
					map.put((double) c.get(c.MINUTE),
							map.containsKey((double) c.get(c.MINUTE)) ? map.get((double) c.get(c.MINUTE)) + 1 : 1);
					break;
			}
			actTimeBins.get(i).put(act, map);
		}

	}

	private void removeMultiGranTimeStamps(LinkedList<XEvent> queueMultiGran) {
		Set<String> acts = new HashSet<String>();
		for (XEvent e : queueMultiGran)
			acts.add(XConceptExtension.instance().extractName(e));
		multiGranWindow.remove(acts);

	}

	private void removeSameTimeStamps(LinkedList<XEvent> queueSameTime) {
		Set<String> acts = new HashSet<String>();
		for (XEvent e : queueSameTime)
			acts.add(XConceptExtension.instance().extractName(e));
		sameSetTimeStamps.remove(acts);

	}

	private static int valsAboveThreshold(SimpleMatrix mat, int r) {
		List<Integer> items = new ArrayList<Integer>();
		int thresholdVal = 0;
		double total = 0.;
		for (int c = 0; c < mat.numCols(); c++) {
			total += mat.get(r, c);
			items.add((int) (mat.get(r, c)));
		}
		Collections.sort(items);
		double sumTemp = 0.;
		for (int i : items) {
			sumTemp += i;
			if (((double) sumTemp / (double) total) > 0.7)
				return i;
		}
		return -1;
	}

	public List<Reason> getReasons() {
		return reasons;
	}

	public void setReasons(List<Reason> reasons) {
		this.reasons = reasons;
	}

	private static void compare() {

//		IAnalysisFactory af = IAnalysisFactory.create();
//		IHistogramFactory hf = af.createHistogramFactory(af.createTreeFactory().create());
//
//		IHistogram1D h2 = null;
//		double[] d = { 1., 1. };
//		ComparisonData d1 = new ComparisonData(d, d, 1);
//		ComparisonData d2 = new ComparisonData(d, d, 1);
//		KuiperComparisonAlgorithm s = new KuiperComparisonAlgorithm();
//		s.compare(d1, d2, "");
	}

	public List<Reason> getWhiteListedReasons() {
		return whiteListedReasons;
	}

	public void setWhiteListedReasons(List<Reason> whiteListedReasons) {
		this.whiteListedReasons = whiteListedReasons;
	}
}
