package org.processmining.dataquality;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.RowFilter.ComparisonType;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.processmining.dataquality.Reasons.Reason;
import org.processmining.dataquality.Reasons.ReasonMapping;
import org.processmining.dataquality.Reasons.ReasonMapping.Reasons;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.processanalytics.SwingGraph;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class DetectionPanel extends ProMSplitPane {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3697882196100566798L;
	//view names
	public final String selectview = "Table view";
	public final String comparemodellogview = "Compare model-log view";
	public final String exploretracevarinats = "Explore trace variants (LogEnhancements)";
	public final String projectalignmenttolog = "Project Alignment to Log(PNetReplayer)";
	public final String projectmodelwithalignments = "Model Projected with Alignments(PNetReplayer)";

	//view names end

	private PNetVisualizerController controller;
	private InteractivePetriNet ipn;
	private DefaultTableModel tableModel;
	public ProMTable table;
	private List<DirectedGraphElement> graphElementsPerRowCondition = new ArrayList<>();
	private List<DirectedGraphElement> graphElementsPerRowConsequence = new ArrayList<>();
	private List<Integer> relationTypePerRow = new ArrayList<>();
	private TableRowSorter<TableModel> sorter;
	private Map<Integer, Float> minValuesPerColumn = new HashMap<>();

	private Dimension singleSize, doubleSize, tripleSize;
	private ProMTextField nameFilter = new ProMTextField("Activities");
	private JCheckBox showWhiteList = new JCheckBox("Show whitelisted items?");
	private String[] relationTypes = { selectview, comparemodellogview, exploretracevarinats, projectalignmenttolog,
			projectmodelwithalignments };
	private JComboBox switchRelationTypeBox = SlickerFactory.instance().createComboBox(relationTypes);
	private JLabel selectedTranPair = new JLabel();
	private JLabel selctedTranPairMetrics = new JLabel();

	JPanel logPanel; //detection log panel

	JPanel dgPanel; //detection graph panel
	//	DetectionGraphPanel dgPanel;
	//	private JButton analyzeResults = new JButton("Explore");
	//	JButton home = new JButton("Home");

	//	private JCheckBox toggleProcessNameBox = SlickerFactory.instance().createCheckBox("Show process names", false);
	//	private ProMTextField nameFilter = new ProMTextField("Filter by name");
	//	private ProMTextField minSupport = new ProMTextField("Minimum Support");
	//	private ProMTextField minConfidence = new ProMTextField("Minimum Confidence");
	//	private ProMTextField minLift = new ProMTextField("Minimum Lift");
	//	private ProMTextField minConviction = new ProMTextField("Minimum Conviction");
	//	private ProMTextField minCosine = new ProMTextField("Minimum Cosine");
	//	private ProMTextField minJaccard = new ProMTextField("Minimum Jaccard");
	//	private ProMTextField minPhi = new ProMTextField("Minimum Phi");
	Set<TransPairAlignment> tranPairs = null;

	//	private ProMTextField minSignificance = new ProMTextField("Minimum Significance");

	@SuppressWarnings("serial")
	public DetectionPanel(PNetVisualizerController controller) {
		super(ProMSplitPane.VERTICAL_SPLIT);
		int width = 150;

		this.dgPanel = new JPanel();
		//dgPanel.setPreferredSize(new Dimension(800, 350));
		this.tableModel = new BooleanTableModel();
		//		this.tableModel = new DefaultTableModel();
		this.table = new ProMTable(tableModel);
		this.ipn = controller.getIPN();
		this.controller = controller;
		this.tranPairs = ipn.getTranPairs();
		JPanel panel = new JPanel(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.weightx = 5;
		constraints.gridwidth = 2;
		panel.add(switchRelationTypeBox, constraints);
		constraints.weightx = 3;
		constraints.gridwidth = 3;
		panel.add(nameFilter, constraints);
		//		panel.add(analyzeResults, constraints);
		panel.add(selectedTranPair, constraints);
		panel.add(selctedTranPairMetrics, constraints);
		panel.add(showWhiteList, constraints);
		panel.setPreferredSize(new Dimension(width, 23));
		panel.setMinimumSize(new Dimension(width, 23));
		panel.setBackground(Color.LIGHT_GRAY);

		//top comp
		setTopComponent(panel);
		setResizeWeight(0.0);
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));

		int columnCount = table.getColumnModel().getColumnCount();
		singleSize = new Dimension((width - 150) / (columnCount + 4), 23);
		doubleSize = new Dimension((width - 40) * 2 / (columnCount + 4), 23);
		tripleSize = new Dimension((width - 120) * 3 / (columnCount + 4), 23);
		switchRelationTypeBox.setMinimumSize(singleSize);
		switchRelationTypeBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateSelectedView();
			}

		});
		nameFilter.getDocument().addDocumentListener(new NameFilterDocumentListener());

		//top comp ends

		//		try {
		//			ImageIcon img = ImageHelper.loadImage("home.png");
		//			home.setIcon(img);
		//		} catch (Exception ex) {
		//			System.out.println(ex);
		//		}

		setBottomComponent(table);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getSelectionModel().addListSelectionListener(new RowListener());
		table.setMinimumSize(new Dimension(200, 40));
		table.setPreferredSize(new Dimension(1000, 450));
		table.getTableHeader().setPreferredSize(new Dimension(1000, 20));
		table.getTableHeader().setReorderingAllowed(false);

		sorter = new TableRowSorter<TableModel>(table.getTable().getModel());
		table.setRowSorter(sorter);

		tableModel.addColumn("id");
		tableModel.addColumn("Issue");
		tableModel.addColumn("Associated activities");
		//		tableModel.addColumn("Number of cases");
		tableModel.addColumn("Affected number of instances");
		//		tableModel.addColumn("Total number of events");
		//		tableModel.addColumn("% of events");
		tableModel.addColumn("Reason");
		tableModel.addColumn("WhiteList?");
		//		metricsTableModel.addColumn("Activities");
		//		metricsTableModel.addColumn("Fitness");
		//		metricsTableModel.addColumn("Precision");
		//		metricsTableModel.addColumn("Severity");
		//		metricsTableModel.addColumn("Average Duration");

		TableColumn tc = table.getColumnModel().getColumn(5);
		tc.setCellEditor(table.getTable().getDefaultEditor(Boolean.class));
		tc.setCellRenderer(table.getTable().getDefaultRenderer(Boolean.class));
		tc.setHeaderRenderer(new CheckBoxHeader(new CheckBoxListner()));

		Enumeration<TableColumn> columns = table.getColumnModel().getColumns();
		while (columns.hasMoreElements()) {
			final TableColumn column = columns.nextElement();
			column.setPreferredWidth(width / (columnCount + 4));
			column.setCellEditor(new DefaultCellEditor(new JCheckBox()) {
				//				public void addCellEditorListener(CellEditorListener l) {
				//
				////					if (!column.getHeaderValue().equals("WhiteList?")) {
				////						this.cancelCellEditing();
				////					} else {
				////						//update whitlists here
				////						updateWhiteListItems();
				////					}
				//				}

				protected void fireEditingStopped() {
					super.fireEditingStopped();
					//					if (!column.getHeaderValue().equals("WhiteList?")) {
					//						this.cancelCellEditing();
					//					} else {
					//update whitlists here
					updateWhiteListItems();
					//					}

				}

				protected void fireEditingCanceled() {
					super.fireEditingCanceled();
				}

			});
		}

		//		table.getColumnModel().getColumn(0).setPreferredWidth(3 * (width / (columnCount + 4)));
		//		table.getColumnModel().getColumn(1).setPreferredWidth(3 * (width / (columnCount + 4)));

		addRelations();

		List<RowSorter.SortKey> sortKeys = new ArrayList<>(25);
		sortKeys.add(new RowSorter.SortKey(4, SortOrder.ASCENDING));
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		sorter.setSortKeys(sortKeys);

	}

	private void updateWhiteListItems() {
		DetectionModel dm = ipn.getDqModel().getDetectionModel();
		String val = table.getValueAt(table.getSelectedRow(), table.getSelectedColumn()).toString();
		String id = table.getValueAt(table.getSelectedRow(), 0).toString();
		String reason = table.getValueAt(table.getSelectedRow(), 1).toString();
		String activities = table.getValueAt(table.getSelectedRow(), 2).toString();

		Reason removeReason = null;
		boolean add = true;
		for (Reason r : dm.getWhiteListedReasons()) {
			if (r.getActivities().toString().equals(activities) && r.getReason().toString().equals(reason)) {
				if (val.equalsIgnoreCase("n")) {
					removeReason = r; //remove
				} else if (val.equalsIgnoreCase("y")) {
					add = false; //already present no need to add
				}
			}
		}
		if (removeReason != null)
			dm.getWhiteListedReasons().remove(removeReason);
		else if (add && (val.equalsIgnoreCase("y"))) {
			dm.getWhiteListedReasons().add(dm.getReasons().get(Integer.valueOf(id)));
		}
	}

	private void updateSelectedView() {
		TransPairAlignment tpa = getTranPairAlignment();
		if (tpa == null) {
			JOptionPane.showMessageDialog(null, "Please select a valid row from the table to proceed",
					"No value selected", JOptionPane.ERROR_MESSAGE);
			switchRelationTypeBox.setSelectedItem(selectview);
			return;
		}

	}

	private TransPairAlignment getTranPairAlignment() {
		TransPairAlignment tpa = null;
		String activities = selectedTranPair.getText();
		String[] activitiesList = activities.split(",");
		Set<String> stringSet = new HashSet<String>();
		for (int i = 0; i < activitiesList.length; i++) {
			String act = activitiesList[i];
			if (i == 0) {
				if (act == "") //if no act selected
					return null;
				act = act.substring(1);
			} else if (i == activitiesList.length - 1) {
				act = act.substring(0, act.length() - 1);
			}
			act = act.replaceAll("^\\s+", "");
			act = act.replaceAll("\\s+$", "");
			stringSet.add(act);
		}

		for (TransPairAlignment t : tranPairs) {
			if (t.getActivities().containsAll(stringSet))
				tpa = t;
		}

		return tpa;
	}

	private void addRelations() {
		final DetectionModel dm = ipn.getDqModel().getDetectionModel();

		for (final Reason r : ipn.getDqModel().getDetectionModel().getReasons()) {
			int totalNumOfEvts = 1;
			switch (r.getReason()) {
				case CoarseGrainedTimeStamps :
					break;
				case DirectlyRelationsIssues :
					break;
				case EventualRelationsIssues :
					break;
				case MixedGranularityOfDifferentEvents :
				case MixedGranularityOfDifferentEventsSequence :
				case MixedGranularityOfSameEvent :
				case NoTimeStampValue :
				case SameTimeStampValues :
					for (String s : r.getActivities())
						for (Map<String, Integer> g : dm.granularity)
							totalNumOfEvts += g.containsKey(s) ? g.get(s) : 0;

					break;
				case StatsRelatedDayOfMonth :
					break;
				case StatsRelatedDayOfWeek :
					break;
				case StatsRelatedHour :
					break;
				case StatsRelatedMin :
					break;
				case StatsRelatedMonth :
					break;
				default :
					break;

			}
			boolean add = true;
			Reasons reason = r.getReason();
			List<String> activities = r.getActivities();

			for (Reason r2 : dm.getWhiteListedReasons())
				if (r2.getActivities().equals(activities) && r2.getReason().equals(reason))
					add = false;

			if (add || !showWhiteList.isSelected())
				tableModel.addRow(new Object[] { dm.getReasons().indexOf(r), r.getReason(), r.getActivities(),
						r.getNumIssues(), ReasonMapping.getReason(r.getReason()), Boolean.FALSE });

		}
	}

	private RowFilter<Object, Object> createMinValueFilter() {
		RowFilter<Object, Object> filter = null;
		List<RowFilter<Object, Object>> filters = new ArrayList<>();
		for (Integer index : minValuesPerColumn.keySet()) {
			List<RowFilter<Object, Object>> greaterEqual = new ArrayList<>();
			greaterEqual.add(RowFilter.numberFilter(ComparisonType.AFTER, minValuesPerColumn.get(index), index));
			greaterEqual.add(RowFilter.numberFilter(ComparisonType.EQUAL, minValuesPerColumn.get(index), index));
			filters.add(RowFilter.orFilter(greaterEqual));
		}

		if (filters.size() > 0) {
			filter = RowFilter.andFilter(filters);
		}

		return filter;
	}

	private RowFilter<Object, Object> createCombinedFilter() {
		List<RowFilter<Object, Object>> filters = new ArrayList<>();
		//		filters.add(createRelationTypeFilter());
		//		filters.add(createNameFilter());

		RowFilter<Object, Object> valueFilter = createMinValueFilter();
		if (valueFilter != null) {
			filters.add(valueFilter);
		}

		return RowFilter.andFilter(filters);
	}

	public JLabel getSelectedTranPair() {
		return selectedTranPair;
	}

	public void setSelectedTranPair(JLabel selectedTranPair) {
		this.selectedTranPair = selectedTranPair;
	}

	private class NameFilterDocumentListener implements DocumentListener {

		private void newFilter() {
			RowFilter<TableModel, Object> rf = null;
			//If current expression doesn't parse, don't update.
			try {
				rf = RowFilter.regexFilter(nameFilter.getText(), 2);
			} catch (java.util.regex.PatternSyntaxException e) {
				return;
			}
			sorter.setRowFilter(rf);
		}

		public void insertUpdate(DocumentEvent e) {
			newFilter();

		}

		public void removeUpdate(DocumentEvent e) {
			newFilter();

		}

		public void changedUpdate(DocumentEvent e) {
			newFilter();

		}

	}

	private class RowListener implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent event) {

			//TODO get the graph here!
			if (table.getSelectedRow() < 0) {
				return;
			}

			int modelIndex = table.getTable().convertRowIndexToModel(table.getSelectedRow());
			if (event.getValueIsAdjusting()) {
				return;
			}

			int index = controller.getIPN().getDqModel().getDetectionModel().getReasons()
					.indexOf(tableModel.getValueAt(modelIndex, 0));

			Reason r = controller.getIPN().getDqModel().getDetectionModel().getReasons()
					.get((int) tableModel.getValueAt(modelIndex, 0));

			Map<String, Map<String, Double>> hist = new HashMap<String, Map<String, Double>>();

			DetectionModel dm = controller.getIPN().getDqModel().getDetectionModel();
			int i2 = -1;
			String a1, a2;
			int a1i, a2i;
			double count;
			switch (r.getReason()) {
				case CoarseGrainedTimeStamps :
					break;
				case DirectlyRelationsIssues :

					a1 = r.getActivities().get(0);
					a2 = r.getActivities().get(1);
					a1i = dm.matIndexMap.get(a1);
					a2i = dm.matIndexMap.get(a2);
					hist.put(a1 + "->" + a2, new HashMap<String, Double>());
					hist.get(a1 + "->" + a2).put(a1 + "->" + a2, dm.dfMat.get(a1i, a2i));
					count = 0;
					for (int j = 0; j < dm.dfMat.numCols(); j++)
						if (j != a2i)
							count += dm.dfMat.get(a1i, j);
					hist.put(a1 + "->other", new HashMap<String, Double>());
					hist.get(a1 + "->other").put(a1 + "->other", count);
					break;
				case EventualRelationsIssues :
					a1 = r.getActivities().get(0);
					a2 = r.getActivities().get(1);
					a1i = dm.matIndexMap.get(a1);
					a2i = dm.matIndexMap.get(a2);
					hist.put(a1 + "->" + a2, new HashMap<String, Double>());
					MatrixUtils.printMatrix(dm.epMat);
					hist.get(a1 + "->" + a2).put(a1 + "->" + a2, dm.efMat.get(a1i, a2i));
					count = 0;
					for (int j = 0; j < dm.efMat.numCols(); j++)
						if (j != a2i)
							count += dm.efMat.get(a1i, j);
					hist.put(a1 + "->other", new HashMap<String, Double>());
					hist.get(a1 + "->other").put(a1 + "->other", count);
					break;
				case MixedGranularityOfDifferentEvents :
				case MixedGranularityOfDifferentEventsSequence :
				case MixedGranularityOfSameEvent :
				case NoTimeStampValue :
				case SameTimeStampValues :
					for (String s : r.getActivities())
						for (int i = 0; i < dm.granularity.size(); i++) { //month,day,hour,min,sec
							Map<String, Integer> gran = dm.granularity.get(i);
							for (String a : gran.keySet()) {
								if (r.getActivities().contains(a) || r.getActivities().equals(a)) {
									if (!hist.containsKey(a))
										hist.put(a, new HashMap<String, Double>());
									Map<String, Double> map = hist.get(a);
									double val = gran.get(a);
									switch (i) {
										case 0 :
											map.put("month", map.containsKey("month") ? val + 1 : val);
											break;
										case 1 :
											map.put("day", map.containsKey("day") ? val + 1 : val);
											break;
										case 2 :
											map.put("hour", map.containsKey("hour") ? val + 1 : val);
											break;
										case 3 :
											map.put("min", map.containsKey("min") ? val + 1 : val);
											break;
										case 4 :
											map.put("sec", map.containsKey("sec") ? val + 1 : val);
											break;

									}
								}
							}
						}

					break;
				case StatsRelatedMonth :
					i2 = 0;
					break;
				case StatsRelatedDayOfMonth :
					i2 = 1;
					break;
				case StatsRelatedDayOfWeek :
					i2 = 2;
					break;
				case StatsRelatedHour :
					i2 = 3;
					break;
				case StatsRelatedMin :
					i2 = 4;

					break;
				default :
					break;

			}
			if (i2 != -1) {
				hist.put("others", new HashMap<String, Double>());
				for (String s : r.getActivities())
					if (!hist.containsKey(s))
						hist.put(s, new HashMap<String, Double>());
				for (String s : dm.actTimeBins.get(i2).keySet()) {
					for (Double key : dm.actTimeBins.get(i2).get(s).keySet())
						if (r.getActivities().contains(s) || r.getActivities().equals(s)) {
							hist.get(s).put(key.toString(), dm.actTimeBins.get(i2).get(s).get(key));
						} else {
							hist.get("others").put(
									key.toString(),
									hist.get("others").containsKey(key.toString()) ? hist.get("others").get(
											key.toString())
											+ dm.actTimeBins.get(i2).get(s).get(key) : dm.actTimeBins.get(i2).get(s)
											.get(key));
						}
				}
			}

			SwingGraph.createHistogramWithStringCategory(getDgPanel(), hist);
			//			controller.getDgPanel().add();
			//			controller.getDgPanel().revalidate();
			//			controller.getDgPanel().repaint();
			//pmd
			TransPairAlignment tpa = null;
			String activities = tableModel.getValueAt(modelIndex, 0).toString();
			activities.replace("[", "");
			activities.replace("]", "");
			String[] activitiesList = activities.split(",");
			Set<String> stringSet = new HashSet<String>();
			for (String act : activitiesList)
				stringSet.add(act);

			System.out.println("here!! ");
			selectedTranPair.setText(activities);
		}
	}

	private static class AlignmentUtils {
		private static double round(double value, int places) {
			if (places < 0)
				throw new IllegalArgumentException();

			long factor = (long) Math.pow(10, places);
			value = value * factor;
			long tmp = Math.round(value);
			return (double) tmp / factor;
		}
	}

	public JPanel getDgPanel() {
		return dgPanel;
	}

	public void setDgPanel(JPanel dgPanel) {
		this.dgPanel = dgPanel;
	}

	class BooleanTableModel extends DefaultTableModel {
		//
		// This method is used by the JTable to define the default
		// renderer or editor for each cell. For example if you have
		// a boolean data it will be rendered as a check box. A
		// number value is right aligned.
		//
		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex == 5)
				return Boolean.class;
			else if (columnIndex == 0 || columnIndex == 3)
				return Integer.class;
			else
				return String.class;
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return columnIndex == 5;
		}
	}

	class CheckBoxHeader extends JCheckBox implements TableCellRenderer, MouseListener {
		protected CheckBoxHeader rendererComponent;
		protected int column;
		protected boolean mousePressed = false;

		public CheckBoxHeader(ItemListener itemListener) {
			rendererComponent = this;
			rendererComponent.addItemListener(itemListener);
		}

		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
				boolean hasFocus, int row, int column) {
			if (table != null) {
				JTableHeader header = table.getTableHeader();
				if (header != null) {
					rendererComponent.setForeground(header.getForeground());
					rendererComponent.setBackground(header.getBackground());
					rendererComponent.setFont(header.getFont());
					header.addMouseListener(rendererComponent);
				}
			}
			setColumn(column);
			rendererComponent.setText("WhiteList?");
			setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			return rendererComponent;
		}

		protected void setColumn(int column) {
			this.column = column;
		}

		public int getColumn() {
			return column;
		}

		protected void handleClickEvent(MouseEvent e) {
			if (mousePressed) {
				mousePressed = false;
				JTableHeader header = (JTableHeader) (e.getSource());
				JTable tableView = header.getTable();
				TableColumnModel columnModel = tableView.getColumnModel();
				int viewColumn = columnModel.getColumnIndexAtX(e.getX());
				int column = tableView.convertColumnIndexToModel(viewColumn);

				if (viewColumn == this.column && e.getClickCount() == 1 && column != -1) {
					doClick();
				}
			}
		}

		public void mouseClicked(MouseEvent e) {
			handleClickEvent(e);
			((JTableHeader) e.getSource()).repaint();
		}

		public void mousePressed(MouseEvent e) {
			mousePressed = true;
		}

		public void mouseReleased(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}
	}

	class CheckBoxListner implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			Object source = e.getSource();
			if (source instanceof AbstractButton == false)
				return;
			boolean checked = e.getStateChange() == ItemEvent.SELECTED;
			for (int x = 0, y = table.getTable().getRowCount(); x < y; x++) {
				table.getTable().setValueAt(new Boolean(checked), x, 5);
			}
		}
	}
}
