package org.processmining.dataquality;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.ejml.simple.SimpleMatrix;

public class DetectionUtils {

	public static int getDateGranularity(Date d) {
		Calendar c = Calendar.getInstance();
		c.setTime(d);

		if (c.get(Calendar.SECOND) != 0)
			return Calendar.SECOND;
		if (c.get(Calendar.MINUTE) != 0)
			return Calendar.MINUTE;
		if (c.get(Calendar.HOUR) != 0)
			return Calendar.HOUR;
		if (c.get(Calendar.DAY_OF_WEEK) != 0)
			return Calendar.DAY_OF_WEEK;
		if (c.get(Calendar.MONTH) != 0)
			return Calendar.MONTH;
		if (c.get(Calendar.YEAR) != 0)
			return Calendar.YEAR;

		return -1;
	}

	public static void addRelations(XEvent evt1, XEvent evt2, Map<String, Map<String, Integer>> map) {
		String act1 = XConceptExtension.instance().extractName(evt1);
		String act2 = XConceptExtension.instance().extractName(evt2);

		if (map.containsKey(act1))
			map.put(act1, new HashMap<String, Integer>());
		map.get(act1).put(act2, map.containsKey(act2) ? map.get(act1).get(act2) + 1 : 1);
	}

	public static void updateDirectRelations(XEvent evt1, XEvent evt2, SimpleMatrix dfMat, SimpleMatrix dpMat,
			Map<String, Integer> posMap) {
		String act1 = XConceptExtension.instance().extractName(evt1);
		String act2 = XConceptExtension.instance().extractName(evt2);
		dfMat.set(posMap.get(act1), posMap.get(act2), dfMat.get(posMap.get(act1), posMap.get(act2)) + 1); //follows relation
		dpMat.set(posMap.get(act2), posMap.get(act1), dpMat.get(posMap.get(act2), posMap.get(act1)) + 1); //precedes relation
	}

	public static void updateEventualAndCoRelations(Set<XEvent> prevEvt, XEvent evt1, SimpleMatrix efMat,
			SimpleMatrix epMat, SimpleMatrix coMat, Map<String, Integer> posMap) {
		String act1 = XConceptExtension.instance().extractName(evt1);
		for (XEvent evt2 : prevEvt) {
			String act2 = XConceptExtension.instance().extractName(evt2);
			efMat.set(posMap.get(act1), posMap.get(act2), efMat.get(posMap.get(act1), posMap.get(act2)) + 1); //follows relation
			epMat.set(posMap.get(act2), posMap.get(act1), epMat.get(posMap.get(act2), posMap.get(act1)) + 1); //precedes relation
			coMat.set(posMap.get(act2), posMap.get(act1), coMat.get(posMap.get(act2), posMap.get(act1)) + 1); //cooccurs relation
			coMat.set(posMap.get(act1), posMap.get(act2), coMat.get(posMap.get(act1), posMap.get(act2)) + 1); //cooccurs relation
		}
	}

	
}
