package org.processmining.dataquality;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.RowFilter.ComparisonType;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class LogPanel extends ProMSplitPane {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3697882196100566798L;
	//view names
	public final String selectview = "Table view";
	public final String comparemodellogview = "Compare model-log view";
	public final String exploretracevarinats = "Explore trace variants (LogEnhancements)";
	public final String projectalignmenttolog = "Project Alignment to Log(PNetReplayer)";
	public final String projectmodelwithalignments = "Model Projected with Alignments(PNetReplayer)";

	//view names end

	private PNetVisualizerController controller;
	private InteractivePetriNet ipn;
	private DefaultTableModel tableModel;
	public ProMTable table;
	private List<DirectedGraphElement> graphElementsPerRowCondition = new ArrayList<>();
	private List<DirectedGraphElement> graphElementsPerRowConsequence = new ArrayList<>();
	private List<Integer> relationTypePerRow = new ArrayList<>();
	private TableRowSorter<TableModel> sorter;
	private Map<Integer, Float> minValuesPerColumn = new HashMap<>();

	private Dimension singleSize, doubleSize, tripleSize;
	private ProMTextField nameFilter = new ProMTextField("Activities");
	private String[] relationTypes = { selectview, comparemodellogview, exploretracevarinats, projectalignmenttolog,
			projectmodelwithalignments };
	private JComboBox switchRelationTypeBox = SlickerFactory.instance().createComboBox(relationTypes);
	private JLabel selectedTranPair = new JLabel();
	private JLabel selctedTranPairMetrics = new JLabel();
	//	private JButton analyzeResults = new JButton("Explore");
	//	JButton home = new JButton("Home");

	//	private JCheckBox toggleProcessNameBox = SlickerFactory.instance().createCheckBox("Show process names", false);
	//	private ProMTextField nameFilter = new ProMTextField("Filter by name");
	//	private ProMTextField minSupport = new ProMTextField("Minimum Support");
	//	private ProMTextField minConfidence = new ProMTextField("Minimum Confidence");
	//	private ProMTextField minLift = new ProMTextField("Minimum Lift");
	//	private ProMTextField minConviction = new ProMTextField("Minimum Conviction");
	//	private ProMTextField minCosine = new ProMTextField("Minimum Cosine");
	//	private ProMTextField minJaccard = new ProMTextField("Minimum Jaccard");
	//	private ProMTextField minPhi = new ProMTextField("Minimum Phi");
//	Set<TransPairAlignment> tranPairs = null;

	//	private ProMTextField minSignificance = new ProMTextField("Minimum Significance");

	@SuppressWarnings("serial")
	public LogPanel(PNetVisualizerController controller,DefaultTableModel metricsTableModel) {
		super(ProMSplitPane.VERTICAL_SPLIT);

		int width = 1150;

		this.tableModel = metricsTableModel;
		this.table = new ProMTable(metricsTableModel);
		this.ipn = controller.getIPN();
		this.controller = controller;
//		this.tranPairs = ipn.getTranPairs();
		JPanel panel = new JPanel(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.weightx = 5;
		constraints.gridwidth = 2;
//		panel.add(switchRelationTypeBox, constraints);
		constraints.weightx = 3;
		constraints.gridwidth = 3;
//		panel.add(nameFilter, constraints);
		//		panel.add(analyzeResults, constraints);
		panel.add(selectedTranPair, constraints);
		panel.add(selctedTranPairMetrics, constraints);
		panel.setPreferredSize(new Dimension(width, 23));
		panel.setMinimumSize(new Dimension(width, 23));
		panel.setBackground(Color.LIGHT_GRAY);

		//top comp
		setTopComponent(panel);
		setResizeWeight(0.0);
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));

		int columnCount = table.getColumnModel().getColumnCount();
		singleSize = new Dimension((width - 150) / (columnCount + 4), 23);
		doubleSize = new Dimension((width - 40) * 2 / (columnCount + 4), 23);
		tripleSize = new Dimension((width - 120) * 3 / (columnCount + 4), 23);
		switchRelationTypeBox.setMinimumSize(singleSize);
		switchRelationTypeBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateSelectedView();
			}

		});
		nameFilter.getDocument().addDocumentListener(new NameFilterDocumentListener());
		//top comp ends

		//		try {
		//			ImageIcon img = ImageHelper.loadImage("home.png");
		//			home.setIcon(img);
		//		} catch (Exception ex) {
		//			System.out.println(ex);
		//		}

		setBottomComponent(table);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getSelectionModel().addListSelectionListener(new RowListener());
		table.setMinimumSize(new Dimension(300, 40));
		table.setPreferredSize(new Dimension(width, 450));
		table.getTableHeader().setPreferredSize(new Dimension(1000, 20));
		table.getTableHeader().setReorderingAllowed(false);

		sorter = new TableRowSorter<TableModel>(table.getTable().getModel());
		table.setRowSorter(sorter);

//		metricsTableModel.addColumn("Case ID");
//		metricsTableModel.addColumn("Activity");
//		metricsTableModel.addColumn("Time");
//		metricsTableModel.addColumn("Severity");
//		metricsTableModel.addColumn("Average Duration");

		Enumeration<TableColumn> columns = table.getColumnModel().getColumns();
		while (columns.hasMoreElements()) {
			final TableColumn column = columns.nextElement();
			column.setPreferredWidth(width / (columnCount + 4));
			column.setCellEditor(new DefaultCellEditor(new JTextField()) {

				protected void fireEditingStopped() {
					this.cancelCellEditing();
					super.fireEditingStopped();
				}

				protected void fireEditingCanceled() {
					super.fireEditingCanceled();
				}

			});
		}
		table.getColumnModel().getColumn(0).setPreferredWidth(3 * (width / (columnCount + 4)));
		table.getColumnModel().getColumn(1).setPreferredWidth(3 * (width / (columnCount + 4)));

//		addRelations();

		List<RowSorter.SortKey> sortKeys = new ArrayList<>(25);
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		sortKeys.add(new RowSorter.SortKey(2, SortOrder.ASCENDING));
		sorter.setSortKeys(sortKeys);

	}

	private void updateSelectedView() {
		TransPairAlignment tpa = getTranPairAlignment();
		if (tpa == null) {
			JOptionPane.showMessageDialog(null, "Please select a valid row from the table to proceed",
					"No value selected", JOptionPane.ERROR_MESSAGE);
			switchRelationTypeBox.setSelectedItem(selectview);
			return;
		}

	}

	private TransPairAlignment getTranPairAlignment() {
		TransPairAlignment tpa = null;
		String activities = selectedTranPair.getText();
		String[] activitiesList = activities.split(",");
		Set<String> stringSet = new HashSet<String>();
		for (int i = 0; i < activitiesList.length; i++) {
			String act = activitiesList[i];
			if (i == 0) {
				if (act == "") //if no act selected
					return null;
				act = act.substring(1);
			} else if (i == activitiesList.length - 1) {
				act = act.substring(0, act.length() - 1);
			}
			act = act.replaceAll("^\\s+", "");
			act = act.replaceAll("\\s+$", "");
			stringSet.add(act);
		}

//		for (TransPairAlignment t : tranPairs) {
//			if (t.getActivities().containsAll(stringSet))
//				tpa = t;
//		}

		return tpa;
	}

//	private void addRelations() {
//		XLog log = ipn.getLog();
//		
//		for(XTrace t: log){
//			for(XEvent e: t){
//				
//			}
//		}
//		for (TransPairAlignment tp : tranPairs) {
//			double severity = 0;
//			double syncMove = 0;
//			double errorMove = 0;
//			for (Transition t : tp.getSyncMoveMap().keySet())
//				syncMove += tp.getSyncMoveMap().get(t);
//			for (Transition t : tp.getModelMove().keySet())
//				errorMove += tp.getModelMove().get(t);
//			for (String t : tp.getLogMoveMap().keySet())
//				errorMove += tp.getLogMoveMap().get(t);
//
//			severity = AlignmentUtils.round((errorMove / (syncMove + errorMove)), 2);
//
//			tableModel.addRow(new Object[] { tp.getSortedActivities(), tp.getFitness(), tp.getPrecision(), severity,
//					"5" });
//		}

//	}

	private RowFilter<Object, Object> createMinValueFilter() {
		RowFilter<Object, Object> filter = null;
		List<RowFilter<Object, Object>> filters = new ArrayList<>();
		for (Integer index : minValuesPerColumn.keySet()) {
			List<RowFilter<Object, Object>> greaterEqual = new ArrayList<>();
			greaterEqual.add(RowFilter.numberFilter(ComparisonType.AFTER, minValuesPerColumn.get(index), index));
			greaterEqual.add(RowFilter.numberFilter(ComparisonType.EQUAL, minValuesPerColumn.get(index), index));
			filters.add(RowFilter.orFilter(greaterEqual));
		}

		if (filters.size() > 0) {
			filter = RowFilter.andFilter(filters);
		}

		return filter;
	}

	private RowFilter<Object, Object> createCombinedFilter() {
		List<RowFilter<Object, Object>> filters = new ArrayList<>();
		//		filters.add(createRelationTypeFilter());
		//		filters.add(createNameFilter());

		RowFilter<Object, Object> valueFilter = createMinValueFilter();
		if (valueFilter != null) {
			filters.add(valueFilter);
		}

		return RowFilter.andFilter(filters);
	}

	public JLabel getSelectedTranPair() {
		return selectedTranPair;
	}

	public void setSelectedTranPair(JLabel selectedTranPair) {
		this.selectedTranPair = selectedTranPair;
	}

	private class NameFilterDocumentListener implements DocumentListener {

		private void newFilter() {
			RowFilter<TableModel, Object> rf = null;
			//If current expression doesn't parse, don't update.
			try {
				rf = RowFilter.regexFilter(nameFilter.getText(), 1);
			} catch (java.util.regex.PatternSyntaxException e) {
				return;
			}
			sorter.setRowFilter(rf);
		}

		public void insertUpdate(DocumentEvent e) {
			newFilter();

		}

		public void removeUpdate(DocumentEvent e) {
			newFilter();

		}

		public void changedUpdate(DocumentEvent e) {
			newFilter();

		}

	}

	private class RowListener implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent event) {
			if (table.getSelectedRow() < 0) {
				return;
			}

			int modelIndex = table.getTable().convertRowIndexToModel(table.getSelectedRow());
			if (event.getValueIsAdjusting()) {
				return;
			}

			//pmd
			TransPairAlignment tpa = null;
			String activities = tableModel.getValueAt(modelIndex, 0).toString();
			activities.replace("[", "");
			activities.replace("]", "");
			String[] activitiesList = activities.split(",");
			Set<String> stringSet = new HashSet<String>();
			for (String act : activitiesList)
				stringSet.add(act);

			System.out.println("here!! ");
			selectedTranPair.setText(activities);
		}
	}

	private static class AlignmentUtils {
		private static double round(double value, int places) {
			if (places < 0)
				throw new IllegalArgumentException();

			long factor = (long) Math.pow(10, places);
			value = value * factor;
			long tmp = Math.round(value);
			return (double) tmp / factor;
		}
	}
}
