package org.processmining.dataquality.Reasons;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.dataquality.RepairBox;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.log.ReSortLog;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.alignments.AlignmentCalculations;
import org.processmining.prabhakardixit.alignments.ProjectedAveragedAlignments;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;

public class CalculateAlignments {
	//	static Set<Long> caseStart = new HashSet<Long>();
	//	static Set<Long> caseEnd = new HashSet<Long>();
	static Map<String, List<Long>> acts1 = new HashMap<String, List<Long>>();
	static Map<String, List<Long>> acts2 = new HashMap<String, List<Long>>();
	static String actToRepair;

	//	static Map<String, Long> acts1 = new HashMap<String, Long>();
	//	static Map<String, Long> acts2 = new HashMap<String, Long>();

	public static PNRepResult calculateAlignments() {
		PNetVisualizerController controller = PNetVisualizerController.getInstance();
		InteractivePetriNet ipn = controller.getIPN();
		Set<String> acts = new HashSet<String>();
		for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
			acts.add(t.getLabel());
		XLog log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), acts, true);
		return calculateAlignments(log);
	}

	public static PNRepResult calculateAlignments(XLog log) {
		PNetVisualizerController controller = PNetVisualizerController.getInstance();

		InteractivePetriNet ipn = controller.getIPN();

		PNRepResult result = AlignmentCalculations.calculateNAlignment1(ipn.getContext(), log, ipn
				.getselectedPetriNetMap().getPetriNet(), 100, false, null, ipn.getDqModel().getRepairModel()
				.getWeights(), false, true);
		System.out.println("The fitness is :" + result.getInfo().get(PNRepResult.TRACEFITNESS));
		return result;
	}

	public static void doRepair() {
		PNetVisualizerController controller = PNetVisualizerController.getInstance();
		InteractivePetriNet ipn = controller.getIPN();
		Set<String> acts = new HashSet<String>();
		acts1.clear();
		acts2.clear();

		//initiate act1 and act2 boxes
		for (RepairBox rb : controller.getIPN().getDqModel().getRepairModel().getRepairs()) {
			if (!rb.getActs1().equals("none"))
				acts1.put(rb.getActs1().getSelectedItem().toString(), new ArrayList<Long>());
			if (!rb.getActs2().equals("none"))
				acts2.put(rb.getActs2().getSelectedItem().toString(), new ArrayList<Long>());
		}

		acts1.put("Case start", new ArrayList<Long>());
		acts2.put("Case end", new ArrayList<Long>());
		actToRepair = ipn.getDqModel().getRepairModel().getActToRepair().getSelectedItem().toString();

		for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
			acts.add(t.getLabel());
		XLog log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), acts, false);
		PNRepResult result = calculateAlignments(log);

		//initial logged containing only problematic traces
		XLog log2 = cloneErroneousLog(ipn.getLog(), log, result, true);

		//get all the repair boxes
		List<RepairBox> rbs = ipn.getDqModel().getRepairModel().getRepairs();
		List<XLog> logs = new ArrayList<XLog>();
		logs.add(log2);
		//		saveLog(log2);
		for (int i = 0; i < rbs.size(); i++) {
			//for each repair box, create an event log
			XLog lTemp;
			if (i == 0)
				lTemp = log2;
			else
				lTemp = logs.get(i - 1);

			//find and annotate problematic events first 
			annotateProblematicEvents(lTemp, rbs.get(i));

			//do the change corresponding to repair box in lTemp - change only problematic events
			changeLog(lTemp, rbs.get(i), controller);

			//re-sort the event log to reflect new timestamps - this is used by alignments calculations
			lTemp = ReSortLog.removeEdgePoints(ipn.getContext(), lTemp);
			//get the alignment scores for changed log
			result = calculateAlignments(lTemp);

			//clone the log to remove the traces 'fixed' until now, and keep only those traces which have issues
			XLog l = cloneErroneousLog(ipn.getLog(), lTemp, result, false);
			logs.add(l);
		}

		XLog newLog = createNewLog(ipn.getLog(), logs);
		newLog = ReSortLog.removeEdgePoints(ipn.getContext(), newLog);
		ipn.getDqModel().getRepairModel().getRepairStatsPanel().addNewTab(newLog);

		saveLog(newLog);

	}

	private static void saveLog(XLog newLog) {
		//first clear the att values
		clearAttributes(newLog);

		PNetVisualizerController controller = PNetVisualizerController.getInstance();
		controller.getIPN().getContext().getProvidedObjectManager()
				.createProvidedObject("Extracted Log", newLog, XLog.class, controller.getIPN().getContext());
		controller.getIPN().getUIContext().getGlobalContext().getResourceManager().getResourceForInstance(newLog)
				.setFavorite(true);
		controller.getIPN().setLog(newLog);
	}

	private static XLog createNewLog(XLog log, List<XLog> logs) {
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		Set<String> tracesChangedIds = new HashSet<String>();
		int eventsAdded = 0;
		int eventsChanged = 0;
		int eventsRemved = 0;

		XLog newLog = factory.createLog();
		Set<Integer> skippedTraces = new HashSet<Integer>();
		Set<Integer> addedTraces = new HashSet<Integer>();

		//populate the skipped traces first - these are the traces present in the LAST log
		for (XTrace t : logs.get(logs.size() - 1)) {
			skippedTraces.add(Integer.valueOf(t.getAttributes().get("OrgId").toString()));
		}

		//		XAttributeBoolean defaultFalse = factory.createAttributeBoolean("false", false, null);

		//start from second last log in logs - and add traces iteratively
		for (int i = logs.size() - 2; i >= 0; i--) {
			for (XTrace t : logs.get(i)) {
				if (!(t.getAttributes().containsKey("removeEvtsTrace"))
						|| !((XAttributeBoolean) t.getAttributes().get("removeEvtsTrace")).getValue()) {
					int id = Integer.valueOf(t.getAttributes().get("OrgId").toString());
					if (!skippedTraces.contains(id) && !addedTraces.contains(id)) {
						XTrace tNew = factory.createTrace(log.get(id).getAttributes());
						tNew.getAttributes().put("OrgId",
								factory.createAttributeLiteral("OrgId", String.valueOf(id), null));
						for (XEvent e : log.get(id)) { //the original log
							XEvent eNew = (XEvent) e.clone();//  factory.createEvent(e.getAttributes());
							tNew.add(eNew);

							String orgEvtId = (e.getID().toString());
							XAttribute att2 = factory
									.createAttributeLiteral("orgEvtId", String.valueOf(orgEvtId), null);
							eNew.getAttributes().put("orgEvtId", att2);
						}

						for (XEvent e : t) {
							if ((e.getAttributes().containsKey("newlyAdded"))
									&& ((XAttributeBoolean) e.getAttributes().get("newlyAdded")).getValue()) {
								//							if (((XAttributeBoolean) e.getAttributes().getOrDefault("newlyAdded", defaultFalse))
								//									.getValue()) {//add this newly created event
								tNew.add(e); //add additional events
								eventsAdded++;
								tracesChangedIds.add(String.valueOf(id));
							} else { //update the timestamp from newly created logs
								String evtId = (e.getAttributes().get("orgEvtId").toString());
								XTimeExtension.instance().assignTimestamp(getEventUsingXIDAsAttribute(evtId, tNew),
										XTimeExtension.instance().extractTimestamp(e)); //change time of events
								eventsChanged++;
								tracesChangedIds.add(String.valueOf(id));
							}

						}
						newLog.add(tNew);
						addedTraces.add(id);
					}

				} else {
					{
						for (XEvent e : t) {
							String evtId = (e.getAttributes().get("orgEvtId").toString());
							String OrgId = (e.getAttributes().get("traceId").toString());
							XTrace tNew = getTraceUsingOrgIDAsAttribute(newLog, OrgId);

							//							if (tNew == null) {
							//								System.out.println();
							//								getTraceUsingOrgIDAsAttribute(newLog, OrgId);
							//								eventsRemved++;
							//							} else {
							tNew.remove(getEventUsingXIDAsAttribute(evtId, tNew));
							tracesChangedIds.add(OrgId);
							eventsRemved++;
							//							}
						}
					}
				}
			}
		}
		//add the remaining traces
		for (int i = 0; i < log.size(); i++) {
			if (!addedTraces.contains(i)) {
				XTrace tNew = factory.createTrace(log.get(i).getAttributes());
				tNew.getAttributes().put("OrgId", factory.createAttributeLiteral("OrgId", String.valueOf(i), null));
				for (XEvent e : log.get(i)) { //the original log
					XEvent eNew = (XEvent) e.clone();//  factory.createEvent(e.getAttributes());
					tNew.add(eNew);

					String orgEvtId = (e.getID().toString());
					XAttribute att2 = factory.createAttributeLiteral("orgEvtId", String.valueOf(orgEvtId), null);
					eNew.getAttributes().put("orgEvtId", att2);
				}
				newLog.add(tNew);
			}
		}

		//add stats
		newLog.getAttributes().put("casesChanged",
				factory.createAttributeLiteral("casesChanged", String.valueOf(tracesChangedIds.size()), null));
		newLog.getAttributes().put("eventsAdded",
				factory.createAttributeLiteral("eventsAdded", String.valueOf(eventsAdded), null));
		newLog.getAttributes().put("eventsRemoved",
				factory.createAttributeLiteral("eventsRemoved", String.valueOf(eventsRemved), null));
		newLog.getAttributes().put("eventsChanged",
				factory.createAttributeLiteral("eventsChanged", String.valueOf(eventsChanged), null));
		return newLog;
	}

	private static XTrace getTraceUsingOrgIDAsAttribute(XLog newLog, String orgId) {
		for (XTrace t : newLog)
			if (((XAttributeLiteral) t.getAttributes().get("OrgId")).getValue().equals(orgId)) {
				return t;
			}
		return null;
	}

	//currently only log moves are annotated as problems, - add implies more model moves , remove implies more log moves
	private static void annotateProblematicEvents(XLog log, RepairBox rb) {
		String a1 = rb.getActs1().getSelectedItem().toString();
		String a2 = rb.getActs2().getSelectedItem().toString();

		PNetVisualizerController controller = PNetVisualizerController.getInstance();

		PNRepResult result = calculateAlignments(log);
		for (SyncReplayResult sync : result) {
			int add = 0;
			int remove = 0;
			boolean addTraces = false;
			for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
				StepTypes type = sync.getStepTypes().get(actIndex);
				Object stepObj = sync.getNodeInstance().get(actIndex);
				switch (type) {
					case L :
						if (stepObj.toString().equals(actToRepair)) {
							remove++;
							addTraces = true;
						}
						break; // unable to project this case, stop
					case MREAL :
						if (stepObj.toString().equals(actToRepair)) {
							add++;
							addTraces = true;
						}
						break; // unable to project this case, stop
				}

			}
			if (addTraces) {
				for (int id : sync.getTraceIndex()) {
					int tot = add;
					XFactory factory = XFactoryRegistry.instance().currentDefault();
					XTrace t = log.get(id);
					int i = 0;
					Set<Integer> parsedEvents = new HashSet<Integer>();
					//navigate again - if log move or model move - then move next to trace, if log move and seleced act - then repair event position
					XEvent prevValidAct1 = null;
					boolean annotateAct2 = false;
					boolean annotateAct1 = true;
					long beforeCount = 0;
					for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
						StepTypes type = sync.getStepTypes().get(actIndex);
						Object stepObj = sync.getNodeInstance().get(actIndex);
						XAttribute atttrue = factory.createAttributeBoolean("repair", true, null);
						XAttribute attfalse = factory.createAttributeBoolean("repair", false, null);
						//						XAttribute attAct1 = factory.createAttributeBoolean("act1", true, null);
						//						XAttribute attAct2 = factory.createAttributeBoolean("act2", true, null);
						XAttribute removeAtt = factory.createAttributeBoolean("remove", true, null);
						//						XAttribute addAfterAtt = factory.createAttributeBoolean("addAfter", true, null);
						//						XAttribute addBeforeAtt = factory.createAttributeBoolean("addBefore", true, null);

						//						XAttribute defaultZero = factory.createAttributeDiscrete("zero", 0, null);
						switch (type) {
							case L :
								t.get(i).getAttributes().put("repair", attfalse);
								if (stepObj.toString().equals(actToRepair)) {
									if (tot > 0) {
										t.get(i).getAttributes().put("repair", atttrue);
									} else {
										t.get(i).getAttributes().put("remove", removeAtt);

									}
									tot--;
								}
								parsedEvents.add(i++);
								break;
							case LMGOOD :
								t.get(i).getAttributes().put("repair", attfalse);
								if (a1.equals(stepObj.toString())) {
									//									t.get(i).getAttributes().put("act1", attAct1);
									prevValidAct1 = t.get(i);
								}
								if (a2.equals(stepObj.toString())) {
									//									t.get(i).getAttributes().put("act2", attAct2);
									if (annotateAct2) {
										//										long l = ((XAttributeDiscrete) t.get(i).getAttributes()
										//												.getOrDefault("addAfter", defaultZero)).getValue();
										long l = t.get(i).getAttributes().containsKey("addBefore") ? ((XAttributeDiscrete) t
												.get(i).getAttributes().get("addBefore")).getValue()
												: 0;
										l += beforeCount;
										XAttribute addBeforeCount = factory.createAttributeDiscrete("addBefore", l,
												null);
										t.get(i).getAttributes().put("addBefore", addBeforeCount);
										annotateAct2 = false;
									}
								}
								parsedEvents.add(i++);
								break;
							case MREAL : //in case of model move, and add>remove, annotated the 
								if (stepObj.toString().equals(actToRepair)) {
									//									add--; //one less item to add
									if (prevValidAct1 != null) {
										//										long l = ((XAttributeDiscrete) prevValidAct1.getAttributes().getOrDefault(
										//												"addAfter", defaultZero)).getValue();
										long l = prevValidAct1.getAttributes().containsKey("addAfter") ? ((XAttributeDiscrete) prevValidAct1
												.getAttributes().get("addAfter")).getValue() : 0;
										l++;
										XAttribute addAfterCount = factory.createAttributeDiscrete("addAfter", l, null);
										prevValidAct1.getAttributes().put("addAfter", addAfterCount);
									}
									if (!a2.equalsIgnoreCase("none")) {
										annotateAct2 = true;
										beforeCount++;
									}
								}

						}
					}
				}
			}
		}

	}

	//if orgLog is null, use log index as orgId - also used to populate the maps and values of act1 differences, act2 differences, case start,end differences
	private static XLog cloneErroneousLog(XLog orgLog, XLog log, PNRepResult result, boolean init) {
		PNetVisualizerController controller = PNetVisualizerController.getInstance();
		String actToRepair = controller.getIPN().getDqModel().getRepairModel().getActToRepair().getSelectedItem()
				.toString();

		//create a smaller event log, with only erroneous cases
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog subLog = factory.createLog((XAttributeMap) log.getAttributes().clone());

		//		XAttributeBoolean defaultFalse = factory.createAttributeBoolean("false", false, null);
		//Walk thought each sync of the replay result - clone and add traces to logs
		for (SyncReplayResult sync : result) {
			boolean addTraces = false;

			for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
				int logMoveCount = 0;
				int modelMoveCount = 0;
				StepTypes type = sync.getStepTypes().get(actIndex);
				Object stepObj = sync.getNodeInstance().get(actIndex);
				switch (type) {
					case L :

					case MREAL :
						if (stepObj.toString().equals(actToRepair))
							addTraces = true;
						break;

				}

			}
			if (addTraces) {
				subLog.getClassifiers().addAll(log.getClassifiers());
				subLog.getExtensions().addAll(log.getExtensions());
				subLog.getGlobalTraceAttributes().addAll(log.getGlobalTraceAttributes());
				subLog.getGlobalEventAttributes().addAll(log.getGlobalEventAttributes());

				for (int i : sync.getTraceIndex()) {
					XTrace trace = log.get(i);
					XAttribute att;
					XTrace orgTrace;
					int orgTraceId;

					//					if (!((XAttributeBoolean) trace.getAttributes().getOrDefault("removeEvtsTrace", defaultFalse))
					//							.getValue()) {
					if (!(trace.getAttributes().containsKey("removeEvtsTrace"))
							|| !((XAttributeBoolean) trace.getAttributes().get("removeEvtsTrace")).getValue()) {
						orgTrace = orgLog.get(Integer.valueOf(trace.getAttributes().get("OrgId").toString()));

						XTrace trace2 = factory.createTrace((XAttributeMap) trace.getAttributes().clone());
						for (XEvent event : trace) {
							if (!(event.getAttributes().containsKey("newlyAdded"))
									|| !((XAttributeBoolean) event.getAttributes().get("newlyAdded")).getValue()) {
								//							if (!((XAttributeBoolean) event.getAttributes().getOrDefault("newlyAdded", defaultFalse))
								//									.getValue()) {
								String orgEvtId = (event.getAttributes().get("orgEvtId").toString());
								XAttribute att2 = factory.createAttributeLiteral("orgEvtId", String.valueOf(orgEvtId),
										null);
								XEvent event2 = (XEvent) getEventUsingXID(orgEvtId, orgTrace).clone();
								event2.getAttributes().put("orgEvtId", att2);
								trace2.add(event2);
							} else {
								trace2.add(event);
							}
						}
						subLog.add(trace2);
					}
				}
			} else if (init) {
				//update the differences - initialziation only
				for (int id : sync.getTraceIndex()) {
					XTrace t = log.get(id);
					Date dateAct = null;
					Map<String, Long> acts1Temp = new HashMap<String, Long>();
					Map<String, Long> acts2Temp = new HashMap<String, Long>();
					int i = 0;
					for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {

						int logMoveCount = 0;
						int modelMoveCount = 0;

						StepTypes type = sync.getStepTypes().get(actIndex);
						Object stepObj = sync.getNodeInstance().get(actIndex);
						switch (type) { //stepObj is tran in case of syn move - so use it in all the maps 
							case LMGOOD :

								logMoveCount++;
								if (acts1.containsKey(stepObj.toString()))
									acts1Temp.put(stepObj.toString(),
											XTimeExtension.instance().extractTimestamp(t.get(i)).getTime());

								if (acts2.containsKey(stepObj.toString()))
									acts2Temp.put(stepObj.toString(),
											XTimeExtension.instance().extractTimestamp(t.get(i)).getTime());
								if (controller.getIPN().getDqModel().getRepairModel().getActToRepair()
										.getSelectedItem().toString().equals(stepObj.toString()))
									dateAct = XTimeExtension.instance().extractTimestamp(t.get(i)); //here we are interested in the event!
								i++;
								break;
							case MREAL :
								break; // unable to project this case, stop
							case L :
								i++;
								break;
						}

						if (dateAct != null) {
							Date caseStartDt = XTimeExtension.instance().extractTimestamp(
									orgLog.get(Integer.valueOf(t.getAttributes().get("OrgId").toString())).get(0));
							Date caseEndDt = XTimeExtension.instance().extractTimestamp(
									orgLog.get(Integer.valueOf(t.getAttributes().get("OrgId").toString())).get(
											t.size() - 1));

							for (String a1 : acts1Temp.keySet()) {
								acts1.get(a1).add(dateAct.getTime() - acts1Temp.get(a1));

							}
							for (String a2 : acts2Temp.keySet())
								acts2.get(a2).add(acts2Temp.get(a2) - dateAct.getTime());

							acts1.get("Case start").add(dateAct.getTime() - caseStartDt.getTime());
							acts2.get("Case end").add(caseEndDt.getTime() - dateAct.getTime());
						}
					}
				}

			}

		}
		return subLog;
	}

	private static void changeLog(XLog lTemp, RepairBox rb, PNetVisualizerController controller) {
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		//		XAttribute defaultFalse = factory.createAttributeBoolean("act1", false, null);
		boolean add = controller.getIPN().getDqModel().getRepairModel().addMissingActs();
		boolean remove = controller.getIPN().getDqModel().getRepairModel().removeExtraActs();
		String actToRepair = controller.getIPN().getDqModel().getRepairModel().getActToRepair().getSelectedItem()
				.toString();

		XTrace removeEventsTrace = factory.createTrace(); //dummy trace contraining only events to be remvoed
		XAttribute removeEvtsTrace = factory.createAttributeBoolean("removeEvtsTrace", true, null);
		removeEventsTrace.getAttributes().put("removeEvtsTrace", removeEvtsTrace);
		for (XTrace t : lTemp) {
			XEvent act1 = null;
			XEvent act2 = null;
			XEvent evtToRepair = null;
			//			XAttribute defaultZero = factory.createAttributeDiscrete("zero", 0, null);
			if (rb.getActs1().getSelectedItem().equals("Case start"))
				act1 = t.get(0);
			if (rb.getActs1().getSelectedItem().equals("Case end"))
				act2 = t.get(t.size() - 1);

			Set<XEvent> eventsToRepair = new HashSet<XEvent>();
			List<XEvent> eventsAddAfterOrBefore = new ArrayList<XEvent>();
			Set<XEvent> eventsToRemove = new HashSet<XEvent>();
			for (XEvent e : t) {
				if ((e.getAttributes().containsKey("repair"))
						&& ((XAttributeBoolean) e.getAttributes().get("repair")).getValue())
					//				if (((XAttributeBoolean) e.getAttributes().getOrDefault("repair", defaultFalse)).getValue())
					eventsToRepair.add(e);
				if ((e.getAttributes().containsKey("remove"))
						&& ((XAttributeBoolean) e.getAttributes().get("remove")).getValue())
					//				if (((XAttributeBoolean) e.getAttributes().getOrDefault("remove", defaultFalse)).getValue())
					eventsToRemove.add(e);
				long l = e.getAttributes().containsKey("addAfter") ? ((XAttributeDiscrete) e.getAttributes().get(
						"addAfter")).getValue() : 0;
				//				if (!(e.getAttributes().containsKey("addAfter")))
				//					((XAttributeDiscrete) e.getAttributes().getOrDefault("addAfter", defaultZero)).getValue();
				if (l == 0)
					l = e.getAttributes().containsKey("addBefore") ? ((XAttributeDiscrete) e.getAttributes().get(
							"addBefore")).getValue() : 0;
				//					l = ((XAttributeDiscrete) e.getAttributes().getOrDefault("addBefore", defaultZero)).getValue();
				for (int i = 0; i < l; i++)
					eventsAddAfterOrBefore.add(e);
			}

			//repair first
			boolean addAct1 = rb.getActs1().getSelectedItem().toString().equals("None");
			for (XEvent e : eventsToRepair) {
				if (!eventsAddAfterOrBefore.isEmpty()) {
					XEvent e2 = eventsAddAfterOrBefore.get(0);
					if (!addAct1)
						changeTime(e, e2, null, rb);
					else
						changeTime(e, null, e2, rb);
					eventsAddAfterOrBefore.remove(0);
				}

			}

			// add 
			if (add) {
				if (!eventsAddAfterOrBefore.isEmpty()) {
					for (XEvent e2 : eventsAddAfterOrBefore) {
						//						long l = ((XAttributeDiscrete) e2.getAttributes().getOrDefault("addAfter", defaultZero))
						//								.getValue();
						//						if (l == 0)
						//							l = ((XAttributeDiscrete) e2.getAttributes().getOrDefault("addBefore", defaultZero))
						//									.getValue();
						//						for (int i = 0; i < l; i++) {
						XEvent eNew = factory.createEvent();
						XAttribute newlyAdded = factory.createAttributeBoolean("newlyAdded", true, null);
						XConceptExtension.instance().assignName(eNew, actToRepair);
						XLifecycleExtension.instance().assignStandardTransition(eNew, StandardModel.COMPLETE);
						eNew.getAttributes().put("newlyAdded", newlyAdded);
						t.add(eNew);
						if (!addAct1)
							changeTime(eNew, e2, null, rb);
						else
							changeTime(eNew, null, e2, rb);
						//							eventsAddAfterOrBefore.remove(e2);
						//						}
					}
				}
			}

			//TODO remove extra activities - these activities are removed from the original trace, and added to the "remove trace column"
			if (remove)
				for (XEvent e : eventsToRemove) {
					if (!(t.getAttributes().containsKey("removeEvtsTrace"))
							|| !((XAttributeBoolean) t.getAttributes().get("removeEvtsTrace")).getValue()) {
						//					if (!((XAttributeBoolean) t.getAttributes().getOrDefault("removeEvtsTrace", defaultFalse))
						//							.getValue()) {
						t.remove(e);
						String orgId = ((XAttributeLiteral) t.getAttributes().get("OrgId")).getValue();
						XAttribute OrgId = factory.createAttributeLiteral("traceId", orgId, null);
						e.getAttributes().put("traceId", OrgId);
						removeEventsTrace.add(e);
					}
				}

		}
		//finally add remove extra activities trace
		lTemp.add(removeEventsTrace);

	}

	private static void changeTime(XEvent evtToRepair, XEvent act1, XEvent act2, RepairBox rb) {
		Calendar c = Calendar.getInstance();
		if (act1 != null)
			c.setTime(XTimeExtension.instance().extractTimestamp(act1));
		else if (act2 != null)
			c.setTime(XTimeExtension.instance().extractTimestamp(act2));
		int val = Integer.valueOf(rb.getUnit().getText().toString().equals("") ? "0" : rb.getUnit().getText()
				.toString());
		if (act1 == null)
			val = -val;
		switch (rb.getRepairMethod()) {
			case AbsoluteDays :
				c.add(Calendar.DATE, val);
				XTimeExtension.instance().assignTimestamp(evtToRepair, (c.getTime().getTime()));
				break;
			case AbsoluteHours :
				c.add(Calendar.HOUR, val);
				XTimeExtension.instance().assignTimestamp(evtToRepair, (c.getTime().getTime()));
				break;
			case AbsoluteMinutes :
				c.add(Calendar.MINUTE, val);
				XTimeExtension.instance().assignTimestamp(evtToRepair, (c.getTime().getTime()));
				break;
			case AbsoluteMonths :
				c.add(Calendar.MONTH, val);
				XTimeExtension.instance().assignTimestamp(evtToRepair, (c.getTime().getTime()));
				break;
			case AbsoluteSeconds :
				c.add(Calendar.SECOND, val);
				XTimeExtension.instance().assignTimestamp(evtToRepair, (c.getTime().getTime()));
				break;
			case AbsoluteWeeks :
				c.add(Calendar.DATE, val * 7);
				XTimeExtension.instance().assignTimestamp(evtToRepair, (c.getTime().getTime()));
				break;
			case Mean :
				if (act1 != null) {
					Long mean;
					if (rb.getActs1().getSelectedItem().equals("Case start"))
						mean = CalculationUtils.getMean(acts1.get("Case start"));
					else
						mean = CalculationUtils.getMean(acts1.get(XConceptExtension.instance().extractName(act1)));
					Long newTime = XTimeExtension.instance().extractTimestamp(act1).getTime() + mean;
					XTimeExtension.instance().assignTimestamp(evtToRepair, newTime);
				} else if (act2 != null) {
					Long mean;
					if (rb.getActs2().getSelectedItem().equals("Case end"))
						mean = CalculationUtils.getMean(acts2.get("Case end"));
					else
						mean = CalculationUtils.getMean(acts2.get(XConceptExtension.instance().extractName(act2)));

					Long newTime = XTimeExtension.instance().extractTimestamp(act2).getTime() - mean;
					XTimeExtension.instance().assignTimestamp(evtToRepair, newTime);
				}
				break;
			case Median :
				if (act1 != null) {
					Long med = CalculationUtils.getMedian(acts1.get(XConceptExtension.instance().extractName(act1)));
					Long newTime = XTimeExtension.instance().extractTimestamp(act1).getTime() + med;
					XTimeExtension.instance().assignTimestamp(evtToRepair, newTime);
				} else if (act2 != null) {
					Long med = CalculationUtils.getMedian(acts2.get(XConceptExtension.instance().extractName(act2)));
					Long newTime = XTimeExtension.instance().extractTimestamp(act2).getTime() - med;
					XTimeExtension.instance().assignTimestamp(evtToRepair, newTime);
				}
				break;
			default :
				break;

		}
	}

	private static XEvent getEventUsingXID(String xid, XTrace t) {
		for (XEvent e : t)
			if (e.getID().toString().equals(xid))
				return e;

		return null;
	}

	private static XEvent getEventUsingXIDAsAttribute(String xid, XTrace t) {
		for (XEvent e : t)
			if (e.getAttributes().get("orgEvtId").toString().equals(xid))
				return e;

		return null;
	}

	public static void renameDuplicates() {
		PNetVisualizerController controller = PNetVisualizerController.getInstance();
		InteractivePetriNet ipn = controller.getIPN();
		Set<String> acts = new HashSet<String>();
		XLog orgLog = ipn.getLog();
		for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
			acts.add(t.getLabel());
		XLog log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), acts, false);
		PNRepResult result = calculateAlignments(log);

		for (SyncReplayResult sync : result) {
			for (int id : sync.getTraceIndex()) {
				int traceId = Integer
						.valueOf(((XAttributeLiteral) log.get(id).getAttributes().get("OrgId")).getValue());
				int evntCount = 0;
				for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
					StepTypes type = sync.getStepTypes().get(actIndex);
					Object stepObj = sync.getNodeInstance().get(actIndex);
					if (type.equals(StepTypes.LMGOOD)) {
						String evtId = (((XAttributeLiteral) log.get(id).get(evntCount).getAttributes().get("orgEvtId"))
								.getValue());
						XEvent orgEvt = getEventUsingXID(evtId, orgLog.get(traceId));
						XConceptExtension.instance().assignName(orgEvt, stepObj.toString());
						evntCount++;
					} else if (type.equals(StepTypes.L))
						evntCount++;

				}
			}
		}
	}

	private static void clearAttributes(XLog newLog) {
		for (XTrace t : newLog) {
			t.getAttributes().remove("OrgId");

			for (XEvent e : t) {
				e.getAttributes().remove("orgEvtId");
				e.getAttributes().remove("addAfter");
				e.getAttributes().remove("addBefore");
				e.getAttributes().remove("newlyAdded");
				e.getAttributes().remove("repair");

			}
		}

	}
}
