package org.processmining.dataquality.Reasons;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

public class CalculationUtils {

	public static int getEditDistance(XLog orgLog, XLog log2) {

		Map<String, Character> actCode = new HashMap<String, Character>();

		int dis = 0;
		for (XTrace t : log2) {
			String s1 = "";
			String s2 = "";
			char alphabet = 'A';
			for (XEvent e : t) {
				String s = XConceptExtension.instance().extractName(e);
				if (!actCode.containsKey(s))
					actCode.put(s, alphabet++);
				s1 += actCode.get(s);
			}
			XTrace orgTrace = orgLog.get(Integer.valueOf(t.getAttributes().get("OrgId").toString()));
			for (XEvent e : orgTrace) {
				CharSequence s = XConceptExtension.instance().extractName(e);
				if (actCode.containsKey(s))
					s2 += actCode.get(s);
			}
			dis += levenshteinDistance(s1, s2);
		}
		return dis;

	}

	public static int levenshteinDistance(CharSequence lhs, CharSequence rhs) {
		int len0 = lhs.length() + 1;
		int len1 = rhs.length() + 1;

		// the array of distances                                                       
		int[] cost = new int[len0];
		int[] newcost = new int[len0];

		// initial cost of skipping prefix in String s0                                 
		for (int i = 0; i < len0; i++)
			cost[i] = i;

		// dynamically computing the array of distances                                  

		// transformation cost for each letter in s1                                    
		for (int j = 1; j < len1; j++) {
			// initial cost of skipping prefix in String s1                             
			newcost[0] = j;

			// transformation cost for each letter in s0                                
			for (int i = 1; i < len0; i++) {
				// matching current letters in both strings                             
				int match = (lhs.charAt(i - 1) == rhs.charAt(j - 1)) ? 0 : 1;

				// computing cost for each transformation                               
				int cost_replace = cost[i - 1] + match;
				int cost_insert = cost[i] + 1;
				int cost_delete = newcost[i - 1] + 1;

				// keep minimum cost                                                    
				newcost[i] = Math.min(Math.min(cost_insert, cost_delete), cost_replace);
			}

			// swap cost/newcost arrays                                                 
			int[] swap = cost;
			cost = newcost;
			newcost = swap;
		}

		// the distance is the cost for transforming all letters in both strings        
		return cost[len0 - 1];
	}

	public static long getMedian(List<Long> values) {
		List<Long> val = new ArrayList<Long>(values);
		Collections.sort(val);
		return val.get(val.size() / 2);
	}

	public static long getMean(List<Long> values) {
		Long tot = 0l;
		for (Long l : values)
			tot += l;

		return tot / values.size();
	}
}
