package org.processmining.discovery;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.alignments.AlignmentCalculations;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;
import org.processmining.prabhakardixit.processanalytics.ProcessAnalyticsUtils;
import org.processmining.reduction.Reduction;

public class DiscoverUsingSynthesis {

	public static void discover(InteractivePetriNet ipn) {
		Petrinet pn = ipn.getselectedPetriNetMap().getPetriNet();

		Set<String> activities = ipn.getLogInfo().getActivities();

		for (String act : activities) {
			//first get a sublog of the current activities and the prepost corresponding to trans from the net
			Set<String> prePostActs = new HashSet<String>();
			XLog log = DiscoveryUtils.getPrePostMergedLog(pn, act, ipn.getLog(), prePostActs);

			//reduce the net to keep only pre post acts in the net
			Set<PetrinetNode> transToKeep = new HashSet<PetrinetNode>();
			Set<Transition> tranSet = new HashSet<Transition>();
			PetriNetMap pnmap = new PetriNetMap(ipn.getselectedPetriNetMap().getPetriNet());
			for (Transition t : pnmap.getPetriNet().getTransitions())
				if (prePostActs.contains(t.getLabel())) {
					tranSet.add(t);
					transToKeep.add(t);
				}
			//reduce petrinet now
			Reduction rd = new Reduction(pnmap.getPetriNet(), transToKeep);

			//calculate alignments
			PNRepResult replay = AlignmentCalculations.calculateNAlignment1(ipn.getPromContext(), log,
					pnmap.getPetriNet(), 5, false, tranSet);

			//keep track of pre and post trans
			SyncReplayAnalysis sra = getLogMovePlaces(replay, pnmap.getPetriNet(), act);

			//find the best location using logMovePlaces
			
		}

	}

	private static SyncReplayAnalysis getLogMovePlaces(PNRepResult replay, Petrinet pn, String newAct) {
		int maxNbrOfLogMoves = 0, minNbrOfLogMoves = 99, averageNbrOfLogMoves = 0;
		Set<Place> marking = new HashSet<Place>();
		int totalLogMoves = 0;
		int totalNbrOfTraces = 0;
		Map<Set<Place>, Integer> logMovesPlacesCount = new HashMap<Set<Place>, Integer>();
		Map<Transition, Integer> syncTrans = new HashMap<Transition, Integer>();
		for (SyncReplayResult sync : replay) {
			int nbrOfLogMoves = 0;
			int nbrOfTraces = sync.getTraceIndex().size();
			marking.clear();
			marking.addAll(PetrinetUtils.getInitialMarking(pn));
			for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
				StepTypes type = sync.getStepTypes().get(actIndex);
				Object stepObj = sync.getNodeInstance().get(actIndex);
				String actName = stepObj.toString();
				switch (type) {
					case L :
						if (!actName.equals(newAct))
							break;
						boolean added = false;
						for (Set<Place> key : logMovesPlacesCount.keySet()) {
							if (key.equals(marking)) {
								logMovesPlacesCount.put(key, logMovesPlacesCount.get(key) + nbrOfTraces);
								added = true;
							}
						}
						if (!added)
							logMovesPlacesCount.put(new HashSet<Place>(marking), nbrOfTraces);
						nbrOfLogMoves++;
						break;
					case LMGOOD :
						syncTrans.put((Transition) stepObj, syncTrans.containsKey(stepObj) ? syncTrans.get(stepObj)
								+ nbrOfTraces : nbrOfTraces);
						ProcessAnalyticsUtils.execTran((Transition) stepObj, pn, marking);
						break;
					case MINVI :
					case MREAL :
						ProcessAnalyticsUtils.execTran((Transition) stepObj, pn, marking);
						break;
					case LMNOGOOD :
					case LMREPLACED :
					case LMSWAPPED :
						// unable to project this case, stop
				}
			}
			if (nbrOfLogMoves < minNbrOfLogMoves)
				minNbrOfLogMoves = nbrOfLogMoves;
			if (nbrOfLogMoves > maxNbrOfLogMoves)
				maxNbrOfLogMoves = nbrOfLogMoves;
			totalNbrOfTraces += nbrOfTraces;
			totalLogMoves += nbrOfTraces * nbrOfLogMoves;
		}
		averageNbrOfLogMoves = totalNbrOfTraces / totalLogMoves;
		return new SyncReplayAnalysis(logMovesPlacesCount, syncTrans, maxNbrOfLogMoves, minNbrOfLogMoves,
				averageNbrOfLogMoves);

	}
}
