package org.processmining.discovery;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class DiscoveryUtils {

	public static XLog getPrePostMergedLog(Petrinet pn, String newAct, XLog completeLog, Set<String> prePostActivities) {
		Set<String> actsInNet = new HashSet<String>();
		for (Transition t : pn.getTransitions())
			actsInNet.add(t.getLabel());

		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog subLog = factory.createLog((XAttributeMap) completeLog.getAttributes().clone());
		subLog.getClassifiers().addAll(completeLog.getClassifiers());
		subLog.getExtensions().addAll(completeLog.getExtensions());
		subLog.getGlobalTraceAttributes().addAll(completeLog.getGlobalTraceAttributes());

		//Walk thought each trace of the original log
		XEvent prevEvent = null;
		boolean addNext = false;

		for (XTrace trace : completeLog) {
			boolean addTrace = false;
			XTrace trace2 = factory.createTrace((XAttributeMap) trace.getAttributes().clone());
			for (XEvent event : trace) {
				String currAct = XConceptExtension.instance().extractName(event).toString();
				if (currAct.equals(newAct)) {
					addTrace = true;
					addNext = true;
					if (prevEvent != null)
						trace2.add(prevEvent);
					trace2.add((XEvent) event.clone());
				} else if (addNext) {
					prePostActivities.add(currAct);
					addNext = false;
					trace2.add((XEvent) event.clone());
				} else {
					prePostActivities.add(currAct);
					prevEvent = (XEvent) event.clone();
				}
			}
			if (addTrace)
				subLog.add(trace2);
		}

		return subLog;
	}

	public static Set<String> getPrePostActivities(Petrinet pn, String newAct, XLog completeLog, boolean addPrecedes,
			boolean addFollows) {
		Set<String> prePostActivities = new HashSet<String>();
		Set<String> actsInNet = new HashSet<String>();
		for (Transition t : pn.getTransitions())
			actsInNet.add(t.getLabel());

		//Walk thought each trace of the original log
		XEvent prevEvent = null;
		for (XTrace trace : completeLog) {
			boolean addNext = false;
			for (XEvent event : trace) {
				String currAct = XConceptExtension.instance().extractName(event).toString();
				if (actsInNet.contains(currAct) || currAct.equals(newAct))
					if (currAct.equals(newAct)) {
						addNext = true;
						if (prevEvent != null && addPrecedes)
							prePostActivities.add(XConceptExtension.instance().extractName(prevEvent).toString());
					} else if (addNext) {
						if (addFollows)
							prePostActivities.add(currAct);
						addNext = false;
					} else {
						//						prePostActivities.add(currAct);
						prevEvent = (XEvent) event.clone();
					}
			}
		}

		return prePostActivities;
	}
}
