package org.processmining.discovery;

import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.processmining.prabhakardixit.fcnets.models.InteractiveMatrixNet;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.LDSets;
import org.processmining.prabhakardixit.fcnets.synthesis.VizToPnetMapper;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class SynthesizeNets {

	public static int tranCount = 3;
	public static int placeCount = 3;

	public static void addParallelPlace(InteractivePetriNet ipn) {
		List<LDSets> placeLD = ipn.getplaceLDNonSiphons();
		boolean added = false;
		Random random = new Random();
		int id = random.nextInt(placeLD.size());
		while (!added) {
			InteractiveMatrixNet imn = ipn.getInteractiveMatNet();
			Set<Integer> startTranSet = new HashSet<Integer>();
			Set<Integer> endTranSet = new HashSet<Integer>();
			added = true;

			startTranSet.addAll(placeLD.get(id).getPositiveNodes(imn));
			endTranSet.addAll(placeLD.get(id).getNegativeNodes(imn));
			//			System.out.println("Parallel Place: " + startTranSet + " End Set " + endTranSet);
			for (int i : startTranSet)
				if (PetrinetUtils.finalTransition(ipn.getTranBasedOnID(i)))
					added = false;

			if (startTranSet.containsAll(endTranSet) || startTranSet.equals(endTranSet))
				added = false;

			if (added) {
				VizToPnetMapper.addParallelPlace(startTranSet, endTranSet, "newPlace", ipn, placeCount++);
			}
		}
	}

	public static void addChoiceTran(InteractivePetriNet ipn) {
		Set<LDSets> tranLD = ipn.getAllTranLDFCs();
		boolean added = false;
		Random random = new Random();
		while (!added) {
			InteractiveMatrixNet imn = ipn.getInteractiveMatNet();
			int id = random.nextInt(tranLD.size() + 1);
			if (id != 0)
				id = id - 1;
			int c = 0;
			LDSets ldt = null;
			for (LDSets ldt2 : tranLD) {
				if (c == id)
					ldt = ldt2;
				c++;
			}

			Set<Integer> startTranSet = new HashSet<Integer>();
			Set<Integer> endTranSet = new HashSet<Integer>();
			added = true;

			startTranSet.addAll(ldt.getNegativeNodes(imn));
			endTranSet.addAll(ldt.getPositiveNodes(imn));
			for (int i : startTranSet)
				if (PetrinetUtils.initialOrFinalPlace(ipn.getPlaceBasedOnID(i), ipn.getselectedPetriNetMap()
						.getPetriNet()))
					added = false;

			for (int i : endTranSet)
				if (PetrinetUtils.initialOrFinalPlace(ipn.getPlaceBasedOnID(i), ipn.getselectedPetriNetMap()
						.getPetriNet()))
					added = false;

			if (startTranSet.containsAll(endTranSet) || startTranSet.equals(endTranSet))
				added = false;
			if (added) {
				VizToPnetMapper.addChoiceTransition(startTranSet, endTranSet, "", ipn, tranCount++, true);

			}
		}
	}

	
	
	public static void addFirstActAsSequence(InteractivePetriNet ipn, String actName) {
		Set<Integer> sourceNodes = new HashSet<Integer>();
		Set<Integer> targetNodes = new HashSet<Integer>();
		sourceNodes.add(0);
		targetNodes.add(1);
		VizToPnetMapper.addNewTransition(sourceNodes, targetNodes, actName,actName, ipn, false, placeCount++, tranCount++, false);
	}

}
