package org.processmining.discovery;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.processmining.prabhakardixit.fcnets.utils.MapUtils;
import org.processmining.processtree.Block;
import org.processmining.processtree.Block.And;
import org.processmining.processtree.Block.Or;
import org.processmining.processtree.Block.Seq;
import org.processmining.processtree.Block.Xor;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task.Automatic;

public class TreePriority {
	/*
	 * Each tree contains the relation between two activities (one from the
	 * model and one from the log) Duplicate activities from the model are used
	 * with alignments (new log created wrt aliognments), and use each activitiy
	 * separately to form the relationship by keeping map (eg a,b1; a,b2;
	 * a,b3...)
	 */

	public static Map<ProcessTree, Integer> getPTPriority(Set<ProcessTree> actPTree, String newAct) {
		Map<ProcessTree, Integer> sortedActMap = new HashMap<ProcessTree, Integer>();
		for (ProcessTree pt : actPTree) {
			sortedActMap.put(pt, getRank(pt));

		}
		return MapUtils.sortByValue(sortedActMap, false);
	}

	/**
	 * gets the rank of the process tree based on the relation between two
	 * activities in the model (a- model act, b-log act) 0- Direct sequences -
	 * without skips : a->b || 1 - Parallel - without skips : a ^ b || 2 - IOR -
	 * without skips : a v b || 3 - XORLOOP - without skips (a) (b) (tau) || 4 -
	 * XORLOOP - without skips (b) (a) (tau) * how to handle this || 5 - XORLOOP
	 * - without skips (a) (tau) (b) 6 - Seq - with log act skip - a -> (b x
	 * tau) - this is possible xor candidate in the model 7 - Seq - with model
	 * act skip - (a x tau) -> b - how to handle this 8 - AOB
	 *
	 */
	private static int getRank(ProcessTree pt) {
		int i = -1;
		boolean inc = true;

		//-1
		if (!(pt.getRoot() instanceof Block) || (!containsOpNodeAsChild(pt) || !(pt instanceof Seq)))
			return i;
		i++;
		//0 - Direct sequences -  without skips : a->b 1 
		if (pt.getRoot() instanceof Seq && pt.getNodes().size() == 3) {
			return i;
		}
		//1 - Parallel - without skips : a ^ b 
		i++;
		if (pt.getRoot() instanceof And && pt.getNodes().size() == 3)
			return i;

		//2 - IOR - without skips : a v b 
		i++;
		if (pt.getRoot() instanceof Or && pt.getNodes().size() == 3)
			return i;

		//3 - XORLOOP - without skips (a) (b) (tau)
		i++;
		if (pt.getRoot() instanceof XorLoop && pt.getNodes().size() == 4
				&& ((XorLoop) pt.getRoot()).getChildren().get(2) instanceof Automatic)
			return i;

		//4 -  XORLOOP - without skips (b) (a) (tau) - TODO handle this!
		i++;
		if (pt.getRoot() instanceof XorLoop && pt.getNodes().size() == 4
				&& ((XorLoop) pt.getRoot()).getChildren().get(2) instanceof Automatic)
			return i;

		//5 - XORLOOP - without skips (a) (tau) (b) 
		i++;
		if (pt.getRoot() instanceof XorLoop && pt.getNodes().size() == 4
				&& ((XorLoop) pt.getRoot()).getChildren().get(1) instanceof Automatic)
			return i;

		//6 - Seq - with log act skip - a -> (b x tau) - possible xor
		i++;
		if (pt.getRoot() instanceof Seq && pt.getNodes().size() == 3 && ((Seq) pt.getRoot()).getChildren().get(1) instanceof Xor)
			return i;

		// 7 - Seq - with model act skip - (a x tau) -> b 
		i++;
		if (pt.getRoot() instanceof Seq && pt.getNodes().size() == 3 && ((Seq) pt.getRoot()).getChildren().get(0) instanceof Xor)
			return i;

		return -1;
	}

	private static boolean containsOpNodeAsChild(ProcessTree pt) {
		boolean opNode = false;
		for (Node n : ((Block) pt.getRoot()).getChildren()) {
			if (n instanceof Block)
				opNode = true;
		}

		return opNode;
	}

}
