package org.processmining.discoverycontest;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

@Plugin(name = "Filter event log based on choice", parameterLabels = { "Petrinet ", "Event Log", }, returnLabels = { "Petri net" }, returnTypes = { Petrinet.class })
public class FilterPlugin {

	/**
	 * The method that does the heavy lifting for your plug-in.
	 * 
	 * Note that this method only uses the boolean which is stored in the
	 * configuration. Nevertheless, it could have used the integer and/or the
	 * String as well.
	 * 
	 * @param context
	 *            The context where to run in.
	 * @param input1
	 *            The first input.
	 * @param dclist
	 * @param log
	 * @param input2
	 *            The second input.
	 * @param configuration
	 *            The configuration to use.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	static Set<String> events = new HashSet<String>();
	public static boolean stop = false;

	public Petrinet yourPrivatePlugin(UIPluginContext context, XLog logOriginal, XLog logNew) throws IllegalAccessException,
			InstantiationException {
		Set<Integer> positive = new HashSet<Integer>();

		for (XTrace trace : logNew) {
			Set<String> eventsToKeep = new HashSet<String>();
			for (XEvent event : trace) {
				String name = XConceptExtension.instance().extractName(event);
				eventsToKeep.add(name);
			}

			//filter original log to contain traces with all the prev events only
			Set<Integer> cases = new HashSet<Integer>();
			for (XTrace trace1 : logOriginal) {
				cases.add(logOriginal.indexOf(trace1));
			}
			for (XTrace trace1 : logOriginal) {
				for (XEvent event : trace1) {
					String name = XConceptExtension.instance().extractName(event);
					if (!eventsToKeep.contains(name)) {
						cases.remove(logOriginal.indexOf(trace1));
					}
				}
			}

			Set<Integer> casesToRemove = new HashSet<Integer>();
			for (int casee : cases) {
				if (logOriginal.get(casee).size() != eventsToKeep.size())
					casesToRemove.add(casee);
			}

			cases.removeAll(casesToRemove);
			if (cases.size() > 0)
				positive.add(logNew.indexOf(trace) + 1);
			
			
			System.out.println(" Trace " + logNew.indexOf(trace)  + " cases " + cases);
		}
		System.out.println("Positive cases  " + positive.size() + " cases " + positive  );
		return null;
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Reduce petrinet 22", requiredParameterLabels = { 0, 1 })
	public Petrinet yourDefaultPlugin(UIPluginContext context, XLog logOrg, XLog logNew) throws IllegalAccessException,
			InstantiationException {

		return yourPrivatePlugin(context, logOrg, logNew);

	}
}
