package org.processmining.discoverycontest;

import java.util.Collection;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;

@Plugin(name = "Relabel choice petri nets", parameterLabels = { "Petrinet ", }, returnLabels = { "Petri net" }, returnTypes = { Petrinet.class })
public class RelabelChoiceTransPlugin {

	public Petrinet yourPrivatePlugin(UIPluginContext context, Petrinet pn2) throws IllegalAccessException,
			InstantiationException {
		PetriNetMap pnmap = new PetriNetMap(pn2);
		Petrinet pn = pnmap.getPetriNet();
		for (Transition t : pn.getTransitions()) {
			if (t.isInvisible())
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getInEdges(t)) {
					Place p = (Place) e2.getSource();
					Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> eOut = pn.getOutEdges(p);
					if (eOut.size() > 1) {
						String name = "";
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : eOut) {
							if (!e.getTarget().equals(t))
								name += e.getTarget().getLabel();
						}
						t.getAttributeMap().put(AttributeMap.LABEL, "NOT " + name);
					}
				}
		}

		return pn;
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Reduce petrinet 22", requiredParameterLabels = { 0 })
	public Petrinet yourDefaultPlugin(UIPluginContext context, Petrinet pn) throws IllegalAccessException,
			InstantiationException {

		return yourPrivatePlugin(context, pn);

	}
}
