package org.processmining.discoverycontest;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

@Plugin(name = "Rename duplicates with ids", parameterLabels = { "Petrinet ", }, returnLabels = { "Petri net" }, returnTypes = { Petrinet.class })
public class RenameDuplicatesPlugin {

	/**
	 * The method that does the heavy lifting for your plug-in.
	 * 
	 * Note that this method only uses the boolean which is stored in the
	 * configuration. Nevertheless, it could have used the integer and/or the
	 * String as well.
	 * 
	 * @param context
	 *            The context where to run in.
	 * @param input1
	 *            The first input.
	 * @param dclist
	 * @param log
	 * @param input2
	 *            The second input.
	 * @param configuration
	 *            The configuration to use.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	static Set<String> events = new HashSet<String>();
	public static boolean stop = false;

	public Petrinet yourPrivatePlugin(UIPluginContext context, Petrinet pn) throws IllegalAccessException,
			InstantiationException {

		Map<String, Integer> tranMap = new HashMap<String, Integer>();
		Map<Transition, Integer> tranMapId = new HashMap<Transition, Integer>();
		for (Transition t : pn.getTransitions()) {

			int id = tranMap.containsKey(t.getLabel()) ? tranMap.get(t.getLabel()) + 1 : 1;
			tranMap.put(t.getLabel(), id);
			tranMapId.put(t, id);
		}

		for (Transition t : tranMapId.keySet()) {
			if (t.getLabel().equals("o") || t.getLabel().equals("p"))
				t.getAttributeMap().put(org.processmining.models.graphbased.AttributeMap.LABEL,
						t.getLabel() + tranMapId.get(t));
			if (t.getLabel().equals("t")) {
				if (tranMapId.get(t) == 1) {
					t.getAttributeMap().put(org.processmining.models.graphbased.AttributeMap.LABEL, t.getLabel() + 2);
				} else {
					t.getAttributeMap().put(org.processmining.models.graphbased.AttributeMap.LABEL, t.getLabel() + 1);
				}
			}
		}
		return pn;
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Reduce petrinet 22", requiredParameterLabels = { 0 })
	public Petrinet yourDefaultPlugin(UIPluginContext context, Petrinet pn) throws IllegalAccessException,
			InstantiationException {

		return yourPrivatePlugin(context, pn);

	}
}
