package org.processmining.discoverycontest.layout;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Shape;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.processmining.framework.util.Pair;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.gui.action.FitGroupBoundsAction;
import org.processmining.humbertogarciacaballero.models.gui.action.NodeTypeShapeAction;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LabelDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomEdgeRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomLabelRendereForEdgesBPMN;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomShapeRendererBPMN;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.SoundnessInformation;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayoutOrientation;
import org.processmining.humbertogarciacaballero.plugins.PluginUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.Action;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.action.layout.Layout;
import prefuse.action.layout.SpecifiedLayout;
import prefuse.controls.DragControl;
import prefuse.controls.FocusControl;
import prefuse.controls.PanControl;
import prefuse.controls.WheelZoomControl;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.GraphicsLib;
import prefuse.util.PrefuseLib;
import prefuse.util.StrokeLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;


public class BPMNGraph extends Display {

	private static final String LABEL_DECORATOR_GROUP_NAME = "labelgroup";
	public static final String EDGE_DECORATORS = "edgeDeco";
	private static final Schema DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
	public static final String EDGES = "graph.edges";
	public static final String AGGR_DECORATORS = "aggrDeco";
	public static final String AGGR = "aggregates";
	private static JPanel panel;
	static {
		DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.gray(128));
		DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Tahoma", 75));
	}

	public BPMNGraph(LayoutAlgorithmReductionRules lar, PetrinetToBPMN pbpm) {

		super(new Visualization());
		this.setSize(800, 500);
		addControlListener(new WheelZoomControl(true, true));
		addControlListener(new PanControl());
		addControlListener(new DragControl("decorators"));
		setHighQuality(true);

		Petrinet pn = lar.getWs().net.getNet();
		Set<Place> initMarking = new HashSet<Place>();
		initMarking.add(PetrinetUtils.getInitialPlace(pn));
		Marking m = new Marking();
		SoundnessInformation information = new SoundnessInformation();

		int i = 0;
		Map<PetrinetNode, Integer> idMap = new HashMap<PetrinetNode, Integer>();
		for (Place p : pn.getPlaces())
			idMap.put(p, i++);
		for (Transition t : pn.getTransitions())
			idMap.put(t, i++);

		Graph vt = PluginUtils.netToGraph(null, pn, null, information, null, idMap);
		vt.addColumn("x", double.class);
		vt.addColumn("y", double.class);
		vt.addColumn("ior", boolean.class);
		vt.addColumn("and", boolean.class);
		vt.addColumn("xor", boolean.class);
		vt.addColumn("start", boolean.class);
		vt.addColumn("end", boolean.class);
		vt.addColumn(VisualItem.LABEL, String.class);
		Iterator iter = vt.nodes();
		//add bpmn and layout info
		while (iter.hasNext()) {
			Node n = (Node) iter.next();
			int id = n.getInt("ID");
			PetrinetNode pnode = null;
			for (PetrinetNode pnode1 : idMap.keySet())
				if (idMap.get(pnode1) == id)
					pnode = pnode1;
			Pair<Double, Double> xy = lar.getXY(pnode);
			n.setBoolean("start", false);
			n.setBoolean("end", false);
			if (pnode instanceof Place) {
				n.setBoolean(PNConstants.NODE_TAU_TRANSITION_KEY, true);

				if ((PetrinetUtils.getInitialPlace(pn).equals(pnode))) {
					n.setBoolean(PNConstants.NODE_TAU_TRANSITION_KEY, false);
					n.setBoolean("start", true);
				}
				if ((PetrinetUtils.getFinalPlace(pn).equals(pnode))) {
					n.setBoolean(PNConstants.NODE_TAU_TRANSITION_KEY, false);
					n.setBoolean("end", true);
				}
			}
			if (pnode instanceof Transition && PetrinetUtils.getInitialTranistion(pn).equals(pnode))
				System.out.println("here");

			n.set("x", xy.getFirst());
			n.set("y", xy.getSecond());

			//update ior xor and and gateways
			System.out.println(pnode.getLabel().toString().equals("iorsplit") + " : "
					+ pnode.getLabel().toString().equals("andsplit"));
			if (pbpm != null) {
				if (pbpm.getIorSplits().contains(pnode)) {
					n.set("label", "o");
					n.set("type", "IOR");
					n.setBoolean(PNConstants.NODE_TAU_TRANSITION_KEY, false);
				}
				if (pbpm.getAndSplits().contains(pnode)) {
					n.set("label", "+");
					n.set("type", "AND");
					n.setBoolean(PNConstants.NODE_TAU_TRANSITION_KEY, false);
				}
				if (pbpm.getXorSplits().contains(pnode)) {
					n.set("label", "x");
					n.set("type", "XOR");
					n.setBoolean(PNConstants.NODE_TAU_TRANSITION_KEY, false);
				}
			}
			n.setBoolean("ior", pbpm.getIorSplits().contains(pnode));
			n.setBoolean("and", pbpm.getAndSplits().contains(pnode));
			n.setBoolean("xor", pbpm.getXorSplits().contains(pnode));
		}

		Iterator iter2 = vt.edges();
		//hide edges and add bends info
		//		while (iter2.hasNext()) {
		//			Edge e = (Edge) iter2.next();
		//			int id1 = e.getSourceNode().getInt("ID");
		//			int id2 = e.getTargetNode().getInt("ID");
		//			PetrinetNode source = null;
		//			PetrinetNode target = null;
		//
		//			for (PetrinetNode pnode : idMap.keySet())
		//				if (idMap.get(pnode) == id1)
		//					source = pnode;
		//				else if (idMap.get(pnode) == id2)
		//					target = pnode;
		//
		//			System.out.println(source);
		//			System.out.println(target);
		//			
		//			if (target instanceof Place && pn.getInEdges(target).size() == 1 && pn.getOutEdges(target).size() == 1) {
		//				Transition targetTran = null;
		//				Node newTarget = null;
		//
		//				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> petriEdge : pn.getOutEdges(target)) {
		//					targetTran = (Transition) petriEdge.getTarget();
		//				}
		//
		//				iter = vt.nodes();
		//				while (iter.hasNext()) {
		//					Node n = (Node) iter.next();
		//					if (idMap.get(targetTran).equals(n.getInt("ID")))
		//						newTarget = n;
		//				}
		//				Edge e2 = vt.addEdge(e.getSourceNode(), newTarget);
		//
		//				List<Point2D> bends = new ArrayList<Point2D>();
		//				bends.add(new Point2D.Double(lar.getXY(target).getFirst(), lar.getXY(target).getSecond()));
		//				//								bends.addAll(lar.getXYEdges(source, target));
		//				//								bends.addAll(lar.getXYEdges(target, targetTran));
		//				//				bends.add(new Point2D.Double(lar.getXY(targetTran).getFirst(), lar.getXY(targetTran).getSecond()));
		//				vt.removeEdge(e);
		//				//				e2.set(PNConstants.EDGE_BENDS_KEY, lar.getEdgePoints(source, targetTran, source, target));
		//				e2.set(PNConstants.EDGE_BENDS_KEY, bends);
		//			} else if (source instanceof Place && !PetrinetUtils.getInitialPlace(pn).equals(source)
		//					&& pn.getInEdges(source).size() <= 1 && pn.getOutEdges(source).size() == 1) {
		//				vt.removeEdge(e);
		//			} else {
		//				e.set(PNConstants.EDGE_BENDS_KEY, lar.getXYEdges(source, target));
		//				//								e.set(PNConstants.EDGE_BENDS_KEY, new ArrayList<Point2D>());
		//			}
		//		}

		iter2 = vt.edges();
		//contains no places - remove unwanted silent transitions (silent trans have only one incoming and one outgoing edges at this point)
		while (iter2.hasNext()) {
			Edge e = (Edge) iter2.next();
			Node n1 = e.getSourceNode();
			Node n2 = e.getTargetNode();
			if (!n1.getBoolean(PNConstants.NODE_TAU_TRANSITION_KEY)) {
				if (n2.getBoolean(PNConstants.NODE_TAU_TRANSITION_KEY)) {
					boolean validTarget = false;
					List<Point2D> bends = new ArrayList<Point2D>();
					Node n3 = null;
					Edge e3 = e;
					while (!validTarget) {
						bends.add(new Point2D.Double(n2.getDouble("x"), n2.getDouble("y")));
						Iterator iter3 = vt.edges();
						while (iter3.hasNext()) {
							e3 = (Edge) iter3.next();
							if (e3.getSourceNode().equals(n2))
								n3 = e3.getTargetNode();
						}
						if (n3.getBoolean(PNConstants.NODE_TAU_TRANSITION_KEY) && n3.getOutDegree() > 0) {
							n2 = n3;
						} else {
							bends.add(new Point2D.Double(n2.getDouble("x"), n2.getDouble("y")));
							validTarget = true;
						}

					}
					Edge e2 = vt.addEdge(n1, n3);
					e2.set(PNConstants.EDGE_BENDS_KEY, bends);
					if (pbpm.getGuards().containsKey(n3.get("label"))) {
						e2.set(VisualItem.LABEL, pbpm.getGuards().get(n3.get("label")));
					}
					//					e2.set(VisualItem.LABEL, "total ll l ");
				}
			}
		}
		//
		//		//remove all tau edges
		iter2 = vt.edges();
		while (iter2.hasNext()) {
			Edge e = (Edge) iter2.next();
			Node n1 = e.getSourceNode();
			Node n2 = e.getTargetNode();
			if (n1.getBoolean(PNConstants.NODE_TAU_TRANSITION_KEY)
					|| n2.getBoolean(PNConstants.NODE_TAU_TRANSITION_KEY))
				vt.removeEdge(e);
		}

		final Schema LABEL_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, true);
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Arial",Font.BOLD,  25));
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.color(Color.BLACK));

		//		m_vis.addGraph("graph", graph);
		m_vis.addGraph("graph", vt);
		ColorAction fill = new ColorAction("graph.nodes", VisualItem.FILLCOLOR, ColorLib.color(Color.WHITE));
		ColorAction edges = new ColorAction("graph.edges", VisualItem.STROKECOLOR, ColorLib.color(Color.BLACK));
		ColorAction edges2 = new ColorAction("graph.edges", VisualItem.FILLCOLOR, ColorLib.color(Color.BLACK));
		StrokeAction edgeStrokeSize = new StrokeAction("graph.edges",
				StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
		StrokeAction borderSize = new StrokeAction("graph.nodes", StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
		ColorAction border = new ColorAction("graph.nodes", VisualItem.STROKECOLOR, PNConstants.STROKE_COLOR);

		NodeTypeShapeAction shape = new NodeTypeShapeAction("graph.nodes");
		ActionList color = new ActionList();
		color.add(fill);
		color.add(edges);
		color.add(edges2);
		color.add(edgeStrokeSize);
		color.add(borderSize);
		color.add(border);
		color.add(shape);

		Action orientationAction = new Action() {
			private LayoutOrientation lo = null;

			@Override
			public void run(double frac) {
				if (lo == null) {
					lo = PNetVisualizerController.getInstance().getLayoutOrientation();
					PNetVisualizerController.getInstance().setLayoutOrientation(LayoutOrientation.TOP_TO_BOTTOM);
				} else {
					PNetVisualizerController.getInstance().setLayoutOrientation(lo);
					lo = null;
				}
			}
		};
		ActionList decorators = new ActionList();
		decorators.add(new LabelDecoratorLayout("labeldecorator"));
		decorators.add(new LabelDecoratorLayout(LABEL_DECORATOR_GROUP_NAME));
		decorators.add(new LabelLayout2(EDGE_DECORATORS));
		decorators.add(new RepaintAction());

		ActionList layout = new ActionList();
		layout.add(orientationAction);
		layout.add(new SpecifiedLayout("graph.nodes", "x", "y"));
		layout.add(decorators);
		layout.add(new FitGroupBoundsAction("graph", 10, 0, false));
		layout.add(new RepaintAction());
		layout.add(orientationAction);

		layout.add(new RepaintAction());
		// once actions have been created (see above)
		// they are added to the Visualization.
		m_vis.putAction("decorators", decorators);
		m_vis.putAction("color", color);
		m_vis.putAction("layout", layout);
		m_vis.putAction(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME, shape);
		m_vis.run("color");
		m_vis.run("layout");
		m_vis.run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);

		m_vis.addDecorators(LABEL_DECORATOR_GROUP_NAME, GraphVisualization.GRAPH_NODES_GROUP, LABEL_DECORATOR_SCHEMA);
		m_vis.addDecorators(EDGE_DECORATORS, EDGES, DECORATOR_SCHEMA);
		//		m_vis.addDecorators(AGGR_DECORATORS, AGGR, new HoverPredicate(), DECORATOR_SCHEMA);

		//		Renderer nodeR = new ShapeRenderer();
		CustomShapeRendererBPMN r = new CustomShapeRendererBPMN();
		//		EdgeRenderer edgeR = new EdgeRenderer2(prefuse.Constants.EDGE_TYPE_CURVE, prefuse.Constants.EDGE_ARROW_FORWARD);
		EdgeRenderer edgeR = new CustomEdgeRenderer();
		//		edgeR.setEdgeType(Constants.EDGE_TYPE_CURVE);

		DefaultRendererFactory drf = new DefaultRendererFactory();
		drf.setDefaultRenderer(r);
		drf.setDefaultEdgeRenderer(edgeR);
		drf.add(new InGroupPredicate(LABEL_DECORATOR_GROUP_NAME), new LabelRenderer("label"));
		drf.add(new InGroupPredicate(EDGE_DECORATORS), new CustomLabelRendereForEdgesBPMN(VisualItem.LABEL));

		m_vis.setRendererFactory(drf);

		addControlListener(new focusRenderer());

		this.requestFocus();
		
		this.addKeyListener(new KeyListener() {

			public void keyTyped(java.awt.event.KeyEvent e) {
				// TODO Auto-generated method stub

			}

			public void keyReleased(java.awt.event.KeyEvent e) {
				// TODO Auto-generated method stub

			}

			public void keyPressed(java.awt.event.KeyEvent e) {
				if (e.getKeyCode() == java.awt.event.KeyEvent.VK_F11) {
					
				}

			}
		});
	}

	public static JPanel getLogView(LayoutAlgorithmReductionRules lar, PetrinetToBPMN pbpm) {

		BPMNGraph ex = new BPMNGraph(lar, pbpm);

		JPanel panel2 = new JPanel(new BorderLayout());
		panel2.add(ex, BorderLayout.CENTER);
		panel = panel2;
		return panel2;
	}

	public static void main(String[] argv) {
		BPMNGraph ex = new BPMNGraph(null, null);
		JFrame frame = new JFrame("prefuse example");
		frame.getContentPane().add(ex);
		frame.pack(); // layout components in window
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true); // show the window
	}

	class LabelLayout2 extends Layout {
		public LabelLayout2(String group) {
			super(group);
		}

		public void run(double frac) {
			Iterator iter = m_vis.items(m_group);
			while (iter.hasNext()) {
				DecoratorItem decorator = (DecoratorItem) iter.next();
				VisualItem decoratedItem = decorator.getDecoratedItem();
				Rectangle2D bounds = decoratedItem.getBounds();
				double x = bounds.getCenterX();
				double y = bounds.getCenterY();
				/* modification to move edge labels more to the arrow head */
				//				double x2 = 0, y2 = 0;
				//				if (decoratedItem instanceof EdgeItem) {
				//					VisualItem dest = ((EdgeItem) decoratedItem).getTargetItem();
				//					x2 = dest.getX();
				//					y2 = dest.getY();
				//					x = (x + x2) / 2;
				//					y = (y + y2) / 2;
				//				}

				setX(decorator, null, x);
				setY(decorator, null, y);
			}
		}
	}

	class EdgeRenderer2 extends EdgeRenderer {
		public EdgeRenderer2(int edgeTypeCurve, int edgeArrowForward) {
			super(edgeTypeCurve, edgeArrowForward);
		}

		protected Shape getRawShape(VisualItem item) {
			EdgeItem edge = (EdgeItem) item;
			VisualItem item1 = edge.getSourceItem();
			VisualItem item2 = edge.getTargetItem();

			int type = 0;
			if (item1.equals(item2))
				type = 1;

			getAlignedPoint(m_tmpPoints[0], item1.getBounds(), m_xAlign1, m_yAlign1);
			getAlignedPoint(m_tmpPoints[1], item2.getBounds(), m_xAlign2, m_yAlign2);
			m_curWidth = (float) (m_width * getLineWidth(item));

			// create the arrow head, if needed
			EdgeItem e = (EdgeItem) item;
			if (e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE) {
				// get starting and ending edge endpoints
				boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
				Point2D start = null, end = null;
				start = m_tmpPoints[forward ? 0 : 1];
				end = m_tmpPoints[forward ? 1 : 0];

				// compute the intersection with the target bounding box
				VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
				int i = GraphicsLib.intersectLineRectangle(start, end, dest.getBounds(), m_isctPoints);
				if (i > 0)
					end = m_isctPoints[0];

				// create the arrow head shape
				AffineTransform at = getArrowTrans(start, end, m_curWidth);
				m_curArrow = at.createTransformedShape(m_arrowHead);

				// update the endpoints for the edge shape
				// need to bias this by arrow head size
				Point2D lineEnd = m_tmpPoints[forward ? 1 : 0];
				lineEnd.setLocation(0, -m_arrowHeight);
				at.transform(lineEnd, lineEnd);
			} else {
				m_curArrow = null;
			}

			// create the edge shape
			Shape shape = null;
			double n1x = m_tmpPoints[0].getX();
			double n1y = m_tmpPoints[0].getY();
			double n2x = m_tmpPoints[1].getX();
			double n2y = m_tmpPoints[1].getY();
			switch (type) {
				case Constants.EDGE_TYPE_LINE :
					m_line.setLine(n1x, n1y, n2x, n2y);
					shape = m_line;
					break;
				case Constants.EDGE_TYPE_CURVE :
					getCurveControlPoints(edge, m_ctrlPoints, n1x, n1y, n2x, n2y);
					m_cubic.setCurve(n1x, n1y, n1x, n1y + 100, n2x, n2y + 20, n2x, n2y);
					shape = m_cubic;
					break;
				default :
					throw new IllegalStateException("Unknown edge type");
			}

			// return the edge shape
			return shape;
		}

	}

	class focusRenderer extends FocusControl {
		public focusRenderer() {
			super();
		}
		
		public void mouseClicked(MouseEvent e) {
			if (SwingUtilities.isMiddleMouseButton(e)){
				System.out.println("xxx");
				FileOutputStream fileOutPutStream;
				try {
					fileOutPutStream = new FileOutputStream("img100.png");
					//		ExportAsPDF.convertToPdf(this, this.WIDTH, this.HEIGHT, "");
					BPMNGraph.this.saveImage(fileOutPutStream, "png", 7.);
					fileOutPutStream.flush();
					fileOutPutStream.close();
					System.out.println("xxx22");
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		}

		public void keyPressed(java.awt.event.KeyEvent e) {
			System.out.println("here");
			if (e.getKeyCode() == KeyEvent.VK_F2) {
//				try {
//					try {
//						ExportAsPDF.convertToPdf(panel, 1, 1, "");
//					} catch (DocumentException e1) {
						// TODO Auto-generated catch block
//						e1.printStackTrace();
//					}
//				} catch (FileNotFoundException e1) {
					// TODO Auto-generated catch block
//					e1.printStackTrace();
//				}
			}
		}
	}
}