package org.processmining.discoverycontest.layout;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class BPMNUtils {
	public static void removeNodes(PetrinetToBPMN pbpm, Petrinet pn) {
		for (PetrinetNode t : pbpm.getHiddenNodes()) {
			if (t instanceof Transition && ((Transition) t).isInvisible()) {
				//remove this silent transition if it has one incoming and one outgoing edge
				Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edgesIn = pn.getInEdges(t);
				Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edgesOut = pn.getOutEdges(t);
				if (edgesOut.size() > 1) {
					pn.removeTransition((Transition) t);
					pn.removeNode(t);
				}
			}
		}

	}

	public static void makeSilentSplitsVisible(PetrinetToBPMN pbpm, Petrinet pn) {
		for (Transition t : pn.getTransitions()) {
			if (t.isInvisible()) {
				if (pbpm.getAndSplits().contains(t)) {
					t.setInvisible(false);
					t.getAttributeMap().put(AttributeMap.LABEL, "andsplit");
				}
				if (pbpm.getIorSplits().contains(t)) {
					t.setInvisible(false);
					t.getAttributeMap().put(AttributeMap.LABEL, "iorsplit");
				}
			}
		}
	}

	public static void updateSilentLabels(PetrinetToBPMN pbpm, Petrinet pn) {
		for (Transition t : pn.getTransitions()) {
			if (t.isInvisible()) {
				if (pbpm.getAndSplits().contains(t)
						&& (t.getAttributeMap().get(AttributeMap.LABEL).equals("andsplit") || t.getAttributeMap()
								.get(AttributeMap.LABEL).equals("iorsplit"))) {
					t.setInvisible(true);
				}
			}
		}
	}

	//add silent transitions before and after splits and joins (in order to increase the spacing)
	public static void addSilentPrePostSplitJoins(Petrinet pn) {
		Set<Transition> tAll = new HashSet<Transition>(pn.getTransitions());
		Set<PetrinetNode> alreadyAdded = new HashSet<PetrinetNode>();
		for (Transition t : tAll) {
			if (t.isInvisible()) {
				if (pn.getOutEdges(t).size() > 1 || pn.getInEdges(t).size() > 1) {
					//between tran and target place
					Set<Place> targetPlaces = new HashSet<Place>();
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t))
						targetPlaces.add((Place) e.getTarget());
					for (Place p : targetPlaces) {
						//						if (!alreadyAdded.contains(p)) {
						pn.removeArc(t, p);
						Place p2 = pn.addPlace("tau");
						Transition t2 = pn.addTransition("tau");
						t2.setInvisible(true);
						pn.addArc(t, p2);
						pn.addArc(p2, t2);
						pn.addArc(t2, p);
						alreadyAdded.add(p);
						//						}
					}

					//between source place and tran
					Set<Place> sourcePlaces = new HashSet<Place>();
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t))
						sourcePlaces.add((Place) e.getSource());
					for (Place p : sourcePlaces) {
						//						if (!alreadyAdded.contains(p)) {
						pn.removeArc(p, t);
						Place p2 = pn.addPlace("tau");
						Transition t2 = pn.addTransition("tau");
						t2.setInvisible(true);
						pn.addArc(p, t2);
						pn.addArc(t2, p2);
						pn.addArc(p2, t);
						alreadyAdded.add(p);
						//						}
					}
				}
			}
		}
		Set<Place> pAll = new HashSet<Place>(pn.getPlaces());
		Set<Transition> validTrans = validTransitions(pn);
		for (Place p : pAll) {
			if (pn.getOutEdges(p).size() > 1 || pn.getInEdges(p).size() > 1) {
				//between tran and target place
				Set<Transition> targetTrans = new HashSet<Transition>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(p))
					targetTrans.add((Transition) e.getTarget());
				for (Transition t : targetTrans) {
					//					if (!t.isInvisible() || (t.isInvisible() && pn.getOutEdges(t).size() > 1)) {
					if (validTrans.contains(t)) {
						pn.removeArc(p, t);
						Place p2 = pn.addPlace("tau");
						Transition t2 = pn.addTransition("tau");
						t2.setInvisible(true);
						pn.addArc(p, t2);
						pn.addArc(t2, p2);
						pn.addArc(p2, t);
					}
				}

				//between source place and tran
				Set<Transition> sourceTrans = new HashSet<Transition>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(p))
					sourceTrans.add((Transition) e.getSource());
				for (Transition t : sourceTrans) {
					//					if (!t.isInvisible()) {
					if (validTrans.contains(t)) {
						pn.removeArc(t, p);
						Place p2 = pn.addPlace("tau");
						Transition t2 = pn.addTransition("tau");
						t2.setInvisible(true);
						pn.addArc(t, p2);
						pn.addArc(p2, t2);
						pn.addArc(t2, p);
					}
				}
			}

		}
	}

	private static Set<Transition> validTransitions(Petrinet pn) {
		Set<Transition> validTrans = new HashSet<Transition>();
		for (Transition t : pn.getTransitions()) {
			if (!t.isInvisible())
				validTrans.add(t);
			else {
				if (pn.getInEdges(t).size() == 1 && pn.getOutEdges(t).size() == 1) {
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t)) {
						if (pn.getOutEdges(e.getSource()).size() > 1) {
							for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getOutEdges(t)) {
								if (pn.getInEdges(e2.getTarget()).size() > 1) {
									validTrans.add(t); //this is a choice skip tran
								}
							}
						}
					}
				}
			}
		}
		return validTrans;
	}

	public static void makeIORsVisible(PetrinetToBPMN pbpm, Petrinet net) {
		for (Transition t : pbpm.getIorSplits()) {
			t.setInvisible(false);
			t.getAttributeMap().put(AttributeMap.LABEL, "IORSPLIT");
		}

	}
}
