package org.processmining.discoverycontest.layout;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.processmining.framework.util.Pair;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.gui.action.FitGroupBoundsAction;
import org.processmining.humbertogarciacaballero.models.gui.action.NodeTypeShapeAction;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LabelDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomEdgeRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomShapeRendererBPMN;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.SoundnessInformation;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayoutOrientation;
import org.processmining.humbertogarciacaballero.plugins.PluginUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.Action;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.action.layout.Layout;
import prefuse.action.layout.SpecifiedLayout;
import prefuse.controls.DragControl;
import prefuse.controls.PanControl;
import prefuse.controls.ZoomControl;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.GraphicsLib;
import prefuse.util.PrefuseLib;
import prefuse.util.StrokeLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;

public class CopyOfBPMNGraph extends Display {

	private static final String LABEL_DECORATOR_GROUP_NAME = "labelgroup";
	public static final String EDGE_DECORATORS = "edgeDeco";
	private static final Schema DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
	public static final String EDGES = "graph.edges";
	public static final String AGGR_DECORATORS = "aggrDeco";
	public static final String AGGR = "aggregates";
	static {
		DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.gray(128));
		DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Tahoma", 16));
	}

	public CopyOfBPMNGraph(LayoutAlgorithmReductionRules lar, PetrinetToBPMN pbpm) {

		super(new Visualization());

		addControlListener(new ZoomControl());
		addControlListener(new PanControl());
		addControlListener(new DragControl("decorators"));
		setHighQuality(true);

		Petrinet pn = lar.getWs().net.getNet();
		Set<Place> initMarking = new HashSet<Place>();
		initMarking.add(PetrinetUtils.getInitialPlace(pn));
		Marking m = new Marking();
		SoundnessInformation information = new SoundnessInformation();

		int i = 0;
		Map<PetrinetNode, Integer> idMap = new HashMap<PetrinetNode, Integer>();
		for (Place p : pn.getPlaces())
			idMap.put(p, i++);
		for (Transition t : pn.getTransitions())
			idMap.put(t, i++);

		Graph vt = PluginUtils.netToGraph(null, pn, null, information, null, idMap);
		vt.addColumn("x", double.class);
		vt.addColumn("y", double.class);
		vt.addColumn("ior", boolean.class);
		vt.addColumn("and", boolean.class);
		vt.addColumn("xor", boolean.class);
		Iterator iter = vt.nodes();

		Map<Double, Double> increaseTheSizeForGateWays = new HashMap<Double, Double>();
		Map<PetrinetNode, Double> newPositionForGateways = new HashMap<PetrinetNode, Double>();
		Double minX = 100.;
		while (iter.hasNext()) {
			Node n = (Node) iter.next();
			int id = n.getInt("ID");
			PetrinetNode pnode = null;
			for (PetrinetNode pnode1 : idMap.keySet())
				if (idMap.get(pnode1) == id)
					pnode = pnode1;
			if (pbpm.getIorSplits().contains(pnode) || pbpm.getXorSplits().contains(pnode)
					|| pbpm.getAndSplits().contains(pnode)) {
				Double x = lar.getXY(pnode).getFirst();

				//get the minimum distance of prev and next nodes
				Double prevMinX = 99.;
				Double nextMinX = 99.;
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pn.getInEdges(pnode)) {
					Double tempDiffX = x - lar.getXY(edge.getSource()).getFirst();
					tempDiffX = tempDiffX < 0 ? tempDiffX * -1. : tempDiffX;
					if (tempDiffX < minX) {
						prevMinX = lar.getXY(edge.getSource()).getFirst();
					}
				}
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pn.getOutEdges(pnode)) {
					Double tempDiffX = x - lar.getXY(edge.getTarget()).getFirst();
					tempDiffX = tempDiffX < 0 ? tempDiffX * -1. : tempDiffX;
					if (tempDiffX < minX)
						nextMinX = lar.getXY(edge.getTarget()).getFirst();
				}

				Double totalPrevDist = 0.;

				if (x - prevMinX < minX) {
					totalPrevDist = minX - (prevMinX - x);
					newPositionForGateways.put(pnode, totalPrevDist);
				}
				if (nextMinX - x < minX) {
					totalPrevDist += minX - (x - nextMinX);
				}
				if (totalPrevDist > 0.) {
					increaseTheSizeForGateWays.put(x, totalPrevDist);
				}

			}
		}

		iter = vt.nodes();
		//add bpmn and layout info
		while (iter.hasNext()) {
			Node n = (Node) iter.next();
			int id = n.getInt("ID");
			PetrinetNode pnode = null;
			for (PetrinetNode pnode1 : idMap.keySet())
				if (idMap.get(pnode1) == id)
					pnode = pnode1;
			Pair<Double, Double> xy = lar.getXY(pnode);

			if (pnode instanceof Place && PetrinetUtils.getInitialPlace(pn).equals(pnode))
				System.out.println("here");

			if (pnode instanceof Transition && PetrinetUtils.getInitialTranistion(pn).equals(pnode))
				System.out.println("here");

			Double x = xy.getFirst();

			for (Double key : increaseTheSizeForGateWays.keySet()) {
				if (key > x) {
					x += increaseTheSizeForGateWays.get(key);
				}

			}
			if (newPositionForGateways.containsKey(pnode))
				x += newPositionForGateways.get(pnode);
			n.set("x", xy.getFirst());
			n.set("y", xy.getSecond());

			//update ior xor and and gateways
			System.out.println(pnode.getLabel().toString().equals("iorsplit") + " : "
					+ pnode.getLabel().toString().equals("andsplit"));
			if (pbpm != null) {
				if (pbpm.getIorSplits().contains(pnode)) {
					n.set("label", "o");
					n.set("type", "IOR");
				}
				if (pbpm.getAndSplits().contains(pnode)) {
					n.set("label", "+");
					n.set("type", "AND");
				}
				if (pbpm.getXorSplits().contains(pnode)) {
					n.set("label", "x");
					n.set("type", "XOR");
				}
			}
			n.setBoolean("ior", pbpm.getIorSplits().contains(pnode));
			n.setBoolean("and", pbpm.getAndSplits().contains(pnode));
			n.setBoolean("xor", pbpm.getXorSplits().contains(pnode));
		}

		Iterator iter2 = vt.edges();
		//hide edges and add bends info
		while (iter2.hasNext()) {
			Edge e = (Edge) iter2.next();
			int id1 = e.getSourceNode().getInt("ID");
			int id2 = e.getTargetNode().getInt("ID");
			PetrinetNode source = null;
			PetrinetNode target = null;

			for (PetrinetNode pnode : idMap.keySet())
				if (idMap.get(pnode) == id1)
					source = pnode;
				else if (idMap.get(pnode) == id2)
					target = pnode;

			if (target instanceof Place && pn.getInEdges(target).size() == 1 && pn.getOutEdges(target).size() == 1) {
				Transition targetTran = null;
				Node newTarget = null;
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> petriEdge : pn.getOutEdges(target)) {
					targetTran = (Transition) petriEdge.getTarget();
				}
				iter = vt.nodes();
				while (iter.hasNext()) {
					Node n = (Node) iter.next();
					if (idMap.get(targetTran).equals(n.getInt("ID")))
						newTarget = n;
				}
				Edge e2 = vt.addEdge(e.getSourceNode(), newTarget);

				List<Point2D> bends = new ArrayList<Point2D>();
				//				bends.add(new Point2D.Double(lar.getXY(source).getFirst(), lar.getXY(source).getSecond()));
				bends.addAll(lar.getXYEdges(source, target));
				bends.addAll(lar.getXYEdges(target, targetTran));
				//				bends.add(new Point2D.Double(lar.getXY(targetTran).getFirst(), lar.getXY(targetTran).getSecond()));
				vt.removeEdge(e);
				//				e2.set(PNConstants.EDGE_BENDS_KEY, lar.getEdgePoints(source, targetTran, source, target));
				e2.set(PNConstants.EDGE_BENDS_KEY, bends);
			} else if (source instanceof Place && !PetrinetUtils.getInitialPlace(pn).equals(source)
					&& pn.getInEdges(source).size() <= 1 && pn.getOutEdges(source).size() == 1) {
				vt.removeEdge(e);
			} else {
				e.set(PNConstants.EDGE_BENDS_KEY, lar.getXYEdges(source, target));
				//				e.set(PNConstants.EDGE_BENDS_KEY, new ArrayList<Point2D>());
			}
		}

		final Schema LABEL_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, true);
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", 25));
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.color(Color.BLACK));

		//		m_vis.addGraph("graph", graph);
		m_vis.addGraph("graph", vt);
		ColorAction fill = new ColorAction("graph.nodes", VisualItem.FILLCOLOR, ColorLib.color(Color.WHITE));
		ColorAction edges = new ColorAction("graph.edges", VisualItem.STROKECOLOR, ColorLib.getCategoryPalette(10,
				0.9f, 0.2f, 1.f, 1.0f)[1]);
		ColorAction edges2 = new ColorAction("graph.edges", VisualItem.FILLCOLOR, ColorLib.getCategoryPalette(10, 0.9f,
				0.2f, 1.f, 1.0f)[1]);
		StrokeAction edgeStrokeSize = new StrokeAction("graph.edges",
				StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
		StrokeAction borderSize = new StrokeAction("graph.nodes", StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
		ColorAction border = new ColorAction("graph.nodes", VisualItem.STROKECOLOR, PNConstants.STROKE_COLOR);
		//		ShapeAction shape = new ShapeAction("graph.nodes") {
		//			@Override
		//			public int getShape(VisualItem item) {
		//				NodeType type = NodeType.valueOf(item.getString(PNConstants.NODE_TYPE_KEY));
		//				if (type == NodeType.PLACE) {
		//					return Constants.SHAPE_ELLIPSE;
		//				} else {
		//					return PNConstants.;
		//				}
		//			}
		//		};

		NodeTypeShapeAction shape = new NodeTypeShapeAction("graph.nodes");
		ActionList color = new ActionList();
		color.add(fill);
		color.add(edges);
		color.add(edges2);
		color.add(edgeStrokeSize);
		color.add(borderSize);
		color.add(border);
		color.add(shape);

		Action orientationAction = new Action() {
			private LayoutOrientation lo = null;

			@Override
			public void run(double frac) {
				if (lo == null) {
					lo = PNetVisualizerController.getInstance().getLayoutOrientation();
					PNetVisualizerController.getInstance().setLayoutOrientation(LayoutOrientation.TOP_TO_BOTTOM);
				} else {
					PNetVisualizerController.getInstance().setLayoutOrientation(lo);
					lo = null;
				}
			}
		};
		ActionList decorators = new ActionList();
		decorators.add(new LabelDecoratorLayout("labeldecorator"));
		decorators.add(new LabelDecoratorLayout(LABEL_DECORATOR_GROUP_NAME));
		decorators.add(new LabelLayout2(EDGE_DECORATORS));
		decorators.add(new RepaintAction());

		ActionList layout = new ActionList();
		layout.add(orientationAction);
		layout.add(new SpecifiedLayout("graph.nodes", "x", "y"));
		layout.add(decorators);
		layout.add(new FitGroupBoundsAction("graph", 10, 0, false));
		layout.add(new RepaintAction());
		layout.add(orientationAction);

		layout.add(new RepaintAction());
		// once actions have been created (see above)
		// they are added to the Visualization.
		m_vis.putAction("decorators", decorators);
		m_vis.putAction("color", color);
		m_vis.putAction("layout", layout);
		m_vis.putAction(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME, shape);
		m_vis.run("color");
		m_vis.run("layout");
		m_vis.run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);

		m_vis.addDecorators(LABEL_DECORATOR_GROUP_NAME, GraphVisualization.GRAPH_NODES_GROUP, LABEL_DECORATOR_SCHEMA);
		m_vis.addDecorators(EDGE_DECORATORS, EDGES, DECORATOR_SCHEMA);
		//		m_vis.addDecorators(AGGR_DECORATORS, AGGR, new HoverPredicate(), DECORATOR_SCHEMA);

		//		Renderer nodeR = new ShapeRenderer();
		CustomShapeRendererBPMN r = new CustomShapeRendererBPMN();
		//		EdgeRenderer edgeR = new EdgeRenderer2(prefuse.Constants.EDGE_TYPE_CURVE, prefuse.Constants.EDGE_ARROW_FORWARD);
		EdgeRenderer edgeR = new CustomEdgeRenderer();
		//		edgeR.setEdgeType(Constants.EDGE_TYPE_CURVE);

		DefaultRendererFactory drf = new DefaultRendererFactory();
		drf.setDefaultRenderer(r);
		drf.setDefaultEdgeRenderer(edgeR);
		drf.add(new InGroupPredicate(LABEL_DECORATOR_GROUP_NAME), new LabelRenderer("label"));
		drf.add(new InGroupPredicate(EDGE_DECORATORS), new LabelRenderer(VisualItem.LABEL));

		//		drf.add(new InGroupPredicate(AGGR_DECORATORS), new LabelRenderer(VisualItem.LABEL));
		m_vis.setRendererFactory(drf);

		//		drf.add(new InGroupPredicate("label"), new CustomLabelRenderer("label"));

		//		int[] palette = new int[] { ColorLib.rgb(255, 180, 180), ColorLib.rgb(190, 190, 255) };

		//		DataColorAction nFill = new DataColorAction("graph.nodes", "flag", Constants.NOMINAL, VisualItem.FILLCOLOR,

		//	        r.setTransitionsInOutDegreeEnabled(true);
		//		SimplifiedEdgeRenderer simplifiedRenderer = new SimplifiedEdgeRenderer();
		//		simplifiedRenderer.setArrowHeadSize(3, 2);
		//		DefaultRendererFactory factory = new DefaultRendererFactory();
		//		factory.setDefaultEdgeRenderer(simplifiedRenderer);
		//
		//		factory.add(new InGroupPredicate("labeldecorator"), new CustomLabelRenderer(PNConstants.NODE_LABEL_KEY) {
		//			public String getText(VisualItem item) {
		//				return "Label";
		//			}
		//		});
		//		factory.add(new InGroupPredicate("indegreedecorator"), new LabelRenderer() {
		//			public String getText(VisualItem item) {
		//				return "X";
		//			}
		//		});
		//		factory.add(new InGroupPredicate("outdegreedecorator"), new LabelRenderer() {
		//			public String getText(VisualItem item) {
		//				return "Y";
		//			}
		//		});
		//
		//		final Schema DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		//		DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		//		DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, PNConstants.FONT_COLOR);
		//		DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Arial", 2));
		//
		//		final Schema DEGREE_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		//		DEGREE_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		//		DEGREE_DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, PNConstants.FONT_COLOR);
		//		DEGREE_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Arial", 2));
		//
		//		m_vis.addDecorators("labeldecorator", "graph.nodes", PNConstants.TRANSITION_NODE_PREDICATE, DECORATOR_SCHEMA);
		//		m_vis.addDecorators("indegreedecorator", "graph.nodes", PNConstants.TRANSITION_NODE_PREDICATE,
		//				DEGREE_DECORATOR_SCHEMA);
		//		m_vis.addDecorators("outdegreedecorator", "graph.nodes", PNConstants.TRANSITION_NODE_PREDICATE,
		//				DEGREE_DECORATOR_SCHEMA);
		//
		//		m_vis.setRendererFactory(factory);

	}

	public static JPanel getLogView(LayoutAlgorithmReductionRules lar, PetrinetToBPMN pbpm) {

		CopyOfBPMNGraph ex = new CopyOfBPMNGraph(lar, pbpm);

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(ex, BorderLayout.CENTER);
		return panel;
	}

	public static void main(String[] argv) {
		CopyOfBPMNGraph ex = new CopyOfBPMNGraph(null, null);
		JFrame frame = new JFrame("prefuse example");
		frame.getContentPane().add(ex);
		frame.pack(); // layout components in window
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true); // show the window
	}

	class LabelLayout2 extends Layout {
		public LabelLayout2(String group) {
			super(group);
		}

		public void run(double frac) {
			Iterator iter = m_vis.items(m_group);
			while (iter.hasNext()) {
				DecoratorItem decorator = (DecoratorItem) iter.next();
				VisualItem decoratedItem = decorator.getDecoratedItem();
				Rectangle2D bounds = decoratedItem.getBounds();
				double x = bounds.getCenterX();
				double y = bounds.getCenterY();
				/* modification to move edge labels more to the arrow head */
				//				double x2 = 0, y2 = 0;
				//				if (decoratedItem instanceof EdgeItem) {
				//					VisualItem dest = ((EdgeItem) decoratedItem).getTargetItem();
				//					x2 = dest.getX();
				//					y2 = dest.getY();
				//					x = (x + x2) / 2;
				//					y = (y + y2) / 2;
				//				}

				setX(decorator, null, x);
				setY(decorator, null, y);
			}
		}
	}

	class EdgeRenderer2 extends EdgeRenderer {
		public EdgeRenderer2(int edgeTypeCurve, int edgeArrowForward) {
			super(edgeTypeCurve, edgeArrowForward);
		}

		protected Shape getRawShape(VisualItem item) {
			EdgeItem edge = (EdgeItem) item;
			VisualItem item1 = edge.getSourceItem();
			VisualItem item2 = edge.getTargetItem();

			int type = 0;
			if (item1.equals(item2))
				type = 1;

			getAlignedPoint(m_tmpPoints[0], item1.getBounds(), m_xAlign1, m_yAlign1);
			getAlignedPoint(m_tmpPoints[1], item2.getBounds(), m_xAlign2, m_yAlign2);
			m_curWidth = (float) (m_width * getLineWidth(item));

			// create the arrow head, if needed
			EdgeItem e = (EdgeItem) item;
			if (e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE) {
				// get starting and ending edge endpoints
				boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
				Point2D start = null, end = null;
				start = m_tmpPoints[forward ? 0 : 1];
				end = m_tmpPoints[forward ? 1 : 0];

				// compute the intersection with the target bounding box
				VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
				int i = GraphicsLib.intersectLineRectangle(start, end, dest.getBounds(), m_isctPoints);
				if (i > 0)
					end = m_isctPoints[0];

				// create the arrow head shape
				AffineTransform at = getArrowTrans(start, end, m_curWidth);
				m_curArrow = at.createTransformedShape(m_arrowHead);

				// update the endpoints for the edge shape
				// need to bias this by arrow head size
				Point2D lineEnd = m_tmpPoints[forward ? 1 : 0];
				lineEnd.setLocation(0, -m_arrowHeight);
				at.transform(lineEnd, lineEnd);
			} else {
				m_curArrow = null;
			}

			// create the edge shape
			Shape shape = null;
			double n1x = m_tmpPoints[0].getX();
			double n1y = m_tmpPoints[0].getY();
			double n2x = m_tmpPoints[1].getX();
			double n2y = m_tmpPoints[1].getY();
			switch (type) {
				case Constants.EDGE_TYPE_LINE :
					m_line.setLine(n1x, n1y, n2x, n2y);
					shape = m_line;
					break;
				case Constants.EDGE_TYPE_CURVE :
					getCurveControlPoints(edge, m_ctrlPoints, n1x, n1y, n2x, n2y);
					m_cubic.setCurve(n1x, n1y, n1x, n1y + 100, n2x, n2y + 20, n2x, n2y);
					shape = m_cubic;
					break;
				default :
					throw new IllegalStateException("Unknown edge type");
			}

			// return the edge shape
			return shape;
		}

	}

}