package org.processmining.discoverycontest.layout;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class IncrementalLayoutCalculations {
	//base line is x=0
	private static Map<PetrinetNode, Pair<Short, Short>> nodePos;
	private static Set<Pair<Short, Short>> placeHolderPos;

	static Petrinet pn;

	public IncrementalLayoutCalculations(Petrinet net) {
		pn = net;
		nodePos = new HashMap<PetrinetNode, Pair<Short, Short>>();
		placeHolderPos = new HashSet<Pair<Short, Short>>();
		for (Place p : pn.getPlaces()) {
			if (PetrinetUtils.getInitialPlace(net).equals(p))
				nodePos.put(p, new Pair<Short, Short>((short) 0, (short) 1));
			else if (PetrinetUtils.getFinalPlace(net).equals(p))
				nodePos.put(p, new Pair<Short, Short>((short) 0, (short) 5));
			else
				nodePos.put(p, new Pair<Short, Short>((short) 0, (short) 3));
		}
		for (Transition t : pn.getTransitions()) {
			if (PetrinetUtils.getInitialTranistion(net).equals(t))
				nodePos.put(t, new Pair<Short, Short>((short) 0, (short) 2));
			else
				nodePos.put(t, new Pair<Short, Short>((short) 0, (short) 4));
		}

	}

	public static void useLDRule(Set<PetrinetNode> preNodes, Set<PetrinetNode> postNodes, PetrinetNode node) {
		short layer = -1;
		short posi = -1;
		for (PetrinetNode t : preNodes) {
			if (nodePos.get(t).getFirst() > layer)
				layer = nodePos.get(t).getFirst();
			if (nodePos.get(t).getSecond() > posi)
				posi = nodePos.get(t).getSecond();
		}

		short maxPosi = -1;
		for (PetrinetNode t : postNodes) {
			if (nodePos.get(t).getSecond() > maxPosi)
				maxPosi = nodePos.get(t).getSecond();
		}

		//check if layer and posi combi is already occupied - in nodePos and/or placeHolderPosi - chose smarter layer posi in that case
		layer = (short) (layer + 1);
		nodePos.put(node, new Pair<Short, Short>(layer, ((short) (posi + 1))));

		//add placeholders
		for (int i = posi + 2; i < maxPosi; i++) {
			placeHolderPos.add(new Pair<Short, Short>(layer, ((short) (i))));
		}
	}

	public static void useAbstractionRule(Set<Transition> prevNode, Set<Place> nextNodes, Transition newTran,
			Place newPlace) {
		short layer = 99;
		short posi = 999;
		for (Place p : nextNodes) {
			if (nodePos.get(p).getFirst().equals(0)) {
				layer = 0;
			} else if (layer != 0 && layer > nodePos.get(p).getFirst()) {
				layer = nodePos.get(p).getFirst();
			}
		}
		for (Transition t : prevNode)
			posi = nodePos.get(t).getSecond() < posi ? nodePos.get(t).getSecond() : posi;
		Pair<Short, Short> placePos = new Pair<Short, Short>(layer, (short) (posi + 1));
		Pair<Short, Short> tranPos = new Pair<Short, Short>(layer, (short) (posi + 2));

		for (PetrinetNode node : nodePos.keySet()) {
			//			if (nodePos.get(node).getFirst().equals(layer) && nodePos.get(node).getSecond() >= posi + 1) {
			if (nodePos.get(node).getSecond() >= posi + 2) {
				nodePos.put(node, new Pair<Short, Short>(layer, (short) (nodePos.get(node).getSecond() + 2)));
			}
		}
		nodePos.put(newTran, tranPos);
		nodePos.put(newPlace, placePos);

		//if placeholder contained this exact position and layer, remove the posi from place holder
		Set<Pair<Short, Short>> placeHolderPosTemp = new HashSet<Pair<Short, Short>>(placeHolderPos);
		for (Pair<Short, Short> pos : placeHolderPosTemp) {
			if (pos.getFirst().equals(layer) && (pos.getSecond().equals(posi + 2) || pos.getSecond().equals(posi + 1)))
				placeHolderPos.remove(pos);
		}

		placeHolderPosTemp = new HashSet<Pair<Short, Short>>(placeHolderPos);
		//update the placeholders posi of all the following posis
		for (Pair<Short, Short> pos : placeHolderPosTemp) {
			if (pos.getSecond() >= posi + 1) {
				if (!(pos.getSecond() == posi + 2) && !(pos.getSecond() == posi + 1))
					placeHolderPos.remove(pos);
				placeHolderPos.add(new Pair<Short, Short>(pos.getFirst(), (short) (pos.getSecond() + 2)));
			}

		}

		boolean loop = true;
		PetrinetNode node = null;
		//		updateAllNonLoopPositions(pn, null, nodePos);
	}

	private static void updateAllNonLoopPositions(Petrinet pn, PetrinetNode node,
			Map<PetrinetNode, Pair<Short, Short>> nodePos) {
		if (node == null)
			node = PetrinetUtils.getInitialPlace(pn);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(node)) {
			if (nodePos.get(node).getSecond() < nodePos.get(e.getTarget()).getSecond()) {
				updateAllNonLoopPositions(pn, e.getTarget(), nodePos);
			} else {
				//update all the positions of the layer
				updateAllValsForLayer(nodePos.get(e.getTarget()).getFirst(), nodePos.get(node).getSecond());
			}
		}

	}

	private static void updateAllValsForLayer(Short layerNbr, Short minX) {
		for (PetrinetNode node : nodePos.keySet()) {
			if (nodePos.get(node).getFirst().equals(layerNbr) && nodePos.get(node).getSecond() > minX) {
				nodePos.put(node, new Pair<Short, Short>(layerNbr, (short) (minX + 2)));
			}
		}

	}

	public void useAbstractionRule(Set<Integer> att1, Set<Integer> att2, InteractivePetriNet ipn, int newPlaceID,
			int newTranID) {
		Set<Transition> sourceNodes = new HashSet<Transition>();
		Set<Place> targetNodes = new HashSet<Place>();
		Place newPlace = ipn.getPlaceBasedOnID(newPlaceID);
		Transition newTran = ipn.getTranBasedOnID(newTranID);
		for (int i : att1) {
			sourceNodes.add(ipn.getTranBasedOnID(i));
		}
		for (int i : att2) {
			targetNodes.add(ipn.getPlaceBasedOnID(i));
		}
		useAbstractionRule(sourceNodes, targetNodes, newTran, newPlace);
	}

	public void useLDRule(InteractivePetriNet ipn, Set<Integer> preInt, Set<Integer> postInt, int newPlaceID,
			boolean isPlaceLD) {
		PetrinetNode newNode;
		Set<PetrinetNode> preNodes = new HashSet<PetrinetNode>();
		Set<PetrinetNode> postNodes = new HashSet<PetrinetNode>();
		if (isPlaceLD) {
			newNode = ipn.getPlaceBasedOnID(newPlaceID);
			for (int i : preInt)
				preNodes.add(ipn.getTranBasedOnID(i));
			for (int i : postInt)
				postNodes.add(ipn.getTranBasedOnID(i));
		} else {
			newNode = ipn.getTranBasedOnID(newPlaceID);
			for (int i : preInt)
				preNodes.add(ipn.getPlaceBasedOnID(i));
			for (int i : postInt)
				postNodes.add(ipn.getPlaceBasedOnID(i));
		}
		useLDRule(preNodes, postNodes, newNode);
	}

	public static Map<PetrinetNode, Pair<Short, Short>> getNodePos() {
		return nodePos;
	}

	public static void setNodePos(Map<PetrinetNode, Pair<Short, Short>> nodePos) {
		IncrementalLayoutCalculations.nodePos = nodePos;
	}

}
