package org.processmining.discoverycontest.layout;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;

public class PetrinetToBPMN {
	Set<Transition> andSplits = new HashSet<Transition>();
	Set<Transition> iorSplits = new HashSet<Transition>();
	Set<Place> xorSplits = new HashSet<Place>();
	Set<PetrinetNode> hiddenNodes = new HashSet<PetrinetNode>();
	Map<String, String> guards = new HashMap<String, String>();

	public Map<String, String> getGuards() {
		return guards;
	}

	public void setLongTermVal(Map<String, String> longTermVal) {
		this.guards = guards;
	}

	public Set<Transition> getAndSplits() {
		return andSplits;
	}

	public void setAndSplits(Set<Transition> andSplits) {
		this.andSplits = andSplits;
	}

	public Set<Transition> getIorSplits() {
		return iorSplits;
	}

	public void setIorSplits(Set<Transition> iorSplits) {
		this.iorSplits = iorSplits;
	}

	public Set<Place> getXorSplits() {
		return xorSplits;
	}

	public void setXorSplits(Set<Place> xorSplits) {
		this.xorSplits = xorSplits;
	}

	public Set<PetrinetNode> getHiddenNodes() {
		return hiddenNodes;
	}

	public void setHiddenNodes(Set<PetrinetNode> hiddenNodes) {
		this.hiddenNodes = hiddenNodes;
	}

	public PetrinetToBPMN(Petrinet pn, PetriNetWithData net1) {
		//		Petrinet pn = pnmap.getPetriNet();
		iorSplits.clear();
		xorSplits.clear();
		hiddenNodes.clear();
		andSplits.clear();
		Map<Transition, Set<Transition>> iorTrans = updateHiddenIORTrans(pn);
		Set<Transition> hiddenTrans = new HashSet<Transition>();

		for (Transition t : iorTrans.keySet()) {
			hiddenTrans.addAll(iorTrans.get(t));
		}

		if (net1 == null) {
			//ior joins
			for (Transition t : iorTrans.keySet()) {
				//find the common outgoing transition - as the ior join tran
				Set<Place> allEndPlaces = new HashSet<Place>();
				Set<Place> duplPlaces = new HashSet<Place>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t)) {
					duplPlaces.add((Place) e.getTarget());
				}
				for (Transition t2 : iorTrans.get(t)) {
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t2)) {
//						if (allEndPlaces.contains(e.getTarget()))
//							duplPlaces.add((Place) e.getTarget());
//						else
							allEndPlaces.add((Place) e.getTarget());
					}
				}
				allEndPlaces.removeAll(duplPlaces);
				for (Place p : allEndPlaces) {
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(p)) {
						iorSplits.add((Transition) e.getTarget());
					}
				}
			}

			//ior splits
			Set<Transition> IORTrans = iorTrans.keySet();
			for (Transition t : pn.getTransitions()) {
				if (t.isInvisible()) {
					if (hiddenTrans.contains(t)) {
						hiddenNodes.add(t);
					} else if (IORTrans.contains(t))
						iorSplits.add(t);
				}
			}
		}else{
			for(Transition t: pn.getTransitions())
				if(t.getLabel().equals("IORSPLIT")){
					iorSplits.add(t);
					t.setInvisible(true);
				}
					
		}
		//xor choices
		for (Place p : pn.getPlaces())
			if (pn.getInEdges(p).size() > 1 || pn.getOutEdges(p).size() > 1)
				xorSplits.add(p);
			else
				hiddenNodes.add(p);

		//and splits
		for (Transition t : pn.getTransitions()) {
			if (t.isInvisible()) {
				if (!hiddenTrans.contains(t) && !iorSplits.contains(t)) {
					if (pn.getInEdges(t).size() > 1 || pn.getOutEdges(t).size() > 1)
						andSplits.add(t);
					else
						hiddenNodes.add(t);
				}
			}
		}

		//load data attributes based on labes
		if (net1 != null) {
			guards = ExtractGuards.extract(net1);
		}

	}

	//get ior trans (both splits and joins)
	private static Map<Transition, Set<Transition>> updateHiddenIORTrans2(Petrinet pn) {
		Map<Transition, Set<Transition>> iorTrans = new HashMap<Transition, Set<Transition>>();

		for (Transition t : pn.getTransitions()) {
			Set<Transition> possibleIORs = new HashSet<Transition>();
			if (t.isInvisible() && pn.getOutEdges(t).size() > 1 && !contains(t, iorTrans)) {
				//if more than one outgoing edges, try to find if prev place is a choice split with at least one additional silent outoging tran
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t)) {
					Place p = (Place) e.getSource();
					if (pn.getOutEdges(p).size() > 1) {
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOutP : pn.getOutEdges(p)) {
							Transition t2 = (Transition) eOutP.getTarget();
							if (t2.isInvisible() && !t.equals(t2))
								possibleIORs.add(t2);
						}
					}
				}

				if (!possibleIORs.isEmpty()) {
					//get the outgoing Transitions of the current transition
					Set<Transition> tranOut = new HashSet<Transition>();
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e1 : pn.getOutEdges(t))
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getOutEdges(e1
								.getTarget()))
							tranOut.add((Transition) e2.getTarget());

					Set<Transition> possibleIORTemp = new HashSet<Transition>(possibleIORs);
					Set<Transition> tranOutCommon = new HashSet<Transition>();
					for (Transition t2 : possibleIORTemp) {
						boolean iorTran = false;
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e1 : pn.getOutEdges(t2))
							for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getOutEdges(e1
									.getTarget()))
								if (tranOut.contains(e2.getTarget())) {
									tranOutCommon.add((Transition) e2.getTarget());
									iorTran = true;
								}
						if (!iorTran)
							possibleIORs.remove(t2);
					}

					Transition commonOutPutTran = null;
					Set<Transition> tranOutTemp = new HashSet<Transition>(tranOut);

				}

			}
		}

		return null;
	}

	private static Map<Transition, Set<Transition>> updateHiddenIORTrans(Petrinet pn) {
		Set<Transition> hiddenTrans = new HashSet<Transition>();
		Set<Transition> iORTrans = new HashSet<Transition>();
		//		Petrinet pn = ipn.getselectedPetriNetMap().getPetriNet();
		Map<Transition, Set<Transition>> iorTrans = new HashMap<Transition, Set<Transition>>();
		Set<Set<Transition>> iorSets = new HashSet<Set<Transition>>();

		for (Transition t : pn.getTransitions()) {
			Set<Transition> possibleIORs = new HashSet<Transition>();
			if (t.isInvisible() && pn.getOutEdges(t).size() > 1 && !contains(t, iorSets)) {
				//if more than one outgoing edges, try to find if prev place is a choice split with at least one additional silent outoging tran
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t)) {
					Place p = (Place) e.getSource();
					if (pn.getOutEdges(p).size() > 1) {
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOutP : pn.getOutEdges(p)) {
							Transition t2 = (Transition) eOutP.getTarget();
							if (t2.isInvisible() && !t.equals(t2))
								possibleIORs.add(t2);
						}
					}
				}

				if (!possibleIORs.isEmpty()) {
					//get the Transitions of the current transition
					Set<Transition> tranOut = new HashSet<Transition>();
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e1 : pn.getOutEdges(t))
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getOutEdges(e1
								.getTarget()))
							tranOut.add((Transition) e2.getTarget());

					Transition commonTran = null;
					Set<Transition> possibleIORTemp = new HashSet<Transition>(possibleIORs);
					Set<Transition> tranOutCommon = new HashSet<Transition>();

					for (Transition t2 : possibleIORTemp) {
						boolean iorTran = false;
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e1 : pn.getOutEdges(t2))
							for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getOutEdges(e1
									.getTarget()))
								if (tranOut.contains(e2.getTarget())) {
									tranOutCommon.add((Transition) e2.getTarget());
									iorTran = true;
								}
						if (!iorTran)
							possibleIORs.remove(t2);
					}

					if (possibleIORs.size() > 1) {
						hiddenTrans.addAll(possibleIORs);
						iORTrans.add(t);
						for (Transition t2 : tranOutCommon)
							if (t2.isInvisible())
								iORTrans.add(t2);
						Set<Transition> iorSet = new HashSet<Transition>(possibleIORs);
						iorSets.add(iorSet);
						iorTrans.put(t, new HashSet<Transition>(possibleIORs));
					}
				}
			}
		}

		//make the central transition key - AND 
		Map<Transition, Set<Transition>> iorTrans2 = new HashMap<Transition, Set<Transition>>();

		for (Transition t : iorTrans.keySet()) {
			Map<Transition, Set<Place>> tranPlacesMap = new HashMap<Transition, Set<Place>>();
			Map<Place, Integer> placeCount = new HashMap<Place, Integer>();
			Set<Place> places = new HashSet<Place>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t)) {
				places.add((Place) e.getTarget());
				placeCount.put((Place) e.getTarget(),
						placeCount.containsKey(e.getTarget()) ? placeCount.get(e.getTarget()) + 1 : 1);
			}
			tranPlacesMap.put(t, new HashSet<Place>(places));
			for (Transition t2 : iorTrans.get(t)) {
				places.clear();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t2)) {
					places.add((Place) e.getTarget());
					placeCount.put((Place) e.getTarget(),
							placeCount.containsKey(e.getTarget()) ? placeCount.get(e.getTarget()) + 1 : 1);
				}
				tranPlacesMap.put(t2, new HashSet<Place>(places));
			}

			Set<Place> commonPlaces = new HashSet<Place>();
			int max=0;
			for(Place p: placeCount.keySet())
				if(placeCount.get(p)>max)
					max=placeCount.get(p);
			
			for (Place p : placeCount.keySet())
				if (placeCount.get(p) == max)
					commonPlaces.add(p);

			Transition andTran = null;
			for (Transition t3 : tranPlacesMap.keySet())
				if (commonPlaces.equals(tranPlacesMap.get(t3)) || commonPlaces.containsAll(tranPlacesMap.get(t3)))
					andTran = t3;
			Set<Transition> transToRemove = new HashSet<Transition>(tranPlacesMap.keySet());
			transToRemove.remove(andTran);
			iorTrans2.put(andTran, transToRemove);

		}

		return iorTrans2;
	}

	private static boolean contains(Transition t, Set<Set<Transition>> iorSets) {
		for (Set<Transition> tranSet : iorSets)
			if (tranSet.contains(t))
				return true;
		return false;
	}

	private static boolean contains(Transition t2, Map<Transition, Set<Transition>> iorTrans) {
		for (Transition t : iorTrans.keySet())
			if (iorTrans.get(t).contains(t2) || t.equals(t2))
				return true;

		return false;
	}

}
