package org.processmining.discoverycontest.layout;

import java.util.HashSet;
import java.util.Set;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Temp plugin PMD", parameterLabels = { "PND", }, returnLabels = { "XLog" }, returnTypes = { PetriNetWithData.class })
public class TempPlugin {

	/**
	 * The method that does the heavy lifting for your plug-in.
	 * 
	 * Note that this method only uses the boolean which is stored in the
	 * configuration. Nevertheless, it could have used the integer and/or the
	 * String as well.
	 * 
	 * @param context
	 *            The context where to run in.
	 * @param input1
	 *            The first input.
	 * @param dclist
	 * @param log
	 * @param input2
	 *            The second input.
	 * @param configuration
	 *            The configuration to use.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	static Set<String> events = new HashSet<String>();
	public static boolean stop = false;

	public PetriNetWithData yourPrivatePlugin(UIPluginContext context, PetriNetWithData pn)
			throws IllegalAccessException, InstantiationException {

		Place p = null;

		//remove initial place and tran
		for (Place p2 : pn.getPlaces())
			if (pn.getInEdges(p2).size() == 0)
				p = p2;

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(p)) {
			pn.removeNode(e.getTarget());
		}
		pn.removeNode(p);

		//remove final place and tran
		for (Place p2 : pn.getPlaces())
			if (pn.getOutEdges(p2).size() == 0)
				p = p2;
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(p)) {
			pn.removeNode(e.getSource());
		}
		pn.removeNode(p);

		Marking initial = new Marking();
		Marking[] finalMarking = new Marking[1];
		Marking finalMarkings = new Marking();
		for (Place p2 : pn.getPlaces()) {
			if (pn.getInEdges(p2).size() == 0)
				initial.add(p2);
			if (pn.getOutEdges(p2).size() == 0)
				finalMarkings.add(p2);
		}
		finalMarking[0] = finalMarkings;
		pn.setInitialMarking(initial);
		pn.setFinalMarkings(finalMarking);

		for (Transition t : pn.getTransitions()) {
			if (t.isInvisible())
				t.getAttributeMap().put(AttributeMap.LABEL, "tau");
		}

		Transition t = null;
		for (Transition t3 : pn.getTransitions()) {
			if (t3.getLabel().equals("aa")) {
				t = t3;
			}

		}
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t)) {
			p = (Place) e.getTarget();
		}
		pn.removeNode(p); 
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t)) {
			p = (Place) e.getSource();
		}
		pn.removeNode(p); 
		pn.removeNode(t); 
		
		return pn;
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Reduce petrinet 22", requiredParameterLabels = { 0 })
	public PetriNetWithData yourDefaultPlugin(UIPluginContext context, PetriNetWithData pn)
			throws IllegalAccessException, InstantiationException {

		return yourPrivatePlugin(context, pn);

	}
}
