package org.processmining.exhaustivesearch;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.models.InteractiveMatrixNet;
import org.processmining.prabhakardixit.fcnets.models.LDSets;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class CompareExhaustiveIncrementalApproach {

	public static boolean compare(PluginContext context, InteractiveMatrixNet imn) {
		return compare(imn, getExhaustiveInfo(context,imn));
	}
	
	public static ExhaustiveInfo getExhaustiveInfo(PluginContext context, InteractiveMatrixNet imn) {
		long msecs = System.currentTimeMillis();
		ExhaustiveInfo ei = ExhaustiveLinearDependencyCalc.getLinearDependenciesExhaustively(context, imn
				.getselectedPetriNetMap().getPetriNet());
		msecs = System.currentTimeMillis() - msecs;
		return ei;
	}

	public static boolean compare(InteractiveMatrixNet imn, ExhaustiveInfo ei) {

		Set<Integer> hashOfEiAdded = new HashSet<Integer>();
		Set<Integer> hashOfImnAdded = new HashSet<Integer>();

		//1. navigate through all the lds from imn
		//2. see if similar pair exists in ei - add to the bucket for removal

		//---------LDTRANS---------
		int countInEiLDT = 0;
		int countInImnLDT = 0;
		for (LDSets tld : imn.getAllTranLDFCs()) {
			Set<Place> inputPlaces = new HashSet<Place>();
			Set<Place> outputPlaces = new HashSet<Place>();
			for (int negIdsImn : tld.getNegativeNodes(imn))
				inputPlaces.add(imn.getPlaceBasedOnID(negIdsImn));

			for (int posIdsImn : tld.getPositiveNodes(imn))
				outputPlaces.add(imn.getPlaceBasedOnID(posIdsImn));

			boolean iterate = true;
			Set<Place> ipOp = new HashSet<Place>();
			ipOp.add(PetrinetUtils.getInitialPlace(imn.getselectedPetriNetMap().getPetriNet()));
			ipOp.add(PetrinetUtils.getFinalPlace(imn.getselectedPetriNetMap().getPetriNet()));
			boolean ind = false;
			if (!Collections.disjoint(inputPlaces, ipOp) || !Collections.disjoint(outputPlaces, ipOp))
				iterate = false;
			else {
				countInImnLDT++;
				ind = true;
			}
			//navigate thru the complete ei pairs to find right pair

			Iterator iter = ei.getLdTrans().iterator();
			if (iterate) {
				for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> eip : ei.getLdTrans()) {
					if (eip.getFirst().equals(inputPlaces) && eip.getSecond().equals(outputPlaces)) {
						//					iterate = false; //found element stop iterating
						countInEiLDT++;
						ind = false;
						hashOfEiAdded.add(eip.hashCode());
					}
				}
				hashOfImnAdded.add(tld.hashCode());
				if (ind) {
					//					System.out.println(inputPlaces + "not present for some reason" + outputPlaces);
					//					System.out.println("IMN LDT Not added : " + tld.get);
				}
			}
		}

		//---------LDPLACES---------
		int countInEiLDP = 0;
		int countInImnLDP = 0;
		for (LDSets tld : imn.getplaceLDNonSiphons()) {
			Set<Transition> inputTrans = new HashSet<Transition>();
			Set<Transition> outputTrans = new HashSet<Transition>();
			for (int negIdsImn : tld.getNegativeNodes(imn))
				outputTrans.add(imn.getTranBasedOnID(negIdsImn));

			for (int posIdsImn : tld.getPositiveNodes(imn))
				inputTrans.add(imn.getTranBasedOnID(posIdsImn));

			countInImnLDP++;

			//navigate thru the complete ei pairs to find right pair
			boolean iterate = true;
			Iterator iter = ei.getLdTrans().iterator();
			for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> eip : ei.getLdPlaces()) {
				if (eip.getSecond().equals(inputTrans) && eip.getFirst().equals(outputTrans)) {
					iterate = false; //found element stop iterating
					countInEiLDP++;
				}
			}
		}

		//		for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> eip : ei.getLdTrans())
		//			if (!hashOfEiAdded.contains(eip.hashCode()))
		//				System.out.println("EIP LDT Not added : " + eip);

		//		System.out.println(" EI : " + countInEiLDP + " EI LDP : " + ei.getLdPlaces().size() + " IMN : " + countInImnLDP + " IMN LDP : " + imn.getAllTranLDFCs().size() 
		//				+ " count EI LDT : " + " EI : " + countInEiLDT + " EI LDT : " + ei.getLdTrans().size() + " IMN LDT count : "
		//				+ countInImnLDT + " IMN LDT: " + imn.getTranLDFCs().size());

		//		for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> eip : ei.getLdTrans()) {
		//			System.out.println(eip);
		//		}

		//		System.out.println("--");
		for (LDSets tld : imn.getAllTranLDFCs()) {
			Set<Place> inputPlaces = new HashSet<Place>();
			Set<Place> outputPlaces = new HashSet<Place>();
			for (int negIdsImn : tld.getNegativeNodes(imn))
				inputPlaces.add(imn.getPlaceBasedOnID(negIdsImn));

			for (int posIdsImn : tld.getPositiveNodes(imn))
				outputPlaces.add(imn.getPlaceBasedOnID(posIdsImn));
			//		System.out.println(inputPlaces + "" +  outputPlaces);
		}

		return (countInEiLDP == ei.getLdPlaces().size() && countInImnLDP == countInEiLDP
				&& countInEiLDT == ei.getLdTrans().size() && countInImnLDT == countInEiLDT);
	}
}
