package org.processmining.exhaustivesearch;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ejml.simple.SimpleMatrix;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.Woflan;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.WoflanDiagnosis;
import org.processmining.prabhakardixit.alignments.CombinationUtils;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class ExhaustiveLinearDependencyCalc {

	public static ExhaustiveInfo getLinearDependenciesExhaustively(PluginContext context, Petrinet net) {
		Map<Transition, Integer> tranInciMap = new HashMap<Transition, Integer>();
		Map<Place, Integer> placeInciMap = new HashMap<Place, Integer>();
		SimpleMatrix inciMat = MatrixUtils.createInciMap(tranInciMap, placeInciMap, net); //alsoAugmented TOD
		SimpleMatrix augMat3 = MatrixUtils.getSCIncidenceMatrix2(inciMat, placeInciMap, net);

		ExhaustiveInfo ei = new ExhaustiveInfo(); //write results intot his
		//temp
		Transition t = PetrinetUtils.getInitialTranistion(net);
		t.getAttributeMap().put(AttributeMap.LABEL, "D_I");

		//get ld trans first
		Set<Pair<Set<PetrinetNode>, Set<PetrinetNode>>> alreadyEvaluatedLDTrans = new HashSet<Pair<Set<PetrinetNode>, Set<PetrinetNode>>>();
		Set<Pair<Set<PetrinetNode>, Set<PetrinetNode>>> exhaustiveLDTrans = new HashSet<Pair<Set<PetrinetNode>, Set<PetrinetNode>>>();

		Set<Integer> cols = new HashSet<Integer>();
		for (int i = 0; i < augMat3.numCols(); i++) {
			cols.add(i);
		}

		int numComb = 0;
		//get all the combinations of the transitions - each combination is a new ld tran
		for (int i = 1; i < cols.size() + 1; i++) {
			Set<Set<Integer>> allComb = CombinationUtils.getAllCombs(cols, i);

			for (Set<Integer> comb : allComb) {
				numComb++;
				//calc ld 
				SimpleMatrix optionLinCol = new SimpleMatrix(augMat3.numRows(), 1);
				for (int colId : comb)
					optionLinCol = optionLinCol.plus(augMat3.extractVector(false, colId));

				//				MatrixUtils.printMatrix(optionLinCol);
				if (!alreadyEvaluated(alreadyEvaluatedLDTrans, getPair(optionLinCol, placeInciMap, null))) {
					//create a new net with the lds
					PetriNetMap pnmap = getLDPetriNetMap(getPair(optionLinCol, placeInciMap, null), net, true);

					//validate if it is fc and sound
					if (isFreeChoice(getPair(optionLinCol, placeInciMap, null), net)) {
						if (isSound(context, pnmap.getPetriNet())) //check if soundness check is not needed for ld transitions  
							exhaustiveLDTrans.add(getPair(optionLinCol, placeInciMap, null));
					} else { //see if it is possible to make this FC - by adding self loops
						//get the map
						Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair = updateFCPairByAddingSelfLoopToTrans(
								getPair(optionLinCol, placeInciMap, null), placeInciMap, net);
						PetriNetMap pnmap2 = getLDPetriNetMap(pair, net, true);
						if (!alreadyEvaluated(alreadyEvaluatedLDTrans, pair))
							if (isFreeChoice(pair, net) && isSound(context, pnmap2.getPetriNet())) {
								exhaustiveLDTrans.add(pair);
							}
					}
				}
			}
		}

		//finally get all self loops for transitions

		//create a new net with the lds
		for (Place p : net.getPlaces()) {
			Set<PetrinetNode> negIds = new HashSet<PetrinetNode>();
			Set<PetrinetNode> posIds = new HashSet<PetrinetNode>();
			negIds.add(p);
			posIds.add(p);
			Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair = new Pair<Set<PetrinetNode>, Set<PetrinetNode>>(negIds,
					posIds);

			pair = updateFCPairByAddingSelfLoopToTrans(pair, placeInciMap, net);
			if (!alreadyEvaluated(alreadyEvaluatedLDTrans, pair)) {
				PetriNetMap pnmap2 = getLDPetriNetMap(pair, net, true);
				if (isFreeChoice(pair, net) && isSound(context, pnmap2.getPetriNet())) {
					exhaustiveLDTrans.add(pair);
				}
			}
		}

		//----------------PLACES---------------
		//get ld places 
		Set<Pair<Set<PetrinetNode>, Set<PetrinetNode>>> alreadyEvaluatedLDPlaces = new HashSet<Pair<Set<PetrinetNode>, Set<PetrinetNode>>>();
		Set<Pair<Set<PetrinetNode>, Set<PetrinetNode>>> exhaustiveLDPlaces = new HashSet<Pair<Set<PetrinetNode>, Set<PetrinetNode>>>();
		Set<Integer> rows = new HashSet<Integer>();
		for (int i = 0; i < augMat3.numRows(); i++) {
			rows.add(i);
		}
		//get all the combinations of the places - each combination is a new ld tran
		for (int i = 1; i < rows.size() + 1; i++) {
			Set<Set<Integer>> allComb = CombinationUtils.getAllCombs(rows, i);

			for (Set<Integer> comb : allComb) {
				numComb++;
				//calc ld 
				SimpleMatrix optionLinCol = new SimpleMatrix(1, augMat3.numCols());
				for (int rowId : comb)
					optionLinCol = optionLinCol.plus(augMat3.extractVector(true, rowId));

				//create a new net with the lds
				PetriNetMap pnmap = getLDPetriNetMap(getPair(optionLinCol, null, tranInciMap), net, false);

				if (!alreadyEvaluated(alreadyEvaluatedLDPlaces, getPair(optionLinCol, null, tranInciMap))) {
					//validate if it is fc and sound
					if (isFreeChoiceForNewPlace(getPair(optionLinCol, null, tranInciMap), net)) {
						if (isSound(context, pnmap.getPetriNet())) //check if soundness check is not needed for ld transitions  
							exhaustiveLDPlaces.add(getPair(optionLinCol, null, tranInciMap));
					} else { //see if it is possible to make this FC - by adding self loops
						//get the map
						System.out.println("");
						Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair = updateFCPairByAddingSelfLoopToPlaces(
								getPair(optionLinCol, null, tranInciMap), net);
						PetriNetMap pnmap2 = getLDPetriNetMap(pair, net, false);
						if (!alreadyEvaluated(alreadyEvaluatedLDPlaces, pair))
							if (isFreeChoiceForNewPlace(pair, net) && isSound(context, pnmap2.getPetriNet())) {
								exhaustiveLDPlaces.add(pair);
							}
					}
				}
			}
		}

		System.out.println("Total number of combinations : " + numComb);
		System.out.println("Total number of linear dependencies found : " + exhaustiveLDTrans.size());

		Place pi = PetrinetUtils.getInitialPlace(net);
		Place po = PetrinetUtils.getFinalPlace(net);

		Set<Pair<Set<PetrinetNode>, Set<PetrinetNode>>> pairsToRemove = new HashSet<Pair<Set<PetrinetNode>, Set<PetrinetNode>>>();
		for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> p : exhaustiveLDTrans) {
			if (p.getFirst().contains(pi) || p.getSecond().contains(po) || p.getFirst().contains(po)
					|| p.getSecond().contains(pi)) {
				//				System.out.println(p);
				pairsToRemove.add(p);
			}

		}
		exhaustiveLDTrans.removeAll(pairsToRemove);
		ei.setLdTrans(exhaustiveLDTrans);
		ei.setLdPlaces(exhaustiveLDPlaces);
		System.out.println("Total number of linear dependencies places found : " + exhaustiveLDPlaces.size());
		

//		for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> p : exhaustiveLDPlaces) {
//			{
//				//				System.out.println(p);
//				ei.getLdTrans().add(p);
//			}
//
//		}
		ei.setTotalLDPlacesEvaluated(alreadyEvaluatedLDPlaces.size());
		ei.setTotalTimeForLDTrans(alreadyEvaluatedLDTrans.size());

		return ei;
	}

	private static boolean alreadyEvaluated(Set<Pair<Set<PetrinetNode>, Set<PetrinetNode>>> alreadyEvaluatedLDPairs,
			Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair) {
		for (Pair<Set<PetrinetNode>, Set<PetrinetNode>> p : alreadyEvaluatedLDPairs) {
			if (pair.getFirst().equals(p.getFirst()) && pair.getSecond().equals(p.getSecond()))
				return true;
		}
		alreadyEvaluatedLDPairs.add(pair);
		return false;
	}

	private static Pair<Set<PetrinetNode>, Set<PetrinetNode>> updateFCPairByAddingSelfLoopToPlaces(
			Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair, Petrinet net) {

		Set<PetrinetNode> outTrans = pair.getFirst();
		Set<PetrinetNode> allOutTrans = new HashSet<PetrinetNode>();
		for (PetrinetNode t : outTrans) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : net.getInEdges(t)) {
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : net.getOutEdges(e.getSource())) {
					allOutTrans.add(e2.getTarget());
				}
			}
		}
		//keep only uncommon out trans
		allOutTrans.removeAll(outTrans);

		if (!allOutTrans.isEmpty()) {
			//add the uncommon places as both start and end
			pair.getFirst().addAll(allOutTrans);
			pair.getSecond().addAll(allOutTrans);

		}
		return pair;
	}

	private static boolean isFreeChoiceForNewPlace(Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair, Petrinet net) {
		Set<PetrinetNode> outgoingTrans = pair.getFirst();
		for (PetrinetNode t : outgoingTrans)
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : net.getInEdges(t)) {
				Set<Transition> outTrans = new HashSet<Transition>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : net.getOutEdges(e.getSource()))
					outTrans.add((Transition) e2.getTarget());
				if (!outTrans.equals(outgoingTrans))
					return false;

			}

		return true;
	}

	//updates the pairs to include palces for fc
	private static Pair<Set<PetrinetNode>, Set<PetrinetNode>> updateFCPairByAddingSelfLoopToTrans(
			Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair, Map<Place, Integer> placeInciMap, Petrinet net) {

		Set<Place> allInPlaces = new HashSet<Place>();
		for (PetrinetNode p : pair.getFirst())
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : net.getOutEdges(p))
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : net.getInEdges(e.getTarget()))
					allInPlaces.add((Place) e2.getSource());

		//keep only common places
		allInPlaces.removeAll(pair.getFirst());
		PetriNetMap pnmap = null;
		if (!allInPlaces.isEmpty()) {
			//add the uncommon places as both start and end
			pair.getFirst().addAll(allInPlaces);
			pair.getSecond().addAll(allInPlaces);
			//get petrinet map
			//			pnmap = getLDPetriNetMap(pair, net, true);

		}
		return pair;
	}

	private static PetriNetMap getLDPetriNetMap(Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair, Petrinet net,
			boolean ldtran) {
		PetriNetMap pnmap = null;
		if (ldtran) {
			pnmap = new PetriNetMap(net);
			Petrinet newNet = pnmap.getPetriNet();
			Transition t = newNet.addTransition("");
			for (PetrinetNode p : pair.getFirst())
				newNet.addArc(pnmap.getMapPlaces().get(p), t);
			for (PetrinetNode p : pair.getSecond())
				newNet.addArc(t, pnmap.getMapPlaces().get(p));
		} else { //add new ld Place
			pnmap = new PetriNetMap(net);
			Petrinet newNet = pnmap.getPetriNet();
			Place p = newNet.addPlace("");
			for (PetrinetNode t : pair.getFirst())
				newNet.addArc(p, pnmap.getMapTransitions().get(t));
			for (PetrinetNode t : pair.getSecond())
				newNet.addArc(pnmap.getMapTransitions().get(t), p);
		}

		return pnmap;
	}

	private static Pair<Set<PetrinetNode>, Set<PetrinetNode>> getPair(SimpleMatrix optionLinCol,
			Map<Place, Integer> placeInciMap, Map<Transition, Integer> tranInciMap) {
		Set<PetrinetNode> negIds = new HashSet<PetrinetNode>();
		Set<PetrinetNode> posIds = new HashSet<PetrinetNode>();
		if (placeInciMap != null)
			for (int i = 0; i < optionLinCol.numRows(); i++) {
				if (optionLinCol.get(i, 0) == -1.)
					for (Place place : placeInciMap.keySet())
						if (placeInciMap.get(place) == i)
							negIds.add(place);

				if (optionLinCol.get(i, 0) == 1.)
					for (Place place2 : placeInciMap.keySet()) {
						if (placeInciMap.get(place2) == i)
							posIds.add(place2);
					}

			}
		else if (tranInciMap != null)
			for (int i = 0; i < optionLinCol.numCols(); i++) {
				if (optionLinCol.get(0, i) == -1.)
					for (Transition tran : tranInciMap.keySet())
						if (tranInciMap.get(tran) == i)
							negIds.add(tran);

				if (optionLinCol.get(0, i) == 1.)
					for (Transition tran2 : tranInciMap.keySet()) {
						if (tranInciMap.get(tran2) == i)
							posIds.add(tran2);
					}

			}

		Pair<Set<PetrinetNode>, Set<PetrinetNode>> negPosIds = new Pair<Set<PetrinetNode>, Set<PetrinetNode>>(negIds,
				posIds);
		return negPosIds;
	}

	private static boolean isSound(PluginContext context, Petrinet net) {
		// TODO Use woflan
		Woflan wof = new Woflan();
		WoflanDiagnosis diag = null;
		try {
			diag = wof.diagnose(context, net);
//			wof.diagnose(WoflanState.FREECHOICE);
			//			wof.diagnose(WoflanState.FREECHOICE);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return diag.isSound();
	}

	//assume that the entire net is FC, and find for the new ld, if there is another such place - based on Original Net
	private static boolean isFreeChoice(Pair<Set<PetrinetNode>, Set<PetrinetNode>> pair, Petrinet net) {
		Set<PetrinetNode> inputPlaces = new HashSet<PetrinetNode>();
		inputPlaces.addAll(pair.getFirst());

		for (PetrinetNode p : inputPlaces) {
			//get incoming places of all outgoing tran
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : net.getOutEdges(p)) {
				Set<Place> allInPlaces = new HashSet<Place>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eIn : net.getInEdges(e.getTarget())) {
					allInPlaces.add((Place) eIn.getSource());
				}
				if (!inputPlaces.equals(allInPlaces))
					return false;
			}
		}

		//check if all out trans present in all places
		boolean first = true;
		Set<Transition> outTransFirst = new HashSet<Transition>();
		for (PetrinetNode p : inputPlaces) {
			Set<Transition> outTrans = new HashSet<Transition>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : net.getOutEdges(p)) {
				outTrans.add((Transition) e.getTarget());
			}
			if (first) {
				outTransFirst.addAll(outTrans);
				first = false;
			} else {
				if (!outTrans.equals(outTransFirst))
					return false;
			}
		}

		return true;
	}
}
