package org.processmining.exhaustivesearch;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.prabhakardixit.fcnets.expansion.ExpandUsingTracker;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.fcnets.synthesis.CreateIPN;
import org.processmining.reduction.Reduction;

@Plugin(name = "432 Synthesize and compare net", parameterLabels = { "Petrinet " , "XLog", }, returnLabels = { "Interactive Petri net" }, returnTypes = { Petrinet.class })
public class SynthesisComparePlugin {

	/**
	 * The method that does the heavy lifting for your plug-in.
	 * 
	 * Note that this method only uses the boolean which is stored in the
	 * configuration. Nevertheless, it could have used the integer and/or the
	 * String as well.
	 * 
	 * @param context
	 *            The context where to run in.
	 * @param input1
	 *            The first input.
	 * @param dclist
	 * @param log
	 * @param input2
	 *            The second input.
	 * @param configuration
	 *            The configuration to use.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	static Set<String> events = new HashSet<String>();
	public static boolean stop = false;

	public Petrinet yourPrivatePlugin(UIPluginContext context, Petrinet pn, XLog log) throws IllegalAccessException,
			InstantiationException {
		PetriNetMap pnmap = new PetriNetMap(pn);
		Reduction rd = new Reduction(pnmap.getPetriNet(),context);
		
		InteractivePetriNet ipn = CreateIPN.createIPN(log, context,true,true,true);
		MatrixUtils.initiate(ipn);
		ExpandUsingTracker.expanUsingTracker(rd.getReductionTracker(), ipn);
		if(CompareExhaustiveIncrementalApproach.compare(context, ipn.getInteractiveMatNet()))
			System.out.println("TRUE STORY ");
		else
			System.out.println("FALSE STORY ");
		return ipn.getselectedPetriNetMap().getPetriNet();
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Reduce petrinet 22", requiredParameterLabels = { 0 , 1 })
	public Petrinet yourDefaultPlugin(UIPluginContext context, Petrinet pn, XLog log) throws IllegalAccessException,
			InstantiationException {

		return yourPrivatePlugin(context, pn, log);

	}
}
