package org.processmining.experiments;

import java.util.HashSet;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

@Plugin(name = "Make transitions silent and remove +complete", parameterLabels = { "PN " }, returnLabels = { "APN" }, returnTypes = { AcceptingPetriNet.class })
public class BPMNToPetriNetToAPN {

	/**
	 * The method that does the heavy lifting for your plug-in.
	 * 
	 * Note that this method only uses the boolean which is stored in the
	 * configuration. Nevertheless, it could have used the integer and/or the
	 * String as well.
	 * 
	 * @param context
	 *            The context where to run in.
	 * @param input1
	 *            The first input.
	 * @param dclist
	 * @param log
	 * @param input2
	 *            The second input.
	 * @param configuration
	 *            The configuration to use.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	static Set<String> events = new HashSet<String>();
	public static boolean stop = false;

	public AcceptingPetriNet yourPrivatePlugin(UIPluginContext context, Petrinet pn) throws IllegalAccessException,
			InstantiationException {
		AcceptingPetriNet apn = new AcceptingPetriNetImpl(pn);
		for (Transition t : apn.getNet().getTransitions()) {
			if (t.getLabel().contains("event"))
				t.setInvisible(true);
			else if (t.getLabel().contains("t_task_"))
				t.getAttributeMap().put(org.processmining.models.graphbased.AttributeMap.LABEL,
						t.getLabel().replace("t_task_", ""));
			
			
		}
		return apn;
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Petri Net generati11on", requiredParameterLabels = { 0 })
	public AcceptingPetriNet yourDefaultPlugin(UIPluginContext context, Petrinet log) throws IllegalAccessException,
			InstantiationException {

		return yourPrivatePlugin(context, log);

	}
}
