package org.processmining.experiments;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.pnalignanalysis.conformance.AlignmentPrecGen;
import org.processmining.plugins.pnalignanalysis.conformance.AlignmentPrecGenRes;
import org.processmining.prabhakardixit.alignments.AlignmentCalculations;
import org.processmining.prabhakardixit.alignments.ProjectedAveragedAlignments;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;
import org.processmining.prabhakardixit.alignments.UpdateProjectedAveragedAlignments;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.synthesis.VizToPnetMapper;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.plugins.CompareLog2PetriNetPlugin;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;
import org.processmining.reduction.ReductionTracker;

public class ExpandUsingTrackerRecordAlignmentInfo {

	public static void expanUsingTracker(List<ReductionTracker> redTrack, InteractivePetriNet ipn) {
//		Petrinet pn = ipn.getselectedPetriNetMap().getPetriNet();

		long msecs;
		for (int i = redTrack.size() - 1; i >= 0; i--) {
			long timeForPAC = 0;
			long timeForPCC;
			long timeForTraditional;
			float[] fitPrePAC = new float[2];
			float[] fitPrePCC = new float[2];
			float[] fitPreTrad = new float[2];
			ReductionTracker currRedTrack = redTrack.get(i);
			switch (currRedTrack.getMethodName()) {
				case ABSTRACTION :
					VizToPnetMapper.addNewTransition(currRedTrack.getAtt1(), currRedTrack.getAtt2(),
							currRedTrack.getTranName(), currRedTrack.getPlaceName(), ipn, false,
							currRedTrack.getNewPlaceID(), currRedTrack.getNewTranID(), currRedTrack.isSilent());
					msecs = System.currentTimeMillis();
					UpdateProjectedAveragedAlignments.addSeqAlignmentCal(ipn,
							ipn.getTranBasedOnID(currRedTrack.getNewTranID())); //alignments

					timeForPAC = System.currentTimeMillis() - msecs;
					break;
				case LDPLACE :
					VizToPnetMapper.addParallelPlace(currRedTrack.getAtt1(), currRedTrack.getAtt2(),
							currRedTrack.getPlaceName(), ipn, currRedTrack.getNewPlaceID());
					msecs = System.currentTimeMillis();
//					UpdateProjectedAveragedAlignments.addChoiceAlignmentCal(ipn,
//							ipn.getTranBasedOnID(currRedTrack.getNewTranID()));
					timeForPAC = System.currentTimeMillis() - msecs;
					break;
				case LDTRAN :
					VizToPnetMapper.addChoiceTransition(currRedTrack.getAtt1(), currRedTrack.getAtt2(),
							currRedTrack.getTranName(), ipn, currRedTrack.getNewTranID(), currRedTrack.isSilent());
					msecs = System.currentTimeMillis();
					UpdateProjectedAveragedAlignments.addChoiceAlignmentCal(ipn,
							ipn.getTranBasedOnID(currRedTrack.getNewTranID()));
					timeForPAC = System.currentTimeMillis() - msecs;
					break;
				case RENAME :
					Transition t = ipn.getTranBasedOnID(currRedTrack.getNewTranID());
					t.setInvisible(currRedTrack.isSilent());
					t.getAttributeMap().put(AttributeMap.LABEL, currRedTrack.getTranName());
					msecs = System.currentTimeMillis();
					UpdateProjectedAveragedAlignments.addLinearlyDependentCal(ipn,
							ipn.getTranBasedOnID(currRedTrack.getNewTranID()));
					timeForPAC = System.currentTimeMillis() - msecs;
					break;
			}

			msecs = System.currentTimeMillis();
			Set<String> activities = new HashSet<String>();
			for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
				if (!t.isInvisible())
					activities.add(t.getLabel());
			XLog log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), activities, false);
			try {
				fitPrePCC = calculatedPCC(ipn.getselectedPetriNetMap().getPetriNet(), log);
			} catch (ProjectedMeasuresFailedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			timeForPCC = System.currentTimeMillis() - msecs;

			msecs = System.currentTimeMillis();
			activities = new HashSet<String>();
			for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
				if (!t.isInvisible())
					activities.add(t.getLabel());
			log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), activities, false);
//			fitPrePAC = getTraditionalFitnessAndPrecision(ipn.getContext(), log, ipn.getselectedPetriNetMap()
//					.getPetriNet());
			timeForTraditional = System.currentTimeMillis() - msecs;

			System.out.println("PAC : " + timeForPAC + " PCC : " + timeForPCC + " Trad : " + timeForTraditional);
		}

		int count=0;
		double fit = 0.;
		double pre = 0.;
		for(TransPairAlignment tpa : ipn.getTranPairs()){
			fit+= tpa.getFitness();
			pre+=tpa.getPrecision();
			count++;
		}

		System.out.println( count + " Total Fitness : " + fit/count);
		System.out.println("Total Precision : " + pre/count);
		
	}

	private static float[] calculatedPCC(Petrinet pn, XLog log) throws ProjectedMeasuresFailedException {
		AcceptingPetriNet model = new AcceptingPetriNetImpl(pn);
		float[] fitPre = new float[2];
		CompareLog2PetriNetPlugin compare = new CompareLog2PetriNetPlugin();
		CompareParameters param = new CompareParameters();

		Set<String> act = new HashSet<String>();
		for (Transition t : model.getNet().getTransitions())
			if (!t.isInvisible()) {
				//				System.out.println("\n Transition : " + t);
				act.add(t.getLabel());

			}
		//		System.out.println("\n LOG: ");
		//		for (XTrace t : log)
		//			for (XEvent e : t)
		//				System.out.println(XConceptExtension.instance().extractName(e));
		param.setK(act.size() > 1 ? 2 : 1);
		ProjectedRecallPrecisionResult result = null;
		try {
			System.out.println(param.getK() + " acts " + act);
			if (param.getK() > 1)
				result = compare.measure(log, model, param, new ProMCanceller() {
					public boolean isCancelled() {
						return false;
					}
				});
		} catch (AutomatonFailedException | InterruptedException | ProjectedMeasuresFailedException e) {
			e.printStackTrace();
		}
		if (param.getK() > 1) {
			fitPre[0] = (float) result.getRecall();
			fitPre[1] = (float) result.getPrecision();
		}
		return fitPre;
	}

	public static float[] getTraditionalFitnessAndPrecision(PluginContext context, XLog log, Petrinet pn) {
		float[] fitPre = new float[2];
		PNRepResult replay = AlignmentCalculations.calculateNAlignment1(context, log, pn, 1000, false, null);
		fitPre[0] = ((Double) replay.getInfo().get(PNRepResult.TRACEFITNESS)).floatValue();

		//calculate precision
		AlignmentPrecGen precGen = new AlignmentPrecGen();
		AlignmentPrecGenRes precRes = precGen.measureConformanceAssumingCorrectAlignment(context,
				AlignmentCalculations.transitionToEventClassMapperByLabel(log, pn, null,false), replay, pn,
				PetrinetUtils.getInitialMarking(pn), true);

		fitPre[1] = ((Double) precRes.getPrecision()).floatValue();
		return fitPre;
	}

}
