package org.processmining.experiments;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.models.InteractiveMatrixNet;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.LDSets;
import org.processmining.prabhakardixit.fcnets.synthesis.VizToPnetMapper;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class SynthesizeNets {

	public static int tranCount =3;
	public static int placeCount =3;
	
	public static void addParallelPlace(InteractivePetriNet ipn) {
		InteractiveMatrixNet imn = ipn.getInteractiveMatNet();
		List<LDSets> placeLD = ipn.getplaceLDNonSiphons();
		boolean added = false;
		Random random = new Random();
		int id = random.nextInt(placeLD.size());
		while (!added) {
			Set<Integer> startTranSet = new HashSet<Integer>();
			Set<Integer> endTranSet = new HashSet<Integer>();
			added = true;

			startTranSet.addAll(placeLD.get(id).getPositiveNodes(imn));
			endTranSet.addAll(placeLD.get(id).getNegativeNodes(imn));
//			System.out.println("Parallel Place: " + startTranSet + " End Set " + endTranSet);
			for (int i : startTranSet)
				if (PetrinetUtils.finalTransition(ipn.getTranBasedOnID(i)))
					added = false;

			if (startTranSet.containsAll(endTranSet) || startTranSet.equals(endTranSet))
				added = false;

			if (added) {
//				System.out.println("start.clear(); end.clear();");
//				for (Integer i : startTranSet)
//					System.out.println("start.add(" + i + ");");
//				for (Integer i : endTranSet)
//					System.out.println("end.add(" + i + ");");
//				System.out.println("addPP(start,end,ipn);");
				VizToPnetMapper.addParallelPlace(startTranSet, endTranSet, "newPlace", ipn,placeCount++);
			}
		}
	}

	public static void addChoiceTran(InteractivePetriNet ipn) {
		InteractiveMatrixNet imn = ipn.getInteractiveMatNet();
		Set<LDSets> tranLD = ipn.getAllTranLDFCs();
		boolean added = false;
		Random random = new Random();
		while (!added) {
			int id = random.nextInt(tranLD.size() + 1);
			if (id != 0)
				id=id-1;
			int c=0;
			LDSets ldt = null;
			for(LDSets ldt2 : tranLD){
				if(c==id)
					ldt=ldt2;
				c++;
			}
			
			Set<Integer> startTranSet = new HashSet<Integer>();
			Set<Integer> endTranSet = new HashSet<Integer>();
			added = true;

			startTranSet.addAll(ldt.getNegativeNodes(imn));
			endTranSet.addAll(ldt.getPositiveNodes(imn));
			for (int i : startTranSet)
				if (PetrinetUtils.initialOrFinalPlace(ipn.getPlaceBasedOnID(i), ipn.getselectedPetriNetMap()
						.getPetriNet()))
					added = false;

			for (int i : endTranSet)
				if (PetrinetUtils.initialOrFinalPlace(ipn.getPlaceBasedOnID(i), ipn.getselectedPetriNetMap()
						.getPetriNet()))
					added = false;

			if (startTranSet.containsAll(endTranSet) || startTranSet.equals(endTranSet))
				added = false;
			if (added) {
				//					System.out.println("Choice Tran: " + startTranSet + " End Set " + endTranSet);
//				System.out.println("start.clear(); end.clear();");
//				for (Integer i : startTranSet)
//					System.out.println("start.add(" + i + ");");
//				for (Integer i : endTranSet)
//					System.out.println("end.add(" + i + ");");
//				System.out.println("addChoice(start,end,ipn);");

				VizToPnetMapper.addChoiceTransition(startTranSet, endTranSet, "", ipn,tranCount++,true);

			}
		}
	}

	public static void addSequence(InteractivePetriNet ipn, int seqId) {
		Petrinet pn = ipn.getselectedPetriNetMap().getPetriNet();
		Set<Integer> sourceNodes = new HashSet<Integer>();
		Set<Integer> targetNodes = new HashSet<Integer>();
		boolean added = false;
		switch (seqId) {
			case 1 : // transition and all outgoing places
				for (Transition t : pn.getTransitions()) {
					if (!PetrinetUtils.finalTransition(t) && !added) {
						added = true;
						sourceNodes.add(ipn.getTranId(t));
						Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = pn
								.getOutEdges(t);
						for (PetrinetEdge edge : edges) {
							Place p = (Place) edge.getTarget();
							targetNodes.add(ipn.getPlaceId(p));
						}
					}
				}
//				System.out.println("start.clear(); end.clear();");
//				for (Integer i : sourceNodes)
//					System.out.println("start.add(" + i + ");");
//				for (Integer i : targetNodes)
//					System.out.println("end.add(" + i + ");");
//				System.out.println("addSeq(start,end,ipn);");
				VizToPnetMapper.addNewTransition(sourceNodes, targetNodes, "", "",ipn, false,placeCount++,tranCount++,true);
				break;
			case 2 : //place and all incoming trans
				for (Place p : pn.getPlaces()) {
					if (!PetrinetUtils.initialOrFinalPlace(p, pn) && !added) {
						added = true;
						targetNodes.add(ipn.getPlaceId(p));
						Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = pn
								.getInEdges(p);
						for (PetrinetEdge edge : edges) {
							Transition t = (Transition) edge.getSource();
							sourceNodes.add(ipn.getTranId(t));
						}
					}
				}
//				System.out.println("start.clear(); end.clear();");
//				for (Integer i : sourceNodes)
//					System.out.println("start.add(" + i + ");");
//				for (Integer i : targetNodes)
//					System.out.println("end.add(" + i + ");");
//				System.out.println("addSeq(start,end,ipn);");
				VizToPnetMapper.addNewTransition(sourceNodes, targetNodes, "", "",ipn, false,placeCount++,tranCount++,true);
			case 3 : //set of places and trans
				break;
			default :
				break;
		}
		//		System.out.println("Sequence Place: " + sourceNodes + " End Set " + targetNodes);

	}

	//	public static void addChoiceTranBasedOnRandomPlace(InteractivePetriNet ipn) {
	//		SimpleMatrix tranLDInput = ipn.getTranLDFreeChoiceMatrixInput();
	//		SimpleMatrix tranLDOutput = ipn.getTranLDFreeChoiceMatrixOutput();
	//		boolean added = false;
	//		Random random = new Random();
	//
	//		List<String> ctr = new ArrayList<String>();
	//		int placeId = random.nextInt(ipn.getselectedPetriNetMap().getPetriNet().getPlaces().size());
	//		if (placeId != 0 && placeId != 2) {
	//			Set<Integer> startCols = new HashSet<Integer>();
	//			for (int col = 0; col < tranLDOutput.numCols(); col++)
	//				if (tranLDOutput.get(placeId, col) == -1) {
	//					startCols.add(col);
	//					boolean addedStart = false;
	//					boolean addedEnd = false;
	//					String startStr = "";
	//					String endStr = "";
	//					for (int row = 0; row < tranLDOutput.numRows(); row++) {
	//						if (tranLDOutput.get(row, col) == -1) {
	//							if (addedStart)
	//								startStr += ",";
	//							startStr += row;
	//							addedStart = true;
	//						}
	//					}
	//					for (int row = 0; row < tranLDInput.numRows(); row++) {
	//						if (tranLDInput.get(row, col) == 1) {
	//							if (addedEnd)
	//								endStr += ",";
	//							endStr += row;
	//							addedEnd = true;
	//						}
	//					}
	//					ctr.add(startStr + ";" + endStr);
	//				}
	//			int rand = random.nextInt(ctr.size());
	//			String[] allStr = ctr.get(rand).split(";");
	//			Set<Integer> startTranSet = new HashSet<Integer>();
	//			Set<Integer> endTranSet = new HashSet<Integer>();
	//			for (String str : allStr[0].split(","))
	//				startTranSet.add(Integer.parseInt(str));
	//			for (String str : allStr[1].split(","))
	//				endTranSet.add(Integer.parseInt(str));
	//			System.out.println("start.clear(); end.clear();");
	//			for (Integer i : startTranSet)
	//				System.out.println("start.add(" + i + ");");
	//			for (Integer i : endTranSet)
	//				System.out.println("end.add(" + i + ");");
	//			System.out.println("addChoice(start,end,ipn);");
	//			VizToPnetMapper.addChoiceTransition(startTranSet, endTranSet, "", ipn);
	//
	//		}
	//	}

}
