package org.processmining.expts;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class FACSSynthesizeNets {
	static int count = 0;
	static long msecsiss;
	static long msecsbf;
	static String analysis = "";

	public static void synthesize(Petrinet pn) {
		Map<Byte, Place> placeIdMap = new HashMap<Byte, Place>();
		Map<Byte, Transition> tranIdMap = new HashMap<Byte, Transition>();
		ISSVectors iss = new ISSVectors();
		iss.TT.clear();
		iss.PP.clear();
		byte[] tt1 = new byte[1000];
		byte[] pp1 = new byte[1000];

		for (int l = 0; l < 9; l++)
			iss.PP1.add(new byte[2]);

		Set<byte[]> removeBytes = new HashSet<byte[]>();
		for (byte[] b : iss.PP1) {
			removeBytes.add(b);
		}

		iss.PP1.removeAll(removeBytes);

		//		tt1.add((byte) 0);
		//		pp1.add((byte) 0);

		iss.TT.add(tt1);
		iss.PP.add(pp1);

		analysis += "\r\n new net";

		for (Place p : pn.getPlaces())
			placeIdMap.put((byte) 0, p);
		for (Transition t : pn.getTransitions())
			tranIdMap.put((byte) 0, t);

		Random r = new Random();
		//		for (int l = 0; l < 3; l++)
		//			addSequence(pn, placeIdMap, tranIdMap);
		long msecs = System.currentTimeMillis();
		int randomInt = 4;
		for (int k = 0; k < 21; k++) {
			System.out.println(k);
			if (k == 1 || k == 2)
				randomInt = 4;
			String str = "";
			switch (randomInt) {
				case 1 :
					addTransition(iss, pn, placeIdMap, tranIdMap);
					str = "TT";
					break;
				case 2 :
					addPlace(iss, pn, placeIdMap, tranIdMap);
					str = "PP";
					break;
				default :
					addSequence(pn, placeIdMap, tranIdMap);
					str = "SEQ";
					break;
			}
			randomInt = r.nextInt(3);
			msecsbf = System.currentTimeMillis();
			BruteForceComputation.computeBF(pn, placeIdMap, tranIdMap);
			msecsbf = System.currentTimeMillis() - msecsbf;

			writeTime(str);
		}
		System.out.println("Total time : " + (System.currentTimeMillis() - msecs));

		//TT
		try {
			PrintWriter writer = new PrintWriter("Analysis_03081.txt", "UTF-8");
			writer.println(analysis);
			writer.close();
		} catch (IOException e) {
			// do something
		}

	}

	private static void writeTime(String str) {
		analysis += "\r\n " + str + " ISS " + msecsiss + " BF " + msecsbf;
	}

	public static void addSequence(Petrinet pn, Map<Byte, Place> placeIdMap, Map<Byte, Transition> tranIdMap) {
		//add seq - select random tran
		Set<Transition> tranSet = new HashSet<Transition>();
		Set<Place> placeSet = new HashSet<Place>();
		Random r = new Random();
		int randomInt = r.nextInt(pn.getTransitions().size());
		int i = 0;
		for (Transition t : pn.getTransitions()) {

			if (i == randomInt)
				//			if (tranIdMap.get((byte) 0) == t)
				tranSet.add(t);
			i++;
		}

		for (Transition t : tranSet) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t)) {
				placeSet.add((Place) e.getTarget());
			}
		}

		Place pNew = pn.addPlace("" + count++);
		Transition tNew = pn.addTransition("" + count++);
		for (Transition t : tranSet) {
			for (Place p : placeSet) {
				pn.removeArc(t, p);
				pn.addArc(tNew, p);
			}
			pn.addArc(t, pNew);
		}
		pn.addArc(pNew, tNew);
		placeIdMap.put((byte) placeIdMap.size(), pNew);
		tranIdMap.put((byte) tranIdMap.size(), tNew);

		msecsiss = System.currentTimeMillis();
		ISSVectors.addNewLDNodes(pn, pNew, tNew, placeIdMap, tranIdMap);
		msecsiss = System.currentTimeMillis() - msecsiss;
	}

	public static void addTransition(ISSVectors iss, Petrinet pn, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {

		//init temp backup info
//		PetriNetMap pnmap = new PetriNetMap(pn);
//		Map<Byte, Place> placeIdMapNew = new HashMap<Byte, Place>();
//		Map<Byte, Transition> tranIdMapNew = new HashMap<Byte, Transition>();
//		for (byte b : placeIdMap.keySet())
//			placeIdMapNew.put(b, pnmap.getMapPlaces().get(placeIdMap.get(b)));
//
//		for (byte b : tranIdMap.keySet())
//			tranIdMapNew.put(b, pnmap.getMapTransitions().get(tranIdMap.get(b)));
//		byte[] ttTemp = null;

		//init ends

		boolean added = false;
		Random r = new Random();
		while (!added) {
			r = new Random();
			//			System.out.println(iss.TT.size());
			int randomInt = r.nextInt(iss.TT.size());
			int i = 0;
			for (byte[] tt : iss.TT) {
				i++;
				if (i == randomInt) {
					Place p = null;
					Set<Place> negatives = new HashSet<Place>();
					Set<Place> positives = new HashSet<Place>();
					for (byte j = 0; j < pn.getPlaces().size(); j++) {
						if (tt[j] == -1) {
							negatives.add(placeIdMap.get(j));
							p = placeIdMap.get(j);
						} else if (tt[j] == 1)
							positives.add(placeIdMap.get(j));
					}

					if (positives.size() != 0) {
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(p))
							for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eIn : pn.getInEdges(e
									.getTarget()))
								if (negatives.add((Place) eIn.getSource()))
									positives.add((Place) eIn.getSource());

						if (VectorUtils.isFCTransVector(pn, negatives, positives)) {
							//						temp
//							ttTemp = tt;
							//temp

							//tt is chosen, update pp
							msecsiss = System.currentTimeMillis();
							ISSVectors.updatePP(pn, tt, placeIdMap, tranIdMap);
							msecsiss = System.currentTimeMillis() - msecsiss;
							//						for (Place p : negatives)
							addTransition(pn, negatives, positives, tranIdMap, placeIdMap);

							//remove invalid PP's
							Set<byte[]> removePP = new HashSet<byte[]>();
							for (byte[] pp : iss.PP) {
								//							byte[] vec = new byte[pp.size()];
								//							for (int l = 0; l < pp.size(); l++)
								//								vec[l] = pp.get(l);
								if (!VectorUtils.isValidISSPlaceVector(tranIdMap, pn, pp)) {
									removePP.add(pp);
								}
							}
							added = true;
							iss.PP.removeAll(removePP);

							//check linear dependence of pp
							//						byte[] vec = new byte[tt.size()];
							//						for (int l = 0; l < pn.getPlaces().size(); l++)
							//							vec[l] = tt.get(l);
							//						if (!VectorUtils.isLinearlyDependentTran(pnmap.getPetriNet(), pp, placeIdMapNew, tranIdMapNew)) {
							//							VectorUtils.isLinearlyDependentTran(pnmap.getPetriNet(), vec, placeIdMapNew, tranIdMapNew);
							//							System.err.println("Double Blasphemy!");
							//						}
						}

					}
				}
				added = true;
			}
		}
		//		for (byte[] pp : iss.PP) {
		//			byte[] vec = new byte[pp.size()];
		//			for (int l = 0; l < pp.size(); l++)
		//				vec[l] = pp.get(l);
		//			if (!VectorUtils.isLinearlyDependentPlace(pn, vec, placeIdMap, tranIdMap)) {
		//				System.err.println("Blasphemy!");
		//				byte[] vec2 = new byte[vec.length - 1];
		//				List<Byte> ppTemp = new ArrayList<Byte>();
		//				for (int l = 0; l < vec.length - 1; l++) {
		//					vec2[l] = vec[l];
		//					ppTemp.add(vec[l]);
		//				}
		//				if (!VectorUtils.isLinearlyDependentPlace(pnmap.getPetriNet(), vec2, placeIdMapNew, tranIdMapNew)) {
		//					System.err.println("x Blasphemy!");
		//				}
		//				if (!VectorUtils.isValidISSPlaceVector(tranIdMap, pn, vec))
		//					System.out.println("inValid");
		//				updatePPTemp(pnmap.getPetriNet(), ttTemp, placeIdMapNew, tranIdMapNew, ppTemp);
		//			}
		//		}
	}

	public static void addTransition(Petrinet pn, Set<Place> pIn, Set<Place> pOut, Map<Byte, Transition> tranIdMap,
			Map<Byte, Place> placeIdMap) {
		boolean added = false;

		Random r = new Random();
		int randomInt = r.nextInt(pn.getTransitions().size());
		int i = 0;
		Transition tNew = pn.addTransition("LDT" + (pIn.size() + pOut.size()));

		for (Place p : pIn)
			pn.addArc(p, tNew);
		for (Place p : pOut)
			pn.addArc(tNew, p);

		tranIdMap.put((byte) tranIdMap.size(), tNew);

	}

	public static void addPlace(ISSVectors iss, Petrinet pn, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {

		//init temp backup info
//		PetriNetMapL pnmap = new PetriNetMapL(pn);
//		Map<Byte, Place> placeIdMapNew = new HashMap<Byte, Place>();
//		Map<Byte, Transition> tranIdMapNew = new HashMap<Byte, Transition>();
//		for (byte b : placeIdMap.keySet())
//			placeIdMapNew.put(b, pnmap.getMapPlaces().get(placeIdMap.get(b)));
//
//		for (byte b : tranIdMap.keySet())
//			tranIdMapNew.put(b, pnmap.getMapTransitions().get(tranIdMap.get(b)));
//		byte[] ppTemp = null;
		//init ends

		boolean added = false;
		Random r = new Random();
		while (!added) {
			r = new Random();
			int randomInt = r.nextInt(iss.PP.size());
			int i = 0;
			for (byte[] pp : iss.PP) {
				i++;
				Transition t = null;
				if (i == randomInt) {
					Set<Transition> negatives = new HashSet<Transition>();
					Set<Transition> positives = new HashSet<Transition>();
					for (byte j = 0; j < pn.getTransitions().size(); j++) {
						if (pp[j] == -1)
							negatives.add(tranIdMap.get(j));
						else if (pp[j] == 1) {
							positives.add(tranIdMap.get(j));
							t = tranIdMap.get(j);
						}
					}
					if (negatives.size() != 0) {
						for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t))
							for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOut : pn.getOutEdges(e
									.getSource()))
								if (positives.add((Transition) eOut.getTarget()))
									negatives.add((Transition) eOut.getTarget());

						if (VectorUtils.isFCPlaceVector(pn, negatives, positives)) {
							//temp
//							ppTemp = pp;
							//						System.out.println(pp);
							//tt is chosen, update pp
							msecsiss = System.currentTimeMillis();
							ISSVectors.updateTT(pn, pp, placeIdMap, tranIdMap);
							msecsiss = System.currentTimeMillis() - msecsiss;
							addPlace(pn, negatives, positives, tranIdMap, placeIdMap);

							//remove invalid TT's
							Set<byte[]> removeTT = new HashSet<byte[]>();
							for (byte[] tt : iss.TT) {
								if (!VectorUtils.isValidISSTranVector(placeIdMap, pn, tt)) {
									removeTT.add(tt);
								}
							}

							iss.TT.removeAll(removeTT);

							added = true;
						}
						//check linear dependence of pp
						//						byte[] vec = new byte[pp.size()];
						//						for (int l = 0; l < pp.size(); l++)
						//							vec[l] = pp.get(l);
						//						if (!VectorUtils
						//								.isLinearlyDependentPlace(pnmap.getPetriNet(), vec, placeIdMapNew, tranIdMapNew)) {
						//							VectorUtils.isLinearlyDependentPlace(pnmap.getPetriNet(), vec, placeIdMapNew, tranIdMapNew);
						//							System.err.println("Double Blasphemy!");
						//						}
					}

				}
			}
		}

		//		for (byte[] tt : iss.TT) {
		//			byte[] vec = new byte[pn.getPlaces().size()];
		//			for (int l = 0; l < vec.length; l++)
		//				vec[l] = tt[l];
		//			if (!VectorUtils.isLinearlyDependentTran(pn, vec, placeIdMap, tranIdMap)) {
		//				System.err.println("Blasphemy!");
		//				if (!VectorUtils.isLinearlyDependentTran(pnmap.getPetriNet(), vec, placeIdMapNew, tranIdMapNew)) {
		//					System.err.println("Blasphemy!");
		//				}
		//				updateTTTemp(pnmap.getPetriNet(), ppTemp, placeIdMapNew, tranIdMapNew, tt);
		//				System.out.println();
		//			}
		//		}

	}

	public static void addPlace(Petrinet pn, Set<Transition> tOut, Set<Transition> tIn,
			Map<Byte, Transition> tranIdMap, Map<Byte, Place> placeIdMap) {

		Place pNew = pn.addPlace("LDT" + (tIn.size() + tOut.size()));

		for (Transition t : tIn)
			pn.addArc(t, pNew);
		for (Transition t : tOut)
			pn.addArc(pNew, t);

		placeIdMap.put((byte) placeIdMap.size(), pNew);

	}

//	public static void updatePPTemp(Petrinet pn, List<Byte> newAddedTran, Map<Byte, Place> placeIdMap,
//			Map<Byte, Transition> tranIdMap, List<Byte> pp) {
//		SimpleMatrix inci = VectorUtils.createInciMap(tranIdMap, placeIdMap, pn);
//
//		SimpleMatrix colVec = new SimpleMatrix(pn.getPlaces().size(), 1);
//
//		for (int i = 0; i < newAddedTran.size(); i++)
//			colVec.set(i, 0, newAddedTran.get(i));
//
//		//		System.out.println("12");
//		PointValuePair mu = ISSVectors.getLDs(inci, colVec);
//		SimpleMatrix muVec = new SimpleMatrix(pn.getTransitions().size(), 1);
//
//		for (int r = 0; r < muVec.numRows(); r++)
//			muVec.set(r, 0, mu.getKey()[r]);
//
//		SimpleMatrix ppVec = new SimpleMatrix(1, pn.getTransitions().size());
//		for (int c = 0; c < ppVec.numCols(); c++) {
//			ppVec.set(0, c, pp.get(c));
//
//		}
//		pp.add(pp.size(), (byte) ppVec.mult(muVec).get(0, 0));
//	}

//	public static void updateTTTemp(Petrinet pn, byte[] newAddedPlace, Map<Byte, Place> placeIdMap,
//			Map<Byte, Transition> tranIdMap, byte[] tt) {
//		SimpleMatrix inci = VectorUtils.createInciMap(tranIdMap, placeIdMap, pn);
//		MatrixUtilsL.printMatrix(inci);
//
//		SimpleMatrix rowVec = new SimpleMatrix(1, pn.getTransitions().size());
//		for (int i = 0; i < pn.getTransitions().size(); i++)
//			rowVec.set(0, i, newAddedPlace[i]);
//
//		PointValuePair lam = ISSVectors.getLDs(inci.transpose(), rowVec.transpose());
//		SimpleMatrix lamVec = new SimpleMatrix(1, pn.getPlaces().size());
//
//		for (int c = 0; c < lamVec.numCols(); c++)
//			lamVec.set(0, c, lam.getKey()[c]);
//
//		SimpleMatrix ttVec = new SimpleMatrix(pn.getPlaces().size(), 1);
//		for (int r = 0; r < ttVec.numRows(); r++) {
//			ttVec.set(r, 0, tt[r]);
//		}
//		tt[pn.getPlaces().size()] = (byte) lamVec.mult(ttVec).get(0, 0);
//
//	}
}
