package org.processmining.expts;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.math3.optim.MaxIter;
import org.apache.commons.math3.optim.PointValuePair;
import org.apache.commons.math3.optim.linear.LinearConstraint;
import org.apache.commons.math3.optim.linear.LinearConstraintSet;
import org.apache.commons.math3.optim.linear.LinearObjectiveFunction;
import org.apache.commons.math3.optim.linear.NoFeasibleSolutionException;
import org.apache.commons.math3.optim.linear.NonNegativeConstraint;
import org.apache.commons.math3.optim.linear.Relationship;
import org.apache.commons.math3.optim.linear.SimplexSolver;
import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
import org.ejml.simple.SimpleMatrix;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class ISSVectors {

	public static Set<byte[]> TT = new HashSet<byte[]>();
	public static Set<byte[]> PP = new HashSet<byte[]>();

	public static Set<byte[]> PP1 = new HashSet<byte[]>();

	//called before actually adding the place 
	public static void updateTT(Petrinet pn, byte[] newAddedPlace, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {
		SimpleMatrix inci = VectorUtils.createInciMap(tranIdMap, placeIdMap, pn);
//		MatrixUtilsL.printMatrix(inci);

		SimpleMatrix rowVec = new SimpleMatrix(1, pn.getTransitions().size());
		for (int i = 0; i < pn.getTransitions().size(); i++)
			rowVec.set(0, i, newAddedPlace[i]);

		PointValuePair lam = getLDs(inci.transpose(), rowVec.transpose());
		SimpleMatrix lamVec = new SimpleMatrix(1, pn.getPlaces().size());

		for (int c = 0; c < lamVec.numCols(); c++)
			lamVec.set(0, c, lam.getKey()[c]);

		for (byte[] tt : TT) {
			SimpleMatrix ttVec = new SimpleMatrix(pn.getPlaces().size(), 1);
			for (int r = 0; r < ttVec.numRows(); r++) {
				ttVec.set(r, 0, tt[r]);
			}
			tt[pn.getPlaces().size()] = (byte) lamVec.mult(ttVec).get(0, 0);

		}
	}

	//called before actually adding the place 
	public static void updatePP(Petrinet pn, byte[] newAddedTran, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {
		SimpleMatrix inci = VectorUtils.createInciMap(tranIdMap, placeIdMap, pn);

		SimpleMatrix colVec = new SimpleMatrix(pn.getPlaces().size(), 1);

		for (int i = 0; i < pn.getPlaces().size(); i++)
			colVec.set(i, 0, newAddedTran[i]);

//		System.out.println("12");
		PointValuePair mu = getLDs(inci, colVec);
		SimpleMatrix muVec = new SimpleMatrix(pn.getTransitions().size(), 1);

		for (int r = 0; r < muVec.numRows(); r++)
			muVec.set(r, 0, mu.getKey()[r]);

		for (byte[] pp : PP) {
			SimpleMatrix ppVec = new SimpleMatrix(1, pn.getTransitions().size());
			for (int c = 0; c < ppVec.numCols(); c++) {
				ppVec.set(0, c, pp[c]);

			}
			pp[pn.getTransitions().size()] = (byte) ppVec.mult(muVec).get(0, 0);
		}
	}

	// the new nodes are already added
	public static void addNewLDNodes(Petrinet pn, Place p, Transition t, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {
		byte[] tArr = new byte[pn.getPlaces().size()];
		byte[] pArr = new byte[pn.getTransitions().size()];
		Set<Transition> inTrans = new HashSet<Transition>(), outTrans = new HashSet<Transition>();
		Set<Place> inPlaces = new HashSet<Place>(), outPlaces = new HashSet<Place>();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eIn : pn.getInEdges(p))
			inTrans.add((Transition) eIn.getSource());

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOut : pn.getOutEdges(p))
			outTrans.add((Transition) eOut.getTarget());

		for (byte c = 0; c < pArr.length; c++) {
			if (inTrans.contains(tranIdMap.get((byte) c))) {
				if (outTrans.contains(tranIdMap.get((byte) c)))
					pArr[c] = 0;
				pArr[c] = 1;
			} else if (outTrans.contains(tranIdMap.get((byte) c)))
				pArr[c] = -1;
		}

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eIn : pn.getInEdges(t))
			inPlaces.add((Place) eIn.getSource());

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOut : pn.getOutEdges(t))
			outPlaces.add((Place) eOut.getTarget());

		for (byte i = 0; i < tArr.length; i++) {
			if (inPlaces.contains(placeIdMap.get(i))) {
				if (outPlaces.contains(placeIdMap.get(i)))
					tArr[i] = 0;
				tArr[i] = -1;
			} else if (outPlaces.contains(placeIdMap.get(i)))
				tArr[i] = 1;
		}

		Set<byte[]> ttNew = new HashSet<byte[]>();
		for (byte[] tt : TT) {
			tt[pn.getPlaces().size()] = (byte) 0;
			//			tt.set(tt.size(), (byte) 0); //default increase
			byte[] tt1 = Arrays.copyOf(tt, tt.length), tt2 = Arrays.copyOf(tt, tt.length);

			for (int i = 0; i < pn.getPlaces().size(); i++) {
				tt1[i] = (byte) (tt1[i] + tArr[i]);
				tt2[i] = (byte) (tt2[i] - tArr[i]);

			}

			if (VectorUtils.isValidISSTranVector(placeIdMap, pn, tt1))
				ttNew.add(tt1);
			if (VectorUtils.isValidISSTranVector(placeIdMap, pn, tt2))
				ttNew.add(tt2);
		}
		TT.addAll(ttNew);

		Set<byte[]> ppNew = new HashSet<byte[]>();
		for (byte[] pp : PP) {
			pp[pn.getTransitions().size()] = (byte) 0;//default increase
			//			pp.set(pp.size(), (byte) 0); //default increase
			byte[] pp1 = Arrays.copyOf(pp, pp.length), pp2 = Arrays.copyOf(pp, pp.length);
			for (int i = 0; i < pn.getTransitions().size(); i++) {
				//				System.out.println(pp1.size() + " Arr : " + pArr.length);
				pp1[i] = (byte) (pp1[i] + pArr[i]);
				pp2[i] = (byte) (pp2[i] - pArr[i]);
			}

			if (VectorUtils.isValidISSPlaceVector(tranIdMap, pn, pp1))
				ppNew.add(pp1);
			if (VectorUtils.isValidISSPlaceVector(tranIdMap, pn, pp2))
				ppNew.add(pp2);
		}
		PP.addAll(ppNew);
	}

	public static PointValuePair getLDs(SimpleMatrix inciMatrixWithoutLastCol, SimpleMatrix linDepMatrix) {
		SimpleMatrix inciMatrix = new SimpleMatrix(inciMatrixWithoutLastCol);
		//		MatrixUtilsL.printMatrix(inciMatrixWithoutLastCol);
		//		MatrixUtilsL.printMatrix(linDepMatrix);
		double[] coeff = new double[inciMatrix.numCols()];
		LinearObjectiveFunction f = new LinearObjectiveFunction(coeff, 0);
		Collection<LinearConstraint> constraints = new ArrayList<LinearConstraint>();
		boolean isLD = true;
		for (int r = 0; r < inciMatrix.numRows(); r++) {
			double[] inciMat = new double[inciMatrix.numCols()];
			for (int c = 0; c < inciMatrix.numCols(); c++) {
				inciMat[c] = inciMatrix.get(r, c);
			}
			constraints.add(new LinearConstraint(inciMat, Relationship.EQ, linDepMatrix.get(r, 0)));
		}
		SimplexSolver solver = new SimplexSolver();
		PointValuePair solution = null;
		try {
			solution = solver.optimize(new MaxIter(1000), f, new LinearConstraintSet(constraints), GoalType.MINIMIZE,
					new NonNegativeConstraint(true));
		} catch (NoFeasibleSolutionException ex) {
			isLD = false;
		}
		return solution;
	}

}
