package org.processmining.expts;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ejml.simple.SimpleMatrix;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;

public class VectorUtils {

	public static boolean isLinearlyDependentTran(Petrinet pn, byte[] vec, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {
		SimpleMatrix inci = createInciMap(tranIdMap, placeIdMap, pn);

		SimpleMatrix colVec = new SimpleMatrix(pn.getPlaces().size(), 1);

		for (int i = 0; i < pn.getPlaces().size(); i++)
			colVec.set(i, 0, vec[i]);

		//		MatrixUtilsL.printMatrix(inci);
		//		MatrixUtilsL.printMatrix(colVec);

		return MatrixUtils.isAugMatLD(inci, colVec);
	}

	public static boolean isLinearlyDependentPlace(Petrinet pn, byte[] vec, Map<Byte, Place> placeIdMap,
			Map<Byte, Transition> tranIdMap) {

		SimpleMatrix inci = createInciMap(tranIdMap, placeIdMap, pn);

		SimpleMatrix rowVec = new SimpleMatrix(1, pn.getTransitions().size());

		for (int i = 0; i < vec.length; i++)
			rowVec.set(0, i, vec[i]);

		if (!MatrixUtils.isAugMatLD(inci.transpose(), rowVec.transpose())) {
			//			MatrixUtilsL.printMatrix(inci.transpose());
			//			MatrixUtilsL.printMatrix(rowVec.transpose());
			return false;
		} else
			return true;
	}

	public static boolean isValidISSTranVector(Map<Byte, Place> placeId, Petrinet pn, byte[] vec) {
		SimpleMatrix m = new SimpleMatrix(vec.length, 1);
		for (byte b : vec) {
			m.set(0, 0, b);
		}
		Set<Place> negatives = new HashSet<Place>();
		Set<Place> positives = new HashSet<Place>();
		for (byte i = 0; i < placeId.size(); i++) {
			byte b = vec[i];
			if (b == -1) {
				negatives.add(placeId.get(i));
			} else if (b == 1) {
				positives.add(placeId.get(i));
			}

		}
		if (!isValidSpan(vec))
			return false;
		if (isFCTransVector(pn, negatives, positives) || isAbsPossibleTranVector(pn, negatives, positives))
			return true;
		return false;

	}

	public static boolean isValidISSPlaceVector(Map<Byte, Transition> tranId, Petrinet pn, byte[] vec) {

		Set<Transition> negatives = new HashSet<Transition>();
		Set<Transition> positives = new HashSet<Transition>();
		for (byte i = 0; i < tranId.size(); i++) {
			byte b = vec[i];
			if (b == -1) {
				negatives.add(tranId.get(i));
			} else if (b == 1) {
				positives.add(tranId.get(i));
			}

		}

		if (!isValidSpan(vec))
			return false;
		if (isFCPlaceVector(pn, negatives, positives) || isAbsPossiblePlaceVector(pn, negatives, positives))
			return true;
		return false;

	}

	public static boolean isFCTransVector(Petrinet pn, Set<Place> negatives, Set<Place> positives) {
		for (Transition t : pn.getTransitions()) {
			Set<Place> inPlaces = new HashSet<Place>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(t)) {
				inPlaces.add((Place) e.getSource());
			}
			if (!Collections.disjoint(negatives, inPlaces)) {
				if (!Collections.disjoint(positives, inPlaces) || inPlaces.size() < negatives.size())
					return false;
			}
		}
		return true;
	}

	public static boolean isFCPlaceVector(Petrinet pn, Set<Transition> negatives, Set<Transition> positives) {
		for (Place p : pn.getPlaces()) {
			Set<Transition> outTrans = new HashSet<Transition>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(p)) {
				outTrans.add((Transition) e.getTarget());
			}
			if (!Collections.disjoint(positives, outTrans)) {
				if (!Collections.disjoint(negatives, outTrans) || outTrans.size() < positives.size())
					return false;
			}
		}
		return true;
	}

	private static boolean isAbsPossiblePlaceVector(Petrinet pn, Set<Transition> negatives, Set<Transition> positives) {
		Set<Place> outPlaces = new HashSet<Place>();
		if (positives.size() > 1)
			for (Transition t : positives) {
				Set<Place> outPlacesTemp = new HashSet<Place>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t)) {
					outPlacesTemp.add((Place) e.getTarget());
				}
				if (outPlaces.size() == 0) {
					outPlaces.addAll(outPlacesTemp);
				} else {
					if (!outPlacesTemp.equals(outPlaces)) {
						return false;
					}
				}
			}
		return true;
	}

	private static boolean isAbsPossibleTranVector(Petrinet pn, Set<Place> negatives, Set<Place> positives) {
		Set<Transition> inTrans = new HashSet<Transition>();
		if (negatives.size() > 1)
			for (Place p : negatives) {
				Set<Transition> inTransTemp = new HashSet<Transition>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(p)) {
					inTransTemp.add((Transition) e.getSource());
				}
				if (inTrans.size() == 0) {
					inTrans.addAll(inTransTemp);
				} else {
					if (!inTransTemp.equals(inTrans)) {
						return false;
					}
				}
			}
		return true;
	}

	//create incidenceMat and the map of the trans and places in inciMat
	public static SimpleMatrix createInciMap(Map<Byte, Transition> tranIdMap, Map<Byte, Place> placeIdMap, Petrinet pn) {
		SimpleMatrix inciMat = new SimpleMatrix(pn.getPlaces().size(), pn.getTransitions().size());
		//create inciMat
		for (Place p : pn.getPlaces()) {
			int row = -1;
			for (byte b : placeIdMap.keySet())
				if (placeIdMap.get(b) == p)
					row = b;

			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = pn.getInEdges(p);
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = pn.getOutEdges(p);

			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : inEdges) {
				int col = -1;
				for (byte b : tranIdMap.keySet())
					if (tranIdMap.get(b) == e.getSource())
						col = b;
				inciMat.set(row, col, 1);
				//				newRow.set(0, tranInciMap.get(e.getSource()), 1);
			}
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : outEdges) {
				int col = -1;
				for (byte b : tranIdMap.keySet())
					if (tranIdMap.get(b) == e.getTarget())
						col = b;
				inciMat.set(row, col, -1);
				//				newRow.set(0, tranInciMap.get(e.getTarget()), newRow.get(0, tranInciMap.get(e.getTarget())) - 1);
			}
			//			inciMat = inciMat.combine(inciMat.numRows(), 0, newRow);
		}
		return inciMat;
	}

	public static boolean isValidSpan(byte[] vec) {
		for (byte b : vec)
			if (b != 0 && b != 1 && b != -1)
				return false;
		return true;

	}

}
