package org.processmining.humbertogarciacaballero.models.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.action.AggregateNodesByLayer;
import org.processmining.humbertogarciacaballero.models.gui.action.AggregateNodesBySimplifiedLayer;
import org.processmining.humbertogarciacaballero.models.gui.action.AggregateNodesSimplifier;
import org.processmining.humbertogarciacaballero.models.gui.action.DepthColorAction;
import org.processmining.humbertogarciacaballero.models.gui.action.FitGroupBoundsAction;
import org.processmining.humbertogarciacaballero.models.gui.action.NodeTypeShapeAction;
import org.processmining.humbertogarciacaballero.models.gui.action.PNColorAction;
import org.processmining.humbertogarciacaballero.models.gui.action.RotateAction;
import org.processmining.humbertogarciacaballero.models.gui.action.TextColorAction;
import org.processmining.humbertogarciacaballero.models.gui.decorator.BarDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.CentreDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LabelDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LayerSimplifierDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LongEdgeTextSourceDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LongEdgeTextTargetDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.NodeDegreeLabelDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.TextBarDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.decorator.TokenTextDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.gui.render.BarRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomEdgeRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomLabelRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.CustomShapeRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.DegreeLabelRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.LayerSimplifierControlRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.LongEdgeDecoratorRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.SimplifiedEdgeRenderer;
import org.processmining.humbertogarciacaballero.models.gui.render.VizToAlignmentMapper;
import org.processmining.humbertogarciacaballero.models.model.NodeType;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LineType;
import org.processmining.humbertogarciacaballero.models.util.Utils;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;

import prefuse.Constants;
import prefuse.Visualization;
import prefuse.action.Action;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.animate.ColorAnimator;
import prefuse.action.animate.LocationAnimator;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.activity.Activity;
import prefuse.activity.ActivityAdapter;
import prefuse.activity.SlowInSlowOutPacer;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.expression.AndPredicate;
import prefuse.data.expression.Predicate;
import prefuse.data.expression.parser.ExpressionParser;
import prefuse.data.search.RegexSearchTupleSet;
import prefuse.data.search.SearchTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.render.NullRenderer;
import prefuse.render.PolygonRenderer;
import prefuse.render.ShapeRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.PrefuseLib;
import prefuse.util.StrokeLib;
import prefuse.visual.AggregateItem;
import prefuse.visual.AggregateTable;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;
import prefuse.visual.expression.VisiblePredicate;
import prefuse.visual.tuple.TableDecoratorItem;

public class GraphVisualization {

	// Constants
	public static final String GRAPH_GROUP = "graph";
	public static final String GRAPH_NODES_GROUP = GRAPH_GROUP + ".nodes";
	public static final String GRAPH_EDGES_GROUP = GRAPH_GROUP + ".edges";

	public static final String LABEL_DECORATOR_GROUP_NAME = "nodedescription";
	public static final String INDEGREE_INFO_DECORATOR_GROUP_NAME = "nodeindegreeinfo";
	public static final String OUTDEGREE_INFO_DECORATOR_GROUP_NAME = "nodeoutdegreeinfo";

	public static final String SIMPLIFICATION_BARS_DECORATOR_NAME = "nodebars";
	public static final String SIMPLIFICATION_BARS_TEXT_DECORATOR_NAME = "nodebarstext";
	public static final String LAYER_SIMPLIFIER_DECORATOR_NAME = "layersimplifier";

	public static final String STARTING_NODES_DECORATOR_GROUP_NAME = "startingnodesdecorator";
	public static final String ENDING_NODES_DECORATOR_GROUP_NAME = "endingnodesdecorator";

	public static final String GRAPH_NODES_INCOMING_HIGHLIGHT = "incomingnodeshighlighted";
	public static final String GRAPH_NODES_OUTGOING_HIGHLIGHT = "outgoingnodeshighlighted";

	public static final String GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL = "incomingnodeshighlightedpathcontrol";
	public static final String GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL = "outgoingnodeshighlightedpathcontrol";

	public static final String GRAPH_NODES_PROBLEMATIC_HIGHLIGHT = "graphnodesproblematichihglight";
	public static final String GRAPH_NODES_PROBLEMATIC_FIT = "graphnodesproblematicfit";

	public static final String GRAPH_NODES_FORWARD_INTERACTION_FOCUS_GROUP = "GRAPH_NODES_FORWARD_INTERACTION_FOCUS_GROUP";
	public static final String GRAPH_NODES_MISSING_PLACE_FOCUS_GROUP = "GRAPH_NODES_MISSING_PLACE_FOCUS_GROUP";

	public static final String BAR_DECORATOR_AGGREGATE_GROUP = "aggregatenodes";
	public static final String BAR_TEXT_DECORATOR_AGGREGATE_GROUP = "bartextdecoratoraggregategroup";
	public static final String LAYER_SIMPLIFIER_AGGREGATE_GROUP = "layersimplifiercontrol";
	public static final String SELECTED_NODES = "selectednodes";
	public static final String RUBBER_BAND_GROUP = "rubberbandgroup";

	public static final String LONG_EDGES_TEXT_SOURCE_DECORATOR_GROUP_NAME = "longedgestextsourcedecoratorgroupname";
	public static final String LONG_EDGES_TEXT_TARGET_DECORATOR_GROUP_NAME = "longedgestexttargetdecoratorgroupname";

	public static final String TOKENS_TEXT_DECORATOR_GROUP_NAME = "tokenstextdecoratorgroupname";

	//PMD Starts
	private static final String GRAPH_NODES_INPUT_GROUP = "inputnodesgroup";
	private static final String GRAPH_NODES_OUTPUT_GROUP = "singleoutputnodesgroup";
	private static final String GRAPH_NODES_MULTIPLE_OUTPUT_GROUP = "multipleoutputnodesgroup";
	private static final String GRAPH_NODES_MULTIPLE_COLOR_GROUP = "multicolor";
	private static final String GRAPH_NODES_INPUT_OUTPUT_GROUP = "inputoutput";
	private static final String GRAPH_EDGES_SELECTED_EDGES_GROUP = "selectededges";
	private static final String GRAPH_NODES_CONFORMANCE_CHECK = "selectedactivitiesforconformance";
	public static final String GRAPH_NEXT_ACTIVITY = "nextactivity";
	public static final String GRAPH_NODES = "graph.nodes";
	//PMD ends

	public static final Predicate PREDICATE_VISIBLE_TRANSITIONS = new AndPredicate(
			PNConstants.TRANSITION_NODE_PREDICATE, VisiblePredicate.TRUE);
	public static final Predicate PREDICATE_TRANSITIONS_LABEL = new AndPredicate(PREDICATE_VISIBLE_TRANSITIONS,
			ExpressionParser.predicate("[" + PNConstants.NODE_TAU_TRANSITION_KEY + "] = FALSE"));
	public static final Predicate PREDICATE_PLACES = ExpressionParser.predicate("[" + PNConstants.NODE_TYPE_KEY
			+ "] = \"" + NodeType.PLACE + "\"");
	public static final Predicate PREDICATE_VISIBLE_PLACES = new AndPredicate(PREDICATE_PLACES, VisiblePredicate.TRUE);
	public static final Predicate HIDDEN_PREDICATE = ExpressionParser.predicate("[" + PNConstants.SIMPLIFIED_KEY
			+ "] == TRUE");
	public static final Predicate ANIMATE_LAYOUT_PREDICATE = ExpressionParser.predicate("INGROUP('" + GRAPH_NODES_GROUP
			+ "') OR INGROUP('" + LABEL_DECORATOR_GROUP_NAME + "') OR INGROUP('" + INDEGREE_INFO_DECORATOR_GROUP_NAME
			+ "') OR INGROUP('" + OUTDEGREE_INFO_DECORATOR_GROUP_NAME + "') OR INGROUP('"
			+ SIMPLIFICATION_BARS_DECORATOR_NAME + "') OR INGROUP('" + STARTING_NODES_DECORATOR_GROUP_NAME
			+ "') OR INGROUP('" + ENDING_NODES_DECORATOR_GROUP_NAME + "')");
	public static final Predicate IN_SELECTED_REMOVE_GROUP = new InGroupPredicate(SELECTED_NODES);
	public static final Predicate LABEL_GROUPS_PREDICATE = ExpressionParser.predicate("INGROUP('"
			+ LABEL_DECORATOR_GROUP_NAME + "') OR INGROUP('" + INDEGREE_INFO_DECORATOR_GROUP_NAME + "') OR INGROUP('"
			+ OUTDEGREE_INFO_DECORATOR_GROUP_NAME + "') OR INGROUP('" + TOKENS_TEXT_DECORATOR_GROUP_NAME + "')");
	public static final Predicate UNBOUNDED_PLACES_PREDICATE = ExpressionParser.predicate("[" + VisualItem.HIGHLIGHT
			+ "] = TRUE AND [" + PNConstants.NODE_UNBOUNDED_PLACE_KEY + "] = TRUE");
	public static final Predicate DEAD_TRANSITIONS_PREDICATE = ExpressionParser.predicate("[" + VisualItem.HIGHLIGHT
			+ "] = TRUE AND [" + PNConstants.NODE_DEAD_TRANSITION_KEY + "] = TRUE");
	public static final Predicate NONLIVE_TRANSITIONS_PREDICATE = ExpressionParser.predicate("[" + VisualItem.HIGHLIGHT
			+ "] = TRUE AND [" + PNConstants.NODE_NON_LIVE_TRANSITION_KEY + "] = TRUE");
	public static final String SELECTED_REMOVE_GROUP = "selectedremovegroup";;

	// The controller
	private PNetVisualizerController controller;
	// The data
	private Graph graph;
	// The Visualization
	private Visualization vis;
	// The rotation action
	private RotateAction rotationAction;
	private Schema nodeSchema;
	// Renderer
	private CustomShapeRenderer customShapeRenderer;
	private SimplifiedEdgeRenderer simplifiedRenderer;
	private ShapeRenderer shapeRenderer;
	
	private CustomEdgeRenderer customEdgeRenderer;

	private PNColorAction edgesColorAction;

	private DegreeLabelRenderer indegreeRenderer;
	private DegreeLabelRenderer outdegreeRenderer;
	private Action indegreeDecorator;
	private Action outdegreeDecorator;

	// Searcher
	private SearchTupleSet m_searcher;
	private String[] m_fields;
	private boolean m_autoIndex = true;

	// Aggregate structures
	private AggregateTable aggregateTable;
	private Action aggregateAction;
	private AggregateTable aggregateTable2;
	private Action aggregateAction2;
	private AggregateTable aggregateTableLayerNodes;
	private Action aggregateActionLayerNodes;

	// Sizes
	private int arrowWidth = 6;
	private int arrowHeight = 5;
	private int transitionsWidth = PNConstants.TRANSITIONS_DEFAULT_WIDTH;
	private int placesSize = PNConstants.PLACES_DEFAULT_SIZE;
	private int transitionsHeight = PNConstants.TRANSITIONS_DEFAULT_HEIGHT;
	private int fontSize = PNConstants.FONT_DEFAULT_SIZE ;
	private int degreeFontSize = PNConstants.FONT_DEGREE_DEFAULT_SIZE;

	// Animations
	private LocationAnimator dragAnimator;
	private long animationLength = PNConstants.ANIMATION_LENGTH_DEFAULT;
	private ActionList animateLayout;
	private ActionList positionAnimate;

	private TransPairAlignment tpa; //PMD alignments- if tpa is present then its mini graph

	public GraphVisualization(Graph graph) {
		controller = PNetVisualizerController.getInstance();
		this.tpa = null; //set tpa as null for main view
		init(graph);
	}

	/**
	 * This method is used only for generating the small graph view for reduced
	 * alignments
	 * 
	 * @param graph
	 * @param controller2
	 * @param net
	 */
	public GraphVisualization(Graph graph, PNetVisualizerController controller2, TransPairAlignment tpa) {
		controller = controller2;
		this.tpa = tpa; //get net as a param
		controller.setActivitiesFromAlignmentView(tpa.getActivities());
		init(graph);
	}

	public TransPairAlignment getTpa() {
		return tpa;
	}

	private void init(Graph graph) {
		setUpData(graph);
		setUpVisualization();
		setUpRenderers();
		setUpSearcher();
		setUpHighlightGroups();
		setUpActions();

		// We have to start the ActionLists that we
		// added to the visualization
		vis.run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
		vis.run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
		vis.run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
	}

	// -- 1. The data --------------------------------------------------
	private void setUpData(Graph graph) {
		this.graph = graph;
	}

	// -- 2. the visualization -----------------------------------------
	private void setUpVisualization() {
		// Create the Visualization object.
		vis = new Visualization();

		nodeSchema = PrefuseLib.getVisualItemSchema();
		Schema edgeSchema = PrefuseLib.getVisualItemSchema();

		nodeSchema.setDefault(VisualItem.STROKE, StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
		nodeSchema.setDefault(VisualItem.STROKECOLOR, PNConstants.STROKE_COLOR);
		edgeSchema.setDefault(VisualItem.STROKE, StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));

		vis.addGraph(GRAPH_GROUP, graph, null, nodeSchema, edgeSchema);
		vis.setValue(GRAPH_NODES_GROUP, null, VisualItem.FILLCOLOR, -1);
		vis.setValue(GRAPH_EDGES_GROUP, null, VisualItem.FILLCOLOR, -1);
		vis.setValue(GRAPH_EDGES_GROUP, null, VisualItem.STROKECOLOR, -1);
		vis.setValue(GRAPH_NODES_GROUP, PNConstants.TRANSITION_NODE_PREDICATE, PNConstants.NODE_WIDTH_KEY,
				PNConstants.TRANSITIONS_DEFAULT_WIDTH);

		aggregateTableLayerNodes = vis.addAggregates(BAR_TEXT_DECORATOR_AGGREGATE_GROUP);
		aggregateTableLayerNodes.addColumn(PNConstants.NODE_LAYER_KEY, int.class, 0);
		aggregateTableLayerNodes.addColumn(PNConstants.NODE_MAX_LAYER_KEY, int.class, 0);
		aggregateTableLayerNodes.addColumn(PNConstants.EDGE_LONG_EDGE_KEY, boolean.class, false);

		aggregateTable = vis.addAggregates(BAR_DECORATOR_AGGREGATE_GROUP);
		aggregateTable.addColumn(PNConstants.NODE_LAYER_KEY, int.class, 0);
		aggregateTable.addColumn(PNConstants.NODE_MAX_LAYER_KEY, int.class, 0);
		aggregateTable.addColumn(PNConstants.EDGE_LONG_EDGE_KEY, boolean.class, false);

		aggregateTable2 = vis.addAggregates(LAYER_SIMPLIFIER_AGGREGATE_GROUP);
		aggregateTable2.addColumn(PNConstants.NODE_LAYER_KEY, int.class, 0);
		aggregateTable2.addColumn(PNConstants.NODE_MAX_LAYER_KEY, int.class, 0);
		aggregateTable2.addColumn(PNConstants.SIMPLIFIED_KEY, boolean.class, false);

		// Disable interaction with edges
		//		vis.setInteractive(GRAPH_EDGES_GROUP, null, false);
	}

	// -- 3. the renderers and renderer factory -------------------------
	private void setUpRenderers() {
		customShapeRenderer = new CustomShapeRenderer();
		customEdgeRenderer = new CustomEdgeRenderer();
//		PolylineEdgeRenderer customEdgeRenderer = new PolylineEdgeRenderer();		//PMD 2607
		customEdgeRenderer.setArrowHeadSize(arrowWidth, arrowHeight);

		simplifiedRenderer = new SimplifiedEdgeRenderer();
		simplifiedRenderer.setArrowHeadSize(arrowWidth, arrowHeight);

		DefaultRendererFactory factory = new DefaultRendererFactory();

		factory.setDefaultRenderer(customShapeRenderer);
		factory.setDefaultEdgeRenderer(customEdgeRenderer);
		factory.add(
				new AndPredicate(new InGroupPredicate(GRAPH_EDGES_GROUP), ExpressionParser.predicate("["
						+ PNConstants.EDGE_BIDIRECTIONAL_DRAW + "] = FALSE")), new NullRenderer());
		factory.add(new AndPredicate(new InGroupPredicate(GRAPH_EDGES_GROUP), HIDDEN_PREDICATE), simplifiedRenderer);

		factory.add(new InGroupPredicate(LABEL_DECORATOR_GROUP_NAME), new CustomLabelRenderer(
				PNConstants.NODE_LABEL_KEY));
		factory.add(new InGroupPredicate(INDEGREE_INFO_DECORATOR_GROUP_NAME),
				indegreeRenderer = new DegreeLabelRenderer() {
					public String getText(VisualItem item) {
						NodeItem n = (NodeItem) ((TableDecoratorItem) item).getDecoratedItem();
						int visibleInDegree = n.getInt(PNConstants.VISIBLE_IN_DEGREE);
						int inDegree = n.getInDegree();

						return String.valueOf(visibleInDegree == -1 ? inDegree : visibleInDegree);
					}
				});
		factory.add(new InGroupPredicate(OUTDEGREE_INFO_DECORATOR_GROUP_NAME),
				outdegreeRenderer = new DegreeLabelRenderer() {
					public String getText(VisualItem item) {
						NodeItem n = (NodeItem) ((TableDecoratorItem) item).getDecoratedItem();
						int visibleOutDegree = n.getInt(PNConstants.VISIBLE_OUT_DEGREE);
						int outDegree = n.getOutDegree();

						return String.valueOf(visibleOutDegree == -1 ? outDegree : visibleOutDegree);
					}
				});
		factory.add(new InGroupPredicate(SIMPLIFICATION_BARS_DECORATOR_NAME), new BarRenderer());
		LabelRenderer barTextRenderer = new LabelRenderer() {
			@Override
			public String getText(VisualItem item) {
				DecoratorItem decorator = (DecoratorItem) item;
				AggregateItem aggregate = (AggregateItem) decorator.getDecoratedItem();
				int layer = aggregate.getInt(PNConstants.NODE_LAYER_KEY);
				int maxLayer = aggregate.getInt(PNConstants.NODE_MAX_LAYER_KEY);

				if (layer < maxLayer && aggregate.getBoolean(PNConstants.EDGE_LONG_EDGE_KEY)) {
					return Utils.toName(layer + 1);
				}

				return null;
			}
		};
		barTextRenderer.setHorizontalTextAlignment(Constants.CENTER);
		factory.add(new InGroupPredicate(SIMPLIFICATION_BARS_TEXT_DECORATOR_NAME), barTextRenderer);
		factory.add(new InGroupPredicate(BAR_DECORATOR_AGGREGATE_GROUP), new NullRenderer());
		factory.add(new InGroupPredicate(BAR_TEXT_DECORATOR_AGGREGATE_GROUP), new NullRenderer());

		factory.add(new InGroupPredicate(LAYER_SIMPLIFIER_DECORATOR_NAME), new LayerSimplifierControlRenderer());
		factory.add(new InGroupPredicate(LAYER_SIMPLIFIER_AGGREGATE_GROUP), new NullRenderer());

		factory.add(new InGroupPredicate(RUBBER_BAND_GROUP), new PolygonRenderer(Constants.POLY_TYPE_LINE));
		factory.add(new InGroupPredicate(STARTING_NODES_DECORATOR_GROUP_NAME), shapeRenderer = new ShapeRenderer());
		factory.add(new InGroupPredicate(ENDING_NODES_DECORATOR_GROUP_NAME), shapeRenderer);
		factory.add(new InGroupPredicate(Visualization.SELECTED_ITEMS), shapeRenderer);

		LongEdgeDecoratorRenderer longEdgesTextSourceRenderer = new LongEdgeDecoratorRenderer() {
			@Override
			public String getText(VisualItem item) {
				// Item is an EdgeITem
				EdgeItem edge = (EdgeItem) ((DecoratorItem) item).getDecoratedItem();
				int fromLayer = edge.getSourceItem().getInt(PNConstants.NODE_LAYER_KEY);
				int toLayer = edge.getTargetItem().getInt(PNConstants.NODE_LAYER_KEY);

				if (fromLayer < toLayer) {
					return Utils.toName(toLayer);
				} else {
					return Utils.toName(toLayer + 1);
				}
			}
		};
		longEdgesTextSourceRenderer.setHorizontalTextAlignment(Constants.CENTER);
		longEdgesTextSourceRenderer.setDimensions(arrowWidth, 1);
		factory.add(new InGroupPredicate(LONG_EDGES_TEXT_SOURCE_DECORATOR_GROUP_NAME), longEdgesTextSourceRenderer);

		LongEdgeDecoratorRenderer longEdgesTextTargetRenderer = new LongEdgeDecoratorRenderer() {
			@Override
			public String getText(VisualItem item) {
				// Item is an EdgeITem
				EdgeItem edge = (EdgeItem) ((DecoratorItem) item).getDecoratedItem();
				int fromLayer = edge.getSourceItem().getInt(PNConstants.NODE_LAYER_KEY);
				int toLayer = edge.getTargetItem().getInt(PNConstants.NODE_LAYER_KEY);

				if (fromLayer < toLayer) {
					return Utils.toName(fromLayer + 1);
				} else {
					return Utils.toName(fromLayer);
				}
			}
		};
		longEdgesTextTargetRenderer.setHorizontalTextAlignment(Constants.CENTER);
		longEdgesTextTargetRenderer.setDimensions(arrowWidth, 1);
		factory.add(new InGroupPredicate(LONG_EDGES_TEXT_TARGET_DECORATOR_GROUP_NAME), longEdgesTextTargetRenderer);

		factory.add(new InGroupPredicate(TOKENS_TEXT_DECORATOR_GROUP_NAME), new LabelRenderer() {
			@Override
			public String getText(VisualItem item) {
				return String.valueOf(((DecoratorItem) item).getDecoratedItem()
						.getInt(PNConstants.NODE_TOKEN_COUNT_KEY));
			}
		});

		// PMD

		factory.add(new InGroupPredicate(GRAPH_NODES), new VizToAlignmentMapper(controller, this));

		factory.add(new InGroupPredicate(GRAPH_NODES), new CustomShapeRenderer() {
			public void render(Graphics2D g, VisualItem vitem) {

				TupleSet ts = vitem.getVisualization().getGroup(GRAPH_NODES);
				String selectedActivity = controller.getActivity();
				//String selectedActivity = "x2";
				TupleSet ts2 = vitem.getVisualization().getGroup(GRAPH_NEXT_ACTIVITY);
				//						Iterator it = ts.tuples(PREDICATE_TRANSITIONS);
				if (false) {

					boolean highlighted = vitem.isHighlighted();
					if (vitem.get(PNConstants.NODE_TYPE_KEY).equals("TRANSITION") && !highlighted) {
						//							setTransitionsInOutDegreeEnabled(true);
						//while (it.hasNext()) {
						//NodeItem item = (NodeItem) it.next();
						NodeItem item = (NodeItem) vitem;
						String prcedesVal = "0";
						String followsVal = "0";
						String coOccursVal = "0";
						// get precedes and follows relations

						String getSet = (String) item.get("Follows");
						if (getSet != null && selectedActivity != vitem.get("label") && selectedActivity != "") {
							boolean added = false;
							List<String> allSet = Arrays.asList(getSet.split(";"));
							allSet = Arrays.asList(getSet.split(";"));
							for (String t : allSet) {
								if (!added) {
									List<String> tranSet = Arrays.asList(t.split(","));
									if (tranSet.get(0).equals(selectedActivity))
										added = true;
									followsVal = tranSet.get(1);
								}
							}
							getSet = (String) item.get("CoOccurs");
							added = false;
							allSet = Arrays.asList(getSet.split(";"));
							for (String t : allSet) {
								if (!added) {
									List<String> tranSet = Arrays.asList(t.split(","));
									if (tranSet.get(0).equals(selectedActivity))
										added = true;
									coOccursVal = tranSet.get(1);
								}
							}

							// ends

							// Double coOccurs =
							// Double.valueOf(coOccursVal);
							Double follows = Double.valueOf(followsVal);
							// render the edge line
							Shape shape = getRawShape(item);
							if (shape != null) {
								Shape tmpnod = shape.getBounds();
								Double w = ((RectangularShape) tmpnod).getWidth();
								Double w1 = w * follows;
								Double w2 = w - w1;

								Shape r = new Rectangle2D.Double(((RectangularShape) tmpnod).getX(),
										((RectangularShape) tmpnod).getY(), w1, ((RectangularShape) tmpnod).getHeight());
								Shape r2 = new Rectangle2D.Double(((RectangularShape) tmpnod).getX() + w1,
										((RectangularShape) tmpnod).getY(), w2, ((RectangularShape) tmpnod).getHeight());

								// ColorLib.gray(100);
								// int i =(int) Math.round(100*coOccurs);
								Float f = Float.valueOf(coOccursVal);
								//									g.draw(shape);
								drawShape(g, vitem, shape);
								Color c1 = new Color(1f, 0f, 1f, f);
								Color c2 = new Color(1f, 1f, 0f, f);

								g.setColor(c2);
								g.fill(r2);
								g.draw(r2);
								g.setColor(c1);
								g.draw(r);
								g.fill(r);
								g.setColor(ColorLib.getColor(PNConstants.EDGE_DEFAULT_COLOR));
								//									g.draw(shape);

							}
						} else {

							Shape shape = getRawShape(item);
							Shape tmpnod = shape.getBounds();
							Shape r = new Rectangle2D.Double(((RectangularShape) tmpnod).getX(),
									((RectangularShape) tmpnod).getY(), ((RectangularShape) tmpnod).getWidth(),
									((RectangularShape) tmpnod).getHeight());

							// int i =(int) Math.round(100*coOccurs);

							// Color c = (Color) ColorLib.gray(100);

							g.setColor(ColorLib.getColor(PNConstants.EDGE_DEFAULT_COLOR));
							String label = (String) item.get("label");
							if (label == "") {
								g.setColor(Color.BLACK);
								g.fill(r);
							}
							//								g.draw(r);
							drawShape(g, vitem, shape);
						}
						// render the edge arrow head, if appropriate
						g.setPaint(ColorLib.getColor(item.getFillColor()));
						//}
					} else {
						NodeItem item = (NodeItem) vitem;
						Shape shape = getRawShape(item);
						g.setColor(ColorLib.getColor(PNConstants.EDGE_DEFAULT_COLOR));
						g.draw(shape);
						if (vitem.get(PNConstants.NODE_TYPE_KEY).equals("PLACE")
								&& !(shape instanceof Ellipse2D.Double)) {
							item.setShape(Constants.SHAPE_ELLIPSE);
							shape = getShape(item);
							Ellipse2D ellipse = (Ellipse2D) shape;
							Ellipse2D el = new Ellipse2D.Double(ellipse.getHeight(), ellipse.getWidth(),
									ellipse.getX(), ellipse.getY());
							g.setColor(ColorLib.getColor(PNConstants.EDGE_DEFAULT_COLOR));
							g.draw((Shape) el);
							g.setPaint(ColorLib.getColor(vitem.getFillColor()));
						} else {
							if (shape != null)
								drawShape(g, vitem, shape);
						}
					}
				}

			}
		});
		// PMD

		vis.setRendererFactory(factory);

		// -- Decorators
		final Schema LABEL_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", fontSize));

		final Schema DEGREE_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		DEGREE_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		DEGREE_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", degreeFontSize));

		final Schema BAR_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		BAR_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		BAR_DECORATOR_SCHEMA.setDefault(VisualItem.FILLCOLOR, PNConstants.EDGE_DEFAULT_COLOR);
		BAR_DECORATOR_SCHEMA.setDefault(VisualItem.STROKECOLOR, ColorLib.setAlpha(PNConstants.EDGE_DEFAULT_COLOR, 90));
		BAR_DECORATOR_SCHEMA.setDefault(VisualItem.STROKE, StrokeLib.getStroke(0.2f));

		final Schema BAR_TEXT_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		BAR_TEXT_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		BAR_TEXT_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", Font.BOLD, fontSize + 2));
		BAR_TEXT_DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, PNConstants.FONT_COLOR);

		final Schema LAYER_SIMPLIFIER_SCHEMA = PrefuseLib.getVisualItemSchema();
		LAYER_SIMPLIFIER_SCHEMA.setDefault(VisualItem.INTERACTIVE, true);
		LAYER_SIMPLIFIER_SCHEMA.setDefault(VisualItem.STROKECOLOR, ColorLib.rgba(255, 255, 255, 0));
		LAYER_SIMPLIFIER_SCHEMA.setDefault(VisualItem.STROKE, StrokeLib.getStroke(0.3f));
		LAYER_SIMPLIFIER_SCHEMA.addColumn(PNConstants.SIMPLIFIER_DECORATOR_WIDTH, double.class, 0);
		LAYER_SIMPLIFIER_SCHEMA.addColumn(PNConstants.SIMPLIFIER_DECORATOR_HEIGHT, double.class, 0);

		final Schema START_NODES_SCHEMA = PrefuseLib.getVisualItemSchema();
		START_NODES_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		START_NODES_SCHEMA.setDefault(VisualItem.FILLCOLOR, ColorLib.color(Color.BLACK));
		START_NODES_SCHEMA.setDefault(VisualItem.SIZE, PNConstants.PLACES_DEFAULT_SIZE
				* PNConstants.STARTING_NODE_SHAPE_PERCENTAGE_SIZE / shapeRenderer.getBaseSize());
		START_NODES_SCHEMA.setDefault(VisualItem.SHAPE, Constants.SHAPE_ELLIPSE);

		final Schema END_NODES_SCHEMA = PrefuseLib.getVisualItemSchema();
		END_NODES_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		END_NODES_SCHEMA.setDefault(VisualItem.STROKECOLOR, ColorLib.color(Color.BLACK));
		END_NODES_SCHEMA.setDefault(VisualItem.STROKE, StrokeLib.getStroke(0.3f));
		END_NODES_SCHEMA.setDefault(VisualItem.SIZE, PNConstants.PLACES_DEFAULT_SIZE
				* PNConstants.ENDING_NODE_SHAPE_PERCENTAGE_SIZE / shapeRenderer.getBaseSize());
		END_NODES_SCHEMA.setDefault(VisualItem.SHAPE, Constants.SHAPE_ELLIPSE);

		final Schema LONG_EDGES_TEXT_SCHEMA = PrefuseLib.getVisualItemSchema();
		LONG_EDGES_TEXT_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		LONG_EDGES_TEXT_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", Font.BOLD, fontSize + 1));
		LONG_EDGES_TEXT_SCHEMA.setDefault(VisualItem.TEXTCOLOR, PNConstants.FONT_COLOR);

		final Schema TOKENS_TEXT_SCHEMA = PrefuseLib.getVisualItemSchema();
		TOKENS_TEXT_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		TOKENS_TEXT_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", Font.BOLD, fontSize + 1));

		vis.addDecorators(LABEL_DECORATOR_GROUP_NAME, GRAPH_NODES_GROUP, PREDICATE_TRANSITIONS_LABEL,
				LABEL_DECORATOR_SCHEMA);
		vis.addDecorators(INDEGREE_INFO_DECORATOR_GROUP_NAME, GRAPH_NODES_GROUP, PREDICATE_TRANSITIONS_LABEL,
				DEGREE_DECORATOR_SCHEMA);
		vis.addDecorators(OUTDEGREE_INFO_DECORATOR_GROUP_NAME, GRAPH_NODES_GROUP, PREDICATE_TRANSITIONS_LABEL,
				DEGREE_DECORATOR_SCHEMA);
		// Bars
		vis.addDecorators(SIMPLIFICATION_BARS_DECORATOR_NAME, BAR_DECORATOR_AGGREGATE_GROUP, BAR_DECORATOR_SCHEMA);
		// Text of bars
		vis.addDecorators(SIMPLIFICATION_BARS_TEXT_DECORATOR_NAME, BAR_TEXT_DECORATOR_AGGREGATE_GROUP,
				BAR_TEXT_DECORATOR_SCHEMA);

		vis.addDecorators(LAYER_SIMPLIFIER_DECORATOR_NAME, LAYER_SIMPLIFIER_AGGREGATE_GROUP, LAYER_SIMPLIFIER_SCHEMA);
		vis.addDecorators(STARTING_NODES_DECORATOR_GROUP_NAME, GRAPH_NODES_GROUP, PNConstants.STARTING_NODE_PREDICATE,
				START_NODES_SCHEMA);
		vis.addDecorators(ENDING_NODES_DECORATOR_GROUP_NAME, GRAPH_NODES_GROUP, PNConstants.ENDING_NODE_PREDICATE,
				END_NODES_SCHEMA);

		vis.addDecorators(
				LONG_EDGES_TEXT_SOURCE_DECORATOR_GROUP_NAME,
				GRAPH_EDGES_GROUP,
				ExpressionParser.predicate("[" + PNConstants.EDGE_LONG_EDGE_KEY + "] = TRUE AND ["
						+ PNConstants.SIMPLIFIED_KEY + "] = TRUE"), LONG_EDGES_TEXT_SCHEMA);
		vis.addDecorators(
				LONG_EDGES_TEXT_TARGET_DECORATOR_GROUP_NAME,
				GRAPH_EDGES_GROUP,
				ExpressionParser.predicate("[" + PNConstants.EDGE_LONG_EDGE_KEY + "] = TRUE AND ["
						+ PNConstants.SIMPLIFIED_KEY + "] = TRUE"), LONG_EDGES_TEXT_SCHEMA);

		vis.addDecorators(TOKENS_TEXT_DECORATOR_GROUP_NAME, GRAPH_NODES_GROUP,
				ExpressionParser.predicate("[" + PNConstants.NODE_TOKEN_COUNT_KEY + "] > 1"), LABEL_DECORATOR_SCHEMA);
	}

	private void setUpActions() {
		edgesColorAction = new PNColorAction(GRAPH_EDGES_GROUP, new String[] { VisualItem.STROKECOLOR,
				VisualItem.FILLCOLOR }, PNConstants.EDGE_DEFAULT_COLOR) {

			@Override
			public int getColor(VisualItem item) {
				int color = super.getColor(item);
				EdgeItem e = (EdgeItem) item;
				int depth = Integer.MAX_VALUE;

//				if (e.getSourceItem().getInt(PNConstants.NODE_DEPTH_KEY) > 0) {
//					depth = e.getSourceItem().getInt(PNConstants.NODE_DEPTH_KEY);
//				}
//				if (e.getTargetItem().getInt(PNConstants.NODE_DEPTH_KEY) > 0) {
//					depth = Math.min(depth, e.getTargetItem().getInt(PNConstants.NODE_DEPTH_KEY));
//				}
//
//				if (depth != Integer.MAX_VALUE) {
//					color = Utils.changeAlphaDepth(color, depth);
//				}

				return color;
			}
		};
		edgesColorAction.add("[" + VisualItem.HIGHLIGHT + "] = TRUE", PNConstants.OUTGOING_HIGHLIGHTED_EDGE_COLOR);
		edgesColorAction.add("INGROUP('" + GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL + "')",
				PNConstants.INCOMING_HIGHLIGHTED_EDGE_COLOR);
		edgesColorAction.add("INGROUP('" + GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL + "')",
				PNConstants.OUTGOING_HIGHLIGHTED_EDGE_COLOR);
		edgesColorAction.add(HIDDEN_PREDICATE, PNConstants.EDGE_COLOR_KEY);

		final StrokeAction edgeStrokeAction = new StrokeAction(GRAPH_EDGES_GROUP,
				StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
		edgeStrokeAction.add("INGROUP('" + GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL + "') OR INGROUP('"
				+ GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL + "')", StrokeLib.getDerivedStroke(
				edgeStrokeAction.getDefaultStroke(), PNConstants.STROKE_DEFAULT_WIDTH + 1.6f));
		edgeStrokeAction.add("[" + VisualItem.HIGHLIGHT + "] = TRUE",
				StrokeLib.getDerivedStroke(edgeStrokeAction.getDefaultStroke(), PNConstants.STROKE_DEFAULT_WIDTH + 2f));
		edgeStrokeAction.add("[" + PNConstants.SIMPLIFIED_KEY + "] = TRUE", StrokeLib.getDerivedStroke(
				edgeStrokeAction.getDefaultStroke(), PNConstants.STROKE_DEFAULT_WIDTH + 1.3f));

		NodeTypeShapeAction shape = new NodeTypeShapeAction(GRAPH_NODES_GROUP);

		// Layout
		ActionList noPaintingLayout = new ActionList(); // Copy of
														// layout
														// action
														// without RepaintAction
		final ActionList layout = new ActionList();
		noPaintingLayout.add(controller.getLayouter());
		layout.add(controller.getLayouter());
		noPaintingLayout.add(edgesColorAction);
		layout.add(edgesColorAction);
		noPaintingLayout.add(edgeStrokeAction);
		layout.add(edgeStrokeAction);
		layout.add(new RepaintAction());		//PMD 2607
		
		// Decorator layouts
		ActionList decorators = new ActionList();
		decorators.add(new LabelDecoratorLayout(LABEL_DECORATOR_GROUP_NAME));
		decorators
				.add(indegreeDecorator = new NodeDegreeLabelDecoratorLayout(INDEGREE_INFO_DECORATOR_GROUP_NAME, true));
		decorators.add(outdegreeDecorator = new NodeDegreeLabelDecoratorLayout(OUTDEGREE_INFO_DECORATOR_GROUP_NAME,
				false));
		decorators.add(aggregateAction = new AggregateNodesBySimplifiedLayer(GRAPH_NODES_GROUP, aggregateTable));
		decorators.add(new BarDecoratorLayout(SIMPLIFICATION_BARS_DECORATOR_NAME));
		decorators.add(aggregateActionLayerNodes = new AggregateNodesByLayer(GRAPH_NODES_GROUP,
				aggregateTableLayerNodes));
		decorators.add(new TextBarDecoratorLayout(SIMPLIFICATION_BARS_TEXT_DECORATOR_NAME));
		setBarDecoratorEnabled(false);

		decorators.add(aggregateAction2 = new AggregateNodesSimplifier(GRAPH_NODES_GROUP, aggregateTable2));
		setSimplifierControlEnabled(false);
		Action simplifierDecoratorLayout = new LayerSimplifierDecoratorLayout(LAYER_SIMPLIFIER_DECORATOR_NAME);
		decorators.add(simplifierDecoratorLayout);
		decorators.add(new CentreDecoratorLayout(STARTING_NODES_DECORATOR_GROUP_NAME));
		decorators.add(new TokenTextDecoratorLayout(TOKENS_TEXT_DECORATOR_GROUP_NAME));
		decorators.add(new CentreDecoratorLayout(ENDING_NODES_DECORATOR_GROUP_NAME));
		decorators.add(new LongEdgeTextSourceDecoratorLayout(LONG_EDGES_TEXT_SOURCE_DECORATOR_GROUP_NAME));
		decorators.add(new LongEdgeTextTargetDecoratorLayout(LONG_EDGES_TEXT_TARGET_DECORATOR_GROUP_NAME));

		noPaintingLayout.add(decorators);
		layout.add(decorators);

		Action fitContainerAction = new FitGroupBoundsAction(GRAPH_GROUP, 3, 0, true);
		layout.add(fitContainerAction);

		final Action repaintAction = new RepaintAction();
		layout.add(repaintAction);

		animateLayout = new ActionList(animationLength);
		animateLayout.setPacingFunction(new SlowInSlowOutPacer());
		LocationAnimator animator = new LocationAnimator();
		animator.setFilterPredicate(ANIMATE_LAYOUT_PREDICATE);
		animateLayout.add(animator);
		ColorAnimator edgesFillAnimator = new ColorAnimator(GRAPH_EDGES_GROUP, new String[] { VisualItem.STROKECOLOR,
				VisualItem.FILLCOLOR });
		animateLayout.add(edgesFillAnimator);
		//        animateLayout.add(fitContainerAction);
		animateLayout.add(repaintAction);
		animateLayout.addActivityListener(new ActivityAdapter() {
			public void activityCancelled(Activity a) {
				// if animation is canceled, set colors to final state
				layout.run(1.0);
			}
		});

		// Position animate
		positionAnimate = new ActionList(animationLength);
		positionAnimate.setPacingFunction(new SlowInSlowOutPacer());
		dragAnimator = new LocationAnimator();
		positionAnimate.add(dragAnimator);
		positionAnimate.add(decorators);
		// positionAnimate.add(fitContainerAction);
		positionAnimate.add(repaintAction);

		// Simplifier decorator
		ActionList simplifierDecorator = new ActionList();
		simplifierDecorator.add(aggregateAction2);
		simplifierDecorator.add(simplifierDecoratorLayout);
		simplifierDecorator.add(repaintAction);

		// Fit in bounds actions
		ActionList fitBounds = new ActionList();
		fitBounds.add(fitContainerAction);
		fitBounds.add(repaintAction);

		// Layout decorators
		ActionList layoutDecorators = new ActionList();
		layoutDecorators.add(decorators);
		layoutDecorators.add(repaintAction);

		// Rotation
		ActionList rotation = new ActionList();
		rotation.add(rotationAction = new RotateAction(GRAPH_GROUP));
		rotation.add(decorators);
		rotation.add(fitContainerAction);
		rotation.add(repaintAction);

		// Highlighting
		final ColorAction fill = new ColorAction(GRAPH_NODES_GROUP, VisualItem.FILLCOLOR, ColorLib.rgb(255, 255, 255));
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_INPUT_OUTPUT_GROUP + "')",
				PNConstants.INPUT_OUTPUT_HIGHLIGHTED_NODE_COLOR); // PMD
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_CONFORMANCE_CHECK + "')",
				PNConstants.CONFORMANCE_ACTIVITY_NODE_COLOR); // PMD
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_EDGES_SELECTED_EDGES_GROUP + "')",
				PNConstants.OUTGOING_HIGHLIGHTED_NODE_COLOR); // PMD
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_INPUT_GROUP + "')",
				PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR); // PMD
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_OUTPUT_GROUP + "')",
				PNConstants.OUTGOING_HIGHLIGHTED_NODE_COLOR);// PMD
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_MULTIPLE_OUTPUT_GROUP + "')",
				PNConstants.HIGHLIGHTED_NODE_COLOR);// PMD
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_INCOMING_HIGHLIGHT + "')",
				PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR); // PMD commented
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('" + GRAPH_NODES_OUTGOING_HIGHLIGHT + "')",
				PNConstants.OUTGOING_HIGHLIGHTED_NODE_COLOR); // PMD commented
		fill.add("[" + PNConstants.NODE_SELECTED_TRANSITION + "]", ColorLib.hex("#fec44f"));
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND (INGROUP('" + GRAPH_NODES_INCOMING_HIGHLIGHT + "') OR INGROUP('"
				+ GRAPH_NODES_MISSING_PLACE_FOCUS_GROUP + "'))", PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR);
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND (INGROUP('" + GRAPH_NODES_OUTGOING_HIGHLIGHT + "') OR INGROUP('"
				+ GRAPH_NODES_FORWARD_INTERACTION_FOCUS_GROUP + "'))", PNConstants.OUTGOING_HIGHLIGHTED_NODE_COLOR);
		fill.add("[" + VisualItem.HIGHLIGHT + "] AND [" + PNConstants.NODE_DEPTH_KEY + "] <= 0",
				PNConstants.HIGHLIGHTED_NODE_COLOR);
		//		fill.add("INGROUP('" + Visualization.SEARCH_ITEMS + "')", PNConstants.HIGHLIGHTED_NODE_COLOR);
		fill.add("[" + PNConstants.NODE_TYPE_KEY + "] = \"" + NodeType.TRANSITION + "\" AND ["
				+ PNConstants.NODE_TAU_TRANSITION_KEY + "]", PNConstants.TAU_TRANSITIONS_BG_COLOR);
		fill.add("[type] = 'TRANSITION'",
				new ColorAction(GRAPH_NODES_GROUP, ExpressionParser.predicate("[type] = 'TRANSITION'"),
						VisualItem.FILLCOLOR) {
					private Color[] palette = ColorBrewer.getSequentialColorPalettes(false)[4].getColorPalette(10);
					private Color[] palette2 = ColorBrewer.getSequentialColorPalettes(false)[8].getColorPalette(10);
					private Color[] palette3 = ColorBrewer.getSequentialColorPalettes(false)[6].getColorPalette(10);

					@Override
					public int getColor(VisualItem item) {
						if (!controller.isAlignmentMode())
							return ColorLib.rgb(255, 255, 255);
						Transition t = null;

						t = controller.getIPN().getTranBasedOnID(Integer.valueOf(item.get("ID").toString()));

						// Choose the right structure depending on the selected mode

//						if (ProcessAnalyticsViz.selectedActivity == null
//								|| (ProcessAnalyticsViz.selectedActivity != null && ProcessAnalyticsViz.selectedActivity
//										.equals(t.getLabel()))) {
//							if (ProcessAnalyticsViz.mode == 0) {
//								float sync = 0;
//								if (ProcessAnalyticsViz.syncMovesFrequencyCurrent.containsKey(t)) {
//									sync = ProcessAnalyticsViz.syncMovesFrequencyCurrent.get(t);
//								}
//								float model = 0;
//								if (ProcessAnalyticsViz.modelMovesFrequencyCurrent.containsKey(t)) {
//									model = ProcessAnalyticsViz.modelMovesFrequencyCurrent.get(t);
//								}
//								if (sync != 0 || model != 0) {
//									int index = ((int) (sync / (sync + model)) * 10);
//									return ColorLib.color(palette[index > 0 ? index - 1 : 0]);
//								}
//							} else if (ProcessAnalyticsViz.mode == 1) {
//								//conformance view
//
//								boolean mainNet = false;
//
//								if (tpa != null) { //update t corresponding to tran of current net
////									t = tpa.getPetriNetMap().getMapTransitions().get(t);
//								}
//
//								int freq = 0;
//								int highestFreq = 0;
//								//get highest Freq and freq for curr tran
//								if (tpa != null) {
//									for (Transition aggT : tpa.getSyncMoveMap().keySet()) {
//										if (tpa.getSyncMoveMap().get(aggT) > highestFreq)
//											highestFreq = tpa.getSyncMoveMap().get(aggT);
//										if (aggT.equals(t)) {
//											freq = tpa.getSyncMoveMap().get(aggT);
//										}
//									}
//								} else {
//									for (Transition aggT : controller.getIPN().getAggrSyncMoves().keySet()) {
//										if (controller.getIPN().getAggrSyncMoves().get(aggT) > highestFreq)
//											highestFreq = controller.getIPN().getAggrSyncMoves().get(aggT);
//										if (aggT.equals(t)) {
//											freq = controller.getIPN().getAggrSyncMoves().get(aggT);
//										}
//									}
//								}
//
//								float val = (((float) freq) / highestFreq);
//								int index = (int) (val * 10);
//								System.out.println(t.getLabel() + " : " + index);
//								return ColorLib.color(palette2[index > 0 ? index - 1 : 0]);
//							} else if (ProcessAnalyticsViz.mode == 2) {
//								// TODO: Finish implementation for performance view!
//								float sync = 0;
//								if (ProcessAnalyticsViz.syncMovesFrequencyCurrent.containsKey(t)) {
//									sync = ProcessAnalyticsViz.syncMovesFrequencyCurrent.get(t);
//								}
//								int totalCases = ProcessAnalyticsViz.getLog().size();
//								int index = (int) ((sync / totalCases) * 10);
//								return ColorLib.color(palette2[index > 0 ? index - 1 : 0]);
//							}
//						}

						return ColorLib.rgb(255, 255, 255);
					}
				});
//		fill.add("[type] = 'PLACE' AND [placeLogMove] = TRUE", ColorLib.rgb(229, 215, 64));
//		fill.add("[type] = 'PLACE'", new ColorAction(GRAPH_NODES_GROUP, ExpressionParser.predicate("[type] = 'PLACE'"),
//				VisualItem.FILLCOLOR) {
//			private Color[] palette = ColorBrewer.getSequentialColorPalettes(false)[4].getColorPalette(10);
//
//			@Override
//			public int getColor(VisualItem item) {
//				if (controller.getIPN() != null)
//					if (ProcessAnalyticsViz.mode == 0) {
//						Place p = (Place) ProcessAnalyticsViz.findNode((NodeItem) item);
//						for (Set<Place> set : ProcessAnalyticsViz.logMovesFrequencyCurrent.keySet()) {
//							if (set.contains(p)) {
//								return ColorLib.color(palette[0]);
//							} else {
//								return ColorLib.color(palette[9]);
//							}
//						}
//					}
//
//				return ColorLib.rgb(255, 255, 255);
//			}
//		});

		final DepthColorAction fillDepth = new DepthColorAction(GRAPH_NODES_GROUP, ExpressionParser.predicate("["
				+ VisualItem.HIGHLIGHT + "] = TRUE"), VisualItem.FILLCOLOR, PNConstants.HIGHLIGHTED_NODE_COLOR);
		fillDepth.add("INGROUP('" + GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL + "') AND INGROUP('"
				+ GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL + "')", ColorLib.rgba(0, 0, 0, 0));
		fillDepth.add("INGROUP('" + GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL + "')",
				PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR);
		fillDepth.add("INGROUP('" + GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL + "')",
				PNConstants.OUTGOING_HIGHLIGHTED_NODE_COLOR);

		final ColorAction soundnessPropertiesFillAction = new ColorAction(GRAPH_NODES_GROUP, VisualItem.FILLCOLOR,
				ColorLib.color(Color.WHITE)) {
			@Override
			public int getColor(VisualItem item) {
				int color = super.getColor(item);

				if (item.getInt(PNConstants.NODE_DEPTH_KEY) > 0) {
					color = Utils.changeAlphaDepth(color, item.getInt(PNConstants.NODE_DEPTH_KEY));
				}

				return color;
			}
		};
		soundnessPropertiesFillAction.add(PNConstants.TAU_TRANSITIONS_PREDICATE, PNConstants.TAU_TRANSITIONS_BG_COLOR);
		soundnessPropertiesFillAction.add(DEAD_TRANSITIONS_PREDICATE, PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR);
		soundnessPropertiesFillAction.add("[" + VisualItem.HIGHLIGHT + "] AND INGROUP('"
				+ GRAPH_NODES_PROBLEMATIC_HIGHLIGHT + "')", PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR);
		soundnessPropertiesFillAction.add(NONLIVE_TRANSITIONS_PREDICATE, PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR);
		soundnessPropertiesFillAction.add(UNBOUNDED_PLACES_PREDICATE, PNConstants.INCOMING_HIGHLIGHTED_NODE_COLOR);

		ColorAction strokeColor = new ColorAction(GRAPH_NODES_GROUP, VisualItem.STROKECOLOR, getShapeBorderColor()) {
			@Override
			public int getColor(VisualItem item) {
				int color = super.getColor(item);

				if (item.getInt(PNConstants.NODE_DEPTH_KEY) > 0) {
					color = Utils.changeAlphaDepth(color, item.getInt(PNConstants.NODE_DEPTH_KEY));
				}

				return color;
			}
		};

		Action fitProblematicGroupAction = new FitGroupBoundsAction(GRAPH_NODES_PROBLEMATIC_FIT, 30, 2000, false);
		Action fitContainerAnimated = new FitGroupBoundsAction(GRAPH_GROUP, 2, 2000, false);

		ActionList highlightActions = new ActionList(200);
		highlightActions.add(new ColorAnimator(GRAPH_NODES_GROUP, VisualItem.FILLCOLOR));
		highlightActions.add(repaintAction);
		highlightActions.addActivityListener(new ActivityAdapter() {
			public void activityCancelled(Activity a) {
				// if animation is canceled, set colors to final state
				// This avoids the blinking problem
				fill.run(1.0);
				fillDepth.run(1.0);
				soundnessPropertiesFillAction.run(1.0);
				repaintAction.run(1.0);
			}
		});

		ActionList highlightEdgesActions = new ActionList(300);
		highlightEdgesActions.add(new ColorAnimator(GRAPH_EDGES_GROUP, new String[] { VisualItem.FILLCOLOR,
				VisualItem.STROKECOLOR }));
		// highlightEdgesActions.add(new
		// ColorAnimator(LAYER_SIMPLIFIER_AGGREGATE_GROUP,
		// VisualItem.STROKECOLOR));
		highlightEdgesActions.add(repaintAction);
		highlightEdgesActions.addActivityListener(new ActivityAdapter() {
			public void activityCancelled(Activity a) {
				// if animation is canceled, set colors to final state
				// This avoids the blinking problem
				edgesColorAction.run(1.0);
				edgeStrokeAction.run(1.0);
				repaintAction.run(1.0);
			}
		});

		ActionList loopModeAnimations = new ActionList(200);
		loopModeAnimations.add(edgesFillAnimator);
		loopModeAnimations.add(repaintAction);
		loopModeAnimations.addActivityListener(new ActivityAdapter() {
			public void activityCancelled(Activity a) {
				// if animation is canceled, set colors to final state
				// This avoids the blinking problem
				edgesColorAction.run(1.0);
				repaintAction.run(1.0);
			}
		});

		TextColorAction textColorAction = new TextColorAction(Visualization.ALL_ITEMS) {
			@Override
			public int getColor(VisualItem item) {
				int color = super.getColor(item);

				if (item instanceof DecoratorItem && ((DecoratorItem) item).getDecoratedItem() instanceof NodeItem) {
					if (item.getInt(PNConstants.NODE_DEPTH_KEY) > 0) {
						color = Utils.changeAlphaDepth(color, item.getInt(PNConstants.NODE_DEPTH_KEY));
					}
				}

				return color;
			}
		};
		textColorAction.setFilterPredicate(LABEL_GROUPS_PREDICATE);

		// add the actions to the visualization
		vis.putAction(PNConstants.VIS_DECORATORS_ACTIONS_NAME, decorators);
		vis.putAction(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME, shape);
		vis.putAction(PNConstants.VIS_LAYOUT_ACTIONS_NAME, layout);
		vis.putAction(PNConstants.VIS_LAYOUT_NO_PAINT_ACTIONS_NAME, noPaintingLayout);
		vis.putAction(PNConstants.VIS_LAYOUT_FIT_BOUNDS_ACTIONS_NAME, fitBounds);
		vis.putAction(PNConstants.VIS_LAYOUT_DECORATORS, layoutDecorators);
		vis.putAction(PNConstants.VIS_LAYOUT_ROTATE, rotation);
		vis.putAction(PNConstants.VIS_HIGHLIGHT_ACTIONS_NAME, highlightActions);
		vis.putAction(PNConstants.VIS_HIGHLIGHT_EDGES_ACTIONS_NAME, highlightEdgesActions);
		vis.putAction(PNConstants.VIS_LOOP_MODE_ANIMATIONS_ACTIONS_NAME, loopModeAnimations);
		vis.putAction(PNConstants.VIS_SIMPLIFIER_DECORATOR_ACTIONS, simplifierDecorator);
		vis.putAction(PNConstants.VIS_REPAINT_ACTIONS_NAME, repaintAction);
		vis.putAction(PNConstants.VIS_ANIMATE_LAYOUT_ACTIONS_NAME, animateLayout);
		vis.putAction(PNConstants.VIS_FILL_NODES_ACTIONS_NAME, fill);
		vis.putAction(PNConstants.VIS_FILL_NODES_DEPTH_ACTIONS_NAME, fillDepth);
		vis.putAction(PNConstants.VIS_FILL_EDGES_ACTIONS_NAME, edgesColorAction);
		vis.putAction(PNConstants.VIS_STROKE_EDGES_ACTIONS_NAME, edgeStrokeAction);
		vis.putAction(PNConstants.VIS_POSITION_ANIMATE_ACTIONS_NAME, positionAnimate);
		vis.putAction(PNConstants.VIS_TEXT_COLOR_ACTIONS_NAME, textColorAction);
		vis.putAction(PNConstants.VIS_SOUNDNESS_COLOR_ACTIONS_NAME, soundnessPropertiesFillAction);
		vis.putAction(PNConstants.VIS_SOUNDNESS_PROBLEMATIC_FIT_ACTIONS_NAME, fitProblematicGroupAction);
		vis.putAction(PNConstants.VIS_LAYOUT_FIT_BOUNDS_ANIMATED_ACTIONS_NAME, fitContainerAnimated);
		vis.putAction(PNConstants.VIS_STROKE_COLOR_ACTIONS_NAME, strokeColor);

		// Always run after actions
		//        vis.alwaysRunAfter(PNConstants.VIS_ANIMATE_LAYOUT_ACTIONS_NAME, PNConstants.VIS_LAYOUT_FIT_BOUNDS_ANIMATED_ACTIONS_NAME);
		vis.alwaysRunAfter(PNConstants.VIS_FILL_NODES_ACTIONS_NAME, PNConstants.VIS_TEXT_COLOR_ACTIONS_NAME);
		vis.alwaysRunAfter(PNConstants.VIS_FILL_NODES_DEPTH_ACTIONS_NAME, PNConstants.VIS_TEXT_COLOR_ACTIONS_NAME);
		vis.alwaysRunAfter(PNConstants.VIS_SOUNDNESS_COLOR_ACTIONS_NAME, PNConstants.VIS_TEXT_COLOR_ACTIONS_NAME);
	}

	private void setUpHighlightGroups() {
		if (vis.getGroup(GRAPH_NODES_FORWARD_INTERACTION_FOCUS_GROUP) == null) {
			vis.addFocusGroup(GRAPH_NODES_FORWARD_INTERACTION_FOCUS_GROUP);
		}
		if (vis.getGroup(GRAPH_NODES_MISSING_PLACE_FOCUS_GROUP) == null) {
			vis.addFocusGroup(GRAPH_NODES_MISSING_PLACE_FOCUS_GROUP);
		}
		if (vis.getGroup(GRAPH_NODES_PROBLEMATIC_HIGHLIGHT) == null) {
			vis.addFocusGroup(GRAPH_NODES_PROBLEMATIC_HIGHLIGHT);
		}

		if (vis.getGroup(GRAPH_NODES_PROBLEMATIC_FIT) == null) {
			vis.addFocusGroup(GRAPH_NODES_PROBLEMATIC_FIT);
		}

		if (vis.getGroup(GRAPH_NODES_INCOMING_HIGHLIGHT) == null) {
			vis.addFocusGroup(GRAPH_NODES_INCOMING_HIGHLIGHT);
		}

		if (vis.getGroup(GRAPH_NODES_OUTGOING_HIGHLIGHT) == null) {
			vis.addFocusGroup(GRAPH_NODES_OUTGOING_HIGHLIGHT);
		}

		if (vis.getGroup(GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL) == null) {
			vis.addFocusGroup(GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL);
		}

		if (vis.getGroup(GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL) == null) {
			vis.addFocusGroup(GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL);
		}

		// PMD starts
		if (vis.getGroup(GRAPH_NODES_INPUT_GROUP) == null) {
			vis.addFocusGroup(GRAPH_NODES_INPUT_GROUP);
		}
		if (vis.getGroup(GRAPH_NODES_OUTPUT_GROUP) == null) {
			vis.addFocusGroup(GRAPH_NODES_OUTPUT_GROUP);
		}
		if (vis.getGroup(GRAPH_NODES_MULTIPLE_OUTPUT_GROUP) == null) {
			vis.addFocusGroup(GRAPH_NODES_MULTIPLE_OUTPUT_GROUP);
		}
		if (vis.getGroup(GRAPH_NEXT_ACTIVITY) == null) {
			vis.addFocusGroup(GRAPH_NEXT_ACTIVITY);
		}
		if (vis.getGroup(GRAPH_NODES_INPUT_OUTPUT_GROUP) == null) {
			vis.addFocusGroup(GRAPH_NODES_INPUT_OUTPUT_GROUP);
		}
		if (vis.getGroup(GRAPH_EDGES_SELECTED_EDGES_GROUP) == null) {
			vis.addFocusGroup(GRAPH_EDGES_SELECTED_EDGES_GROUP);
		}
		if (vis.getGroup(GRAPH_NODES_CONFORMANCE_CHECK) == null) {
			vis.addFocusGroup(GRAPH_NODES_CONFORMANCE_CHECK);
		}
		// PMD Ends

		if (vis.getGroup(SELECTED_REMOVE_GROUP) == null) {
			vis.addFocusGroup(SELECTED_REMOVE_GROUP);
			TupleSet focusGroup = vis.getGroup(SELECTED_REMOVE_GROUP);
			focusGroup.addTupleSetListener(new TupleSetListener() {
				public void tupleSetChanged(TupleSet ts, Tuple[] add, Tuple[] rem) {
					for (int i = 0; i < add.length; i++) {
						VisualItem item = (VisualItem) add[i];
						item.setHighlighted(true);
					}
					for (int i = 0; i < rem.length; i++) {
						VisualItem item = (VisualItem) rem[i];
						item.setHighlighted(false);
					}
					vis.run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
				}
			});
		}

		Table rubberBandTable = new Table();
		rubberBandTable.addColumn(VisualItem.POLYGON, float[].class);
		rubberBandTable.addRow();
		vis.add(RUBBER_BAND_GROUP, rubberBandTable);
		VisualItem rubberBand = (VisualItem) vis.getVisualGroup(RUBBER_BAND_GROUP).tuples().next();
		rubberBand.set(VisualItem.POLYGON, new float[8]);
		rubberBand.setStrokeColor(ColorLib.color(ColorLib.getColor(96, 161, 226)));
		rubberBand.setFillColor(ColorLib.color(ColorLib.getColor(180, 212, 244, 100)));
		rubberBand.setStroke(StrokeLib.getStroke(PNConstants.STROKE_DEFAULT_WIDTH));
	}

	//PMD update nodes in process analytics start
	protected void updateNodesInPetriNet(TupleSet focusGroup) {
		Iterator iter = focusGroup.tuples();
		controller.getProcessAnalytics().getTranSet().clear();
		while (iter.hasNext()) {
			Tuple tuple = (Tuple) iter.next();
			if (tuple.getString(PNConstants.NODE_TYPE_KEY).equals("TRANSITION")) {
				for (PetrinetNode t : controller.getProcessAnalytics().getPetrinet().getNodes()) {
					System.out.println(t.getLocalID().localNodeID().toString() + "      "
							+ tuple.getString(PNConstants.NODE_ID_KEY));
					if (t.getLocalID().localNodeID().toString().equals(tuple.getString(PNConstants.NODE_ID_KEY)))
						controller.getProcessAnalytics().getTranSet().add((Transition) t);
				}
			}
		}
	}

	//PMD update nodes in process analytics end

	private void setUpSearcher() {
		TupleSet source = vis.getGroup(GRAPH_NODES_GROUP);
		m_fields = new String[] { PNConstants.NODE_LABEL_KEY };
		TupleSet search = vis.getGroup(Visualization.SEARCH_ITEMS);

		if (search != null) {
			if (search instanceof SearchTupleSet) {
				m_searcher = (SearchTupleSet) search;
			} else {
				throw new IllegalStateException("Search focus set not instance of SearchTupleSet!");
			}
		} else {
			m_searcher = new RegexSearchTupleSet(false);
			vis.addFocusGroup(Visualization.SEARCH_ITEMS, m_searcher);
		}

		if (m_autoIndex && source != null) {
			// index everything already there
			for (int i = 0; i < m_fields.length; i++)
				m_searcher.index(source.tuples(), m_fields[i]);

			// add a listener to dynamically build search index
			source.addTupleSetListener(new TupleSetListener() {
				public void tupleSetChanged(TupleSet tset, Tuple[] add, Tuple[] rem) {
					if (add != null) {
						for (int i = 0; i < add.length; ++i) {
							for (int j = 0; j < m_fields.length; j++)
								m_searcher.index(add[i], m_fields[j]);
						}
					}
					if (rem != null && m_searcher.isUnindexSupported()) {
						for (int i = 0; i < rem.length; ++i) {
							for (int j = 0; j < m_fields.length; j++)
								m_searcher.unindex(rem[i], m_fields[j]);
						}
					}
				}
			});
		}
	}

	public Visualization getVisualization() {
		return vis;
	}

	public void executeLayout(boolean animate) {
		if (animate) {
			vis.cancel(PNConstants.VIS_ANIMATE_LAYOUT_ACTIONS_NAME);
			vis.run(PNConstants.VIS_LAYOUT_NO_PAINT_ACTIONS_NAME);
			vis.run(PNConstants.VIS_ANIMATE_LAYOUT_ACTIONS_NAME);
		} else {
			vis.run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
		}
	}

	public void fitLayoutBounds() {
		vis.run(PNConstants.VIS_LAYOUT_FIT_BOUNDS_ANIMATED_ACTIONS_NAME);
	}

	public void rotateClockwise() {
		rotationAction.setClockwise(true);
		vis.run(PNConstants.VIS_LAYOUT_ROTATE);
	}

	public void rotateAnticlockwise() {
		rotationAction.setClockwise(false);
		vis.run(PNConstants.VIS_LAYOUT_ROTATE);
	}

	public void runHighlightAction() {
		vis.run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
		vis.run(PNConstants.VIS_HIGHLIGHT_ACTIONS_NAME);
	}

	public void searchFor(String query) {
		synchronized (vis) {
			m_searcher.search(query);
			runHighlightAction();
		}
	}

	public void setLineType(LineType lineType) {
		EdgeRenderer renderer = lineType.getRenderer();
		renderer.setArrowHeadSize(arrowWidth, arrowHeight);
		((DefaultRendererFactory) vis.getRendererFactory()).setDefaultEdgeRenderer(renderer);
	}

	public void setBarDecoratorEnabled(boolean enabled) {
		aggregateAction.setEnabled(enabled);
		aggregateActionLayerNodes.setEnabled(enabled);
	}

	public boolean isBarDecoratorEnabled() {
		return aggregateAction.isEnabled();
	}

	public TupleSet getIncomingFocusGroup() {
		return vis.getGroup(GRAPH_NODES_INCOMING_HIGHLIGHT);
	}

	public TupleSet getOutgoingFocusGroup() {
		return vis.getGroup(GRAPH_NODES_OUTGOING_HIGHLIGHT);
	}

	public TupleSet getIncomingFocusGroupPathControl() {
		return vis.getGroup(GRAPH_NODES_INCOMING_HIGHLIGHT_PATH_CONTROL);
	}

	public TupleSet getOutgoingFocusGroupPathControl() {
		return vis.getGroup(GRAPH_NODES_OUTGOING_HIGHLIGHT_PATH_CONTROL);
	}

	public void setSimplifierControlEnabled(boolean enabled) {
		aggregateAction2.setEnabled(enabled);
	}

	public boolean isSimplifierControlEnabled() {
		return aggregateAction2.isEnabled();
	}

	public AggregateTable getLayerSimplifierAggregate() {
		return aggregateTable2;
	}

	public void runSimplifierDecorator() {
		vis.run(PNConstants.VIS_SIMPLIFIER_DECORATOR_ACTIONS);
	}

	public void setEdgeDirectionEnabled(boolean directed) {
		simplifiedRenderer.setOrientedEnabled(directed);
	}

	public int getTransitionsWidth() {
		return transitionsWidth;
	}

	public void setTransitionsWidth(int width) {
		transitionsWidth = width;
		vis.setValue(GRAPH_NODES_GROUP, PNConstants.TRANSITION_NODE_PREDICATE, PNConstants.NODE_WIDTH_KEY,
				transitionsWidth);
	}

	public int getTransitionsHeight() {
		return transitionsHeight;
	}

	public void setTransitionsHeight(int height) {
		transitionsHeight = height;
		vis.setValue(GRAPH_NODES_GROUP, PNConstants.TRANSITION_NODE_PREDICATE, PNConstants.NODE_HEIGHT_KEY,
				transitionsHeight);
	}

	public int getPlacesSize() {
		return placesSize;
	}

	public void setPlacesSize(int size) {
		placesSize = size;
		vis.setValue(GRAPH_NODES_GROUP, PREDICATE_PLACES, PNConstants.NODE_WIDTH_KEY, placesSize);
		vis.setValue(GRAPH_NODES_GROUP, PREDICATE_PLACES, PNConstants.NODE_HEIGHT_KEY, placesSize);
		vis.setValue(STARTING_NODES_DECORATOR_GROUP_NAME, null, VisualItem.SIZE, placesSize
				* PNConstants.STARTING_NODE_SHAPE_PERCENTAGE_SIZE / shapeRenderer.getBaseSize());
		vis.setValue(ENDING_NODES_DECORATOR_GROUP_NAME, null, VisualItem.SIZE, placesSize
				* PNConstants.ENDING_NODE_SHAPE_PERCENTAGE_SIZE / shapeRenderer.getBaseSize());
	}

	public int getFontSize() {
		return fontSize;
	}

	public void setFontSize(int size) {
		fontSize = size;
		vis.setValue(LABEL_DECORATOR_GROUP_NAME, null, VisualItem.FONT, FontLib.getFont("Arial", fontSize));
	}

	public int getDegreeFontSize() {
		return degreeFontSize;
	}

	public void setDegreeFontSize(int size) {
		degreeFontSize = size;
		vis.setValue(INDEGREE_INFO_DECORATOR_GROUP_NAME, null, VisualItem.FONT,
				FontLib.getFont("Arial", degreeFontSize));
		vis.setValue(OUTDEGREE_INFO_DECORATOR_GROUP_NAME, null, VisualItem.FONT,
				FontLib.getFont("Arial", degreeFontSize));
	}

	public void executePositionAnimate(VisualItem item) {
		dragAnimator.setFilterPredicate(ExpressionParser.predicate("ROW() = " + item.getRow()));
		vis.run(PNConstants.VIS_POSITION_ANIMATE_ACTIONS_NAME);
	}

	public void repaint() {
		vis.repaint();
	}

	public Graph getGraph() {
		return graph;
	}

	public long getAnimationLength() {
		return animationLength;
	}

	//PMD Starts
	public PNetVisualizerController getController() {
		return controller;
	}

	public TupleSet getInputGroup() {
		return vis.getGroup(GRAPH_NODES_INPUT_GROUP);
	}

	public TupleSet getOutputGroup() {
		return vis.getGroup(GRAPH_NODES_OUTPUT_GROUP);
	}

	public TupleSet getInputOutputGroup() {
		return vis.getGroup(GRAPH_NODES_INPUT_OUTPUT_GROUP);
	}

	public TupleSet getSelectedEdgesGroup() {
		return vis.getGroup(GRAPH_EDGES_SELECTED_EDGES_GROUP);
	}

	public TupleSet getSelectedConformanceNodes() {
		return vis.getGroup(GRAPH_NODES_CONFORMANCE_CHECK);
	}

	public TupleSet getMultipleOutputGroup() {
		return vis.getGroup(GRAPH_NODES_MULTIPLE_OUTPUT_GROUP);
	}

	public TupleSet getSelectedActivity() {
		return vis.getGroup(GRAPH_NEXT_ACTIVITY);
	}

	public TupleSet getGraphNodes() {
		return vis.getGroup(GRAPH_NODES);
	}

	//PMD Ends

	public void setAnimationLength(long animationLength) {
		this.animationLength = animationLength;
		animateLayout.setDuration(animationLength);
		positionAnimate.setDuration(animationLength);
	}

	public void setLoopModeEnabled(boolean enabled) {
		edgesColorAction.setLoopModeEnabled(enabled);
		Iterator<Tuple> iter = vis.getGroup(GRAPH_EDGES_GROUP).tuples(PNConstants.LOOP_PREDICATE);

		if (enabled) {
			while (iter.hasNext()) {
				Tuple t = iter.next();
				t.setBoolean(PNConstants.SIMPLIFIED_OLD_KEY, t.getBoolean(PNConstants.SIMPLIFIED_KEY));
			}
			vis.setValue(GRAPH_EDGES_GROUP, PNConstants.LOOP_PREDICATE, PNConstants.EDGE_SIMPLIFICABLE, false);
			vis.setValue(GRAPH_EDGES_GROUP, PNConstants.LOOP_PREDICATE, PNConstants.SIMPLIFIED_KEY, false);
		} else {
			vis.setValue(GRAPH_EDGES_GROUP, PNConstants.LOOP_PREDICATE, PNConstants.EDGE_SIMPLIFICABLE, true);
			while (iter.hasNext()) {
				Tuple t = iter.next();
				t.setBoolean(PNConstants.SIMPLIFIED_KEY, t.getBoolean(PNConstants.SIMPLIFIED_OLD_KEY));
			}
		}

		vis.run(PNConstants.VIS_FILL_EDGES_ACTIONS_NAME);
		vis.run(PNConstants.VIS_DECORATORS_ACTIONS_NAME);
		vis.run(PNConstants.VIS_LOOP_MODE_ANIMATIONS_ACTIONS_NAME);
	}

	public void setTextBarDecoratorVisibility(boolean enabled) {
		vis.setValue(SIMPLIFICATION_BARS_TEXT_DECORATOR_NAME, null, VisualItem.VISIBLE, enabled);
	}

	public boolean isTransitionInOutDegreeEnabled() {
		return customShapeRenderer.isTransitionsInOutDegreeEnabled();
	}

	public void setTransitionInOutDegree(boolean enabled) {
		customShapeRenderer.setTransitionsInOutDegreeEnabled(enabled);
		indegreeDecorator.setEnabled(enabled);
		outdegreeDecorator.setEnabled(enabled);
		indegreeRenderer.setEnabled(enabled);
		outdegreeRenderer.setEnabled(enabled);
	}

	public int getEdgeDefaultColor() {
		return edgesColorAction.getDefaultColor();
	}

	public void setEdgeDefaultColor(int color) {
		edgesColorAction.setDefaultColor(color);
	}

	public int getShapeBorderColor() {
		return (int) nodeSchema.getDefault(VisualItem.STROKECOLOR);
	}

	public void setShapeBorderColor(int color) {
		nodeSchema.setDefault(VisualItem.STROKECOLOR, color);
		vis.setValue(GRAPH_NODES_GROUP, null, VisualItem.STROKECOLOR, color);
	}

	public void setStartingNodeDecoratorEnabled(boolean enabled) {
		vis.setValue(STARTING_NODES_DECORATOR_GROUP_NAME, null, VisualItem.VISIBLE, enabled);
	}

	public void setInformationMode(int mode) {
		vis.run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
		vis.run(PNConstants.VIS_REPAINT_ACTIONS_NAME);
	}

//	public void setSelectedActivity(String activity) {
//		vis.setValue(GRAPH_NODES_GROUP, null, "placeLogMove", false);
//		if (activity != null) {
//			for (Set<Place> set : ProcessAnalyticsViz.logMovesFrequencyCurrent.keySet()) {
//				Map<String, Float> map = ProcessAnalyticsViz.logMovesFrequencyCurrent.get(set);
//				if (map.containsKey(ProcessAnalyticsViz.selectedActivity + "+complete")) {
//					for (Place p : set) {
//						((Tuple) vis
//								.getGroup(GRAPH_NODES_GROUP)
//								.tuples(ExpressionParser.predicate("[originalId] = '"
//										+ p.getLocalID().localNodeID().toString() + "'")).next()).setBoolean(
//								"placeLogMove", true);
//					}
//				}
//			}
//		}
//	}

}
