package org.processmining.humbertogarciacaballero.models.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Hashtable;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.commons.lang3.StringUtils;
import org.processmining.dataquality.DetectionPanel;
import org.processmining.dataquality.LogPanel;
import org.processmining.dataquality.Reasons.CalculateAlignments;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.SoundnessInformation;
import org.processmining.models.graphbased.directed.transitionsystem.CoverabilityGraph;
import org.processmining.prabhakardixit.CalculateLogHeuristics.HeuristicsInfo;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.featureSelection.Evaluators;
import org.processmining.prabhakardixit.processanalytics.ProcessAnalyticsVizViews;
import org.processmining.prabhakardixit.processanalytics.models.ActivityStartOptions;
import org.processmining.prabhakardixit.processanalytics.models.ChartView;
import org.processmining.prabhakardixit.processanalytics.models.GraphData;
import org.processmining.prabhakardixit.processanalytics.models.ProcessView;
import org.processmining.prabhakardixit.processanalytics.models.TimeView;

import prefuse.data.Graph;

import com.fluxicon.slickerbox.factory.SlickerFactory;

@SuppressWarnings("serial")
public class PNetVisualizerGUI extends JFrame {

	// GUI Components
	private JPanel mainGraphPanel;
	private JPanel miniMapPanel;
	private JPanel graphPanel;
	private ToolBarGUI toolBar;
	private JTextField searchField;

	// The controller
	private PNetVisualizerController controller;
	private JLabel lblSearch;
	private JLabel lblSearch2;
	private JPanel panel_3;
	//	private JPanel panel;
	private JPanel panel;

	//PMD Dq panel - added to panel
	//	private JPanel panel1;
	//	private JPanel panel2;

	public PNetVisualizerGUI(Graph graph, InteractivePetriNet net) {
		super("PNet Visualizer");
		setMinimumSize(new Dimension(400, 400));
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
		}

		controller = PNetVisualizerController.getInstance();
		controller.initController(graph, null);
		controller.setIPN(net); //PMD

		init(null);
	}

	public PNetVisualizerGUI(Graph graph, SoundnessInformation soundnessInformation, CoverabilityGraph cg) {
		super("Dynamic Petri Net Visualizer");
		setMinimumSize(new Dimension(10, 10));
		//        LookAndFeel currentLook = UIManager.getLookAndFeel();
		//        try {
		//            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		//        } catch (Exception e) {
		//        }

		controller = PNetVisualizerController.getInstance();
		controller.initController(graph, null);
		//		controller.initController(graph, this, soundnessInformation, cg);

		//		init(soundnessInformation);

		//        try {
		//            UIManager.setLookAndFeel(currentLook);
		//        } catch (UnsupportedLookAndFeelException e) {
		//        }
		init(null);
	}

	//PMD added process analytics starts
	public PNetVisualizerGUI(Graph graph, SoundnessInformation soundnessInformation, CoverabilityGraph cg,
			ProcessAnalyticsVizViews pa) {
		super("Dynamic Petri Net Visualizer");
		setMinimumSize(new Dimension(400, 400));
		controller = PNetVisualizerController.getInstance();
		controller.initController(graph, this, soundnessInformation, cg, pa);
		init(soundnessInformation);
	}

	//PMD added process analytics ends

	//PMD added IPN starts
	public PNetVisualizerGUI(Graph graph, InteractivePetriNet ipn, ProcessAnalyticsVizViews pa) {
		super("Dynamic Petri Net Visualizer");
		setMinimumSize(new Dimension(400, 400));
		controller = PNetVisualizerController.getInstance();
		controller.initController(graph, this, null, null, pa);
		controller.setIPN(ipn); //PMD
		init(null);
	}

	//PMD added IPN ends

	private void init(SoundnessInformation soundnessInformation) {
		// Ensure application exits when window is closed
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Maximizes the frame
		setExtendedState(JFrame.MAXIMIZED_BOTH);

		graphPanel = new JPanel(new BorderLayout());
		getContentPane().add(graphPanel, BorderLayout.CENTER);

		JPanel optionsPanel = new JPanel();
		optionsPanel.setPreferredSize(new Dimension(200, 10));
		//		graphPanel.add(optionsPanel, BorderLayout.EAST);
		optionsPanel.setLayout(new BorderLayout(0, 0));

		JPanel parametersPanel = new JPanel();
		optionsPanel.add(parametersPanel, BorderLayout.CENTER);
		parametersPanel.setLayout(new BoxLayout(parametersPanel, BoxLayout.Y_AXIS));

		JPanel itemInformationPanel = new JPanel();
		itemInformationPanel.setBorder(new TitledBorder(UIManager.getBorder("TitledBorder.border"), "Settings",
				TitledBorder.LEFT, TitledBorder.TOP, null, new Color(0, 0, 0)));
		parametersPanel.add(itemInformationPanel);
		itemInformationPanel.setLayout(new BorderLayout(0, 0));

		panel_3 = new JPanel();
		panel_3.setLayout(new BoxLayout(panel_3, BoxLayout.Y_AXIS));
		itemInformationPanel.add(panel_3, BorderLayout.CENTER);
		graphPanel.add(panel_3, BorderLayout.EAST);
		controller.setInfoPanel(panel_3);

		panel = new JPanel();

		mainGraphPanel = new JPanel(new BorderLayout());

		JPanel heuInfoPanel = new JPanel();
		heuInfoPanel.setBorder(new TitledBorder(UIManager.getBorder("TitledBorder.border"), "Heuristic Info",
				TitledBorder.LEFT, TitledBorder.TOP, null, new Color(0, 0, 0)));

		//PMD if DQ then dont add heuPanel
		if (!controller.getIPN().isDataQuality())
			panel_3.add(heuInfoPanel, BorderLayout.CENTER);

		//		graphPanel.add(mainGraphPanel, BorderLayout.CENTER);

		//        lblSearch2= new JLabel("Search:");
		//		panel.add(lblSearch2);

		JScrollPane scrollPane = new JScrollPane();
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, mainGraphPanel, scrollPane);
		splitPane.setResizeWeight(1d);
		//		splitPane.setLayout(new BorderLayout());
		//		splitPane.add(mainGraphPanel, BorderLayout.CENTER);
		//		splitPane.add(panel, BorderLayout.SOUTH);
		scrollPane.setViewportView(panel);
		controller.setStatisticsPanel(panel);

		//		mainGraphPanel.add(panel, BorderLayout.SOUTH);
		graphPanel.add(splitPane, BorderLayout.CENTER);
		//PMD start adding combo boxes for configuration

		JPanel charInfoPanel = new JPanel();
		charInfoPanel.setBorder(new TitledBorder(UIManager.getBorder("TitledBorder.border"), "Chart Display",
				TitledBorder.LEFT, TitledBorder.TOP, null, new Color(0, 0, 0)));
		JPanel classifyPanel = new JPanel();
		classifyPanel.setBorder(new TitledBorder(UIManager.getBorder("TitledBorder.border"), "Classify",
				TitledBorder.LEFT, TitledBorder.TOP, null, new Color(0, 0, 0)));

		final JComboBox<ProcessView> processView = new JComboBox<ProcessView>(ProcessView.values());
		final JComboBox<TimeView> timeView = new JComboBox<TimeView>(TimeView.values());
		final JComboBox<ChartView> chartView = new JComboBox<ChartView>(ChartView.values());
		final JComboBox<GraphData> graphData = new JComboBox<GraphData>(GraphData.values());
		final JButton execute = new JButton("Calculate");
		final JButton recompute = new JButton("Recompute alignments!");

		final JComboBox<ActivityStartOptions> actStartView = new JComboBox<ActivityStartOptions>(
				ActivityStartOptions.values());
		actStartView.hide();
		actStartView.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().setActivityOption(
						(ActivityStartOptions) actStartView.getSelectedItem());
				controller.executeLayout(false);
			}
		});

		processView.setSelectedItem(controller.getProcessAnalytics().getProcessView());
		processView.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().setProcessView((ProcessView) processView.getSelectedItem());
				controller.executeLayout(false);
				actStartView.show();
			}
		});

		timeView.setSelectedItem(controller.getProcessAnalytics().getTimeView());
		timeView.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().setTimeView((TimeView) timeView.getSelectedItem());
				controller.executeLayout(false);
			}
		});

		chartView.setSelectedItem(controller.getProcessAnalytics().getGraphView());
		chartView.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//				controller.getProcessAnalytics().updatePerformanceDetails();
				//				controller.getProcessAnalytics().getPairwisePerformance().print(controller.getProcessAnalytics().getPetrinet());
				controller.getProcessAnalytics().setGraphView((ChartView) chartView.getSelectedItem());
				controller.executeLayout(false);
			}
		});

		graphData.setSelectedItem(controller.getProcessAnalytics().getGraphData());
		graphData.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().setGraphData((GraphData) graphData.getSelectedItem());
				controller.executeLayout(false);
			}
		});
		execute.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().createGraph();
			}
		});

		recompute.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().recomputeAlignments();
			}
		});

		//classification inputs
		final JSlider fitnessSlider = new JSlider(0, 100);
		Hashtable labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0.0"));
		labelTable.put(new Integer(25), new JLabel("0.25"));
		labelTable.put(new Integer(50), new JLabel("0.5"));
		labelTable.put(new Integer(75), new JLabel("0.75"));
		labelTable.put(new Integer(100), new JLabel("1.0"));
		fitnessSlider.setLabelTable(labelTable);
		fitnessSlider.setMaximumSize(new Dimension(50, 100));
		fitnessSlider
				.setValue((int) controller.getProcessAnalytics().getFeatureSelection().getFitnessThreshold() * 100);
		fitnessSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				//				if (!source.getValueIsAdjusting()) {
				double val = (double) source.getValue() / 100;
				controller.getProcessAnalytics().getFeatureSelection().setFitnessThreshold(val);
				//				}

			}
		});
		final JLabel lblNumAtt = new JLabel("Number of attributes:");
		final JTextField numAtt = new JTextField();
		numAtt.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				if (StringUtils.isNumeric(numAtt.getText()))
					controller.getProcessAnalytics().getFeatureSelection()
							.setNumOfAtts(Integer.valueOf(numAtt.getText()));
			}

			public void removeUpdate(DocumentEvent e) {
				if (StringUtils.isNumeric(numAtt.getText()))
					controller.getProcessAnalytics().getFeatureSelection()
							.setNumOfAtts(Integer.valueOf(numAtt.getText()));
			}

			public void insertUpdate(DocumentEvent e) {
				if (StringUtils.isNumeric(numAtt.getText()))
					controller.getProcessAnalytics().getFeatureSelection()
							.setNumOfAtts(Integer.valueOf(numAtt.getText()));
			}

		});

		final JComboBox<Evaluators> evaluator = new JComboBox<Evaluators>(Evaluators.values());
		evaluator.setSelectedItem(controller.getProcessAnalytics().getFeatureSelection().getEvaluator());
		evaluator.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().getFeatureSelection()
						.setEvaluator((Evaluators) evaluator.getSelectedItem());
				controller.executeLayout(false);
			}
		});

		final JButton classify = new JButton("Classify");
		classify.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.getProcessAnalytics().classify();
			}
		});

		//PMD IPN starts
		//		JPanel itemInformationPanel2 = new JPanel();
		//		itemInformationPanel2.setBorder(new TitledBorder(UIManager.getBorder("TitledBorder.border"), "Heuristics Info",
		//				TitledBorder.LEFT, TitledBorder.TOP, null, new Color(0, 0, 0)));
		//		parametersPanel.add(itemInformationPanel2);
		//		itemInformationPanel2.setLayout(new BorderLayout(0, 0));

		//PMD add combo here starts

		final HeuristicsInfo heuristicsInfo = new HeuristicsInfo();

		//add activityCombo to controller - to be used during prediction

		final JLabel actLbl = new JLabel("Select activity:");
		actLbl.setAlignmentX(Component.CENTER_ALIGNMENT);
		//heuInfoPanel.add(new JLabel("Information Panel:",SwingConstants.LEFT));
		heuInfoPanel.add(heuristicsInfo.getPanel());
		heuInfoPanel.setLayout(new BoxLayout(heuInfoPanel, BoxLayout.Y_AXIS));
		heuInfoPanel.setMaximumSize(heuInfoPanel.getPreferredSize());
		heuInfoPanel.add(actLbl);

		JButton btn = new JButton("Save me");
		btn.addActionListener(new ActionListener() {
			int count = 0;

			public void actionPerformed(ActionEvent e) {
				FileOutputStream fileOutPutStream;
				try {
					fileOutPutStream = new FileOutputStream(count++ + ".png");
					//		ExportAsPDF.convertToPdf(this, this.WIDTH, this.HEIGHT, "");
					controller.getMainView().getDisplay().saveImage(fileOutPutStream, "png", 5.);

					fileOutPutStream.flush();
					fileOutPutStream.close();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}

				System.out.println("xxx22");

			}
		});
		//		panel_3.add(btn);

		JButton btn2 = new JButton("SetAct");
		btn2.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				controller.setActivity("");
			}
		});
		//		panel_3.add(btn2);

		String events[] = new String[controller.getIPN().getSortedEventNames().size() + 1];
		int i = 0;
		events[i] = "";
		//		events[++i] = "";
		if (controller.getIPN() != null)
			for (String event : controller.getIPN().getSortedEventNames())
				events[++i] = event;

		final JComboBox<String> activityCombo = SlickerFactory.instance().createComboBox(events);
		//		final JComboBox<String> activityCombo = new JComboBox<String>(events);
		activityCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.setActivity(activityCombo.getSelectedItem().toString());
				heuristicsInfo.updateDirectRelations(controller);
				heuristicsInfo.updateHeuristics(controller.getIPN());
				controller.executeLayout(false);
			}
		});
		heuInfoPanel.add(activityCombo);
		controller.setActivityCombo(activityCombo);

		if (controller.getIPN().isDataQuality()) //if DQ data quality plugin, then introduce tabbed widnow - for detection and repair
		{
			final JTabbedPane tabePane = new JTabbedPane();
			DetectionPanel dp = new DetectionPanel(controller);
			//			controller.setDetectPane(dp);
			tabePane.add("Detection", dp);
			//			tabePane.add("Detection", new LogPanel(controller, controller.getIPN().getDqModel().getDetectionModel()
			//					.getMetricsTableModel()));
			tabePane.addTab("Repair", controller.getMainView().getDisplay());
			final JPanel p1 = new LogPanel(controller, controller.getIPN().getDqModel().getDetectionModel()
					.getMetricsTableModel());
			//			final JPanel p2 = new JPanel();
			//			controller.setLogPanel(p1);
			//			controller.setDgPanel(p2);
			panel.add(p1);
			panel.add(dp.getDgPanel());
			mainGraphPanel.add(tabePane, BorderLayout.CENTER);
			//			controller.getIPN().getDqModel().getRepairModel().setActCombo(activityCombo);
			tabePane.addChangeListener(new ChangeListener() {

				public void stateChanged(ChangeEvent e) {
					if (tabePane.getSelectedIndex() == 0) {
						DetectionPanel dp = new DetectionPanel(controller);
						//						controller.setDetectPane(dp);

						tabePane.setComponentAt(0, dp);
						//						controller.setDetectPane(new DetectionPanel(controller));
						//						controller.getDetectPane().repaint();
						//						controller.getDetectPane().revalidate();
						panel_3.removeAll();
						panel_3.setPreferredSize(new Dimension(10, 500));
						panel.removeAll();
						panel.add(p1);
						panel.add(dp.getDgPanel());
						revalitdatePanels();

					} else {
						panel_3.add(new JLabel("Repair Configuration"));
						//						addSettingsDialogue();

						panel_3.setPreferredSize(new Dimension(550, 500));
						panel_3.add(controller.getIPN().getDqModel().getRepairModel().createRepairPanel());
						//						panel_3.add(actLbl);
						//						panel_3.add(activityCombo);
						//						controller.getIPN().getDqModel().getRepairModel().setActCombo(activityCombo);
						JButton applyChanges = SlickerFactory.instance().createButton("Apply Changes");
						panel_3.add(applyChanges);

						applyChanges.addActionListener(new ActionListener() {

							public void actionPerformed(ActionEvent e) {
								//apply changes here - make repairs
								CalculateAlignments.doRepair();
							}
						});
						
						JButton applyChanges2 = SlickerFactory.instance().createButton("Apply Changes on complete log");
						applyChanges2.addActionListener(new ActionListener() {
							
							public void actionPerformed(ActionEvent e) {
								//apply changes here - make repairs
								CalculateAlignments.doRepair();
								CalculateAlignments.renameDuplicates();
								
								
							}
						});
						panel_3.add(applyChanges2);
						
						
						panel.removeAll();
						panel.add(controller.getIPN().getDqModel().getRepairModel().getRepairStatsPanel());
						revalitdatePanels();
						//also update statsPanel with repair related information
					}

				}

			});

		} else
			mainGraphPanel.add(controller.getMainView().getDisplay(), BorderLayout.CENTER);

		//PMD add combo here ends

		//PMD IPN ends

		//classification inputs ends
		//		charInfoPanel.add(new JLabel("Graph Based On:"));
		//		charInfoPanel.add(processView);
		//		charInfoPanel.add(actStartView);
		//		charInfoPanel.add(new JLabel("Time View:"));
		//		charInfoPanel.add(timeView);
		//		charInfoPanel.add(new JLabel("Chart Type:"));
		//		charInfoPanel.add(chartView);
		//		charInfoPanel.add(new JLabel("Data to Consider:"));
		//		charInfoPanel.add(graphData);
		//		charInfoPanel.add(execute);
		//		charInfoPanel.setLayout(new BoxLayout(charInfoPanel, BoxLayout.Y_AXIS));
		//		panel_3.add(charInfoPanel, BorderLayout.CENTER);
		//		panel_3.add(recompute);

		//classification starts
		//		classifyPanel.add(lblNumAtt);
		//		classifyPanel.add(numAtt);
		//		classifyPanel.add(evaluator);
		//		classifyPanel.add(classify);
		//		classifyPanel.setLayout(new BoxLayout(classifyPanel, BoxLayout.Y_AXIS));
		//		panel_3.add(classifyPanel, BorderLayout.CENTER);
		//		panel_3.add(fitnessSlider);
		//classification ends

		//		controller.getProcessAnalytics().setPanel(null);//update Panel

		//PMD end adding combo boxes for config

		miniMapPanel = new JPanel(new BorderLayout());
		miniMapPanel.setPreferredSize(new Dimension(10, 150));
		miniMapPanel.add(controller.getMiniMap().getDisplay(), BorderLayout.CENTER);
		optionsPanel.add(miniMapPanel, BorderLayout.SOUTH);

		JPanel panel_1 = new JPanel();
		getContentPane().add(panel_1, BorderLayout.NORTH);
		panel_1.setLayout(new BorderLayout(0, 0));

		// Creates and adds the toolbar
		toolBar = new ToolBarGUI(this, controller.getMainView().getDisplay(), soundnessInformation);
		panel_1.add(toolBar, BorderLayout.WEST);

		JPanel panel_2 = new JPanel();
		FlowLayout flowLayout = (FlowLayout) panel_2.getLayout();
		flowLayout.setVgap(10);
		panel_1.add(panel_2, BorderLayout.EAST);

		searchField = new JTextField();
		SearchFieldListener sfl = new SearchFieldListener();
		searchField.addActionListener(sfl);
		searchField.getDocument().addDocumentListener(sfl);
		searchField.addKeyListener(sfl);

		lblSearch = new JLabel("Search:");
		panel_2.add(lblSearch);
		panel_2.add(searchField);
		searchField.setMargin(new Insets(0, 2, 2, 2));
		searchField.setColumns(30);
		searchField.setText("Ctrl+F to search");
		searchField.setEnabled(false);

		loadEvents();

		// Prepares the window.
		pack();
	}

	private void loadEvents() {
		graphPanel.addComponentListener(new ComponentAdapter() {
			private boolean firstTime = true;

			@Override
			public void componentResized(ComponentEvent e) {
				if (firstTime) {
					firstTime = !firstTime;
				} else {
					resizeHandler();
				}
			}
		});

		graphPanel.registerKeyboardAction(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				toggleSearchBox();
			}
		}, "show search field", KeyStroke.getKeyStroke("ctrl F"), JComponent.WHEN_IN_FOCUSED_WINDOW);

		graphPanel.registerKeyboardAction(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				//                toolBar.undo();
			}
		}, "undo", KeyStroke.getKeyStroke("ctrl Z"), JComponent.WHEN_IN_FOCUSED_WINDOW);

		graphPanel.registerKeyboardAction(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				//                toolBar.redo();
			}
		}, "redo", KeyStroke.getKeyStroke("ctrl Y"), JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	private void resizeHandler() {
		controller.fitViews();
	}

	private void toggleSearchBox() {
		searchField.setEnabled(!searchField.isEnabled());
		if (searchField.isEnabled()) {
			searchField.setText(null);
			searchField.requestFocusInWindow();
		} else {
			searchField.setText("Ctrl+F to search");
		}
	}

	public class SearchFieldListener implements ActionListener, DocumentListener, KeyListener {

		@Override
		public void removeUpdate(DocumentEvent e) {
			if (e.getDocument().getLength() == 0) {
				controller.searchFor(PNConstants.EMPTY_LABEL);
			}
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			controller.searchFor(((JTextField) e.getSource()).getText());
		}

		@Override
		public void keyReleased(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
				toggleSearchBox();
			}
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			// no-op

		}

		@Override
		public void insertUpdate(DocumentEvent e) {
			// no-op
		}

		@Override
		public void keyPressed(KeyEvent e) {
			// no-op
		}

		@Override
		public void keyTyped(KeyEvent e) {
			// no-op
		}
	}

	public void layoutGraph() {
		controller.executeLayout(true);
	}

	//DQ data quality starts
	private void revalitdatePanels() {
		panel.repaint();
		panel.revalidate();
		panel_3.repaint();
		panel_3.revalidate();
	}

	//DQ data quality ends
}
