package org.processmining.humbertogarciacaballero.models.gui.controllers;

import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import javax.swing.SwingUtilities;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;

import prefuse.Display;
import prefuse.controls.DragControl;
import prefuse.visual.VisualItem;

public class CustomDragControl extends DragControl {

    private PNetVisualizerController controller;
    private Point2D from;
    private Point2D to;
    /**
     * Creates a new drag control that invokes an action upon drag events.
     * @param action the action to run when drag events occur.
     */
    public CustomDragControl(String action) {
    	super(action);
    	controller = PNetVisualizerController.getInstance();
    	from = new Point2D.Double();
    	to = new Point2D.Double();
    }
    
    /**
     * @see prefuse.controls.Control#itemPressed(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemPressed(VisualItem item, MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e)) return;
        from.setLocation(item.getX(), item.getY());
        super.itemPressed(item, e);
    }
    
    public void itemReleased(VisualItem item, MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e)) return;
        if ( dragged ) {
            to.setLocation(item.getX(), item.getY());
            controller.dragNode(item, from, to);
        }
        super.itemReleased(item, e);
    }
    
    /**
     * @see prefuse.controls.Control#itemDragged(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemDragged(VisualItem item, MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e)) return;
        dragged = true;
        Display d = (Display)e.getComponent();
        d.getAbsoluteCoordinate(e.getPoint(), temp);
        double dx = temp.getX()-down.getX();
        double dy = temp.getY()-down.getY();
        double x = item.getX();
        double y = item.getY();

        item.setStartX(x);  item.setStartY(y);
        item.setX(x+dx);    item.setY(y+dy);
        item.setEndX(x+dx); item.setEndY(y+dy);

        AffineTransform transform = new AffineTransform();
        transform.translate(dx, dy);
        
        if ( repaint )
            item.getVisualization().repaint();
        
        down.setLocation(temp);
        if ( action != null )
            d.getVisualization().run(action);
    }
}
