package org.processmining.humbertogarciacaballero.models.gui.controllers;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.processmining.discoverycontest.layout.LayoutAlgorithmReductionRules;
import org.processmining.discoverycontest.layout.PetrinetBoxStructureLayouter;
import org.processmining.framework.util.Pair;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.synthesis.VizToPnetMapper;
import org.processmining.prabhakardixit.fcnets.vizutils.InteractiveUtils;

import prefuse.Visualization;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Tuple;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class NodeHighlightControlPMDUtils {

	public static void reduceAbstraction(NodeHighlightControlPMD nhc, int placeId, int tranId, boolean calcSynth,
			boolean runLayout) {
		synchronized (nhc.controller.getData()) {

			Graph g = nhc.controller.getData();
			System.out.println("xxxx Node count : " + g.getNodeCount() + "xxx Edges count : " + g.getEdgeCount());
			Iterator iter = g.getNodes().tuples();
			Node tranToRemove = null;
			Node placeToRemove = null;
			while (iter.hasNext()) {
				Tuple tuple = (Tuple) iter.next();
				Node n = g.getNode(tuple.getRow());
				if (n.get("type") == "TRANSITION" && tranId == (n.getInt("ID")))
					tranToRemove = n;
				else if (n.get("type") == "PLACE" && placeId == n.getInt("ID"))
					placeToRemove = n;
			}

			Set<Node> newTargets = new HashSet<Node>();
			Set<Node> newSources = new HashSet<Node>();
			iter = g.getEdges().tuples();
			while (iter.hasNext()) {
				Tuple tuple = (Tuple) iter.next();
				Edge e = g.getEdge(tuple.getRow());
				if (placeToRemove.equals(e.getTargetNode()))
					newSources.add(e.getSourceNode());
				if (tranToRemove.equals(e.getSourceNode()))
					newTargets.add(e.getTargetNode());
			}

			for (Node source : newSources)
				for (Node target : newTargets)
					g.addEdge(source, target);

			g.removeNode(tranToRemove);
			g.removeNode(placeToRemove);

			if (calcSynth) {
				//TODO calculate SS
				Place p = nhc.controller.getIPN().getPlaceBasedOnID(placeId);
				Transition t = nhc.controller.getIPN().getTranBasedOnID(tranId);
				Set<Transition> tranSet = new HashSet<Transition>();
				Set<Place> placeSet = new HashSet<Place>();
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eIn : nhc.controller.getIPN()
						.getselectedPetriNetMap().getPetriNet().getInEdges(p)) {
					tranSet.add((Transition) eIn.getSource());
				}
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOut : nhc.controller.getIPN()
						.getselectedPetriNetMap().getPetriNet().getOutEdges(t)) {
					placeSet.add((Place) eOut.getTarget());
				}
				for (Transition t2 : tranSet) {
					for (Place p2 : placeSet) {
						nhc.controller.getIPN().getselectedPetriNetMap().getPetriNet().addArc(t2, p2);
					}
				}
				nhc.controller.getIPN().getselectedPetriNetMap().getPetriNet().removeNode(t);
				nhc.controller.getIPN().getselectedPetriNetMap().getPetriNet().removeNode(p);
				nhc.controller.getIPN().setImnIndex(
						nhc.controller.getIPN().getPrevImnIndex(nhc.controller.getIPN().getImnIndex()));
				clearAllHighlights(nhc);
				InteractiveUtils.updateVizParameters(g, nhc.controller);
			}
			if (runLayout)
				layouter(nhc.controller.getVisualization());
			else
				cancelLayouter(nhc.controller.getVisualization());

			clearAllHighlights(nhc);
		}
		//		nhc.controller.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
		//		nhc.controller.getVisualization().run("highlights");
		//		nhc.controller.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
		//		nhc.controller.getVisualization().run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
		//		nhc.controller.getVisualization().run("repaint");
		//		nhc.controller.getVisualization().run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);

	}

	public static boolean removeNodesNotInIPN(NodeHighlightControlPMD nhc) {
		//		Iterator iter = nhc.controller.getData().getNodes().tuples();
		//		Iterator iter = nhc.controller.getVisualization().getGroup("graph.nodes").tuples();

		boolean nodesPresent = false;
		//		NodeItem item = null;

		Graph g = nhc.controller.getData();
		System.out.println("xxxx Node count : " + g.getNodeCount() + "xxx Edges count : " + g.getEdgeCount());
		Iterator iter = g.getNodes().tuples();

		Set<Integer> placesInIPN = new HashSet<Integer>();
		Set<Integer> transInIPN = new HashSet<Integer>();
		for (Place p : nhc.controller.getIPN().getAllPlaceId().keySet())
			placesInIPN.add(nhc.controller.getIPN().getAllPlaceId().get(p));
		for (Transition t : nhc.controller.getIPN().getAllTranId().keySet())
			transInIPN.add(nhc.controller.getIPN().getAllTranId().get(t));

		while (iter.hasNext()) {
			Tuple tuple = (Tuple) iter.next();
			Node n = g.getNode(tuple.getRow());
			if (n.get("type") == "TRANSITION" && !transInIPN.contains(n.getInt("ID")))
				nodesPresent = true;
			else if (n.get("type") == "PLACE" && !placesInIPN.contains(n.getInt("ID")))
				nodesPresent = true;
		}
		return nodesPresent;
	}

	public static void renameActivity(VisualItem item, NodeHighlightControlPMD nhc, String activity, boolean calcSynth) {
		synchronized (nhc.controller.getData()) {
			Petrinet pn = nhc.controller.getIPN().getselectedPetriNetMap().getPetriNet();
			nhc.rename = false;
			Node item2 = (Node) item.getSourceTuple();
			boolean isSilent = false;
			if (activity == "")
				isSilent = true;
			if (calcSynth)
				VizToPnetMapper.changeActivityName((Integer) item.get("ID"), nhc.controller.getIPN(), isSilent);
			item2.set("label", activity);
			item2.set(PNConstants.NODE_TAU_TRANSITION_KEY, isSilent);
			if (calcSynth) {
				Transition updateTran = nhc.controller.getIPN().getTranBasedOnID((Integer) item.get("ID"));
				updateTran.setInvisible(isSilent);
				updateTran.getAttributeMap().put(AttributeMap.LABEL, activity);
				InteractiveUtils.updateVizParameters(((NodeItem) item).getGraph(), nhc.controller); //update viz
			}

			clearAllHighlights(nhc);

		}
	}

	public static void addLinearlyDependentNode(VisualItem item, NodeHighlightControlPMD nhc, int newNodeId,
			String activity, boolean calcSynth, boolean runLayout) {
		synchronized (nhc.controller.getData()) {
			PNetVisualizerController controller = nhc.controller;
			nhc.clicked = false;
			Node nTarget = (Node) item.getSourceTuple();
			Set<Node> selectedInputTuples = new HashSet<Node>();
			Set<Node> selectedOutputTuples = new HashSet<Node>();
			Graph g = nTarget.getGraph();
			System.out.println("xxxx Node count : " + g.getNodeCount() + "xxx Edges count : " + g.getEdgeCount());
			Iterator iter = g.getNodes().tuples();
			Node n = g.addNode();

			while (iter.hasNext()) {
				Tuple tuple = (Tuple) iter.next();
				Node node = g.getNode(tuple.getRow());
				if (node.get("type") != nTarget.get("type"))
					continue;
				if (nhc.getSelectedInput().contains(((int) (node).get("ID"))))
					selectedInputTuples.add(node);
				if (nhc.getSelectedOutput().contains(((int) (node).get("ID"))))
					selectedOutputTuples.add(node);
			}
			Edge e1 = null, e2 = null;
			for (Node node : selectedInputTuples) {
				e1 = g.addEdge(node, n);
				if (!calcSynth)
					e1.set("temp", true);
			}
			for (Node node : selectedOutputTuples) {
				e2 = g.addEdge(n, node);
				if (!calcSynth)
					e2.set("temp", true);
			}

			n.set("label", activity);
			if (nTarget.get("type") == "TRANSITION") {
				//add parallel place
				int newNodeID = newNodeId == -1 ? n.hashCode() : newNodeId;// controller.getIPN().getAllPlaceId().size();
				n.set("ID", newNodeID);
				n.set("type", "PLACE");
				if (calcSynth)
					VizToPnetMapper.addParallelPlace(nhc.getSelectedInput(), nhc.getSelectedOutput(), activity,
							controller.getIPN(), newNodeID);
			} else {
				//add choice trans
				int newNodeID = newNodeId == -1 ? n.hashCode() : newNodeId;//controller.getIPN().getAllTranId().size();
				n.set("ID", newNodeID);
				n.set("type", "TRANSITION");
				n.set(PNConstants.NODE_TAU_TRANSITION_KEY, activity == "");
				if (calcSynth)
					VizToPnetMapper.addChoiceTransition(nhc.getSelectedInput(), nhc.getSelectedOutput(), activity,
							controller.getIPN(), newNodeID, activity == "");

			}

			if (calcSynth)
				InteractiveUtils.updateVizParameters(g, controller);
			else {
				n.set("temp", true);
			}
			controller.executeLayout(true);
			if (runLayout)
				layouter(nhc.controller.getVisualization());
			else
				cancelLayouter(controller.getVisualization());
			//		item.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			//		item.getVisualization().run("highlights");
			//		item.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			//		item.getVisualization().run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
			////		item.getVisualization().run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
			//		item.getVisualization().run("repaint");
		}
	}

	private static void clearAllHighlights(NodeHighlightControlPMD nhc) {

		//		Robot robot = new Robot();
		//		robot.keyPress(nhc.esc);
		if (nhc.selectedEdgeSet.getTupleCount() > 0) {
			Iterator<VisualItem> iter = nhc.selectedEdgeSet.tuples();
			while (iter.hasNext()) {
				iter.next().setHighlighted(false);
			}
			nhc.selectedEdgeSet.clear();
			nhc.selectedEdges.removeAll(nhc.selectedEdges);
		}
		if (nhc.prevSelection != null) {
			nhc.clicked = false;
			nhc.setNeighborHighlight((NodeItem) nhc.prevSelection, false);
			nhc.prevSelection = null;
		}

	}

	public static void addTransAndPlace(NodeHighlightControlPMD nhc, int placeId, int tranId, String activity,
			boolean calcSynth, boolean runLayout) {
		//		synchronized (nhc.controller.getData()) {
		if (!nhc.selectedEdges.isEmpty()) {
			nhc.ctrlPressed = false;
			EdgeItem ed = null;
			Node nSource = null;
			Node nTarget;
			//				VisualItem item = null;
			//				for (VisualItem vi : nhc.selectedEdges) {
			//					item = vi;
			//					nSource = (Node) ((EdgeItem) vi).getSourceItem().getSourceTuple();
			//				}
			Graph g = nhc.controller.getData();
			//			System.out.println("xxxx Node count : " + g.getNodeCount() + "xxx Edges count : " + g.getEdgeCount());
			//			Graph prevGraph = InteractiveUtils.duplicateGraph(g); //create prev graph
			//			controller.getIPN().updateInteractiveMatNet(prevGraph); //create new IMN n update prev graph

			Node n2 = g.addNode();
			Node n = g.addNode();
			int newPlaceID = placeId == -1 ? n.hashCode() : placeId; // controller.getIPN().getAllPlaceId().size();
			int newTranID = tranId == -1 ? n2.hashCode() : tranId; //controller.getIPN().getAllTranId().size();

			Set<Integer> sourceNodes = new HashSet<Integer>();
			Set<Integer> targetNodes = new HashSet<Integer>();
			boolean placeToTran = true;
			//			Set<Node> addedSources = new HashSet<Node>(); //use sets to avoid adding multiple edges
			//			Set<Node> addedTargets = new HashSet<Node>(); 
			for (VisualItem vi : nhc.selectedEdges) {
				nSource = (Node) ((EdgeItem) vi).getSourceItem().getSourceTuple();
				nTarget = (Node) ((EdgeItem) vi).getTargetItem().getSourceTuple();
				g.removeEdge((Edge) vi.getSourceTuple());
				if (sourceNodes.add((Integer) nSource.get("ID"))) //avoid adding multiple edges
					g.addEdge(nSource, n);
				if (targetNodes.add((Integer) nTarget.get("ID"))) //avoid adding multiple edges
					g.addEdge(n2, nTarget);
			}
			Edge e = g.addEdge(n, n2);

			if (!calcSynth) {
				e.set("temp", true);
				n.set("temp", true);
				n2.set("temp", true);
			}
			if (nSource.get("type") == nhc.TRANSITION) {
				n2.set(PNConstants.NODE_TAU_TRANSITION_KEY, activity == "");
				InteractiveUtils.updateBasicNodeInfo(n, nhc.PLACE, newPlaceID, activity);
				InteractiveUtils.updateBasicNodeInfo(n2, nhc.TRANSITION, newTranID, activity);
				placeToTran = false;
			}
			//			else {
			//				if (calcSynth) {
			//					InteractiveUtils.updateBasicNodeInfo(n, nhc.TRANSITION, newTranID, nhc.controller);
			//					InteractiveUtils.updateBasicNodeInfo(n2, nhc.PLACE, newPlaceID, nhc.controller);
			//				}
			//			}
			if (calcSynth) {
				VizToPnetMapper.addNewTransition(sourceNodes, targetNodes, activity, activity, nhc.controller.getIPN(),
						placeToTran, newPlaceID, newTranID, activity == "");
				InteractiveUtils.updateVizParameters(g, nhc.controller);
			}
			//			if (getActivity() != null) {
			if (runLayout)
				layouter(nhc.controller.getVisualization());
			else
				cancelLayouter(nhc.controller.getVisualization());
			//			item.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			//			item.getVisualization().run("highlights");
			//			item.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			//			item.getVisualization().run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
			////			item.getVisualization().run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
			//			item.getVisualization().run("repaint");
			//			}

		}
		//		}
	}

	public static void reduceLinearlyDependentNode(PNetVisualizerController controller, int nodeToRemove,
			boolean calcSynth, boolean removePlace, boolean runLayout) {
		synchronized (controller.getData()) {
			Iterator iter = controller.getVisualization().getVisualGroup("graph.nodes").tuples();
			NodeItem item = null;
			while (iter.hasNext()) {
				VisualItem n = (VisualItem) iter.next();
				if (n instanceof NodeItem && n.get("type") == "TRANSITION") {
					item = (NodeItem) n;
				}
			}
			Node item2 = (Node) item.getSourceTuple();
			Graph g = item2.getGraph();
			System.out.println("xxxx Node count : " + g.getNodeCount() + "xxx Edges count : " + g.getEdgeCount());
			Iterator iter2 = g.getNodes().tuples();
			Node tranToRemove = null;
			Node placeToRemove = null;
			while (iter2.hasNext()) {
				Tuple tuple = (Tuple) iter2.next();
				Node n = g.getNode(tuple.getRow());
				System.out.println(n.get("ID"));
				if (n.get("type") == "TRANSITION" && nodeToRemove == (n.getInt("ID")) && !removePlace)
					tranToRemove = n;
				else if (n.get("type") == "PLACE" && nodeToRemove == n.getInt("ID") && removePlace)
					placeToRemove = n;
			}

			Set<Node> newTargets = new HashSet<Node>();
			Set<Node> newSources = new HashSet<Node>();
			iter = g.getEdges().tuples();
			while (iter.hasNext()) {
				Tuple tuple = (Tuple) iter.next();
				Edge e = g.getEdge(tuple.getRow());
				if (removePlace && placeToRemove.equals(e.getTargetNode()))
					newSources.add(e.getSourceNode());
				if (!removePlace && tranToRemove.equals(e.getSourceNode()))
					newTargets.add(e.getTargetNode());
			}

			for (Node source : newSources)
				for (Node target : newTargets)
					g.addEdge(source, target);

			if (!removePlace)
				g.removeNode(tranToRemove);
			else
				g.removeNode(placeToRemove);

			iter = controller.getVisualization().getVisualGroup("graph.edges").tuples();
			while (iter.hasNext()) {
				Edge e = (Edge) iter.next();
				if (e.getSourceNode() == null || e.getTargetNode() == null)
					g.removeTuple(e);
			}

			if (calcSynth) {
				//TODO calculate SS
				InteractivePetriNet ipn = controller.getIPN();
				if (removePlace) {
					ipn.getselectedPetriNetMap().getPetriNet().removeNode(ipn.getPlaceBasedOnID(nodeToRemove));
				} else {
					ipn.getselectedPetriNetMap().getPetriNet().removeNode(ipn.getTranBasedOnID(nodeToRemove));
				}
				controller.getIPN().setImnIndex(controller.getIPN().getPrevImnIndex(controller.getIPN().getImnIndex()));
				InteractiveUtils.updateVizParameters(g, controller);
			}
			if (runLayout)
				layouter(controller.getVisualization());
			else
				cancelLayouter(controller.getVisualization());
			//		item.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			//		item.getVisualization().run("highlights");
			//		item.getVisualization().run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			//		item.getVisualization().run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
			////		item.getVisualization().run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
			//		item.getVisualization().run("repaint");
			//			clearAllHighlights(controller.get);
		}
	}

	public static void cancelLayouter(Visualization visualization) {
		visualization.cancel(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
		visualization.cancel("highlights");
		visualization.cancel(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
		visualization.cancel(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
		visualization.cancel("repaint");
	}

	public static void layouter(Visualization visualization) {
		//first get the layout information
		synchronized (visualization) {

			LayoutAlgorithmReductionRules lar = PetrinetBoxStructureLayouter.getLayout(visualization);

			Iterator iter = visualization.getGroup("graph.nodes").tuples();
			while (iter.hasNext()) {
				VisualItem n = (VisualItem) iter.next();
				NodeItem item = (NodeItem) n;
				if (n instanceof NodeItem && n.get("type") == "TRANSITION") {
					for (Transition t : lar.getWs().net.getNet().getTransitions())
						if (Integer.valueOf(t.getLabel()).equals(n.get("ID"))) {
							Pair<Double, Double> xy = lar.getXY(t);
							n.set("x", xy.getFirst());
							n.set("y", xy.getSecond());
						}
				} else
					for (Place p : lar.getWs().net.getNet().getPlaces())
						if (Integer.valueOf(p.getLabel()).equals(n.get("ID"))) {
							Pair<Double, Double> xy = lar.getXY(p);
							n.set("x", xy.getFirst());
							n.set("y", xy.getSecond());
						}
			}

			//update bends information
			iter = visualization.getGroup("graph.edges").tuples();
			while (iter.hasNext()) {
				EdgeItem e = (EdgeItem) iter.next();
				PetrinetNode source = null;
				PetrinetNode target = null;
				if (e.getSourceNode().get("type") == "TRANSITION") {
					for (Transition t2 : lar.getWs().net.getNet().getTransitions())
						if (Integer.valueOf(t2.getLabel()).equals(e.getSourceNode().get("ID")))
							source = t2;
					for (Place p2 : lar.getWs().net.getNet().getPlaces())
						if (Integer.valueOf(p2.getLabel()).equals(e.getTargetNode().get("ID")))
							target = p2;
				} else {
					for (Transition t2 : lar.getWs().net.getNet().getTransitions())
						if (Integer.valueOf(t2.getLabel()).equals(e.getTargetNode().get("ID")))
							target = t2;
					for (Place p2 : lar.getWs().net.getNet().getPlaces())
						if (Integer.valueOf(p2.getLabel()).equals(e.getSourceNode().get("ID")))
							source = p2;
				}
				//			e.set(PNConstants.EDGE_BENDS_KEY, lar.getXYEdges(source, target));
			}

			visualization.run(PNConstants.VIS_LAYOUT_ACTIONS_NAME);
			visualization.run("highlights");
			visualization.run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
			visualization.run(PNConstants.VIS_SHAPE_ASSIGNMENT_ACTIONS_NAME);
			visualization.run("repaint");
		}
	}
}
