package org.processmining.humbertogarciacaballero.models.gui.controllers;

import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Queue;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.model.InfoExtractor;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.PlaceInfoExtractor;
import org.processmining.humbertogarciacaballero.models.model.TransitionInfoExtractor;

import prefuse.Visualization;
import prefuse.controls.ControlAdapter;
import prefuse.data.Node;
import prefuse.data.tuple.TupleSet;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpecs;
import com.jgoodies.forms.layout.RowSpec;

public class PathControl extends ControlAdapter implements Observer {

    // Tables of elements used to
    private TupleSet incomingHighlight;
    private TupleSet outgoingHighlight;

    // Controller
    private PNetVisualizerController controller;

    // Information extractors
    private InfoExtractor placeExtractor;
    private InfoExtractor transitionExtractor;

    // Current higlighted node
    private NodeItem highlightedNode;

    // Path mode (incoming, outgoing, both)
    private PathMode pathMode;

    public static enum PathMode {
        INCOMING, OUTGOING, BOTH;
    }

    public PathControl(TupleSet incomingHighlihgt, TupleSet outgoingHighlight) {
        this.incomingHighlight = incomingHighlihgt;
        this.outgoingHighlight = outgoingHighlight;
        controller = PNetVisualizerController.getInstance();
        controller.addObserver(this);
        pathMode = controller.getPathMode();
        placeExtractor = new PlaceInfoExtractor();
        transitionExtractor = new TransitionInfoExtractor();
    }

    /**
     * @see prefuse.controls.Control#itemClicked(prefuse.visual.VisualItem,
     *      java.awt.event.MouseEvent)
     */
    public void itemClicked(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem && e.getClickCount() == 2) {
            NodeItem node = (NodeItem) item;
            handleDoubleClick(node);
        }
    }

    private void handleDoubleClick(NodeItem node) {
        if (highlightedNode == node) {
            node.setHighlighted(false);
            clearInformationPanel();
            hideAllPaths(node);
            highlightedNode = null;
            controller.setSliderEnabled(false);
        } else {
            // Highlighting
            if (highlightedNode != null) {
                highlightedNode.setHighlighted(false);
                hideAllPaths(highlightedNode);
            }

            node.setHighlighted(true);
            highlightedNode = node;

            // Paths
            showPathInvolvingNode(node, controller.getDepth());

            int maxDepth = calculateMaxDepth(node, pathMode);

            // Information
            addInformationToPanel(node, maxDepth);

            controller.setMaxDepth(maxDepth);
            controller.setSliderEnabled(true);
        }

        runAnimations(node.getVisualization());
    }

    private void runAnimations(Visualization vis) {
        // vis.cancel(PNConstants.VIS_HIGHLIGHT_ACTIONS_NAME);
        vis.cancel(PNConstants.VIS_ANIMATE_LAYOUT_ACTIONS_NAME);
        vis.run(PNConstants.VIS_FILL_NODES_ACTIONS_NAME);
        vis.run(PNConstants.VIS_FILL_NODES_DEPTH_ACTIONS_NAME);
        vis.run(PNConstants.VIS_LAYOUT_NO_PAINT_ACTIONS_NAME);
        vis.run(PNConstants.VIS_ANIMATE_LAYOUT_ACTIONS_NAME);
        // vis.run(PNConstants.VIS_HIGHLIGHT_ACTIONS_NAME);
    }

    private void hideAllPaths(NodeItem item) {
        item.getVisualization().setValue(GraphVisualization.GRAPH_EDGES_GROUP, null, PNConstants.EDGE_SIMPLIFICABLE,
                true);
        item.getVisualization().setValue(GraphVisualization.GRAPH_GROUP, null, VisualItem.HIGHLIGHT, false);
        incomingHighlight.clear();
        outgoingHighlight.clear();
    }

    @SuppressWarnings("unchecked")
    private void showPathInvolvingNode(Node node, int depth) {
        if (depth > 0) {
            Iterator<EdgeItem> iter;
            switch (pathMode) {
            case INCOMING:
                iter = node.inEdges();
                break;
            case OUTGOING:
                iter = node.outEdges();
                break;
            default:
                iter = node.edges();
                break;
            }

            while (iter.hasNext()) {
                EdgeItem edge = iter.next();
                edge.setBoolean(PNConstants.EDGE_SIMPLIFICABLE, false);
                NodeItem adjacent = (NodeItem) edge.getAdjacentNode(node);
                adjacent.setHighlighted(true);

                if (edge.getSourceNode().equals(adjacent)) {
                    incomingHighlight.addTuple(adjacent);
                } else {
                    outgoingHighlight.addTuple(adjacent);
                }
                edge.setHighlighted(true);

                showPathInvolvingNode(edge.getAdjacentNode(node), depth - 1);
            }
        }
    }

    @SuppressWarnings("unchecked")
    private int calculateMaxDepth(NodeItem node, PathMode pathMode) {
        int maxDepth = 0;
//        node.getVisualization().setValue(GraphVisualization.GRAPH_NODES_GROUP, null, PNConstants.NODE_DEPTH, -1);
        Queue<Node> queue = new LinkedList<Node>();
        queue.add(node);
//        node.setInt(PNConstants.NODE_DEPTH, 0);

        while (!queue.isEmpty()) {
            Node current = queue.poll();
            Iterator<Node> iter;

            switch (pathMode) {
            case INCOMING:
                iter = current.inNeighbors();
                break;
            case OUTGOING:
                iter = current.outNeighbors();
                break;
            default:
                iter = current.neighbors();
                break;
            }

//            while (iter.hasNext()) {
//                Node neighbor = iter.next();
//                if (neighbor.getInt(PNConstants.NODE_DEPTH) == -1) {
//                    int depth = current.getInt(PNConstants.NODE_DEPTH) + 1;
//                    neighbor.setInt(PNConstants.NODE_DEPTH, depth);
//                    maxDepth = Math.max(maxDepth, depth);
//                    queue.add(neighbor);
//                }
//            }
        }

        return maxDepth;
    }

    public void clearInformationPanel() {
    	JPanel panel = controller.getInfoPanel();
        panel.removeAll();
        panel.revalidate();
        panel.repaint();
    }

    public void addInformationToPanel(NodeItem item, int maxDepth) {
        Map<String, Object> holder = new HashMap<>();
//        if (NodeType.TRANSITION == NodeType.valueOf(item.getString(PNConstants.NODE_TYPE_KEY))) {
//            transitionExtractor.extractInfo(item, maxDepth, holder);
//        } else {
//            placeExtractor.extractInfo(item, maxDepth, holder);
//        }

        JPanel panel = controller.getInfoPanel();
        panel.removeAll();
        FormLayout formLayout = new FormLayout(
                new ColumnSpec[] { FormSpecs.RELATED_GAP_COLSPEC, FormSpecs.DEFAULT_COLSPEC,
                        FormSpecs.RELATED_GAP_COLSPEC, FormSpecs.DEFAULT_COLSPEC, FormSpecs.RELATED_GAP_COLSPEC, },
                new RowSpec[] {});
        panel.setLayout(formLayout);

        int row = 2;
        for (String key : holder.keySet()) {
            formLayout.appendRow(FormSpecs.RELATED_GAP_ROWSPEC);
            formLayout.appendRow(FormSpecs.DEFAULT_ROWSPEC);

            JLabel lblNewLabel = new JLabel(key + ":");
            panel.add(lblNewLabel, "2, " + row + ", right, default");

            JLabel lblNewLabel_1 = new JLabel(String.valueOf(holder.get(key)));
            panel.add(lblNewLabel_1, "4, " + row + ", left, default");
            row += 2;
        }
        formLayout.appendRow(FormSpecs.RELATED_GAP_ROWSPEC);
        panel.revalidate();
        panel.repaint();
    }

    @Override
    public void update(Observable observable, Object value) {
        if (observable instanceof PNetVisualizerController && value instanceof Integer) {
            changePathAndHighlights();
        } else if (observable instanceof PNetVisualizerController && value instanceof PathMode) {
            if (pathMode != controller.getPathMode()) {
                pathMode = controller.getPathMode();
                controller.setMaxDepth(calculateMaxDepth(highlightedNode, pathMode));
            }

            changePathAndHighlights();
        }
    }

    private void changePathAndHighlights() {
        if (highlightedNode != null) {
            // Paths
            hideAllPaths(highlightedNode);
            highlightedNode.setHighlighted(true);
            showPathInvolvingNode(highlightedNode, controller.getDepth());
            runAnimations(highlightedNode.getVisualization());
        }
    }
}
