package org.processmining.humbertogarciacaballero.models.gui.decorator;

import java.awt.geom.Point2D;
import java.util.Iterator;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.model.NodeType;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.util.Utils;

import prefuse.action.layout.Layout;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;

public class LongEdgeTextTargetDecoratorLayout extends Layout {

    private PNetVisualizerController controller;
    private Point2D point, pointAux;

    public LongEdgeTextTargetDecoratorLayout(String group) {
        super(group);
        controller = PNetVisualizerController.getInstance();
        point = new Point2D.Double();
        pointAux = new Point2D.Double();
    }

    @SuppressWarnings("unchecked")
    @Override
    public void run(double frac) {
        Iterator<DecoratorItem> iter = getVisualization().items(getGroup());
        while (iter.hasNext()) {
            DecoratorItem decorator = iter.next();
            EdgeItem edgeItem = (EdgeItem) decorator.getDecoratedItem();
            NodeItem sourceItem = edgeItem.getSourceItem(), targetItem = edgeItem.getTargetItem();
            int fromLayer = edgeItem.getSourceItem().getInt(PNConstants.NODE_LAYER_KEY),
                    toLayer = edgeItem.getTargetItem().getInt(PNConstants.NODE_LAYER_KEY);

            if (controller.isEdgeDirectionEnabled()) {
                if (NodeType.valueOf(targetItem.getString(PNConstants.NODE_TYPE_KEY)) == NodeType.PLACE) {
                    point.setLocation(Utils.calculatePortPoint(edgeItem, targetItem, false));
                    double distance = controller.getEdgeLandingOffset();
                    switch (controller.getLayoutOrientation()) {
                    case TOP_TO_BOTTOM:
                    case BOTTOM_TO_TOP:
                        distance += Utils.getHeight(targetItem) / 2;
                        break;
                    default:
                        distance += Utils.getWidth(targetItem) / 2;
                        break;
                    }
                    pointAux.setLocation(Utils.calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer));
                    double fi = Math.atan2(pointAux.getY() - point.getY(), pointAux.getX() - point.getX());
                    
                    pointAux.setLocation(point);
                    double x2 = pointAux.getX() + distance * Math.cos(fi);
                    double y2 = pointAux.getY() + distance * Math.sin(fi);
                    point.setLocation(x2, y2);
                } else {
                    point.setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer));   
                }
            } else {
                point.setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer));
            }
            
//            Point2D point = Utils.calculateOffsetPortPoint(edgeItem, edgeItem.getTargetItem(), false,
//                    fromLayer > toLayer);

            switch (controller.getLayoutOrientation()) {
            case TOP_TO_BOTTOM:
            case BOTTOM_TO_TOP:
                point.setLocation(point.getX(), point.getY() + ((fromLayer <= toLayer ? -1 : 1) * PNConstants.LONG_EDGES_TEXT_OFFSET));
                break;
            default:
                point.setLocation(point.getX() + ((fromLayer <= toLayer ? -1 : 1) * PNConstants.LONG_EDGES_TEXT_OFFSET), point.getY());
                break;
            }

            setX(decorator, null, point.getX());
            setY(decorator, null, point.getY());
        }
    }

}
