package org.processmining.humbertogarciacaballero.models.gui.render;

import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.model.NodeType;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.util.Utils;

import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;

public class LongEdgeDecoratorRenderer extends LabelRenderer {

	private AffineTransform m_transform;
	private static final double HALF_PI = Math.PI / 2;
	private Polygon polygon;
	private Point2D p1, p2;
	private InGroupPredicate predicate;
	private PNetVisualizerController controller;

	public LongEdgeDecoratorRenderer() {
		m_transform = new AffineTransform();
		p1 = new Point2D.Double();
		p2 = new Point2D.Double();
		predicate = new InGroupPredicate(GraphVisualization.LONG_EDGES_TEXT_SOURCE_DECORATOR_GROUP_NAME);
		controller = PNetVisualizerController.getInstance();
	}

	@Override
	public void render(Graphics2D g, VisualItem item) {
		super.render(g, item);

		// Custom behaviour
		EdgeItem edgeItem = (EdgeItem) ((DecoratorItem) item).getDecoratedItem();
		NodeItem sourceItem = edgeItem.getSourceItem(), targetItem = edgeItem.getTargetItem();
		int fromLayer = sourceItem.getInt(PNConstants.NODE_LAYER_KEY),
				toLayer = targetItem.getInt(PNConstants.NODE_LAYER_KEY);
		double width = edgeItem.getSize();

		if (controller.isEdgeDirectionEnabled()) {
			if (predicate.getBoolean(item)) {
				if (NodeType.valueOf(sourceItem.getString(PNConstants.NODE_TYPE_KEY)) == NodeType.PLACE) {
					p1.setLocation(Utils.calculatePortPoint(edgeItem, sourceItem, true));
					double distance = controller.getEdgeLandingOffset();
					switch (controller.getLayoutOrientation()) {
						case TOP_TO_BOTTOM :
						case BOTTOM_TO_TOP :
							distance += Utils.getHeight(sourceItem) / 2;
							break;
						default :
							distance += Utils.getWidth(sourceItem) / 2;
							break;
					}
					p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer));
					double fi = Math.atan2(p2.getY() - p1.getY(), p2.getX() - p1.getX());

					double x2 = p1.getX() + distance * Math.cos(fi);
					double y2 = p1.getY() + distance * Math.sin(fi);
					p2.setLocation(x2, y2);
				} else {
					p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer));
				}
			} else {
				if (NodeType.valueOf(targetItem.getString(PNConstants.NODE_TYPE_KEY)) == NodeType.PLACE) {
					p2.setLocation(Utils.calculatePortPoint(edgeItem, targetItem, false));
					double distance = controller.getEdgeLandingOffset();
					switch (controller.getLayoutOrientation()) {
						case TOP_TO_BOTTOM :
						case BOTTOM_TO_TOP :
							distance += Utils.getHeight(targetItem) / 2;
							break;
						default :
							distance += Utils.getWidth(targetItem) / 2;
							break;
					}
					p1.setLocation(Utils.calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer));
					double fi = Math.atan2(p1.getY() - p2.getY(), p1.getX() - p2.getX());

					p1.setLocation(p2);
					double x2 = p1.getX() + distance * Math.cos(fi);
					double y2 = p1.getY() + distance * Math.sin(fi);
					p2.setLocation(x2, y2);
				} else {
					p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer));
				}
			}
		} else {
			if (predicate.getBoolean(item)) {
				p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer));
			} else {
				p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer));
			}
		}

		m_transform.setToTranslation(p2.getX(), p2.getY());
		// ROTATION
		if (predicate.getBoolean(item)) {
			p1.setLocation(Utils.calculatePortPoint(edgeItem, sourceItem, true));
			p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer));
		} else {
			p1.setLocation(Utils.calculatePortPoint(edgeItem, targetItem, false));
			p2.setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer));
		}
		m_transform.rotate(-HALF_PI + Math.atan2(p2.getY() - p1.getY(), p2.getX() - p1.getX()));
		if (width > 1) {
			double scalar = width / 4;
			m_transform.scale(scalar, scalar);
		}

		g.setPaint(ColorLib.getColor(edgeItem.getFillColor()));
		g.fill(m_transform.createTransformedShape(polygon));
	}

	public void setDimensions(int w, int h) {
		if (polygon == null) {
			polygon = new Polygon();
		} else {
			polygon.reset();
		}
		polygon.addPoint(-w / 2, 0);
		polygon.addPoint(-w / 2, 1);
		polygon.addPoint(w / 2, 1);
		polygon.addPoint(w / 2, 0);
		polygon.addPoint(0, 0);
	}
}
