package org.processmining.humbertogarciacaballero.models.gui.render;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.util.LinkedList;
import java.util.List;

import org.processmining.humbertogarciacaballero.models.model.NodeType;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.util.Utils;

import prefuse.util.GraphicsLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class StraightLineEdgeRenderer extends PolylineEdgeRenderer {

	public StraightLineEdgeRenderer() {
		super();
	}

	@SuppressWarnings("unchecked")
	@Override
	protected Shape getRawShape(VisualItem item) {
		List<Point2D> bends = item.get(PNConstants.EDGE_BENDS_KEY) != null
				? new LinkedList<>((List<Point2D>) item.get(PNConstants.EDGE_BENDS_KEY)) : EMPTY_LIST;
		EdgeItem edgeItem = (EdgeItem) item;
		NodeItem sourceItem = edgeItem.getSourceItem();
		NodeItem targetItem = edgeItem.getTargetItem();
		int fromLayer = sourceItem.getInt(PNConstants.NODE_LAYER_KEY),
				toLayer = targetItem.getInt(PNConstants.NODE_LAYER_KEY);

		m_tmpPoints[0].setLocation(Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer >= toLayer));
		m_tmpPoints[1].setLocation(Utils.calculatePortPoint(edgeItem, targetItem, false));

		m_curWidth = (float) (m_width * getLineWidth(item));

		Point2D start = null, end = null;
		start = m_tmpPoints[0];
		end = m_tmpPoints[1];

		if (NodeType.PLACE == NodeType.valueOf(targetItem.getString(PNConstants.NODE_TYPE_KEY))) {
			int i = intersectLineCircle(start, end,
					new Point2D.Double(targetItem.getBounds().getCenterX(), targetItem.getBounds().getCenterY()),
					targetItem.getBounds().getWidth() / 2, m_isctPoints);
			if (i > 0)
				end = m_isctPoints[0];
		} else {
			int i = GraphicsLib.intersectLineRectangle(start, end, targetItem.getBounds(), m_isctPoints);
			if (i > 0)
				end = m_isctPoints[0];
		}

		AffineTransform at = getArrowTrans(start, end, m_curWidth);
		m_curArrow = at.createTransformedShape(m_arrowHead);

		Point2D lineEnd = m_tmpPoints[1];
		lineEnd.setLocation(0, -m_arrowHeight);
		at.transform(lineEnd, lineEnd);

		path.reset();
		Point2D point;

		if (NodeType.PLACE == NodeType.valueOf(sourceItem.getString(PNConstants.NODE_TYPE_KEY))) {
			if (intersectLineCircle(
					bends.size() > 0 ? bends.get(0)
							: Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer),
					calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer),
					new Point2D.Double(sourceItem.getBounds().getCenterX(), sourceItem.getBounds().getCenterY()),
					sourceItem.getBounds().getWidth() / 2, m_isctPoints) > 0) {
				point = m_isctPoints[0];
			} else {
				point = calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer);
			}
		} else {
			if (GraphicsLib.intersectLineRectangle(
					calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer),
					bends.size() > 0 ? bends.get(0)
							: Utils.calculateOffsetPortPoint(edgeItem, targetItem, false, fromLayer > toLayer),
					sourceItem.getBounds(), m_isctPoints) > 0)
				point = m_isctPoints[0];
			else
				point = calculateOffsetPortPoint(edgeItem, sourceItem, true, fromLayer <= toLayer);
		}

		path.moveTo(point.getX(), point.getY());

		for (int i = 0; i < bends.size(); i++) {
			point = bends.get(i);
			path.lineTo(point.getX(), point.getY());
		}

		point = m_tmpPoints[0];
		path.lineTo(point.getX(), point.getY());

		point = m_tmpPoints[1];
		path.lineTo(point.getX(), point.getY());

		return path;
	}

	protected Point2D calculateOffsetPortPoint(EdgeItem edgeItem, NodeItem nodeItem, boolean source, boolean downward) {
		//        Point2D port = calculatePortPoint(edgeItem, nodeItem, source);
		Point2D port = new Point2D.Double(nodeItem.getX(), nodeItem.getY());
		//        double landingOffset;
		//        switch (context.getLayoutOrientation()) {
		//        case TOP_TO_BOTTOM:
		//            landingOffset = Utils.getHeight(nodeItem) / 2;
		//            return new Point2D.Double(port.getX(), port.getY() + (downward ? landingOffset : -landingOffset));
		//        case BOTTOM_TO_TOP:
		//            landingOffset = Utils.getHeight(nodeItem) / 2;
		//            return new Point2D.Double(port.getX(), port.getY() + (downward ? -landingOffset : landingOffset));
		//        case LEFT_TO_RIGHT:
		//            landingOffset = Utils.getWidth(nodeItem) / 2;
		//            return new Point2D.Double(port.getX() + (downward ? landingOffset : -landingOffset), port.getY());
		//        default:
		//            landingOffset = Utils.getWidth(nodeItem) / 2;
		//            return new Point2D.Double(port.getX() + (downward ? -landingOffset : landingOffset), port.getY());
		//        }
		return port;
	}
}
