package org.processmining.humbertogarciacaballero.models.model.commands;

import java.awt.geom.Point2D;

import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.util.Utils;

import prefuse.action.Action;
import prefuse.activity.Activity;
import prefuse.activity.ActivityAdapter;
import prefuse.visual.VisualItem;

public class DragCommand extends AbstractCommand {

    private GraphVisualization vis;
    private VisualItem item;
    private Point2D from;
    private Point2D to;
    
    private boolean animateInDoIt;
    
    public DragCommand(GraphVisualization vis, VisualItem item, Point2D from, Point2D to) {
        this.vis = vis;
        this.item = item;
        this.from = from;
        this.to = to;
        animateInDoIt = false;
    }
    
    @Override
    public void doIt() {
        Utils.setPosition(item, to);
        if (animateInDoIt)
            animatePosition();
    }

    @Override
    public void undoIt() {
        Utils.setPosition(item, from);
        animateInDoIt = true;
        animatePosition();
    }

    private void animatePosition() {
        final Action action = vis.getVisualization().getAction(PNConstants.VIS_POSITION_ANIMATE_ACTIONS_NAME);
        action.addActivityListener(new ActivityAdapter() {
            public void activityFinished(Activity a) {
                finish();
                action.removeActivityListener(this);
            }
        });
        vis.executePositionAnimate(item);
    }
}
