package org.processmining.humbertogarciacaballero.models.model.layout;

import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinetreductionlayouter.parameters.LayoutParameters;
import org.processmining.acceptingpetrinetreductionlayouter.plugins.LayoutAcceptingPlugin;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionID;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.CycleRemover;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.CycleRemoverException;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer.Emplacer;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer.EmplacerException;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer.EmplacingStrategy;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer.Layerer;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer.LayererExecption;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer.LayeringStrategy;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.sequencer.Sequencer;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.sequencer.SequencerException;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.sequencer.SequencingStrategy;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.simplifier.ColorMapper;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.simplifier.Simplifier;
import org.processmining.humbertogarciacaballero.models.util.GraphUtil;
import org.processmining.humbertogarciacaballero.models.util.Utils;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.DirectedGraph;

import prefuse.action.ActionList;
import prefuse.action.layout.Layout;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Tuple;
import prefuse.data.tuple.TupleSet;
import prefuse.util.PrefuseLib;
import prefuse.visual.expression.VisiblePredicate;

public class BlockStructuredLayouter extends Layout {

	private static final Logger s_logger = Logger.getLogger(ActionList.class.getName());

	private PluginContext context;
	private CycleRemover cycleRemover;
	private Simplifier simplifier;
	private ColorMapper colorMapper;
	private Layerer layerer;
	private LayeringStrategy layeringStrategy;
	private Sequencer sequencer;
	private SequencingStrategy sequencingStragety;
	private Emplacer emplacer;
	private EmplacingStrategy emplacingStrategy;
	private boolean firstTime = true;

	public BlockStructuredLayouter(String group) {
		super(group);

	}

	public void run(double frac) {
		// Prepares variables and schemas
		Graph graph = (Graph) m_vis.getGroup(m_group);
		Graph graphLayout = GraphUtil.copySparseGraph(graph);
		
		// remove invisible elements
		{
			Iterator<Tuple> iter = graphLayout.tuples(VisiblePredicate.FALSE);
			while (iter.hasNext()) {
				graphLayout.removeTuple(iter.next());
			}
		}

		initNodeSchema(graphLayout.getNodes());
		initEdgeSchema(graphLayout.getEdges());

		// Starts layout steps.
		// 1. Removes (possible) cycles
		long instant = System.currentTimeMillis();
		try {
			cycleRemover.run(graphLayout);
		} catch (Exception e) {
			throw new CycleRemoverException("There is a problem with the cycle remover", e);
		}
		long time = System.currentTimeMillis() - instant;
		s_logger.info("Cycle remover took " + time + " millisencods");

		// 2. Sets up layers
		instant = System.currentTimeMillis();
		try {
			layerer.run(graphLayout);
		} catch (Exception e) {
			throw new LayererExecption("There is a problem with the layerer technique", e);
		}
		time = System.currentTimeMillis() - instant;
		s_logger.info("Layerer took " + time + " millisencods");

		// 3. Minimizes crossings
		instant = System.currentTimeMillis();
		try {
			sequencer.run(graphLayout);
		} catch (Exception e) {
			throw new SequencerException("There is a problem with the sequencer technique", e);
		}
		time = System.currentTimeMillis() - instant;
		s_logger.info("Sequencer took " + time + " millisencods");

		// 4. Assigns coordenates to nodes
		instant = System.currentTimeMillis();
		try {
			emplacer.run(graphLayout);
		} catch (Exception e) {
			throw new EmplacerException("There is a problem with the emplacer technique", e);
		}
		time = System.currentTimeMillis() - instant;
		s_logger.info("Emplacer took " + time + " millisencods");

		// 5. Colors edges
		colorMapper.run(graphLayout);

		// 6. Redoes (possible) cycles
		cycleRemover.redoCycles(graphLayout);

		// 7. Sets up the coordinates
		Iterator<Node> iter = graphLayout.nodes();
		//use block structured layour from Eric here
		AcceptingPetriNet pn = null;
		AcceptingPetriNet x = (new LayoutAcceptingPlugin()).run(context, pn, new LayoutParameters());
		GraphLayoutConnection layoutConnection = findConnection(context.getConnectionManager(), x.getNet().getGraph());

		Iterator<Edge> iter2 = graphLayout.edges();
		while (iter2.hasNext()) {
			Edge e = iter2.next();
			EdgeParams params = getEdgeParams(e);
			Node u = e.getSourceNode();
			Node v = e.getTargetNode();

			Node w = graph.getNodeFromKey(graphLayout.getKey(u.getRow()));
			Node z = graph.getNodeFromKey(graphLayout.getKey(v.getRow()));

			Edge e2 = graph.getEdge(w, z);
			if (e2 == null) {
				e2 = graph.getEdge(z, w);
			}

			e2.setBoolean(PNConstants.SIMPLIFIED_KEY, params.simplified);
			e2.set(PNConstants.EDGE_BENDS_KEY, params.bends);
			e2.setInt(PNConstants.EDGE_COLOR_KEY, params.color);
			e2.setInt(PrefuseLib.getStartField(PNConstants.EDGE_COLOR_KEY), params.color);
			e2.setInt(PrefuseLib.getEndField(PNConstants.EDGE_COLOR_KEY), params.color);
			e2.setFloat(PNConstants.EDGE_SOURCE_PORT_OFFSET, params.sourceOffset);
			e2.setFloat(PNConstants.EDGE_TARGET_PORT_OFFSET, params.targetOffset);
			e2.setBoolean(PNConstants.EDGE_LONG_EDGE_KEY, params.longEdge);
		}

		// Clears this graph to free resources
		graphLayout.clear();

		firstTime = false;
	}

	// ------------------------------------------------------------------------
	// Params

	/**
	 * The data field in which the node parameters used by this layout are
	 * stored.
	 */
	public static final String NODE_PARAMS = "_hierarchicalLayoutNodeParams";
	/**
	 * The data field in which the edge parameters used by this layout are
	 * stored.
	 */
	public static final String EDGE_PARAMS = "_hierarchicalLayoutEdgeParams";
	/**
	 * The schema for the edge parameters used by this layout.
	 */
	public static final Schema NODE_PARAMS_SCHEMA = new Schema();

	static {
		NODE_PARAMS_SCHEMA.addColumn(NODE_PARAMS, NodeParams.class, new NodeParams());
	}

	/**
	 * The schema for the edge parameters used by this layout.
	 */
	public static final Schema EDGE_PARAMS_SCHEMA = new Schema();

	static {
		EDGE_PARAMS_SCHEMA.addColumn(EDGE_PARAMS, EdgeParams.class, new EdgeParams());
	}

	@SuppressWarnings("unchecked")
	private void initNodeSchema(TupleSet ts) {
		ts.addColumns(NODE_PARAMS_SCHEMA);
		Iterator<Tuple> iter = ts.tuples();
		while (iter.hasNext()) {
			Tuple t = iter.next();
			((NodeParams) t.get(NODE_PARAMS)).width = Utils.getWidth(t);
			((NodeParams) t.get(NODE_PARAMS)).height = Utils.getHeight(t);
		}
	}

	private void initEdgeSchema(TupleSet ts) {
		ts.addColumns(EDGE_PARAMS_SCHEMA);
		Iterator<Tuple> iter = ts.tuples();
		while (iter.hasNext()) {
			Tuple t = iter.next();
			((EdgeParams) t.get(EDGE_PARAMS)).forceSimplification = t.getBoolean(PNConstants.EDGE_FORCE_SIMPLIFICATION);
		}
	}

	public static NodeParams getNodeParams(Node n) {
		return ((NodeParams) n.get(NODE_PARAMS));
	}

	public static EdgeParams getEdgeParams(Edge e) {
		return ((EdgeParams) e.get(EDGE_PARAMS));
	}

	/**
	 * Wrapper class holding node parameters used for each node in this layout.
	 */
	public static class NodeParams implements Cloneable {
		public int layer = -1;
		public int rank = -1;
		public boolean dummy = false;
		public float x;
		public float y;
		public int width;
		public int height;
		public boolean visited;
		public Map<Node, List<Node>> dummies = new HashMap<>();
		public boolean simplified;
		public boolean topSimplified;
		public boolean bottomSimplified;
		public int colorAccumulator = 0;

		public NodeParams clone() {
			NodeParams p = null;
			try {
				p = (NodeParams) super.clone();
			} catch (CloneNotSupportedException e) {
				// Never happens
				e.printStackTrace();
			}
			p.dummies = new HashMap<>(this.dummies);
			return p;
		}

		public void addDummy(Node target, Node dummy) {
			List<Node> dummiesList;
			if (dummies.containsKey(target)) {
				dummiesList = dummies.get(target);
			} else {
				dummiesList = new LinkedList<>();
				dummies.put(target, dummiesList);
			}

			dummiesList.add(dummy);
		}
	}

	/**
	 * Wrapper class holding edge parameters used for each edge in this layout.
	 */
	public static class EdgeParams implements Cloneable {
		public boolean bidirectional;
		public boolean bidirectionalCycles;
		public boolean reversed;
		public float sourceOffset;
		public float bidirectionalSourceOffset;
		public float targetOffset;
		public float bidirectionalTargetOffset;
		public boolean type1Conflict;
		public boolean simplified;
		public List<Point2D> bends = new LinkedList<>();
		public int color;
		public boolean longEdge;
		public boolean forceSimplification;

		public EdgeParams clone() {
			EdgeParams p = null;
			try {
				p = (EdgeParams) super.clone();
			} catch (CloneNotSupportedException e) {
				// Never happens
				e.printStackTrace();
			}
			p.bends = new LinkedList<>(this.bends);
			return p;
		}
	}

	protected GraphLayoutConnection findConnection(ConnectionManager manager, DirectedGraph<?, ?> graph) {
		Collection<ConnectionID> cids = manager.getConnectionIDs();
		for (ConnectionID id : cids) {
			Connection c;
			try {
				c = manager.getConnection(id);
			} catch (ConnectionCannotBeObtained e) {
				continue;
			}
			if (c != null && !c.isRemoved() && c instanceof GraphLayoutConnection
					&& c.getObjectWithRole(GraphLayoutConnection.GRAPH) == graph) {
				return (GraphLayoutConnection) c;
			}
		}
		return null;
	}

}
