package org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer;

import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.processmining.humbertogarciacaballero.models.model.layout.HierarchicalLayouter;
import org.processmining.humbertogarciacaballero.models.model.layout.HierarchicalLayouter.EdgeParams;
import org.processmining.humbertogarciacaballero.models.model.layout.HierarchicalLayouter.NodeParams;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayerContainer;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LineType;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.HierarchicalLayoutContext;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.LayoutStage;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer.port.PortCalculator;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.lines.BendCalculator;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;

public abstract class Emplacer extends LayoutStage {

    private LineType lineType;

    private BendCalculator bendCalculator;

    private PortCalculator portCalculator;

    public Emplacer(HierarchicalLayoutContext context) {
        super(context);
        portCalculator = new PortCalculator(context);
        setLineType(LineType.POLYLINE);
    }

    @Override
    public int run(Graph graph) {
        Map<Node, Float> xcoords = calculateCoordinates(graph);

        determineCoordinates(graph, xcoords);
        if (getContext().isReorderNodesEnabled()) {
            readdSimplifiedEdges(graph);
        }
        portCalculator.run(graph);
        removeDummies(graph);

        return 0;
    }

    private void readdSimplifiedEdges(Graph graph) {
        Iterator<Object> iter = getContext().simplificationParameters();
        while (iter.hasNext()) {
            Edge edge = graph.addEdge((Node) iter.next(), (Node) iter.next());
            EdgeParams param = ((EdgeParams) iter.next()).clone();
            param.simplified = true;
            edge.set(HierarchicalLayouter.EDGE_PARAMS, param);
            ((NodeParams) edge.getSourceNode().get(HierarchicalLayouter.NODE_PARAMS)).simplified = true;
            ((NodeParams) edge.getTargetNode().get(HierarchicalLayouter.NODE_PARAMS)).simplified = true;
        }
    }

    // private void centrate(Graph graph, Map<Node, Float> xcoords) {
    // // Calculate balanced center
    // LayerContainer layerContainer = getContext().getLayerContainer();
    // float center = Float.MIN_NORMAL;
    // for (int i = 0; i < layerContainer.getSize(); i++) {
    // center = Math.max(center, calculateCenter(layerContainer.getNodes(i),
    // xcoords));
    // }
    //
    // for (int i = 0; i < layerContainer.getSize(); i++) {
    // float layerCenter = calculateCenter(layerContainer.getNodes(i), xcoords);
    // shift(layerContainer.getNodes(i), xcoords, center - layerCenter);
    // }
    // }
    //
    // private void shift(List<Node> layer, Map<Node, Float> xcoords, float
    // amount) {
    // if (amount > 0) {
    // Node previous = layer.get(layer.size() - 1);
    // if (!((NodeParams) previous.get(HierarchicalLayouter.NODE_PARAMS)).dummy)
    // {
    // xcoords.put(previous, xcoords.get(previous) + amount);
    // }
    //
    // for (int i = layer.size() - 2; i >= 0; i--) {
    // Node node = layer.get(i);
    // if (!((NodeParams) node.get(HierarchicalLayouter.NODE_PARAMS)).dummy
    // && xcoords.get(node) + amount < xcoords.get(previous) +
    // getContext().getIntracellSpacing()) {
    // xcoords.put(node, xcoords.get(node) + amount);
    // }
    // previous = node;
    // }
    // } else {
    // for (int i = 0; i < layer.size(); i++) {
    // Node node = layer.get(i);
    // if (!((NodeParams) node.get(HierarchicalLayouter.NODE_PARAMS)).dummy) {
    // xcoords.put(node, xcoords.get(node) + amount);
    // }
    // }
    // }
    // }
    //
    // private float calculateCenter(List<Node> layer, Map<Node, Float> xcoords)
    // {
    // float maxX = Float.MIN_VALUE;
    // float minX = Float.MAX_VALUE;
    //
    // for (Node node : layer) {
    // if (!((NodeParams) node.get(HierarchicalLayouter.NODE_PARAMS)).dummy) {
    // float x = xcoords.get(node);
    // maxX = Math.max(x, maxX);
    // minX = Math.min(x, minX);
    // }
    // }
    //
    // return (maxX + minX) / 2;
    // }

    protected abstract Map<Node, Float> calculateCoordinates(Graph graph);

    protected void determineCoordinates(Graph graph, Map<Node, Float> xcoords) {
        LayerContainer layerContainer = getContext().getLayerContainer();
        Map<Integer, Integer> heights = calculateMaxHeightPerLayer();
        float layerSpacing = getContext().getLayerSpacing();
        float offsetX;
        float offsetY;
        int verticalPosition = 0;
        switch (getContext().getLayoutOrientation()) {
        case LEFT_TO_RIGHT:
            offsetY = Collections.max(xcoords.values());
            for (int i = 0; i < layerContainer.getSize(); i++) {
                for (Node node : layerContainer.getNodes(i)) {
                    setX(node, verticalPosition);
                    setY(node, Math.abs(xcoords.get(node) - offsetY));
                }
                if (i < layerContainer.getSize() - 1) {
                    verticalPosition += layerSpacing + (heights.get(i) / 2) + (heights.get(i + 1) / 2) + getContext().getEdgeLandingOffset()*2;
                }
            }
            break;
        case RIGHT_TO_LEFT:
            offsetX = layerContainer.getSize();
            offsetY = Collections.max(xcoords.values());
            for (int i = 0; i < layerContainer.getSize(); i++) {
                for (Node node : layerContainer.getNodes(i)) {
                    setX(node, verticalPosition);
                    setY(node, xcoords.get(node));
                }
                if (i < layerContainer.getSize() - 1) {
                    verticalPosition += -layerSpacing - Math.max(heights.get(i), heights.get(i + 1));
                }
            }
            break;
        case TOP_TO_BOTTOM:
            for (int i = 0; i < layerContainer.getSize(); i++) {
                for (Node node : layerContainer.getNodes(i)) {
                    setX(node, xcoords.get(node));
                    setY(node, verticalPosition);
                }
                if (i < layerContainer.getSize() - 1) {
                    verticalPosition += layerSpacing + Math.max(heights.get(i), heights.get(i + 1));
                }
            }
            break;
        case BOTTOM_TO_TOP:
            offsetX = Collections.max(xcoords.values());
            offsetY = layerContainer.getSize();
            for (int i = 0; i < layerContainer.getSize(); i++) {
                for (Node node : layerContainer.getNodes(i)) {
                    setX(node, Math.abs(xcoords.get(node) - offsetX));
                    setY(node, verticalPosition);
                }
                if (i < layerContainer.getSize() - 1) {
                    verticalPosition += -layerSpacing - Math.max(heights.get(i), heights.get(i + 1));
                }
            }
            break;
        }
    }

    private Map<Integer, Integer> calculateMaxHeightPerLayer() {
        Map<Integer, Integer> heights = new HashMap<>();
        LayerContainer layerContainer = getContext().getLayerContainer();

        for (int i = 0; i < layerContainer.getSize(); i++) {
            List<Node> layerNodes = layerContainer.getNodes(i);
            int height = Integer.MIN_VALUE;
            for (Node node : layerNodes) {
                height = Math.max(height, ysize(node));
            }
            heights.put(i, height);
        }

        return heights;
    }

    private void setX(Node n, float x) {
        ((NodeParams) n.get(HierarchicalLayouter.NODE_PARAMS)).x = x;
    }

    private void setY(Node n, float y) {
        ((NodeParams) n.get(HierarchicalLayouter.NODE_PARAMS)).y = y;
    }

    @SuppressWarnings("unchecked")
    protected void removeDummies(Graph graph) {
        LayerContainer layerContainer = getContext().getLayerContainer();
        Iterator<Edge> iter = graph.edges();
        while (iter.hasNext()) {
            Edge e = iter.next();
            if (!isDummy(e.getSourceNode()) && !isDummy(e.getTargetNode())) {
                int layerDifference = layerContainer.getLayer(e.getSourceNode())
                        - layerContainer.getLayer(e.getTargetNode());
                bendCalculator.createBends(e, layerDifference, null);
            }
        }

        List<Node> removeNodes = new LinkedList<>();
        Iterator<Node> iter2 = graph.nodes();
        while (iter2.hasNext()) {
            Node n = iter2.next();
            Map<Node, List<Node>> dummies = ((NodeParams) n.get(HierarchicalLayouter.NODE_PARAMS)).dummies;

            if (dummies.size() > 0) {
                for (Node target : dummies.keySet()) {
                    List<Node> dummiesList = ((NodeParams) n.get(HierarchicalLayouter.NODE_PARAMS)).dummies.get(target);
                    Edge dummyEdge = graph.getEdge(n, dummiesList.get(0));
                    Edge e = graph.addEdge(n, target);
                    e.set(HierarchicalLayouter.EDGE_PARAMS, HierarchicalLayouter.getEdgeParams(dummyEdge).clone());
                    getEdgeParams(e).longEdge = true;
                    calculateOffsets(graph, e, dummiesList);
                    int layerDifference = layerContainer.getLayer(n) - layerContainer.getLayer(target);

                    bendCalculator.createBends(e, layerDifference, dummiesList);

                    removeNodes.addAll(dummiesList);
                }
            }
        }

        for (Node node : removeNodes) {
            layerContainer.removeNode(node);
            graph.removeNode(node);
        }
    }

    private void calculateOffsets(Graph graph, Edge e, List<Node> dummies) {
        Edge sourceDummy = graph.getEdge(e.getSourceNode(), dummies.get(0));
        Edge dummyTarget = graph.getEdge(dummies.get(dummies.size() - 1), e.getTargetNode());
        EdgeParams edgeParams = HierarchicalLayouter.getEdgeParams(e);
        EdgeParams sourceDummyParams = HierarchicalLayouter.getEdgeParams(sourceDummy);
        EdgeParams dummyTargetParams = HierarchicalLayouter.getEdgeParams(dummyTarget);

        edgeParams.sourceOffset = sourceDummyParams.sourceOffset;
        edgeParams.targetOffset = dummyTargetParams.targetOffset;
    }

    protected int xsize(Node u) {
        switch (getContext().getLayoutOrientation()) {
        case TOP_TO_BOTTOM:
        case BOTTOM_TO_TOP:
            return getWidth(u);
        default:
            return getHeight(u);
        }
    }

    protected int ysize(Node u) {
        switch (getContext().getLayoutOrientation()) {
        case TOP_TO_BOTTOM:
        case BOTTOM_TO_TOP:
            return getHeight(u);
        default:
            return getWidth(u);
        }
    }

    protected int getWeight(Edge e) {
        boolean dummySource = isDummy(e.getSourceNode());
        boolean dummyTarget = isDummy(e.getTargetNode());

        if (dummySource && dummyTarget)
            return 8;
        if (dummySource || dummyTarget)
            return 2;
        return 1;
    }

    protected int getWidth(Node n) {
        return HierarchicalLayouter.getNodeParams(n).width;
    }

    protected int getHeight(Node n) {
        return HierarchicalLayouter.getNodeParams(n).height;
    }

    public void setLineType(LineType lineType) {
        Class<? extends BendCalculator> bendCalculatorClass = lineType.getBendCalculator();
        try {
            this.bendCalculator = bendCalculatorClass.getConstructor(HierarchicalLayoutContext.class)
                    .newInstance(getContext());
            this.lineType = lineType;
        } catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException
                | NoSuchMethodException | SecurityException e) {
        }
    }

    public LineType getLineType() {
        return lineType;
    }
}
