package org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.math3.optim.PointValuePair;
import org.apache.commons.math3.optim.linear.LinearConstraint;
import org.apache.commons.math3.optim.linear.LinearConstraintSet;
import org.apache.commons.math3.optim.linear.LinearObjectiveFunction;
import org.apache.commons.math3.optim.linear.NonNegativeConstraint;
import org.apache.commons.math3.optim.linear.Relationship;
import org.apache.commons.math3.optim.linear.SimplexSolver;
import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.HierarchicalLayoutContext;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;

public class SimplexEmplacer extends Emplacer {

	public SimplexEmplacer(HierarchicalLayoutContext context) {
		super(context);
//		context.setIntracellSpacing(10);
	}

	@SuppressWarnings("unchecked")
    @Override
	public Map<Node, Float> calculateCoordinates(Graph graph) {
		double[] unitCoeficients = new double[graph.getEdges().getTupleCount() * 2];
		double[] nullCoeficients = new double[graph.getNodes().getTupleCount()];
		Arrays.fill(unitCoeficients, 1d);
		// calculateCoeficients(graph, unitCoeficients);
		Arrays.fill(nullCoeficients, 0d);

		double[] allCoeficients = new double[unitCoeficients.length + nullCoeficients.length];
		System.arraycopy(unitCoeficients, 0, allCoeficients, 0, unitCoeficients.length);
		System.arraycopy(nullCoeficients, 0, allCoeficients, unitCoeficients.length, nullCoeficients.length);

		LinearObjectiveFunction f = new LinearObjectiveFunction(allCoeficients, 0);

		Collection<LinearConstraint> constraints = new ArrayList<LinearConstraint>();
		Iterator<Edge> iter = graph.edges();
		int index = 0;
		while (iter.hasNext()) {
			Edge e = iter.next();
			Node v = e.getSourceNode();
			Node w = e.getTargetNode();
			int indexV = v.getRow();
			int indexW = w.getRow();

			// Constraint 1
			double[] constraintCoeficients = new double[allCoeficients.length];
			Arrays.fill(constraintCoeficients, 0);
			constraintCoeficients[unitCoeficients.length + indexW] = -getWeight(e);
			constraintCoeficients[unitCoeficients.length + indexV] = getWeight(e);
			constraintCoeficients[index * 2] = 1;
			constraintCoeficients[index * 2 + 1] = -1;
			constraints.add(new LinearConstraint(constraintCoeficients, Relationship.EQ, 0));

			// // Constratin 2
			// double[] constraintCoeficients2 = new
			// double[allCoeficients.length];
			// Arrays.fill(constraintCoeficients2, 0);
			// constraintCoeficients2[i * 2] = 1;
			// constraints.add(new LinearConstraint(constraintCoeficients2,
			// Relationship.GEQ, 0));
			//
			// // Constratin 3
			// double[] constraintCoeficients3 = new
			// double[allCoeficients.length];
			// Arrays.fill(constraintCoeficients3, 0);
			// constraintCoeficients3[i * 2 + 1] = 1;
			// constraints.add(new LinearConstraint(constraintCoeficients3,
			// Relationship.GEQ, 0));
			index++;
		}

		// Separation constraint
		for (int i = 0; i < getContext().getLayerContainer().getSize(); i++) {
			List<Node> layerNodes = getContext().getLayerContainer().getNodes(i);
			for (int j = 1; j < layerNodes.size(); j++) {
				Node a = layerNodes.get(j - 1);
				Node b = layerNodes.get(j);
				int indexA = a.getRow();
				int indexB = b.getRow();

				double[] constraintCoeficients = new double[allCoeficients.length];
				Arrays.fill(constraintCoeficients, 0);
				constraintCoeficients[unitCoeficients.length + indexB] = 1;
				constraintCoeficients[unitCoeficients.length + indexA] = -1;
				constraints.add(new LinearConstraint(constraintCoeficients, Relationship.GEQ, minimumDistance(a, b)));
			}
		}

		SimplexSolver solver = new SimplexSolver();
		PointValuePair pointValuePair = solver.optimize(f, new LinearConstraintSet(constraints), GoalType.MINIMIZE,
				new NonNegativeConstraint(true));

		double[] solution = pointValuePair.getPoint();
		Iterator<Node> iter2 = graph.nodes();
		Map<Node, Float> xcoords = new HashMap<>();
		index = 0;
		while (iter2.hasNext()) {
			Node node = iter2.next();
			xcoords.put(node, (float) solution[unitCoeficients.length + index++]);
		}

		return xcoords;
	}

	public double minimumDistance(Node a, Node b) {
		return (xsize(a) + xsize(b) / 2) + getContext().getIntracellSpacing();
	}

	// public void calculateCoeficients(Graph graph, double[] coeficients) {
	// Iterator<Edge> iter = graph.edges();
	// int i = 0;
	// while (iter.hasNext()) {
	// coeficients[i++] = getWeight(iter.next());
	// }
	// }
}
