package org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;

import org.processmining.humbertogarciacaballero.models.model.NodeType;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.HierarchicalLayoutContext;

import prefuse.data.Graph;
import prefuse.data.Node;

public class ConsecutiveLayerer extends Layerer {

	public ConsecutiveLayerer(HierarchicalLayoutContext context) {
		super(context);
	}

	@Override
	public void calculateLayers(Graph graph) {
		Iterator<Node> rootNodes = getRootNodes(graph, NodeType.PLACE);
		Queue<Node> queue = new LinkedList<>();
		boolean startsWithPlace = rootNodes.hasNext();

		if (startsWithPlace) {
			while (rootNodes.hasNext()) {
				Node node = rootNodes.next();
				setLayer(node, 0);
				queue.add(node);
			}
			rootNodes = getRootNodes(graph, NodeType.TRANSITION);
			while (rootNodes.hasNext()) {
				Node node = rootNodes.next();
				setLayer(node, 1);
				queue.add(node);
			}
		} else {
			rootNodes = getRootNodes(graph, NodeType.TRANSITION);
			while (rootNodes.hasNext()) {
				Node node = rootNodes.next();
				setLayer(node, 0);
				queue.add(node);
			}
		}

		while (!queue.isEmpty()) {
			Node current = queue.poll();
			Iterator<Node> neighbors = getSuccessors(current, true, getNodeType(current).opposite());

			while (neighbors.hasNext()) {
				Node target = neighbors.next();
				if (getLayer(target) == -1) {
					setLayer(target, getLayer(current) + 1);
					queue.add(target);
				}
			}
		}
	}

}
