package org.processmining.humbertogarciacaballero.models.model.layout.stage.sequencer;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayerContainer;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.HierarchicalLayoutContext;

import prefuse.data.Graph;
import prefuse.data.Node;

public class HybridCrossingSequencer extends Sequencer {

    private HybridCrossingComparator comparator;

    public HybridCrossingSequencer(HierarchicalLayoutContext context) {
        super(context);
        comparator = new HybridCrossingComparator();
    }

    @Override
    public int run(Graph graph) {
        LayerContainer layerContainer = getContext().getLayerContainer();

        if (layerContainer.getSize() > 0) {
            // Median and barycenter ordering
            for (int i = 1; i < layerContainer.getSize(); i++) {
                List<Node> toFixNodes = layerContainer.getNodes(i);
                Collections.sort(toFixNodes, comparator);
            }

            // Adjacent exchange
            int improvement;
            do {
                improvement = 0;
                for (int i = 1; i < layerContainer.getSize(); i++) {
                    List<Node> layerNodes = layerContainer.getNodes(i);
                    List<Node> previousLayerNodes = layerContainer.getNodes(i - 1);

                    for (int j = 0; j < layerNodes.size() - 1; j++) {
                        Node u = layerNodes.get(j);
                        Node v = layerNodes.get(j + 1);
                        int currentCrossings = calculateCrossings(layerNodes, previousLayerNodes, u, j, v, j + 1);
                        int newCrossings = calculateCrossings(layerNodes, previousLayerNodes, v, j, u, j + 1);

                        if (newCrossings < currentCrossings) {
                            Collections.swap(layerNodes, j, j + 1);
                            improvement += currentCrossings - newCrossings;
                        }
                    }
                }
            } while (improvement != 0);
        }

        if (PNConstants.DEBUG) {
            s_logger.info("Crossings: " + crossing(layerContainer));
        }

        return 0;
    }

    private class HybridCrossingComparator implements Comparator<Node> {

        @Override
        public int compare(Node n1, Node n2) {
            float medianN1 = medianValue(getContext().getLayerContainer(), n1, -1);
            float medianN2 = medianValue(getContext().getLayerContainer(), n2, -1);

            if (medianN1 > medianN2)
                return 1;
            if (medianN1 < medianN2)
                return -1;

            float barycenterN1 = barycenterValue(getContext().getLayerContainer(), n1, -1);
            float barycenterN2 = barycenterValue(getContext().getLayerContainer(), n2, -1);

            if (barycenterN1 > barycenterN2)
                return 1;
            if (barycenterN1 < barycenterN2)
                return -1;

            if (n1.getDegree() % 2 == 1 && n2.getDegree() % 2 == 0)
                return -1;
            if (n2.getDegree() % 2 == 1 && n1.getDegree() % 2 == 0)
                return 1;

            return 0;
        }
    }
}
