package org.processmining.humbertogarciacaballero.plugins;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.humbertogarciacaballero.algorithms.ProcessMiningAlgorithms;
import org.processmining.humbertogarciacaballero.models.model.NodeType;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.SoundnessInformation;
import org.processmining.humbertogarciacaballero.models.util.Utils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.InhibitorArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.ResetArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.synthesis.VizGraphParametersUtils;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Tuple;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.ObjectLiteral;

public class PluginUtils {

	@SuppressWarnings("unchecked")
	public static Graph netToGraph(PluginContext context, AcceptingPetriNet n, SoundnessInformation information,
			Map<PetrinetNode, Integer> idMap) {
		Graph graph = netToGraph(context, n.getNet(), null, information, idMap);
		Iterator<Place> iter = n.getInitialMarking().iterator();
		while (iter.hasNext()) {
			Place p = iter.next();
			Iterator<Tuple> iter2 = graph.getNodes().tuples(
					new ComparisonPredicate(ComparisonPredicate.EQ, new ColumnExpression(PNConstants.NODE_ID_KEY),
							new ObjectLiteral(p.getLocalID().toString())));

			if (iter2.hasNext()) {
				iter2.next().set(PNConstants.NODE_STARTING_KEY, true);
			}
		}

		Iterator<Marking> markings = n.getFinalMarkings().iterator();
		while (markings.hasNext()) {
			iter = markings.next().iterator();
			while (iter.hasNext()) {
				Place p = iter.next();
				Iterator<Tuple> iter2 = graph.getNodes().tuples(
						new ComparisonPredicate(ComparisonPredicate.EQ, new ColumnExpression(PNConstants.NODE_ID_KEY),
								new ObjectLiteral(p.getLocalID().toString())));

				if (iter2.hasNext()) {
					iter2.next().set(PNConstants.NODE_ENDING_KEY, true);
				}
			}
		}
		return graph;
	}

	//PMD IPN Starts
	public static Graph netToGraph(PluginContext context, PetrinetGraph d, Marking m, SoundnessInformation information,
			Map<PetrinetNode, Integer> idMap) {
		return netToGraph(context, d, m, information, null, idMap);
	}

	//PMD IPN Ends

	public static Graph netToGraph(PluginContext context, PetrinetGraph d, Marking m, SoundnessInformation information,
			InteractivePetriNet ipn, Map<PetrinetNode, Integer> idMap) {
		Graph graph = new Graph(true);
		Map<PetrinetNode, Integer> nodeMap = new HashMap<>();
		Utils.initSchema(graph);
		Map<String, String> labels = ProcessMiningAlgorithms.preprocessDiagnosisPetrinet(context, (Petrinet) d);

		int V = d.getNodes().size();
		PetrinetNode[] nodes = d.getNodes().toArray(new PetrinetNode[V]);

		for (PetrinetNode node : nodes) {
			Node n = graph.addNode();
			nodeMap.put(node, n.getRow());
			if (node instanceof Place) {
				n.set(PNConstants.NODE_TYPE_KEY, NodeType.PLACE.toString());
				n.set(PNConstants.NODE_LABEL_KEY, labels.get(node.getId().toString()));

				//PMD starts
				if (idMap != null) { //if id map provided, then use id from the idmap
					n.set("ID", idMap.get(node));
				} else {
					n.set("ID", ipn.getPlaceId((Place) node));
					n.set("InputOutputNodes",
							VizGraphParametersUtils.addHighlightPlacesBasedOnLDMatrix((Place) node, ipn));
				}
				//PMD ends

			} else {
				//PMD starts
				if (idMap != null) { //if id map provided, then use id from the idmap
					n.set("ID", idMap.get(node));
				} else {
					n.set("ID", (ipn.getTranId((Transition) node)));
					n.set("InputOutputNodes",
							VizGraphParametersUtils.addHighlightTransBasedOnLDMatrix((Transition) node, ipn));
				}
				//PMD ends
				n.set(PNConstants.NODE_TYPE_KEY, NodeType.TRANSITION.toString());
				boolean tau;
				n.set(PNConstants.NODE_TAU_TRANSITION_KEY, (tau = ((Transition) node).isInvisible()));

				if (tau) {
					n.set(PNConstants.NODE_LABEL_KEY, "TAU");
				} else {
					n.set(PNConstants.NODE_LABEL_KEY, labels.get(node.getId().toString()));
				}

			}
			//PMD Starts
			if (ipn != null)
				if (node instanceof Place) {
					String str = "";
					n.set(PNConstants.NODE_TYPE_KEY, NodeType.PLACE.toString());

				} else {
					//add Follows Precedes Coccors relations
					String strtemp = "";
					System.out.println(node);
					n.set(PNConstants.NODE_TYPE_KEY, NodeType.TRANSITION.toString());
				}
			//PMD Ends
			n.setString(PNConstants.NODE_ID_KEY, node.getLocalID().localNodeID().toString());
			
		}

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : d.getEdges()) {
			int source = nodeMap.get(edge.getSource());
			int target = nodeMap.get(edge.getTarget());
			Edge e = graph.getEdge(graph.addEdge(source, target));
			if (edge instanceof ResetArc) {
				e.setBoolean(PNConstants.EDGE_RESET_KEY, true);
			} else if (edge instanceof InhibitorArc) {
				e.setBoolean(PNConstants.EDGE_INHIBITOR_KEY, true);
			}
			if (d.getArc(edge.getTarget(), edge.getSource()) != null) {
				e.setBoolean(PNConstants.EDGE_BIDIRECTIONAL, true);
				if (graph.getEdge(target, source) != -1) {
					e.setBoolean(PNConstants.EDGE_BIDIRECTIONAL_DRAW, false);
				}
			}
		}

		ProcessMiningAlgorithms.postprocessDiagnosisPetrinet(context, (Petrinet) d, labels);

		// Adding extra information
		if (m != null) {
			for (Place p : m) {
				graph.getNode(nodeMap.get(p)).setBoolean(PNConstants.NODE_STARTING_KEY, true);
			}
			markEndingNodes(graph);
		} else {
			markStartingNodes(graph);
			markEndingNodes(graph);
		}

		return graph;
	}

	private static void markEndingNodes(Graph g) {
		Iterator<Node> iter = Utils.getSinkNodes(g);
		while (iter.hasNext()) {
			Node node = iter.next();
			if (NodeType.valueOf(node.getString(PNConstants.NODE_TYPE_KEY)) == NodeType.PLACE)
				node.setBoolean(PNConstants.NODE_ENDING_KEY, true);
		}
	}

	private static void markStartingNodes(Graph g) {
		Iterator<Node> iter = Utils.getRootNodes(g);
		while (iter.hasNext()) {
			Node node = iter.next();
			if (NodeType.valueOf(node.getString(PNConstants.NODE_TYPE_KEY)) == NodeType.PLACE)
				node.setBoolean(PNConstants.NODE_STARTING_KEY, true);
		}
	}
}
