package org.processmining.incrementaldiscovery;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.incrementaldiscovery.models.IncrementalModel;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.pnalignanalysis.conformance.AlignmentPrecGen;
import org.processmining.plugins.pnalignanalysis.conformance.AlignmentPrecGenRes;
import org.processmining.prabhakardixit.alignments.AlignmentCalculations;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.plugins.CompareLog2PetriNetPlugin;
import org.processmining.projectedrecallandprecision.plugins.ComparePetriNet2PetriNetPlugin;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

public class Alignments {

	public static float[] getProjectedFitnessAndPrecision(XLog log, IncrementalModel mnew)
			throws ProjectedMeasuresFailedException {
		Petrinet pn = mnew.getNet();
		AcceptingPetriNet model = new AcceptingPetriNetImpl(pn);
		float[] fitPre = new float[2];
		CompareLog2PetriNetPlugin compare = new CompareLog2PetriNetPlugin();
		CompareParameters param = new CompareParameters();

		Set<String> act = new HashSet<String>();
		for (Transition t : model.getNet().getTransitions())
			if (!t.isInvisible()) {
				//				System.out.println("\n Transition : " + t);
				act.add(t.getLabel());

			}
		//		System.out.println("\n LOG: ");
		//		for (XTrace t : log)
		//			for (XEvent e : t)
		//				System.out.println(XConceptExtension.instance().extractName(e));
		param.setK(act.size() > 1 ? 2 : 1);
		ProjectedRecallPrecisionResult result = null;
		try {
			result = compare.measure(log, model, param, new ProMCanceller() {
				public boolean isCancelled() {
					return false;
				}
			});
		} catch (AutomatonFailedException | InterruptedException | ProjectedMeasuresFailedException e) {
			e.printStackTrace();
		}
		fitPre[0] = (float) result.getRecall();
		fitPre[1] = (float) result.getPrecision();

		mnew.getImn().setProjectedResult(result);

		return fitPre;
	}

	public static float[] getTraditionalFitnessAndPrecision(PluginContext context, XLog log, IncrementalModel mnew) {
		float[] fitPre = new float[2];
		PNRepResult replay = AlignmentCalculations.calculateNAlignment1(context, log, mnew.getNet(), 50, false, null);
		fitPre[0] = ((Double) replay.getInfo().get(PNRepResult.TRACEFITNESS)).floatValue();

		//calculate precision
		AlignmentPrecGen precGen = new AlignmentPrecGen();
		AlignmentPrecGenRes precRes = precGen.measureConformanceAssumingCorrectAlignment(context,
				AlignmentCalculations.transitionToEventClassMapperByLabel(log, mnew.getNet(), null,false), replay,
				mnew.getNet(), PetrinetUtils.getInitialMarking(mnew.getNet()), true);

		fitPre[1] = ((Double) precRes.getPrecision()).floatValue();

		mnew.getImn().setReplayReult(replay);
		return fitPre;
	}

	public static boolean getProjectedModel2ModelComparison(UIPluginContext context, IncrementalModel m2,
			Set<IncrementalModel> models) throws ProjectedMeasuresFailedException, ReductionFailedException {
		Set<Petrinet> nets = new HashSet<Petrinet>();
		AcceptingPetriNet net1 = new AcceptingPetriNetImpl(m2.getNet());

		for (IncrementalModel m : models) {
			float[] fitPre = new float[2];
			ComparePetriNet2PetriNetPlugin compare = new ComparePetriNet2PetriNetPlugin();
			CompareParameters param = new CompareParameters();
			Set<String> act = new HashSet<String>();
			for (Transition t : m2.getNet().getTransitions())
				if (!t.isInvisible())
					act.add(t.getLabel());

			param.setK(act.size() > 1 ? 2 : 1);
			ProjectedRecallPrecisionResult result = null;
			try {
				result = compare.measure(net1, new AcceptingPetriNetImpl(m.getNet()), param, new ProMCanceller() {

					public boolean isCancelled() {
						// TODO Auto-generated method stub
						return false;
					}
				});
			} catch (AutomatonFailedException | InterruptedException | ProjectedMeasuresFailedException e) {
				e.printStackTrace();
			}
			try {
				if (result.getRecall() == 1. && result.getPrecision() == 1.) {
					if (net1.getNet().getEdges().size() < m.getNet().getEdges().size()) { //replace m with m2 if num of edges in m2 is less
						models.remove(m);
						models.add(m2);
					}
					return true;
				}
			} catch (NullPointerException n) {
				context.getProvidedObjectManager().createProvidedObject("net1", net1, AcceptingPetriNet.class, context);
				context.getProvidedObjectManager().createProvidedObject("net2", new AcceptingPetriNetImpl(m.getNet()),
						AcceptingPetriNet.class, context);
				System.out.println("Exported");
				//				context.getGlobalContext().getResourceManager().getResourceForInstance(net1).setFavorite(true);
			}

		}
		models.add(m2);
		return false;
	}
}
