package org.processmining.incrementaldiscovery;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.incrementaldiscovery.models.IncrementalModel;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class DiscoveryPanel extends ProMSplitPane {
	/**
	 * 
	 */
	private static final long serialVersionUID = 3697882196100566798L;

	//view names end

	private PNetVisualizerController controller;
	private InteractivePetriNet ipn;
	private DefaultTableModel tableModel;
	public ProMTable table;
	private TableRowSorter<TableModel> sorter;

	private Dimension singleSize;
	//	private int currSelection;
	private JButton addBtn;
	List<IncrementalModel> incM;
	private Set<String> skipActs;

	GridBagConstraints gbc = new GridBagConstraints();

	@SuppressWarnings("serial")
	public DiscoveryPanel(final DiscoveryPredictor dp) {
		super(ProMSplitPane.VERTICAL_SPLIT);
		controller = dp.getController();
		int width = controller.getStatisticsPanel().getWidth() * 2 / 5;
		int height = controller.getStatisticsPanel().getHeight() - 23;
		this.tableModel = dp.getTableModel();
		this.table = new ProMTable(tableModel);
		this.ipn = controller.getIPN();
		this.incM = dp.getIncM();
		this.skipActs = dp.getSkipActs();

		final JPanel topCom = new JPanel(new GridBagLayout());

		gbc.weightx = 1;
		gbc.gridwidth = 1;

		addBtn = dp.getAddBtn();
		addBtn.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				//TODO add the selected model
				if (dp.getCurrSelection() != -1) {
					ReduceUsingTrackerForVizAndNet.reduceUsingTracker(controller, incM.get(dp.getCurrSelection()),
							false, false);
					ExpandUsingTrackerForVizAndNet.expandUsingTracker(controller, incM.get(dp.getCurrSelection()), true);
					
					//set replay if exists
					controller.getIPN().getInteractiveMatNet()
							.setReplayReult(incM.get(dp.getCurrSelection()).getImn().getReplayReult());
					
					dp.setCurrSelection(-1);
					table.clearSelection();

					//add the current activity as skippable- since its already added
					skipActs.add(controller.getActivity());

					//selection of next activity  - based on policy
					dp.setControllerActivity(false);

			
					try {
						dp.addRelations(ipn);
						System.out.println("done adding!");
					} catch (ProjectedMeasuresFailedException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (ReductionFailedException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}


			}
		});
		topCom.add(addBtn, gbc);

		JButton clearBtn = SlickerFactory.instance().createButton("Clear!");
		clearBtn.setToolTipText("Removes the temporary projection of the activity on the model.");
		clearBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (dp.getCurrSelection() != -1) {
					ReduceUsingTrackerForVizAndNet.reduceUsingTracker(controller, incM.get(dp.getCurrSelection()),
							false, true);
				}
				dp.updateStats("");
			}
		});
		topCom.add(clearBtn, gbc);

		JButton nextAct = SlickerFactory.instance().createButton("Skip act");
		nextAct.setToolTipText("Skips current activity, selects and finds positions <br> for the next activity according to the chosen policy.");
		nextAct.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (dp.getCurrSelection() != -1)
					ReduceUsingTrackerForVizAndNet.reduceUsingTracker(controller, incM.get(dp.getCurrSelection()),
							false, false);
				dp.setCurrSelection(-1);
				table.clearSelection();
				skipActs.add(controller.getActivity());
				dp.setControllerActivity(true); //skip ind true
				try {
					dp.addRelations(ipn);
					System.out.println("done adding!");

				} catch (ProjectedMeasuresFailedException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (ReductionFailedException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}

		});

		topCom.add(nextAct, gbc);

		gbc.gridwidth = 1;
		//		topCom.add(new JLabel("Policy: "));

		//		panel.add(analyzeResults, constraints);
		topCom.setPreferredSize(new Dimension(width, 23));
		topCom.setMinimumSize(new Dimension(width, 23));
		topCom.setBackground(Color.LIGHT_GRAY);

		//top comp
		setTopComponent(topCom);
		setResizeWeight(0.0);
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));

		int columnCount = table.getColumnModel().getColumnCount();
		//		singleSize = new Dimension((width - 150) / (columnCount + 4), 23);

		table.setPreferredSize(new Dimension(width, height));
		table.setMinimumSize(new Dimension(width, height));
		JPanel botCom = new JPanel();
		botCom.setBackground(Color.LIGHT_GRAY);
		botCom.setLayout(new GridBagLayout());
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		botCom.add(table, gbc);

		gbc.gridx = 1;

		//		botCom.add(list,gbc);

		gbc.gridx = 2;
		//		botCom.add(tableConfig,gbc);
		botCom.setPreferredSize(new Dimension(width, height));
		botCom.setMinimumSize(new Dimension(width, height));
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getSelectionModel().addListSelectionListener(new RowListener(dp));
		//		table.setMinimumSize(new Dimension(200, 40));
		//		table.setPreferredSize(new Dimension(1000, 450));
//		table.getTableHeader().setPreferredSize(new Dimension(1000, 20));
		table.getTableHeader().setReorderingAllowed(false);

		sorter = new TableRowSorter<TableModel>(table.getTable().getModel());
		table.setRowSorter(sorter);

		tableModel.addColumn("Rank");
		tableModel.addColumn("Fitness");
		tableModel.addColumn("Precision");

		setBottomComponent(botCom);
		//		Enumeration<TableColumn> columns = table.getColumnModel().getColumns();
		//		while (columns.hasMoreElements()) {
		//			final TableColumn column = columns.nextElement();
		//			column.setPreferredWidth(width / (columnCount + 4));
		//			column.setCellEditor(new DefaultCellEditor(new JTextField()) {
		//
		//				protected void fireEditingStopped() {
		//					this.cancelCellEditing();
		//					super.fireEditingStopped();
		//				}
		//
		//				protected void fireEditingCanceled() {
		//					super.fireEditingCanceled();
		//				}
		//
		//			});
		//		}
		//		table.getColumnModel().getColumn(0).setPreferredWidth(3 * (width / (columnCount + 4)));
		//		table.getColumnModel().getColumn(1).setPreferredWidth(3 * (width / (columnCount + 4)));

		//initiate
		init(dp);

		try {
			dp.addRelations(controller.getIPN());
		} catch (ProjectedMeasuresFailedException | ReductionFailedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		List<RowSorter.SortKey> sortKeys = new ArrayList<>(25);
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		//		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		sorter.setSortKeys(sortKeys);

	}

	private void init(DiscoveryPredictor dp) {
		dp.getPolicyTypeBox().setSelectedItem(dp.heuLtoR);
		dp.setControllerActivity(false);

	}

	//change the policy here
	//	private void updateSelectedView() {
	//
	//		switch (switchRelationTypeBox.getSelectedItem().toString()) {
	//			case selectview :
	//				//do nothing
	//				break;
	//			case alphabetical :
	//				//setBottomComponent(ModelLogView.getResults(controller, tpa));
	//				break;
	//			case heuLtoR :
	//				//setBottomComponent(ExploreTraceVariants.getResult(tpa, ipn.getLog(), ipn.getPromContext()));
	//				break;
	//		}
	//
	//	}

	private class RowListener implements ListSelectionListener {
		DiscoveryPredictor dp;

		public RowListener(DiscoveryPredictor dp) {
			this.dp = dp;
		}

		public void valueChanged(ListSelectionEvent event) {
			if (table.getSelectedRow() < 0) {
				return;
			}

			int modelIndex = table.getTable().convertRowIndexToModel(table.getSelectedRow());
			if (event.getValueIsAdjusting()) {
				return;
			}

			//pmd

			TransPairAlignment tpa = null;
			if (dp.getCurrSelection() != -1)
				ReduceUsingTrackerForVizAndNet.reduceUsingTracker(controller, incM.get(dp.getCurrSelection()), false,
						false);

			int id = Integer.valueOf(tableModel.getValueAt(modelIndex, 0).toString()) - 1;
			dp.setCurrSelection(id);

			if (dp.getCurrSelection() != -1)
				ExpandUsingTrackerForVizAndNet.expandUsingTracker(controller, incM.get(dp.getCurrSelection()), false);

			dp.updateStats("");
			
		}
	}

	private void doAutoComplete() {
		//TODO auto complete functionality

	}

}
