package org.processmining.incrementaldiscovery;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetArrayFactory;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.incrementaldiscovery.models.IncrementalModel;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.prabhakardixit.alignments.CombinationUtils;
import org.processmining.prabhakardixit.alignments.ProjectedAveragedAlignments;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.InteractiveMatrixNet;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.LDSets;
import org.processmining.prabhakardixit.fcnets.synthesis.CreateIPN;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

public class IncrementalDiscoveryBruteForceAlgorithm {
	int k = 0;
	XLog log;
	UIPluginContext context;
	Set<String> addedActs = new HashSet<String>();
	boolean proj;

	public AcceptingPetriNetArray generatePetriNet(XLog log, UIPluginContext context)
			throws ProjectedMeasuresFailedException, ReductionFailedException {
		k = 0;
		this.log = log;
		this.context = context;
		proj = true;

		//init create ipn to generate the basic structure of the net
		InteractivePetriNet ipn = CreateIPN.createIPN(log, context, false, false, true);
		MatrixUtils.initiate(ipn);
		Petrinet pni = ipn.getselectedPetriNetMap().getPetriNet();

		//create activity set
		Set<String> activities = new HashSet<String>();
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
		for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
			activities.add(evt.getId().replace("+complete", ""));
		}

		//create all possible combinations
		Set<IncrementalModel> allModels = new HashSet<IncrementalModel>();

		InteractiveMatrixNet imn = ipn.getInteractiveMatNet();
		Set<IncrementalModel> models = new HashSet<IncrementalModel>();
		models.add(new IncrementalModel(imn));
		int i = 0;
		for (String a : activities) {
			//						if (a.equals("t") || a.equals("a") || a.equals("n")) {
			if (i++ < 5) {
				addedActs.add(a);
				allModels.clear();
				for (IncrementalModel m : models) {
					allModels.addAll(createIncrementalModels(m.getImn(), a, 1));
				}
				//update pn and imn by selecting the best one
				float fit = 0f;
				float pre = 0f;
				float fitPreScore = 0f;
				for (IncrementalModel m : allModels) {
					if (m.getFitness() + m.getPrecision() > fitPreScore) {
						fit = m.getFitness();
						pre = m.getPrecision();
						fitPreScore = fit + pre;
					}
				}
				models.clear();
				for (IncrementalModel m : allModels) {
					if (m.getFitness() + m.getPrecision() == fitPreScore) {
						Alignments.getProjectedModel2ModelComparison(context, m, models); //add to models-if similar model isnt already present, replace if the curr model is better than similar if possible
					}

				}
			}
		}
		float fit = 0f;
		float pre = 0f;
		Set<IncrementalModel> keep = new HashSet<IncrementalModel>();

		//		allModels.retainAll(keep);
		AcceptingPetriNetArray apn = AcceptingPetriNetArrayFactory.createAcceptingPetriNetArray();

		for (IncrementalModel m : models) {
			if (i++ % 1 == 0)
				apn.addNet(new AcceptingPetriNetImpl(m.getNet()));
		}

		return apn;
	}

	private Set<IncrementalModel> createIncrementalModels(InteractiveMatrixNet interactiveMatNet, String label,
			int maxNumOfIter) throws ProjectedMeasuresFailedException {
		List<Set<IncrementalModel>> models = new ArrayList<Set<IncrementalModel>>(); //one set for each iter
		//first create the obj for each iter
		for (int i = 0; i < maxNumOfIter + 1; i++) {
			models.add(new HashSet<IncrementalModel>());
		}
		models.get(0).add(new IncrementalModel(interactiveMatNet));

		for (int i = 0; i < maxNumOfIter; i++) {
			Set<IncrementalModel> nextModels = models.get(i + 1);
			for (IncrementalModel m : models.get(i)) {
				//				if(m.getImn().getInputOutPlaces().size() !=4 && i!=0)
				//					continue;
				//add all sequences
				nextModels.addAll(getAllSequenceModels(m, label));

				//add all choices
				nextModels.addAll(getAllChoiceModels(m, label));

				//add all parallel places (incl the seq tran adding the new place)
				nextModels.addAll(getAllParallelModels(m, label));
			}
		}
		Set<IncrementalModel> allModels = new HashSet<IncrementalModel>();
		for (int i = 1; i < maxNumOfIter + 1; i++)
			allModels.addAll(models.get(i));

		return allModels;
	}

	private Collection<? extends IncrementalModel> getAllParallelModels(IncrementalModel m, String label)
			throws ProjectedMeasuresFailedException {
		Set<IncrementalModel> allModels = new HashSet<IncrementalModel>();

		for (LDSets lds : m.getImn().getplaceLDNonSiphons()) {
			Set<Integer> startTranSet = new HashSet<Integer>();
			Set<Integer> endTranSet = new HashSet<Integer>();
			startTranSet.addAll(lds.getPositiveNodes(m.getImn()));
			endTranSet.addAll(lds.getNegativeNodes(m.getImn()));
			IncrementalModel mnew = new IncrementalModel(m.getImn());
			IncrementalDiscoveryUtils.addParallelPlaceAndSeq(mnew, startTranSet, endTranSet, label, true);
			allModels.add(mnew);
			//alignments
			float[] fitPre = new float[2];
			if (proj)
				fitPre = Alignments.getProjectedFitnessAndPrecision(
						ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);
			else
				fitPre = Alignments.getTraditionalFitnessAndPrecision(context,
						ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);
			mnew.setFitness(fitPre[0]);
			mnew.setPrecision(fitPre[1]);
			System.out.println(k++);

			addSkippableTrans(m, mnew, allModels); //adds new models with silent trans (loop and skip)

		}
		return allModels;
	}

	private Collection<? extends IncrementalModel> getAllChoiceModels(IncrementalModel m, String label)
			throws ProjectedMeasuresFailedException {
		Set<IncrementalModel> allModels = new HashSet<IncrementalModel>();
		Petrinet pn = m.getNet();

		for (LDSets lds : m.getImn().getAllTranLDFCs()) {
			if (lds.getAllNodes(m.getImn()).contains(m.getImn().getPlaceId(PetrinetUtils.getInitialPlace(pn)))
					|| lds.getAllNodes(m.getImn()).contains(m.getImn().getPlaceId(PetrinetUtils.getFinalPlace(pn))))
				continue;
			Set<Integer> startPlaceSet = new HashSet<Integer>();
			Set<Integer> endPlaceSet = new HashSet<Integer>();
			startPlaceSet.addAll(lds.getNegativeNodes(m.getImn()));
			endPlaceSet.addAll(lds.getPositiveNodes(m.getImn()));
			IncrementalModel mnew = new IncrementalModel(m.getImn());
			IncrementalDiscoveryUtils.addChoiceTransition(mnew, startPlaceSet, endPlaceSet, label, true);
			allModels.add(mnew);
			//alignments
			float[] fitPre = new float[2];
			if (proj)
				fitPre = Alignments.getProjectedFitnessAndPrecision(
						ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);
			else
				fitPre = Alignments.getTraditionalFitnessAndPrecision(context,
						ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);
			mnew.setFitness(fitPre[0]);
			mnew.setPrecision(fitPre[1]);

			System.out.println(k++);
		}
		return allModels;
	}

	private Collection<? extends IncrementalModel> getAllSequenceModels(IncrementalModel m, String label)
			throws ProjectedMeasuresFailedException {
		Set<IncrementalModel> allModels = new HashSet<IncrementalModel>();
		Petrinet pn = m.getNet();
		for (Transition t : pn.getTransitions()) {
			Set<Integer> outPlaces = new HashSet<Integer>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(t))
				if (e.getTarget() != PetrinetUtils.getFinalPlace(pn)) {
					outPlaces.add(m.getImn().getPlaceId((Place) e.getTarget()));
				}

			for (int i = 1; i < outPlaces.size() + 1; i++) {
				Set<Set<Integer>> allComb = CombinationUtils.getAllCombs(outPlaces, i);
				for (Set<Integer> placeId : allComb) {
					Set<Integer> tranId = new HashSet<Integer>();
					tranId.add(m.getImn().getTranId(t));
					//add new tran between this tran and the outgoing place 
					IncrementalModel mnew = new IncrementalModel(m.getImn());
					IncrementalDiscoveryUtils.addNewTransition(mnew, tranId, placeId, label, true);
					allModels.add(mnew);
					//alignments
					float[] fitPre = new float[2];
					if (proj)
						fitPre = Alignments.getProjectedFitnessAndPrecision(
								ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);
					else
						fitPre = Alignments.getTraditionalFitnessAndPrecision(context,
								ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);

					mnew.setFitness(fitPre[0]);
					mnew.setPrecision(fitPre[1]);
					System.out.println(k++);

					addSkippableTrans(m, mnew, allModels); //adds new models with silent trans (loop and skip)
				}
			}
		}

		//for places with more than one incoming arcs, add trans too on the combinations
		for (Place p : pn.getPlaces()) {
			Set<Integer> inTransitions = new HashSet<Integer>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getInEdges(p))
				if (e.getTarget() != PetrinetUtils.getFinalPlace(pn)) {
					inTransitions.add(m.getImn().getTranId((Transition) e.getSource()));
				}

			for (int i = 2; i < inTransitions.size() + 1; i++) {
				Set<Set<Integer>> allComb = CombinationUtils.getAllCombs(inTransitions, i);
				for (Set<Integer> tranId : allComb) {
					Set<Integer> placeId = new HashSet<Integer>();
					placeId.add(m.getImn().getPlaceId(p));
					IncrementalModel mnew = new IncrementalModel(m.getImn());
					IncrementalDiscoveryUtils.addNewTransition(mnew, tranId, placeId, label, true);
					allModels.add(mnew);
					//alignments
					float[] fitPre = new float[2];
					if (proj)
						fitPre = Alignments.getProjectedFitnessAndPrecision(
								ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);
					else
						fitPre = Alignments.getTraditionalFitnessAndPrecision(context,
								ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew);

					mnew.setFitness(fitPre[0]);
					mnew.setPrecision(fitPre[1]);
					System.out.println(k++);

					addSkippableTrans(m, mnew, allModels); //adds new models with silent trans (loop and skip)
				}
			}
		}

		return allModels;
	}

	/**
	 * @param m
	 *            - old incremental model
	 * @param mnew
	 *            - new incremental model
	 * @param allModels
	 *            - newly created models will be added to this set
	 * @throws ProjectedMeasuresFailedException
	 */
	private void addSkippableTrans(IncrementalModel m, IncrementalModel mnew, Set<IncrementalModel> allModels)
			throws ProjectedMeasuresFailedException {
		//add a skippable transition for the corresponding seq tran (both choice and loop)
		IncrementalModel mnew1 = new IncrementalModel(mnew.getImn());
		IncrementalModel mnew2 = new IncrementalModel(mnew.getImn());
		//get the new tran id
		int newTranId = -1;
		for (Transition t1 : mnew.getImn().getAllTranId().keySet()) { //search the newly added tran- use this to add new silent tran
			boolean present = false;
			for (Transition t2 : m.getImn().getAllTranId().keySet()) {
				if (m.getImn().getTranId(t2).equals(mnew.getImn().getTranId(t1)))
					present = true;
			}
			if (!present && !PetrinetUtils.getFinalTranistion(mnew.getNet()).equals(t1))		//check if t1 is only in mnew, and if t1 is NOT the final tran
				newTranId = mnew.getImn().getTranId(t1);
		}
		IncrementalDiscoveryUtils.addSilentTransitions(mnew1, mnew2, newTranId, true);
		float[] fitPre = new float[2];
		if (proj)
			fitPre = Alignments.getProjectedFitnessAndPrecision(
					ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew1);
		else
			fitPre = Alignments.getTraditionalFitnessAndPrecision(context,
					ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew1);

		mnew1.setFitness(fitPre[0]);
		mnew1.setPrecision(fitPre[1]);
		allModels.add(mnew1);
		if (proj)
			fitPre = Alignments.getProjectedFitnessAndPrecision(
					ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew2);
		else
			fitPre = Alignments.getTraditionalFitnessAndPrecision(context,
					ProjectedAveragedAlignments.createMergedLog(log, addedActs, false), mnew2);

		mnew2.setFitness(fitPre[0]);
		mnew2.setPrecision(fitPre[1]);
		allModels.add(mnew2);
	}

}
