package org.processmining.incrementaldiscovery;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.prabhakardixit.fcnets.utils.MapUtils;

public class PolicyBasedOnOrdering {

	private Set<String> activities;
	private XLog log;

	public PolicyBasedOnOrdering(Set<String> activities, XLog log) {
		this.activities = activities;
		this.log = log;
	}

	public List<String> getPolicyBasedOnOrdering() {
		List<String> orderedActivities = new ArrayList<String>();
		Map<String, Map<Integer, Integer>> actSeqCount = new HashMap<String, Map<Integer, Integer>>(); //act -> Pos -> Count
		for (String s : activities) {
			actSeqCount.put(s, new HashMap<Integer, Integer>());
		}

		int maxI = -1;
		for (XTrace t : log) {
			int i = 0;
			for (XEvent e : t) {
				String name = XConceptExtension.instance().extractName(e);
				if (actSeqCount.containsKey(name)) {
					actSeqCount.get(name).put(i,
							actSeqCount.get(name).containsKey(i) ? actSeqCount.get(name).get(i) + 1 : 1);
				}
				i++;
			}
			maxI = maxI < i ? i : maxI;
		}

		//get ordered seq lists and add it to the global list
		for (int i = 0; i < maxI; i++) {
			Map<String, Integer> actPosCount = new HashMap<String, Integer>();
			for (String s : actSeqCount.keySet()) {
				if (actSeqCount.get(s).containsKey(i))
					actPosCount.put(s, actSeqCount.get(s).get(i));
			}
			Map<String, Integer> sorted = MapUtils.sortByValue(actPosCount, true);
			for (String s2 : sorted.keySet()) {
				//add only if for current i, the activity has the highest val among all i's
				int maxVal = -1;
				int jOfMax = -1;
				for (int j : actSeqCount.get(s2).keySet()) {
					if (maxVal < actSeqCount.get(s2).get(j)) {
						jOfMax = j;
						maxVal = actSeqCount.get(s2).get(j);
					}
				}
				if (jOfMax == i)
					orderedActivities.add(s2);
			}
		}
		return orderedActivities;
	}
}
