package org.processmining.incrementaldiscovery;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class PredictorActivityListPanel extends ProMSplitPane {

	/**
	 * 
	 */
	private static final long serialVersionUID = 473657835597833153L;

	GridBagConstraints gbc = new GridBagConstraints();
	ProMList<ListModel> list;
	DefaultListModel model;

	public PredictorActivityListPanel(final DiscoveryPredictor dp) {
		super(ProMSplitPane.VERTICAL_SPLIT);
		int width = dp.getController().getStatisticsPanel().getWidth() * 1 / 5;
		int height = dp.getController().getStatisticsPanel().getHeight() - 23;
		final JPanel topCom = new JPanel(new GridBagLayout());
		list = dp.getList();
		model = dp.getActModel();
		setResizeWeight(0.0);
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setMaximumSize(new Dimension(width, 23));
		String[] relationTypes = { dp.selectview, dp.alphabetical, dp.heuLtoR };
		JComboBox switchRelationTypeBox = SlickerFactory.instance().createComboBox(relationTypes);
		dp.setPolicyTypeBox(switchRelationTypeBox);

		list.setPreferredSize(new Dimension(width, height));
		list.setMinimumSize(new Dimension(width, height));
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				if (dp.getCurrSelection() != -1)
					ReduceUsingTrackerForVizAndNet.reduceUsingTracker(dp.getController(),
							dp.getIncM().get(dp.getCurrSelection()), false, false);
				list.requestFocus();
				updateActivityFromList(dp); //update the controller activity with the currently selected activity from list (if not already updated)

			}

		});

		setBottomComponent(list);

		topCom.add(switchRelationTypeBox, gbc);

		switchRelationTypeBox.setMinimumSize(new Dimension(150, 23));
		switchRelationTypeBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//				updateSelectedView();
			}

		});

		JButton autoComplete = SlickerFactory.instance().createButton("Auto Complete!");
		autoComplete.setToolTipText("I'm feeling lucky!");
		autoComplete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JTextField fitness = new JTextField();
				JTextField fitWt = new JTextField();
				JTextField precision = new JTextField();
				JTextField preWt = new JTextField();
				Object[] message = { "Minimum fitness %:", fitness, "Fitness weight %:", fitWt, "Minimum precision %:",
						precision, "Precision weight %:", preWt, };
				int option = JOptionPane.showConfirmDialog(null, message, "Configure", JOptionPane.OK_CANCEL_OPTION);
				if (option == JOptionPane.OK_OPTION) {
					//update values
					dp.setFitWeight(Integer.valueOf(fitWt.getText()));
					dp.setPreWeight(Integer.valueOf(fitWt.getText()));
					dp.setMinFitness(Float.valueOf(fitness.getText()));
					dp.setMinPrecision(Float.valueOf(precision.getText()));

					doAutoComplete(dp);
				} else {
					System.out.println("Login canceled");
				}
				//				String name = JOptionPane.showInputDialog(topCom, "Minimum Fitness?", null);

			}

		});

		topCom.add(new JLabel("Policy:"));
		topCom.add(switchRelationTypeBox);
		topCom.setPreferredSize(new Dimension(width, 23));
		topCom.setMinimumSize(new Dimension(width, 23));
		topCom.setBackground(Color.LIGHT_GRAY);

		topCom.add(autoComplete, gbc);

		//top comp
		setTopComponent(topCom);
	}

	private void updateActivityFromList(DiscoveryPredictor dp) {
		List<ListModel> values = list.getSelectedValuesList();
		String act = "";
		for (Object m : values) {
			int i = model.indexOf(m);
			act = model.get(i).toString();
		}
		if (!dp.getController().getActivity().equals(act)) {
			dp.getController().getActivityCombo().setSelectedItem(act);
			try {
				dp.addRelations(dp.getController().getIPN());
			} catch (ProjectedMeasuresFailedException | ReductionFailedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			//Do nothing
		}
	}

	private void doAutoComplete(DiscoveryPredictor dp) {
		try {
			int max = 0;
			Set<String> addedTrans = new HashSet<String>();
			for (Transition t : dp.getController().getIPN().getselectedPetriNetMap().getPetriNet().getTransitions())
				addedTrans.add(t.getLabel());
			for (String s : dp.getController().getIPN().getLogInfo().getActivities()) {
				if (!addedTrans.contains(s) && !dp.getSkipActs().contains(s))
					max++;
			}
			String str = "";
			int id=0;
			for (int i = 0; i < max; i++) {
				Random generator = new Random(); 
				
				PNetVisualizerController controller = dp.getController();
				dp.addRelations(dp.getController().getIPN());
				if (dp.incM.size() > 0) {
					id = generator.nextInt(dp.incM.size());
					float fitPre[] = Alignments.getTraditionalFitnessAndPrecision(dp.getController().getIPN()
							.getContext(), dp.getController().getIPN().getLog(), dp.incM.get(id));
					str += "\n" + ("Fitness : " + fitPre[0] + " Precision : " + fitPre[1]);
				}
				//always choose the first model and move on
				ExpandUsingTrackerForVizAndNet.expandUsingTracker(controller, dp.incM.get(id), true);
				//add the current activity as skippable- since its already added
				dp.getSkipActs().add(controller.getActivity());
				//selection of next activity  - based on policy
				dp.setControllerActivity(false);
				//update next activity

			}
			System.out.println(str);
		} catch (ProjectedMeasuresFailedException | ReductionFailedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
}
