package org.processmining.incrementaldiscovery;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.controllers.NodeHighlightControlPMDUtils;
import org.processmining.incrementaldiscovery.models.IncrementalModel;
import org.processmining.reduction.ReductionTracker;

import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class ReduceUsingTrackerForVizAndNet {

	/**
	 * @param controller
	 * @param m
	 * @param calcSynth
	 *            : decides if the addition is temporary or permanent -
	 *            calculate the SS or not
	 */
	public static void reduceUsingTracker(PNetVisualizerController controller, IncrementalModel m, boolean calcSynth,
			boolean runLayout) {
		if (!NodeHighlightControlPMDUtils.removeNodesNotInIPN(controller.getMainView().getNodeHighlightControlPMD()))
			return;
		//		NodeHighlightControlPMDUtils.removeNodesNotInIPN(controller.getMainView().getNodeHighlightControlPMD());
		List<ReductionTracker> synthesis = new ArrayList<ReductionTracker>(m.getTrackSynthesis());
		Collections.reverse(synthesis);
		controller.getMainView().getDisplay().requestFocus();
		int i = 0;
		synchronized (controller.getData()) {
			for (ReductionTracker rt : synthesis) {
				i++;
				switch (rt.getMethodName()) {
					case ABSTRACTION :
						reduceAbstraction(controller, rt, calcSynth, runLayout && synthesis.size() == i ? true : false);
						break;
					case LDPLACE :
						reduceLinearDependency(controller, rt, true, calcSynth,
								runLayout && synthesis.size() == i ? true : false);
						break;
					case LDTRAN :
						reduceLinearDependency(controller, rt, false, calcSynth,
								runLayout && synthesis.size() == i ? true : false);
						break;
					case RENAME :
						renameActivity(controller, rt, calcSynth);
						break;
					default :
						break;
				}
			}
		}
	}

	private static void renameActivity(PNetVisualizerController controller, ReductionTracker rt, boolean calcSynth) {
		int tranId = rt.getNewTranID();
		Iterator iter = controller.getVisualization().getVisualGroup("graph.nodes").tuples();
		NodeItem item = null;
		while (iter.hasNext()) {
			VisualItem n = (VisualItem) iter.next();
			if (n instanceof NodeItem && n.get("type") == "TRANSITION" && n.getInt("ID") == rt.getNewTranID()) {
				item = (NodeItem) n;
			}
		}
		if (item != null)
			NodeHighlightControlPMDUtils.renameActivity(item, controller.getMainView().getNodeHighlightControlPMD(),
					rt.isSilent() ? rt.getTranName() : "", calcSynth); //for reduction - reverse the original silence action
	}

	/**
	 * @param controller
	 * @param rt
	 * @param parallePlace
	 *            -is new place
	 * @param runLayout
	 */
	private static void reduceLinearDependency(PNetVisualizerController controller, ReductionTracker rt,
			boolean parallePlace, boolean calcSynth, boolean runLayout) {
		if (parallePlace)
			NodeHighlightControlPMDUtils.reduceLinearlyDependentNode(controller, rt.getNewPlaceID(), calcSynth, true,
					runLayout);
		else
			NodeHighlightControlPMDUtils.reduceLinearlyDependentNode(controller, rt.getNewTranID(), calcSynth, false,
					runLayout);
	}

	private static void reduceAbstraction(PNetVisualizerController controller, ReductionTracker rt, boolean calcSynth,
			boolean runLayout) {
		NodeHighlightControlPMDUtils.reduceAbstraction(controller.getMainView().getNodeHighlightControlPMD(),
				rt.getNewPlaceID(), rt.getNewTranID(), calcSynth, runLayout);

	}
}
