package org.processmining.incrementaldiscovery.processconfiguration;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.ListModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.deckfour.xes.model.XAttribute;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.incrementaldiscovery.DiscoveryPredictor;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ProcessConfigurability extends ProMSplitPane {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5516861929429518962L;
	ProMList<ListModel> list;
	DefaultListModel model;
	GridBagConstraints gbc = new GridBagConstraints();
	List<GlobalAttributeMap> allAttsMap;
	String currAtt = "";

	public ProMTable tableConfig;
	private DiscoveryPredictor dp;

	public ProcessConfigurability(final DiscoveryPredictor dp) {
		super(ProMSplitPane.VERTICAL_SPLIT);
		PNetVisualizerController controller = dp.getController();
		this.dp = dp;
		int width = controller.getStatisticsPanel().getWidth() * 2 / 5;
		int height = controller.getStatisticsPanel().getHeight() - 23;

		this.tableConfig = new ProMTable(dp.getTableModelConfig());

		setResizeWeight(0.0);
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setMaximumSize(new Dimension(width, 23));
		//		setPreferredSize(new Dimension(w, h * 100));

		//int all global attributes including case duration
		allAttsMap = GlobalAttributesUtils.getAllGlobalAtts(controller.getIPN().getLog());

		JPanel topComp = new JPanel();
		topComp.setLayout(new GridBagLayout());
		topComp.setBackground(Color.LIGHT_GRAY);
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridheight = 3;
		topComp.add(new JLabel("Case filtering based on the selected attribute types.", JLabel.RIGHT), gbc);
		gbc.gridx = 2;
		gbc.gridy = 0;
		JLabel helpTxt = new JLabel("<html><font color='red'><b><sup>?</sup></font></html>");
		helpTxt.setToolTipText("<html><p>Click on the buttons to select attribute values. <p><p> The log will be filtered to contain only the chosen attributes. <p><p> The <font color='green'>green</font> colored attribute buttons have at least one selected value. ");
		topComp.add(helpTxt, gbc);
		Set<String> allVals = new HashSet<String>();
		for (Entry<String, XAttribute> x : dp.getController().getIPN().getLog().get(0).getAttributes().entrySet()) {
			if (!x.getKey().contains("log") && !x.getKey().contains("concept"))
				allVals.add(x.getKey());
		}

		String[] relationTypes = new String[allVals.size() + 1];
		int i = 1;
		relationTypes[0] = "Select Global Trace Attribute";
		for (String s : allVals)
			relationTypes[i++] = s;
		final JComboBox switchRelationTypeBox = SlickerFactory.instance().createComboBox(relationTypes);
		switchRelationTypeBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				dp.setAttributeChosen(switchRelationTypeBox.getSelectedItem().toString());

			}
		});

		topComp.add(switchRelationTypeBox);
		topComp.setPreferredSize(new Dimension(width, 23));
		topComp.setMinimumSize(new Dimension(width, 23));

		JButton updateView = SlickerFactory.instance().createButton("Update View");
		updateView.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {

				dp.updateStats("");
			}
		});
		//		topComp.add(updateView);
		setTopComponent(topComp);
		JPanel botComp = new JPanel();
		botComp.setLayout(new GridBagLayout());
		gbc.gridx = 1;
		gbc.gridy = 0;
		tableConfig.setAutoCreateRowSorter(true);

		dp.getTableModelConfig().addColumn("Value");
		dp.getTableModelConfig().addColumn("Num cases");
		dp.getTableModelConfig().addColumn("Avg fitness");
		//		tableModelConfig.addColumn("Avg duration (hrs)");
		dp.getTableModelConfig().addColumn("Num of Perfectly fitting cases");
		//		tableModelConfig.addColumn("% Activities covered");
		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(tableConfig.getTable().getModel());
		TableRowSorter trs = new TableRowSorter(dp.getTableModelConfig());

		trs.setComparator(0, new IntComparator());

		//		tableConfig.setRowSorter(trs);
		tableConfig.setPreferredSize(new Dimension(width, height));
		tableConfig.setMinimumSize(new Dimension(width, height));
		botComp.add(tableConfig, gbc);
		botComp.setPreferredSize(new Dimension(width, height));
		botComp.setMinimumSize(new Dimension(width, height));

		botComp.setBackground(Color.LIGHT_GRAY);

		setBottomComponent(botComp);
	}

	class IntComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			Double int1 = (Double) o1;
			Double int2 = (Double) o2;
			return int1.compareTo(int2);
		}

		public boolean equals(Object o2) {
			return this.equals(o2);
		}
	}
}
