package org.processmining.prabhakardixit.alignments;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.decomposedreplayer.algorithms.DecomposedReplayAlgorithm;
import org.processmining.decomposedreplayer.parameters.DecomposedReplayParameters;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.log.utils.XUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.pnetreplayer.utils.TransEvClassMappingUtils;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;

public class DecomposedAlignments2 extends DecomposedReplayAlgorithm {

	public void decomposedAlignments(InteractivePetriNet ipn) {
		XLog log = ipn.getLog();
		PluginContext context = ipn.getPromContext();
		Petrinet ptnet = ipn.getselectedPetriNetMap().getPetriNet();
		AcceptingPetriNet apn = new AcceptingPetriNetImpl(ptnet);
		DecomposedReplayParameters parameters = new DecomposedReplayParameters(log, apn);
		TransEvClassMapping map = transitionToEventClassMapperByLabel(ipn.getLog(), ptnet, parameters.getClassifier(),parameters);
		parameters.setMapping(map);
		
		PNRepResult result = apply(context, log, apn, parameters);
	}

	private static TransEvClassMapping transitionToEventClassMapperByLabel(XLog log, final Petrinet net,
			XEventClassifier xEventClassifier, DecomposedReplayParameters parameters) {
		TransEvClassMapping map = new TransEvClassMapping(new XEventNameClassifier(), DUMMY);

		List<Transition> listTrans = new ArrayList<Transition>(net.getTransitions());
		Map<String, XEventClass> classes = new HashMap<String, XEventClass>();
		Set<XEventClass> eventClasses = new HashSet<XEventClass>();
		int index = 0;
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, xEventClassifier);
		for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
			classes.put(evt.getId().replace("+complete", ""), evt);
			eventClasses.add(evt);
		}
		eventClasses.add(XUtils.MOVEONMODELACTIVITY);
		TransEvClassMappingUtils.getInstance().getMapping(net, eventClasses, xEventClassifier);
		for (Transition transition : listTrans) {
			if (transition.isInvisible()) {
//				map.put(transition, DUMMY);
				map.put(transition, parameters.getMapping().get(transition));
			} else {
				XEventClass eClass = null;
				if (classes.containsKey(transition.getLabel()))
					eClass = classes.get(transition.getLabel());
				else {
					eClass = new XEventClass(transition.getLabel(), index);
					classes.put(transition.getLabel(), eClass);
					index++;
				}
				map.put(transition, eClass);
			}
		}
//		XEventClass eC = classes.get("c");
//		Transition t=null;
		
//		map.put(t, eC);
		return map;
	}

	// dummy event class (for unmapped transitions)
	public static final XEventClass DUMMY = new XEventClass("[invisible]", -1) {
		public boolean equals(Object o) {
			return this == o;
		}

		public int hashCode() {
			return System.identityHashCode(this);
		}
	};
}
