package org.processmining.prabhakardixit.alignments;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import nl.tue.astar.AStarException;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.manifestreplayer.PNManifestReplayer;
import org.processmining.plugins.petrinet.manifestreplayer.PNManifestReplayerParameter;
import org.processmining.plugins.petrinet.manifestreplayer.TransClass2PatternMap;
import org.processmining.plugins.petrinet.manifestreplayer.algorithms.IPNManifestReplayAlgorithm;
import org.processmining.plugins.petrinet.manifestreplayer.algorithms.PNManifestReplayerILPAlgorithm;
import org.processmining.plugins.petrinet.manifestreplayer.transclassifier.TransClass;
import org.processmining.plugins.petrinet.manifestreplayresult.Manifest;
import org.processmining.plugins.petrinet.replayer.algorithms.IPNReplayAlgorithm;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayer.annotations.PNReplayAlgorithm;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class ManifestAlignmentReplay {
	public static Manifest calculateNAlignment1(PluginContext context, XLog log, Petrinet apn, int maxTraceLength,
			boolean verbose, Set<Transition> tranSets) {
		boolean selectedActivities = false; //if no activities are selected then consider the entire net for analysis
		if (tranSets != null)
			selectedActivities = true;
		//		verbose=true;
		/*
		 * Get cost1 and cost2 corresponding to two activities (tran cost and
		 * event cost) set low costs for misaligned transitions/events (force
		 * good performers to perform good)
		 */
		//		Map<Transition, Integer> transCostMap = new HashMap<Transition, Integer>();
		//		Map<String, Integer> activityCostMap = new HashMap<String, Integer>();
		//		for (Transition t : tranSets)
		//			transCostMap.put(t, misAlignedTrans.contains(t) ? misAlignedTrans.count(t) : 0);
		//
		//		MapUtils.sortByValue(transCostMap, false);
		//		int i = 1;
		//		for (Transition t : transCostMap.keySet()) {
		//			if (transCostMap.get(t) != 0)
		//				transCostMap.put(t, i++); //this val would be subtracted
		//		}
		//
		//		for (Transition act : tranSets)
		//			activityCostMap.put(act.getLabel(),
		//					misAlignedActs.contains(act.getLabel()) ? misAlignedActs.count(act.getLabel()) : 0);
		//
		//		MapUtils.sortByValue(activityCostMap, false);
		//		i = 1;
		//		for (String act : activityCostMap.keySet()) {
		//			if (activityCostMap.get(act) != 0)
		//				activityCostMap.put(act, i++); //this val would be subtracted
		//		}

		//		Petrinet ptnet = apn.getPetriNet();

		Map<String, XEventClass> classes = new HashMap<String, XEventClass>();
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
		Set<XEventClass> evtClasses = new HashSet<XEventClass>();
		for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
			XEventClass evt1 = new XEventClass(evt.getId().replace("+complete", ""), evt.getIndex());
			evtClasses.add(evt1);
			classes.put((evt.getId().replace("+complete", "")), evt1);
			classes.put((evt.getId().replace("+COMPLETE", "")), evt1);
		}
		TransEvClassMapping oldMap = transitionToEventClassMapperByLabel(log, apn, classes);
		CostBasedCompleteParam parameter = new CostBasedCompleteParam(evtClasses, DUMMY, apn.getTransitions(),
				Integer.MAX_VALUE, 1);

		Map<TransClass, Integer> mapTranCost=new HashMap<TransClass, Integer>();
		Map<XEventClass, Integer> mapEvCost=new HashMap<XEventClass, Integer>();
		TransClass2PatternMap mapping=null;
		Marking[] finalMarking = new Marking[1];
		finalMarking[0] = PetrinetUtils.getFinalMarking(apn);

		PNManifestReplayerParameter param = new PNManifestReplayerParameter(mapTranCost, mapEvCost, mapping,
				Integer.MAX_VALUE - 1, PetrinetUtils.getInitialMarking(apn), finalMarking);

		
		
		logInfo.getTransitionClasses().getClasses().iterator();
		Set<String> trans = new HashSet<String>();
		for (Transition t : apn.getTransitions()) {
			
			if (!t.isInvisible() && ((selectedActivities && tranSets.contains(t)) || !selectedActivities)) {
				trans.add(t.toString());
//				mapTranCost.put(t, 5);
				parameter.getMapTrans2Cost().put(t, 5); // low costs are mig aligned trans
			} else
//				mapTranCost.put(t, 0);
				parameter.getMapTrans2Cost().put(t, 0);
		}

		for (XEventClass t : evtClasses) {
			parameter.getMapEvClass2Cost().remove(t);
			if (trans.contains(t.toString().replace("+COMPLETE", ""))) {
				mapEvCost.put(t, 5);
				parameter.getMapEvClass2Cost().put(t, 5);
			} else
				mapEvCost.put(t, 0);
				parameter.getMapEvClass2Cost().put(t, 0);
		}

		parameter.setGUIMode(false);
		parameter.setCreateConn(false);

		Marking initialMarking = new Marking();
		initialMarking.add(PetrinetUtils.getInitialPlace(apn));
	
		parameter.setMaxNumOfStates(2000);
		if (verbose) {
			System.out.println("log: ");
			for (XTrace trace : log) {
				for (XEvent event : trace) {
					System.out.print(event.getAttributes().get("concept:name") + ", ");
				}
				System.out.println();
			}

			System.out.println();
			//			System.out.println("mapping:                        "+oldMap);
			System.out.println("parameter.getMapTrans2Cost():   " + parameter.getMapTrans2Cost());
			System.out.println("parameter.getMapEvClass2Cost(): " + parameter.getMapEvClass2Cost());
		}
		Object[] parameters = new Object[] { parameter.getMapTrans2Cost(), parameter.getMaxNumOfStates(),
				parameter.getMapEvClass2Cost(), 1 };

		// select algorithm without ILP
		//		PNLogMatchInstancesReplayer replayer = new PNLogMatchInstancesReplayer();
		//		NBestOptAlignmentsGraphSamplingAlg alg = new NBestOptAlignmentsGraphSamplingAlg();

		//		PNLogReplayer replayer2 = new PNLogReplayer();
		PNManifestReplayer replayer2 = new PNManifestReplayer();
		//		PNMatchInstancesRepResult pnRepResult = null;
		Manifest result = null;
		try {

//			IPNReplayAlgorithm replayer = new PetrinetReplayerWithILP();
			IPNManifestReplayAlgorithm replayer = new PNManifestReplayerILPAlgorithm(); 
					
			//			IPNReplayAlgorithm[] availAlgorithms = getAvailabelAlgorithms(context, apn, log, oldMap);
			parameter.setFinalMarkings(finalMarking);
			parameter.setInitialMarking(initialMarking);
			result = replayer2.replayLogParameter(context, apn, log, replayer, param);
			

		} catch (AStarException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}

		return result;
	}

	public static TransEvClassMapping transitionToEventClassMapperByLabel(final XLog log, final Petrinet net,
			Map<String, XEventClass> classes) {
		TransEvClassMapping map = new TransEvClassMapping(new XEventNameClassifier(), DUMMY);
		if (classes == null) {
			classes = new HashMap<String, XEventClass>();
			XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
			for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
				XEventClass evt1 = new XEventClass(evt.getId().replace("+complete", ""), evt.getIndex());
				classes.put((evt.getId().replace("+complete", "")), evt1);
				classes.put((evt.getId().replace("+COMPLETE", "")), evt1);
			}
		}
		List<Transition> listTrans = new ArrayList<Transition>(net.getTransitions());
		//		Map<String, XEventClass> classes = new HashMap<String, XEventClass>();
		int index = 0;
		for (Transition transition : listTrans) {
			if (transition.isInvisible() || !classes.containsKey(transition.getLabel())) {
				map.put(transition, DUMMY);
			} else {
				XEventClass eClass = null;
				if (classes.containsKey(transition.getLabel()))
					eClass = classes.get(transition.getLabel());
				//				else {
				//					eClass = new XEventClass(transition.getLabel(), index);
				//					classes.put(transition.getLabel(), eClass);
				//					index++;
				//				}
				map.put(transition, eClass);
			}
		}
		return map;
	}

	// dummy event class (for unmapped transitions)
	public static final XEventClass DUMMY = new XEventClass("[invisible]", -1) {
		public boolean equals(Object o) {
			return this == o;
		}

		public int hashCode() {
			return System.identityHashCode(this);
		}
	};

	protected static IPNReplayAlgorithm[] getAvailabelAlgorithms(PluginContext context, final PetrinetGraph net,
			final XLog log, final TransEvClassMapping mapping) {
		// get all algorithms from the framework
		Set<Class<?>> coverageEstimatorClasses = context.getPluginManager().getKnownClassesAnnotatedWith(
				PNReplayAlgorithm.class);
		IPNReplayAlgorithm[] availAlgorithms = null;
		if (coverageEstimatorClasses != null) {
			List<IPNReplayAlgorithm> algList = new LinkedList<IPNReplayAlgorithm>();
			for (Class<?> coverClass : coverageEstimatorClasses) {
				try {
					IPNReplayAlgorithm alg = (IPNReplayAlgorithm) coverClass.newInstance();
					if (alg.isReqWOParameterSatisfied(context, net, log, mapping)) {
						algList.add(alg);
					}
				} catch (InstantiationException e1) {
					// do nothing
				} catch (IllegalAccessException e1) {
					// do nothing
				} catch (Exception exc) {
					// do nothing
				}
			}
			Collections.sort(algList, new Comparator<IPNReplayAlgorithm>() {

				public int compare(IPNReplayAlgorithm o1, IPNReplayAlgorithm o2) {
					return o1.toString().compareTo(o2.toString());
				}
			});
			availAlgorithms = algList.toArray(new IPNReplayAlgorithm[algList.size()]);

		}
		return availAlgorithms;
	}

	//	public static Map<String, XEventClass> getClassMap(XLog log){
	//		Map<String, XEventClass> classes = new HashMap<String, XEventClass>();
	//		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
	//		Set<XEventClass> evtClasses = new HashSet<XEventClass>();
	//		for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
	//			XEventClass evt1 = new XEventClass(evt.getId().replace("+complete", ""), evt.getIndex());
	//			evtClasses.add(evt1);
	//			classes.put((evt.getId().replace("+complete", "")), evt1);
	//			classes.put((evt.getId().replace("+COMPLETE", "")), evt1);
	//		}
	//		return classes;
	//	}

}
