package org.processmining.prabhakardixit.alignments;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ejml.simple.SimpleMatrix;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

import com.google.common.collect.HashMultiset;
import com.google.common.collect.Multiset;

public class NavigateEmptyTraces {

	public static Multiset<Transition> getModelMoves(ClusterNet cnet, Multiset<Transition> traceMoveMultiset) {
		Multiset<Transition> syncTran = HashMultiset.create();
		Multiset<Transition> syncTran2 = HashMultiset.create();
		Multiset<Transition> execTrans = HashMultiset.create();
		Set<Place> marking = new HashSet<Place>();

		//				System.out.println("");
		//get mapped sync trans
		for (Transition t : traceMoveMultiset) {
			syncTran.add((cnet.getNewMappedTran(t)));
			syncTran2.add((cnet.getNewMappedTran(t)));
		}

		marking.add(PetrinetUtils.getInitialPlace(cnet.getPetriNet()));
		recurseMarkings(cnet, marking, execTrans, syncTran2);

		Multiset<Transition> modelMoves = HashMultiset.create(execTrans);
		for (Transition tran : execTrans) {
			modelMoves.add(tran);
			modelMoves.addAll(cnet.getTranCluster(tran));
		}
		modelMoves.removeAll(syncTran);

		//get original map
		Multiset<Transition> modelMovesTemp = HashMultiset.create(modelMoves);
		for (Transition t : modelMovesTemp.elementSet()) {
			modelMoves.add(cnet.getOriginalMappedTran(t), modelMoves.count(t));
			modelMoves.remove(t);
		}

		return modelMoves;
	}

	public static ClusterNet reduceNetWithClusters(PetriNetMap pnmap) {
		ClusterNet cnet = new ClusterNet(pnmap);
		boolean reduced = true;
		while (reduced) {
			reduced = useAbstractionRule(pnmap.getPetriNet(), cnet);
			if (!reduced)
				reduced = useLDPlaceRule(pnmap.getPetriNet());
		}
		return cnet;
	}

	private static boolean useAbstractionRule(Petrinet pn, ClusterNet cnet) {
		for (Transition t : pn.getTransitions()) {
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = pn.getInEdges(t);
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = pn.getOutEdges(t);
			boolean prevPlaceIsChoiceOrMoreThanOneIncomingArcs = false;
			//if prev place is choice split then cannot apply abstraction
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : inEdges)
				if (pn.getOutEdges(e.getSource()).size() > 1 || pn.getInEdges(e.getSource()).size() == 0) //if choice or init place
					prevPlaceIsChoiceOrMoreThanOneIncomingArcs = true;

			if (pn.getInEdges(t).size() > 1)
				prevPlaceIsChoiceOrMoreThanOneIncomingArcs = true; //if more than one incoming edges, then do not use abstraction
			//reduce using abstraction rule
			if (!prevPlaceIsChoiceOrMoreThanOneIncomingArcs) {
				pn.removeNode(t);
				Set<Transition> preTranSet = new HashSet<Transition>(); //Pre tran set of the pre place of t
				Set<Place> postPlaceSet = new HashSet<Place>(); //Post place sets of t
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : inEdges) {
					Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges2 = pn.getInEdges(e
							.getSource());
					for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : inEdges2) {
						preTranSet.add((Transition) e2.getSource());

					}
					pn.removeNode(e.getSource());
					pn.removeArc(e.getSource(), e.getTarget());

				}
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : outEdges) {
					postPlaceSet.add((Place) e.getTarget());
					pn.removeArc(e.getSource(), e.getTarget());

				}

				for (Transition preTran : preTranSet) {
					for (Place postPlace : postPlaceSet) {
						pn.addArc(preTran, postPlace);
					}

					//add clusters to cnets
					Set<Transition> tranCluster = new HashSet<Transition>(cnet.getTranCluster(t));
					tranCluster.add(t);
					cnet.setTranCluster(preTran, tranCluster);
				}

				return true;
			}

		}
		return false;

	}

	private static boolean useLDPlaceRule(Petrinet pn) {
		Map<Transition, Integer> tranInciMap = new HashMap<Transition, Integer>();
		Map<Place, Integer> placeInciMap = new HashMap<Place, Integer>();
		SimpleMatrix inciMat = MatrixUtils.createInciMap(tranInciMap, placeInciMap, pn);
		SimpleMatrix augMat3 = MatrixUtils.getSCIncidenceMatrix2(inciMat, placeInciMap, pn);

		for (Place p : pn.getPlaces()) {
			if (pn.getInEdges(p).size() < 1) //if init place then continue
				continue;

			int inciIndexOfpNew = placeInciMap.get(p);
			SimpleMatrix inciMatWithoutNewLD = new SimpleMatrix(augMat3.numCols(), augMat3.numRows() - 1);
			SimpleMatrix ldCol = new SimpleMatrix(augMat3.numCols(), 1);
			MatrixUtils.splitInciMat(augMat3.transpose(), inciMatWithoutNewLD, ldCol, inciIndexOfpNew);
			if (MatrixUtils.isAugMatLD(inciMatWithoutNewLD, ldCol)) {

				Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = pn.getInEdges(p);
				Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = pn.getOutEdges(p);
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : inEdges) {

					pn.removeArc(e.getSource(), e.getTarget());
				}
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : outEdges) {

					pn.removeArc(e.getSource(), e.getTarget());
				}
				pn.removePlace(p);

				return true;
			}
		}
		//performance starts
		return false;
	}

	private static void execTran(Transition enabledTran, Petrinet pn, Set<Place> marking) {
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = pn.getInEdges(enabledTran);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : edges) {
			marking.remove(e.getSource());
		}
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges2 = pn.getOutEdges(enabledTran);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : edges2) {
			marking.add((Place) e.getTarget());
		}
	}

	//returns false if final marking reached
	public static void recurseMarkings(ClusterNet cnet, Set<Place> marking, Multiset<Transition> execTrans,
			Multiset<Transition> syncTran2) {
		Petrinet pn = cnet.getPetriNet();

		Set<Transition> enabledTransitions = PetrinetUtils.getEnabledTransitions(pn, marking);
		if (enabledTransitions.isEmpty()) //no tran enabled 
			return;

		/*
		 * check which enabled trans maximally contain the syncMoves if there
		 * are two sets - without complete overlaps, then keep both to be
		 * executed
		 */
		//		Set<Transition> transToExec = new HashSet<Transition>(enabledTransitions);
		Map<Transition, Set<Transition>> commonTransMap = new HashMap<Transition, Set<Transition>>();
		for (Transition et : enabledTransitions) {
			Set<Transition> commonTrans = new HashSet<Transition>(syncTran2);
			commonTrans.retainAll(cnet.getTranCluster(et));
			commonTransMap.put(et, commonTrans);
		}

		Set<Transition> transToExec = new HashSet<Transition>();
		Transition firstTran = null;
		for (Transition et : enabledTransitions) {
			if (transToExec.isEmpty()) {
				firstTran = et;
				transToExec.add(et);
			}
			Set<Transition> transToRetainTemp = new HashSet<Transition>(transToExec);
			for (Transition et2 : transToRetainTemp) {
				if (!et.equals(et2)) {
					Set<Transition> commonTrans1 = commonTransMap.get(et);
					Set<Transition> commonTrans2 = commonTransMap.get(et2);
					if (commonTrans2.containsAll(commonTrans1)) {
						transToExec.remove(et);
						transToExec.add(et2);
					} else if (Collections.disjoint(commonTrans1, commonTrans2)) {
						transToExec.add(et);
					}
				}
			}
		}

		/*
		 * if no cluster contains sync, then choose a tran priority to invi
		 * followed by least model moves
		 */
		if (transToExec.size() == 1 && transToExec.contains(firstTran) && commonTransMap.get(firstTran).size() < 1) {
			for (Transition et : enabledTransitions) {
				Transition tTemp = null;
				for (Transition t : transToExec) {
					tTemp = t;
				}
				if (transToExec.isEmpty())
					if (cnet.getTranCluster(et).size() < cnet.getTranCluster(tTemp).size()
							|| (cnet.getTranCluster(et).size() == cnet.getTranCluster(tTemp).size() && et.isInvisible())) {
						transToExec.clear();
						transToExec.add(et);
					}
			}
		}
		for (Transition enabledTran : transToExec) {
			/*
			 * Remove the already executed transitions (unfold and remove)
			 */
			syncTran2.remove(enabledTran, 1); //remove the already executed trans
			for (Transition t : cnet.getTranCluster(enabledTran)) {
				syncTran2.remove(t, 1);
			}
			if (execTrans.contains(enabledTran)) //TODO check if this is needed
				continue;
			execTrans.add(enabledTran);
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pn.getInEdges(enabledTran))
				marking.remove(edge.getSource());
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pn.getOutEdges(enabledTran))
				marking.add((Place) edge.getTarget());
			recurseMarkings(cnet, marking, execTrans, syncTran2);
		}
		return;
	}

	public static Multiset<Transition> getModelMovesUsingMarkingEq(ClusterNet cnet, Multiset<Transition> traceMoveMultiset, int numPairs) {
		Map<Transition, Integer> tranInciMap = new HashMap<Transition, Integer>();
		Map<Place, Integer> placeInciMap = new HashMap<Place, Integer>();
		SimpleMatrix inciMat = MatrixUtils.createInciMap(tranInciMap, placeInciMap, cnet.getPetriNet());
		
		
		Multiset<Transition> syncTran = HashMultiset.create();
		Multiset<Transition> syncTran2 = HashMultiset.create();
		Multiset<Transition> execTrans = HashMultiset.create();
		Set<Place> marking = new HashSet<Place>();

		//get mapped sync trans
		for (Transition t : traceMoveMultiset) {
			syncTran.add((cnet.getNewMappedTran(t)));
			syncTran2.add((cnet.getNewMappedTran(t)));
		}

		marking.add(PetrinetUtils.getInitialPlace(cnet.getPetriNet()));
		recurseMarkings(cnet, marking, execTrans, syncTran2);

		Multiset<Transition> modelMoves = HashMultiset.create(execTrans);
		for (Transition tran : execTrans) {
			modelMoves.add(tran);
			modelMoves.addAll(cnet.getTranCluster(tran));
		}
		modelMoves.removeAll(syncTran);

		//get original map
		Multiset<Transition> modelMovesTemp = HashMultiset.create(modelMoves);
		for (Transition t : modelMovesTemp.elementSet()) {
			modelMoves.add(cnet.getOriginalMappedTran(t), modelMoves.count(t));
			modelMoves.remove(t);
		}
		return modelMovesTemp;

	}
}
