package org.processmining.prabhakardixit.alignments;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.ejml.simple.SimpleMatrix;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetImpl;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.pnalignanalysis.conformance.AlignmentPrecGen;
import org.processmining.plugins.pnalignanalysis.conformance.AlignmentPrecGenRes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.plugins.ComparePetriNet2PetriNetPlugin;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;
import org.processmining.reduction.Reduction;
import org.processmining.reduction.ReductionUtils;

public class UpdateProjectedAveragedAlignments {

	public static long totalLogTime = 0;
	public static long totalReplayTime = 0;
	public static long totalPrecTime = 0;

	public static void addSeqAlignmentCal(InteractivePetriNet ipn, Transition tNew) {
		if (tNew.isInvisible())
			return;
		Petrinet pn = ipn.getselectedPetriNetMap().getPetriNet();

		Set<TransPairAlignment> newTranAlignments = new HashSet<TransPairAlignment>();

		/*
		 * If tNew labelled activity is already present in the net, then remove
		 * from aggregated info (as these would be re-cauclated)
		 */

		//get all duplicates
		Set<String> duplicateAct = getDuplicateActs(pn);

		//if tNew already present, then remove it 
		if (duplicateAct.contains(tNew.getLabel())) {
			//			Set<TransPairAlignment> transPairAlignmentsWithTNew = getTransPairsWithTNew(ipn, tNew); //remove these from aggregate and pairs
			//			removeFromAggregateAndPairs(ipn, transPairAlignmentsWithTNew);
			ipn.getTranPairs().removeAll(getTransPairsWithTNew(ipn, tNew));
		}

		//calculate all alignments
		calculateAlignments(newTranAlignments, ipn, tNew);
		//		System.out.println("size : " + ipn.getTranPairs().size());
	}

	public static void addLinearlyDependentCal(InteractivePetriNet ipn, Transition tNew) {
		Set<TransPairAlignment> newTranAlignments = new HashSet<TransPairAlignment>();
		for (TransPairAlignment tpa : ipn.getTranPairs()) {
			float[] fitPre = new float[2];
			ComparePetriNet2PetriNetPlugin compare = new ComparePetriNet2PetriNetPlugin();
			CompareParameters param = new CompareParameters();
			Set<String> act = tpa.getActivities();
			PetriNetMap apn = new PetriNetMap(ipn.getselectedPetriNetMap().getPetriNet());
			Set<PetrinetNode> transToKeep = new HashSet<PetrinetNode>();
			for (Transition t : apn.getPetriNet().getTransitions())
				if(tpa.getActivities().contains(t.getLabel()))
					transToKeep.add(t);
//				if (!tpa.getActivities().contains(t.getLabel()))
//					t.setInvisible(true);
			
			Reduction rd = new Reduction(apn.getPetriNet(), transToKeep);
			
			//				else
			//					System.out.println(t.isInvisible());

			//			int i = 1;
			//			for (Transition t : apn.getPetriNet().getTransitions())
			//				if (!t.isInvisible())
			//					i++;
			//
			//			System.out.println("Total count : " + i);
			//
			//			i = 0;
			//			for (Transition t : tpa.getPetriNetMap().getPetriNet().getTransitions())
			//				if (!t.isInvisible())
			//					i++;
			//			if (i < 1)
			//				System.out.println("Total count : " + i);

			param.setK(act.size() > 1 ? 2 : 1);
			ProjectedRecallPrecisionResult result = null;
			try {
				try {
					//					if (param.getK() > 1) {
					result = compare.measure(new AcceptingPetriNetImpl(apn.getPetriNet()), new AcceptingPetriNetImpl(
							tpa.getPetriNetMap().getPetriNet()), param, new ProMCanceller() {

						public boolean isCancelled() {
							// TODO Auto-generated method stub
							return false;
						}

					});
					if (result.getRecall() != 1. || result.getPrecision() != 1.) {
						newTranAlignments.add(tpa);
					}
					//					}
				} catch (ReductionFailedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} catch (AutomatonFailedException | InterruptedException | ProjectedMeasuresFailedException e) {
				e.printStackTrace();
			}

		}
		calculateAlignments(newTranAlignments, ipn, tNew);
	}

	//non loop choices
	public static void addChoiceAlignmentCal(InteractivePetriNet ipn, Transition tNew) {
		int card = 2; //TODO take card as input
		Petrinet pn = ipn.getselectedPetriNetMap().getPetriNet();
		long msecs = -System.currentTimeMillis();
		//use non-sc net - only loop back or choice sets without sc
		Map<Transition, Integer> tranInciMap = new HashMap<Transition, Integer>();
		Map<Place, Integer> placeInciMap = new HashMap<Place, Integer>();
		Set<TransPairAlignment> newTranAlignments = new HashSet<TransPairAlignment>();
		SimpleMatrix inciMat = MatrixUtils.createInciMap(tranInciMap, placeInciMap, ipn.getselectedPetriNetMap()
				.getPetriNet());
		SimpleMatrix augMat3 = inciMat.combine(0, inciMat.numCols(), new SimpleMatrix(inciMat.numRows(), 1));
		//MatrixUtils.getSCIncidenceMatrix2(inciMat, placeInciMap, ipn.getselectedPetriNetMap()
		//				.getPetriNet());
		int inciIndexOftNew = tranInciMap.get(tNew);
		SimpleMatrix inciMatWithoutNewLD = new SimpleMatrix(augMat3.numRows(), augMat3.numCols() - 1);
		SimpleMatrix ldCol = new SimpleMatrix(augMat3.numRows(), 1);
		MatrixUtils.splitInciMat(augMat3, inciMatWithoutNewLD, ldCol, inciIndexOftNew);
		PetriNetMap pnmap = new PetriNetMap(ipn.getselectedPetriNetMap());
		Set<Set<Integer>> ldCols = ReductionUtils.getPossibleDistinctPathsInBetween(inciMatWithoutNewLD, ldCol,
				inciIndexOftNew, false);
		ldCols.addAll(ReductionUtils.getPossibleDistinctPathsInBetween(inciMatWithoutNewLD, ldCol, inciIndexOftNew,
				true));//maximize for loop
		msecs += System.currentTimeMillis();
		System.out.println(" Time taken for reduction : " + msecs);
		/*
		 * Check how many escape routes exists, and check to see if these are
		 * one higher than card already, then no need to do anything for the
		 * pre-exsiting trans
		 */
		Set<Place> inPlaces = new HashSet<Place>();
		Set<Place> outPlaces = new HashSet<Place>();

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> ed : pn.getInEdges(tNew)) {
			inPlaces.add((Place) ed.getSource());
			//			System.out.println("Input place : " + ed.getSource().getLabel());
		}
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> ed : pn.getOutEdges(tNew)) {
			outPlaces.add((Place) ed.getTarget());
			//			System.out.println("Output place : " + ed.getTarget().getLabel());
		}

		if (ldCols.size() < card + 1 && !inPlaces.equals(outPlaces)) {
			Set<Integer> mappedIds = new HashSet<Integer>();
			for (Set<Integer> ldColSet : ldCols) {
				mappedIds.addAll(ldColSet);
			}

			//get the original id from ipn
			Set<Integer> allTransInBw = new HashSet<Integer>();
			for (Transition t2 : tranInciMap.keySet())
				if (mappedIds.contains(tranInciMap.get(t2))) {
					allTransInBw.add(ipn.getTranId(t2));
				}

			boolean changed = true;
			msecs = -System.currentTimeMillis();

			//TODO PMD changed calculator - use - reduced net comparison approach 
			//			while (changed) {
			//				changed = false;
			//				for (Set<Integer> set1 : ipn.getMapLDTrans().keySet()) {
			//					if (allTransInBw.containsAll(set1) || allTransInBw.equals(set1))
			//						if (!allTransInBw.containsAll(ipn.getMapLDTrans().get(set1))) {
			//							allTransInBw.addAll(ipn.getMapLDTrans().get(set1));
			//							changed = true;
			//						} else if (allTransInBw.containsAll(ipn.getMapLDTrans().get(set1))
			//								|| allTransInBw.equals(ipn.getMapLDTrans().get(set1)))
			//							if (!allTransInBw.containsAll(set1)) {
			//								changed = true;
			//								allTransInBw.addAll(set1);
			//							}
			//
			//				}
			//			}
			msecs += System.currentTimeMillis();
			System.out.println(" Time taken for getting all map lds : " + msecs);
			msecs = -System.currentTimeMillis();

			//first get all in between activity label map
			Set<String> allActInBw = new HashSet<String>();

			//			Set<Transition> allTransitionsInBw = new HashSet<Transition>();
			for (Integer i : allTransInBw)
				//this is already the id of the mapped tran
				if (!ipn.getTranBasedOnID(i).isInvisible())
					allActInBw.add(ipn.getTranBasedOnID(i).getLabel());

			Set<String> duplicateActs = getDuplicateActs(ipn.getselectedPetriNetMap().getPetriNet());

			boolean duplicateActsInBw = !Collections.disjoint(allActInBw, duplicateActs);

			//			System.out.println(" All in between aactivities : " + allActInBw);
			/*
			 * Navigate through all pairs and if fitness is not 1 for set of
			 * transitions which are not all in between or not all outside
			 */
			for (TransPairAlignment tpa : ipn.getTranPairs()) {
				//				!allActInBw.containsAll(tpa.getActivities()) &&		//condition exluded to check incorrect removal of choices for in bw
				if ((!Collections.disjoint(allActInBw, tpa.getActivities()))
						|| (duplicateActsInBw && !Collections.disjoint(duplicateActs, tpa.getActivities()))) {
					if (tpa.getFitness() != 1.) {
						//recalculate alignments for this tran pair
						//						PetriNetMap pnmap2 = new PetriNetMap(ipn.getselectedPetriNetMap().getPetriNet());
						//						Set<PetrinetNode> transToKeep = new HashSet<PetrinetNode>();
						//						for (Transition t : pnmap2.getPetriNet().getTransitions())
						//							if (tpa.getActivities().contains(t.getLabel()))
						//								transToKeep.add(t);

						//Reduction rd = new Reduction(pnmap2.getPetriNet(), transToKeep);
						//								ReductionPlugin().reduceNet(ipn.getPromContext(), pnmap2.getPetriNet(),
						//								transToKeep);
						//tpa.setPetriNetMap(pnmap2);
						newTranAlignments.add(tpa);
					}
				}
			}
			msecs += System.currentTimeMillis();
			System.out.println(" Time taken for Reduction of all map lds : " + msecs);
			msecs = -System.currentTimeMillis();
		}

		calculateAlignments(newTranAlignments, ipn, tNew);
		msecs += System.currentTimeMillis();
		System.out.println(" Time taken for alignements : " + msecs + " with size of pairs : "
				+ newTranAlignments.size());
		msecs = -System.currentTimeMillis();
		//		System.out.println("size : " + ipn.getTranPairs().size());
	}

	//	private static void removeModelMovesForNonDuplAct(InteractivePetriNet ipn,
	//			Set<TransPairAlignment> nonDuplRemoveModelMoves, Set<String> nonDuplInBwTrans) {
	//		for (TransPairAlignment tranPair : nonDuplRemoveModelMoves) {
	//			Set<Transition> tranSet = new HashSet<Transition>();
	//			for (Transition t : tranPair.getPetriNetMap().getPetriNet().getTransitions())
	//				if (t.getLabel().equals(tranPair.getAct1()) || t.getLabel().equals(tranPair.getAct2()))
	//					tranSet.add(t);
	//			for (int i = 0; i < tranPair.getReplayResult().toArray().length; i++) {
	//				SyncReplayResult sync = (SyncReplayResult) tranPair.getReplayResult().toArray()[i];
	//				AggregatedTraceLevelPairwiseInfo tlpi = ipn.getPairwiseAlignmentInfoBasedOnIds(sync.getTraceIndex());
	//				for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
	//					StepTypes type = sync.getStepTypes().get(actIndex);
	//					Object stepObj = sync.getNodeInstance().get(actIndex);
	//					if (nonDuplInBwTrans.contains(stepObj.toString()))
	//						switch (type) {
	//							case MREAL :
	//								tlpi.getModelMoves().remove(tranPair.getMappedTransition((Transition) stepObj), 1);
	//								break;
	//							case L :
	//							case LMGOOD :
	//							case MINVI :
	//							case LMNOGOOD :
	//							case LMREPLACED :
	//							case LMSWAPPED :
	//								break;
	//						}
	//				}
	//			}
	//		}
	//	}

	public static Set<String> getDuplicateActs(Petrinet petriNet) {
		Set<String> actsInNet = new HashSet<String>();
		Set<String> duplAct = new HashSet<String>();
		for (Transition t : petriNet.getTransitions()) {
			if (!t.isInvisible())
				if (!actsInNet.add(t.getLabel()))
					duplAct.add(t.getLabel());
		}
		return duplAct;
	}

	private static int getNbrOfIp(ClusterNet cnet, Transition tNewCnet) {
		int sameIpOp = 0;
		Place pIn = null;
		Place pOut = null;
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : cnet.getPetriNet().getInEdges(tNewCnet))
			pIn = (Place) e.getSource();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : cnet.getPetriNet().getOutEdges(tNewCnet))
			pOut = (Place) e.getTarget();

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : cnet.getPetriNet().getOutEdges(pIn))
			if (!((Transition) e.getTarget()).equals(tNewCnet)) {
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : cnet.getPetriNet().getOutEdges(
						e.getTarget())) {
					if (e2.getTarget().equals(pOut)) {
						sameIpOp++;
					}
				}
			}
		return sameIpOp;
	}

	private static Set<TransPairAlignment> getTransPairsWithTNew(InteractivePetriNet ipn, Transition tNew) {
		Set<TransPairAlignment> transPairAlignments = new HashSet<TransPairAlignment>();
		for (TransPairAlignment tpa : ipn.getTranPairs()) {
			if (tpa.containsActivity(tNew.getLabel()))
				transPairAlignments.add(tpa);
		}
		return transPairAlignments;
	}

	private static Set<String> getAllActsInNet(InteractivePetriNet ipn) {
		Set<String> allActivities = new HashSet<String>();
		for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
			if (!t.isInvisible())
				allActivities.add(t.getLabel());
		return allActivities;
	}

	//	private static void removeFromAggregateAndPairs(InteractivePetriNet ipn, Set<TransPairAlignment> transPairsToRemove) {
	//		for (TransPairAlignment tranPair : transPairsToRemove) {
	//			Set<Transition> tranSet = new HashSet<Transition>();
	//			for (Transition t : tranPair.getPetriNetMap().getPetriNet().getTransitions())
	//				if (t.getLabel().equals(tranPair.getAct1()) || t.getLabel().equals(tranPair.getAct2()))
	//					tranSet.add(t);
	//			for (int i = 0; i < tranPair.getReplayResult().toArray().length; i++) {
	//				SyncReplayResult sync = (SyncReplayResult) tranPair.getReplayResult().toArray()[i];
	//				AggregatedTraceLevelPairwiseInfo tlpi = ipn.getPairwiseAlignmentInfoBasedOnIds(sync.getTraceIndex());
	//				for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
	//					StepTypes type = sync.getStepTypes().get(actIndex);
	//					Object stepObj = sync.getNodeInstance().get(actIndex);
	//					if (tranPair.containsActivity(stepObj.toString()))
	//						switch (type) {
	//							case L :
	//								tlpi.getLogMoves().remove(stepObj.toString(), 1);
	//								break;
	//							case LMGOOD :
	//								tlpi.getSyncMoves().remove(tranPair.getMappedTransition((Transition) stepObj), 1);
	//								break;
	//							case MINVI :
	//								break;
	//							case MREAL :
	//								tlpi.getModelMoves().remove(tranPair.getMappedTransition((Transition) stepObj), 1);
	//								break;
	//							case LMNOGOOD :
	//							case LMREPLACED :
	//							case LMSWAPPED :
	//								// unable to project this case, stop
	//						}
	//				}
	//			}
	//		}
	//		ipn.getTranPairs().removeAll(transPairsToRemove);
	//	}

	public static void calculateAlignments(Set<TransPairAlignment> newTranAlignments, InteractivePetriNet ipn,
			Transition tNew) {

		//				System.out.println(" new tran size : " + newTranAlignments.size());
		Set<String> allActInNet = getAllActsInNet(ipn);
		Set<String> duplAct = getDuplicateActs(ipn.getselectedPetriNetMap().getPetriNet());
		boolean isDupl = false;
		if (!(tNew == null && !newTranAlignments.isEmpty())) {
			if (duplAct.contains(tNew.getLabel()))
				isDupl = true;
			if (tNew.isInvisible() && newTranAlignments.isEmpty())
				return;

			//create new tpa set for new act 
			if (!isDupl) {
				if (!tNew.isInvisible())
					for (String act : allActInNet) {
						if (!act.equals(tNew.getLabel())) {
							TransPairAlignment tranPair = new TransPairAlignment();
							tranPair.put(act);
							tranPair.put(tNew.getLabel());
							newTranAlignments.add(tranPair);
						}
					}
			} else { //use the pre existing tpas
				for (TransPairAlignment tranPair : ipn.getTranPairs())
					if (tranPair.getActivities().contains(tNew.getLabel()))
						newTranAlignments.add(tranPair);
			}
		}
		for (TransPairAlignment tranPair : newTranAlignments) {

			//subtract from aggr
			//subtract sync moves first
			for (Transition t : tranPair.getSyncMoveMap().keySet())
				ipn.subFromAggrSyncMoves(t, tranPair.getSyncMoveMap().get(t));

			//subtract log moves
			for (String l : tranPair.getLogMoveMap().keySet())
				ipn.subFromAggrLogMoves(l, tranPair.getLogMoveMap().get(l));

			//subtract model moves
			for (Transition t : tranPair.getModelMove().keySet())
				ipn.subFromAggrModelMoves(t, tranPair.getModelMove().get(t));

			//reset tranPair counters
			tranPair.getSyncMoveMap().clear();
			tranPair.getLogMoveMap().clear();
			tranPair.getModelMove().clear();

			//create new petri net object corresponding to each pair, and reduce it 
			Set<PetrinetNode> transToKeep = new HashSet<PetrinetNode>();
			PetriNetMap pnmap = new PetriNetMap(ipn.getselectedPetriNetMap().getPetriNet());
			for (Transition t : pnmap.getPetriNet().getTransitions())
				if (tranPair.getActivities().contains(t.getLabel()))// || PetrinetUtils.initialOrFinalTran(pnmap.getPetriNet(), t))
					transToKeep.add(t);

			//reduce petrinet now
			Reduction rd = new Reduction(pnmap.getPetriNet(), transToKeep);
			tranPair.setPetriNetMap(pnmap);

			//calculate alignments and precision
			Set<Transition> tranSet = new HashSet<Transition>();
			long msecs = System.currentTimeMillis();
			XLog log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), tranPair.getActivities(), true);
			msecs = System.currentTimeMillis() - msecs;
			totalLogTime += msecs;
			for (Transition t : tranPair.getPetriNetMap().getPetriNet().getTransitions())
				if (tranPair.containsActivity(t.getLabel())) {
					tranSet.add(t);
					//			System.out.println("Transitions : " + t.getLabel());
				} else
					//set all other acts invi
					t.setInvisible(true);
			//calculate alignments
			msecs = System.currentTimeMillis();
			PNRepResult replay = AlignmentCalculations.calculateNAlignment1(ipn.getPromContext(), log, tranPair
					.getPetriNetMap().getPetriNet(), 5, false, tranSet);
			msecs = System.currentTimeMillis() - msecs;
			totalReplayTime += msecs;
			tranPair.setReplayResult(replay);
			tranPair.setFitness((double) replay.getInfo().get(PNRepResult.TRACEFITNESS));

			//calculate precision
			msecs = System.currentTimeMillis();
			AlignmentPrecGen precGen = new AlignmentPrecGen();
			AlignmentPrecGenRes precRes = precGen.measureConformanceAssumingCorrectAlignment(ipn.getPromContext(),
					AlignmentCalculations.transitionToEventClassMapperByLabel(log, tranPair.getPetriNetMap()
							.getPetriNet(), null,false), replay, tranPair.getPetriNetMap().getPetriNet(), PetrinetUtils
							.getInitialMarking(tranPair.getPetriNetMap().getPetriNet()), true);

			msecs = System.currentTimeMillis() - msecs;
			totalPrecTime += msecs;
			tranPair.setPrecision(precRes.getPrecision());
			//			tranPair.setPrecision(1);
			//			System.out.println("Precision is : " + "precRes.getPrecision()" + " Time taken to compute = " + msecs);
			//end prec cal
			//			if (false)
			//				for (int i = 0; i < tranPair.getReplayResult().toArray().length; i++) {
			//					SyncReplayResult sync = (SyncReplayResult) tranPair.getReplayResult().toArray()[i];
			for (SyncReplayResult sync : replay) {
				int val = sync.getTraceIndex().size();
				//				AggregatedTraceLevelPairwiseInfo tlpi = ipn.getPairwiseAlignmentInfoBasedOnIds(sync.getTraceIndex());
				for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
					StepTypes type = sync.getStepTypes().get(actIndex);
					Object stepObj = sync.getNodeInstance().get(actIndex);
					if (tranPair.containsActivity(stepObj.toString()))
						switch (type) {
							case L :
								//								tlpi.setLogMoves(stepObj.toString());
								tranPair.setLogMoves(stepObj.toString());
								//								ipn.getMisalignedActivities().add(stepObj.toString());
								tranPair.incrementLogMoveMap(stepObj.toString(), val);
								break;
							case LMGOOD :
								//								tlpi.getSyncMoves().add(tranPair.getMappedTransition((Transition) stepObj), 1);
								tranPair.getSyncMoves().add(tranPair.getMappedTransition((Transition) stepObj));
								tranPair.incrementSyncMoveMap(tranPair.getMappedTransition((Transition) stepObj), val);
								break;
							case MINVI :
								break;
							case MREAL :
								//								ipn.getMisalignedTransition().add((Transition) stepObj);
								//								tlpi.getModelMoves().add(tranPair.getMappedTransition((Transition) stepObj), 1);
								tranPair.setModelMoveMap((Transition) stepObj, sync.getTraceIndex());
								tranPair.getModelMoves().add(tranPair.getMappedTransition((Transition) stepObj));
								tranPair.incrementModelMove(tranPair.getMappedTransition((Transition) stepObj), val);
								break;
							case LMNOGOOD :
							case LMREPLACED :
							case LMSWAPPED :
								// unable to project this case, stop
						}

				}

			}

			ipn.getTranPairs().add(tranPair);

			//add to aggr
			//add sync moves 
			for (Transition t : tranPair.getSyncMoveMap().keySet()) {
				ipn.addToAggrSyncMoves(t, tranPair.getSyncMoveMap().get(t));
			}
			//add log moves
			for (String l : tranPair.getLogMoveMap().keySet())
				ipn.addToAggrLogMoves(l, tranPair.getLogMoveMap().get(l));

			//add model moves
			for (Transition t : tranPair.getModelMove().keySet()) {
				ipn.addToAggrModelMoves(t, tranPair.getModelMove().get(t));
			}
		}

	}

}
