package org.processmining.prabhakardixit.alignments.visualization;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JPanel;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.pnalignanalysis.visualization.projection.PNLogReplayProjectedVisPanel;
import org.processmining.prabhakardixit.alignments.AlignmentCalculations;
import org.processmining.prabhakardixit.alignments.ProjectedAveragedAlignments;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class AlignmentsGraph {

	public static JComponent getAlignmentsView(InteractivePetriNet ipn, TransPairAlignment tpa, boolean fullModelView,
			String selectedAct) {
		boolean incomplete=true;
		XLog log;
		Petrinet net;
		PNRepResult replay = null;
		if (fullModelView) {
			Set<String> activities = new HashSet<String>();
			for (Transition t : ipn.getselectedPetriNetMap().getPetriNet().getTransitions())
				if (!t.isInvisible())
					activities.add(t.getLabel());
			activities.add(selectedAct);
			log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), activities, true);
			net = ipn.getselectedPetriNetMap().getPetriNet();

		} else {
			log = ProjectedAveragedAlignments.createMergedLog(ipn.getLog(), tpa.getActivities(), false);
			net = tpa.getPetriNetMap().getPetriNet();
			replay = tpa.getReplayResult();
		}
		Map<String, XEventClass> classes = new HashMap<String, XEventClass>();
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
		Set<XEventClass> evtClasses = new HashSet<XEventClass>();
		for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
			XEventClass evt1 = new XEventClass(evt.getId().replace("+complete", ""), evt.getIndex());
			classes.put((evt.getId().replace("+complete", "")), evt1);
			classes.put((evt.getId().replace("+COMPLETE", "")), evt1);
		}

		TransEvClassMapping map = AlignmentCalculations.transitionToEventClassMapperByLabel(log, net, classes,false);

		//calculate replay for full view
		if (fullModelView){
			replay = AlignmentCalculations.calculateNAlignment1(ipn.getPromContext(), log, net, 5, false, null,null,incomplete,false);

		}
		//connections
		GraphLayoutConnection gc = new GraphLayoutConnection(net);
		ipn.getPromContext().getConnectionManager().addConnection(gc);

		try {
			return new PNLogReplayProjectedVisPanel(ipn.getPromContext(), net, PetrinetUtils.getInitialMarking(net),
					log, map, replay);
		} catch (ConnectionCannotBeObtained e) {
			// TODO Auto-generated catch block
			System.out.println(" errored out!! ");
			e.printStackTrace();
		}
		return null;
	}

	public static void getAlignments(PNetVisualizerController controller) {
		JPanel panel = controller.getStatisticsPanel();
		panel.removeAll();
		panel.add(getAlignmentsView(controller.getIPN(), null, true, controller.getActivity()));
		panel.revalidate();
		panel.repaint();

	}

}
