package org.processmining.prabhakardixit.alignments.visualization;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JPanel;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.SoundnessInformation;
import org.processmining.humbertogarciacaballero.plugins.PluginUtils;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.prabhakardixit.alignments.TransPairAlignment;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.DataColorAction;
import prefuse.controls.DragControl;
import prefuse.controls.PanControl;
import prefuse.controls.ToolTipControl;
import prefuse.controls.WheelZoomControl;
import prefuse.controls.ZoomControl;
import prefuse.data.Graph;
import prefuse.util.ColorLib;
import prefuse.visual.VisualItem;

public class GraphView {
	public static JPanel getGraph(PNetVisualizerController controller, TransPairAlignment tpa) {

		// -- 1. load the data ------------------------------------------------

		// load the socialnet.xml file. it is assumed that the file can be
		// found at the root of the java classpath
		Graph graph = null;
		//		Petrinet net = controller.getIPN().getselectedPetriNetMap().getPetriNet();
		Set<Place> initMarking = new HashSet<Place>();
		initMarking.add(PetrinetUtils.getInitialPlace(tpa.getPetriNetMap().getPetriNet()));
		Marking m = new Marking();
		//		m.add(getInitialMarking(n));
		SoundnessInformation information = new SoundnessInformation();
		Petrinet pn = tpa.getPetriNetMap().getPetriNet();
		Map<PetrinetNode, Integer> idMap = getIDMap(controller, tpa);
		//temp starts
//		PetriNetMap pnmap = new PetriNetMap(controller.getIPN().getselectedPetriNetMap());
//		Set<PetrinetNode> keepTrans = new HashSet<PetrinetNode>();
//		for (Transition t : pnmap.getPetriNet().getTransitions())
//			if (tpa.getActivities().contains(t.getLabel()))
//				keepTrans.add(t);
//		Reduction rd = new Reduction(pnmap.getPetriNet(), keepTrans);
//		pn = pnmap.getPetriNet();
//		for (PetrinetNode n : pn.getNodes())
//			idMap.put(n, 0);
		//temp ends

		graph = PluginUtils.netToGraph(null, pn, null, information, controller.getIPN(), idMap);
		GraphVisualization viz = new GraphVisualization(graph, controller, tpa);
		// -- 2. the visualization --------------------------------------------

		// add the graph to the visualization as the data group "graph"
		// nodes and edges are accessible as "graph.nodes" and "graph.edges"
		//		Visualization vis =  new Visualization();
		//		vis.add("graph", viz.getGraph());
		//		vis.setInteractive("graph.edges", null, false);
		Visualization vis = viz.getVisualization();

		// -- 3. the renderers and renderer factory ---------------------------

		// draw the "name" label for NodeItems
		//		LabelRenderer r = new LabelRenderer("name");
		//		r.setRoundedCorner(8, 8); // round the corners
		// create a new default renderer factory
		// return our name label renderer as the default for all non-EdgeItems
		// includes straight line edges for EdgeItems by default

		// -- 4. the processing actions ---------------------------------------

		// create our nominal color palette
		// pink for females, baby blue for males
		int[] palette = new int[] { ColorLib.rgb(255, 180, 180), ColorLib.rgb(190, 190, 255) };
		// map nominal data values to colors using our provided palette
		DataColorAction fill = new DataColorAction("graph.nodes", "gender", Constants.NOMINAL, VisualItem.FILLCOLOR,
				palette);
		// use black for node text
		ColorAction text = new ColorAction("graph.nodes", VisualItem.TEXTCOLOR, ColorLib.gray(0));
		// use light grey for edges
		ColorAction edges = new ColorAction("graph.edges", VisualItem.STROKECOLOR, ColorLib.gray(200));

		// create an action list containing all color assignments
		ActionList color = new ActionList();
		color.add(fill);
		color.add(text);
		color.add(edges);

		// create an action list with an animated layout
		//		ActionList layout = new ActionList(Activity.INFINITY);
		//		layout.add(new ForceDirectedLayout("graph"));
		//		layout.add(new RepaintAction());

		// add the actions to the visualization
		//		vis.putAction("color", color);
		//		vis.putAction("layout", layout);

		// -- 5. the display and interactive controls -------------------------

		Display d = new Display(vis);
		d.setSize(820, 350); // set display size
		// drag individual items around
		d.addControlListener(new DragControl());
		// pan with left-click drag on background
		d.addControlListener(new PanControl());
		// zoom with right-click drag
		d.addControlListener(new ZoomControl());

		WheelZoomControl zoomControl;
		zoomControl = new WheelZoomControl(true, true);
		zoomControl.setMinScale(0.5);
		d.addControlListener(zoomControl);
		//		d.addControlListener(new CustomDragControl(PNConstants.VIS_LAYOUT_DECORATORS));
		d.addControlListener(new ToolTipControl(PNConstants.NODE_LABEL_KEY));

		// -- 6. launch the visualization -------------------------------------

		// create a new window to hold the visualization
		JPanel frame = new JPanel();
		// ensure application exits when window is closed
		frame.add(d);
		//		frame.pack(); // layout components in window
		frame.setVisible(true); // show the window

		// assign the colors
		vis.run("color");
		// start up the animated layout
		vis.run("layout");
		return frame;
	}

	private static Map<PetrinetNode, Integer> getIDMap(PNetVisualizerController controller, TransPairAlignment tpa) {
		Map<PetrinetNode, Integer> idMap = new HashMap<PetrinetNode, Integer>();
		int i = 0;
		for (PetrinetNode n : tpa.getPetriNetMap().getPetriNet().getNodes()) {
			if (n instanceof Transition && !((Transition) n).isInvisible()) {
				idMap.put(n, controller.getIPN().getTranId(tpa.getMappedTransition((Transition) n)));
			} else
				idMap.put(n, i++); //for places id is not important
		}
		return idMap;
	}
}
