package org.processmining.prabhakardixit.alignments.visualization;

import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.humbertogarciacaballero.models.gui.GraphVisualization;
import org.processmining.humbertogarciacaballero.models.gui.decorator.LabelDecoratorLayout;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.DataColorAction;
import prefuse.action.filter.GraphDistanceFilter;
import prefuse.action.layout.Layout;
import prefuse.action.layout.graph.ForceDirectedLayout;
import prefuse.action.layout.graph.NodeLinkTreeLayout;
import prefuse.activity.Activity;
import prefuse.controls.DragControl;
import prefuse.controls.PanControl;
import prefuse.controls.ToolTipControl;
import prefuse.controls.WheelZoomControl;
import prefuse.controls.ZoomControl;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.render.Renderer;
import prefuse.render.ShapeRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.GraphicsLib;
import prefuse.util.PrefuseLib;
import prefuse.util.force.Force;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;

public class LogView extends Display {

	private static final String LABEL_DECORATOR_GROUP_NAME = "labelgroup";
	public static final String EDGE_DECORATORS = "edgeDeco";
	private static final Schema DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
	public static final String EDGES = "graph.edges";
	public static final String AGGR_DECORATORS = "aggrDeco";
	public static final String AGGR = "aggregates";
	static {
		DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.gray(128));
		DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Tahoma", 16));
	}

	private Graph makeGraph(XLog log) {

		// Create tables for node and edge data, and configure their columns.
		Table nodeData = new Table();
		Table edgeData = new Table(0, 1);
		nodeData.addColumn("actColor", int.class);
		nodeData.addColumn("label", String.class);
		nodeData.addColumn("freq", int.class);
		edgeData.addColumn(Graph.DEFAULT_SOURCE_KEY, int.class);
		edgeData.addColumn(Graph.DEFAULT_TARGET_KEY, int.class);
		edgeData.addColumn(VisualItem.LABEL, int.class);
		// Need more data in your nodes or edges?  Just add more
		// columns.

		Graph g = new Graph(nodeData, edgeData, true);

		Node root = g.addNode();

		Map<String, Integer> actColors = new HashMap<String, Integer>();
		root.set("actColor", 1);
		root.set("label", "");
		actColors.put("", 1);
		Node n1 = null;
		Node n2 = null;
		Node endNode = null;
		endNode = g.addNode();
		String actName;
		for (XTrace trace : log) {
			n1 = root;
			for (XEvent event : trace) {
				actName = XConceptExtension.instance().extractName(event);
				if (actName.contains("Relea"))
					System.out.println("");
				boolean added = false;
				Iterator iter = n1.children();
				if (!n1.equals(root) && n1.get("label").equals(actName)) {//add self loop
					added = true;
					n2 = n1;
					if (g.getEdge(n1, n1) == null) {
						Edge e = g.addEdge(n1, n1);
						e.setInt(VisualItem.LABEL, 1);
					}
				} else
					//see if already exists
					while (iter.hasNext()) {
						Node childNode = (Node) iter.next();
						if (childNode.get("label").equals(actName)) {
							n2 = childNode;
							added = true;
						}
					}

				if (!added) {
					n2 = g.addNode();
					n2.set("actColor", 1);
					n2.set("label", actName);
					n2.set("freq", 1);

					Edge e = g.addEdge(n1, n2);
					e.setInt(VisualItem.LABEL, 1);
					if (actColors.containsKey(actName)) {
						n2.set("actColor", actColors.get(actName));
					} else {
						n2.set("actColor", actColors.size() + 1);
						actColors.put(actName, actColors.size() + 1);
					}

				} else {
					g.getEdge(n1, n2).setInt(VisualItem.LABEL, g.getEdge(n1, n2).getInt(VisualItem.LABEL) + 1);
				}

				n1 = n2;

			}
			//add end node

				
//			if(g.getEdge(n1, endNode) != null){
//				g.getEdge(n1, endNode).setInt(VisualItem.LABEL, g.getEdge(n1, endNode).getInt(VisualItem.LABEL) + 1);
//			}else{
//				Edge e = g.addEdge(n1, endNode);
//				e.setInt(VisualItem.LABEL, 1);
//				endNode.set("actColor", 1);
//				endNode.set("label", "");
//			}
//			if (endNode == null) {
//				Edge e = g.addEdge(n1, endNode);
//				e.setInt(VisualItem.LABEL, 1);
//				endNode.set("actColor", 1);
//				endNode.set("label", "");
//			} else {
//				if (g.getEdge(n1, endNode) == null) {
//					Edge e2 = g.addEdge(n1, endNode);
//					e2.setInt(VisualItem.LABEL, 1);
//				} else {
//					g.getEdge(n1, endNode).setInt(VisualItem.LABEL, g.getEdge(n1, endNode).getInt(VisualItem.LABEL) + 1);
//				}
//
//			}
		}
		//		for (String t : actColors.keySet()) {
		//			System.out.println(t + " : " + actColors.get(t));
		//		}
		//		g.addEdge(root, root);
		return g;
	}

	private Graph makeGraphTree() {

		// Create tables for node and edge data, and configure their columns.
		Table nodeData = new Table();
		Table edgeData = new Table(0, 1);
		nodeData.addColumn("flag", boolean.class);
		nodeData.addColumn("label", String.class);
		edgeData.addColumn(Graph.DEFAULT_SOURCE_KEY, int.class);
		edgeData.addColumn(Graph.DEFAULT_TARGET_KEY, int.class);
		edgeData.addColumn("label", String.class);
		// Need more data in your nodes or edges?  Just add more
		// columns.

		// Create Graph backed by those tables.  Note that I'm
		// creating a directed graph here also.
		Graph g = new Graph(nodeData, edgeData, true);
		g.addColumn(VisualItem.LABEL, int.class);
		// Create some nodes and edges, each carrying some data.
		// There are surely prettier ways to do this, but for the
		// example it gets the job done.
		//		Tree t = new Tree();

		Node root = g.addNode();
		Node n1 = g.addNode();
		Node n2 = g.addNode();
		Node n3 = g.addNode();
		n1.setBoolean("flag", false);
		n2.setBoolean("flag", true);
		n3.setBoolean("flag", true);
		n1.set("label", "a");
		n2.set("label", "b");
		n3.set("label", "c");
		Edge e1 = g.addEdge(root, n1);
		Edge e2 = g.addEdge(root, n2);
		Edge e3 = g.addEdge(root, n3);
		e1.setInt(VisualItem.LABEL, 14);
		e2.setInt(VisualItem.LABEL, 22);
		e3.setInt(VisualItem.LABEL, 33);

		//		Edge e4 = g.getEdge(g.addEdge(0, 3));
		//		Edge e5 = g.getEdge(g.addEdge(3, 6));
		//		Edge e6 = g.getEdge(g.addEdge(6, 0));
		//		e4.setString("label", "b");
		//		e5.setString("label", "b");
		//		e6.setString("label", "b");
		return g;
	}

	public LogView(XLog log) {

		super(new Visualization());

		Graph vt = makeGraph(log);
		//		Graph vt = makeGraphTree();
		final Schema LABEL_DECORATOR_SCHEMA = PrefuseLib.getVisualItemSchema();
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.FONT, FontLib.getFont("Serif", 14));
		LABEL_DECORATOR_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.color(Color.BLACK));

		//		m_vis.addGraph("graph", graph);
		m_vis.addGraph("graph", vt);
		m_vis.setInteractive("graph.edges", null, false);
		m_vis.setInteractive("graph.nodes", null, true);
		m_vis.setValue("graph.nodes", null, VisualItem.SHAPE, new Integer(Constants.SHAPE_ELLIPSE));
		m_vis.addDecorators(LABEL_DECORATOR_GROUP_NAME, GraphVisualization.GRAPH_NODES_GROUP, LABEL_DECORATOR_SCHEMA);
		m_vis.addDecorators(EDGE_DECORATORS, EDGES, DECORATOR_SCHEMA);
		//		m_vis.addDecorators(AGGR_DECORATORS, AGGR, new HoverPredicate(), DECORATOR_SCHEMA);

		Renderer nodeR = new ShapeRenderer(20);
		EdgeRenderer edgeR = new EdgeRenderer2(prefuse.Constants.EDGE_TYPE_CURVE, prefuse.Constants.EDGE_ARROW_FORWARD);

		DefaultRendererFactory drf = new DefaultRendererFactory();
		drf.setDefaultRenderer(nodeR);
		drf.setDefaultEdgeRenderer(edgeR);
		drf.add(new InGroupPredicate(LABEL_DECORATOR_GROUP_NAME), new LabelRenderer("label"));
		drf.add(new InGroupPredicate(EDGE_DECORATORS), new LabelRenderer(VisualItem.LABEL));
		//		drf.add(new InGroupPredicate(AGGR_DECORATORS), new LabelRenderer(VisualItem.LABEL));
		m_vis.setRendererFactory(drf);

		//		drf.add(new InGroupPredicate("label"), new CustomLabelRenderer("label"));

		//		int[] palette = new int[] { ColorLib.rgb(255, 180, 180), ColorLib.rgb(190, 190, 255) };
		int[] palette = new int[] { ColorLib.gray(150), ColorLib.rgb(190, 190, 255), ColorLib.rgb(220, 220, 0) };
		ColorAction nStroke = new ColorAction("graph.nodes", VisualItem.STROKECOLOR);
		nStroke.setDefaultColor(ColorLib.gray(100));

		//		DataColorAction nFill = new DataColorAction("graph.nodes", "flag", Constants.NOMINAL, VisualItem.FILLCOLOR,
		//				palette);
		DataColorAction nFill = new DataColorAction("graph.nodes", "actColor", Constants.NOMINAL, VisualItem.FILLCOLOR,
				palette);
		ColorAction edges = new ColorAction("graph.edges", VisualItem.STROKECOLOR, ColorLib.gray(0));
		ColorAction arrow = new ColorAction("graph.edges", VisualItem.FILLCOLOR, ColorLib.gray(0));
		ActionList color = new ActionList();
		color.add(nStroke);
		color.add(nFill);
		color.add(edges);
		color.add(arrow);

		NodeLinkTreeLayout ndl = new NodeLinkTreeLayout("graph");
		ndl.setDepthSpacing(150);
		ndl.setBreadthSpacing(50);

		ForceDirectedLayout fdl = new ForceDirectedLayout("graph");
		Force[] forces = fdl.getForceSimulator().getForces();
		forces[0].setParameter(1, 0);//set dist
		ActionList layout = new ActionList(Activity.INFINITY);
		layout.setStepTime(ABORT);
		//		ActionList decorators = new ActionList();
		layout.add(new LabelDecoratorLayout(LABEL_DECORATOR_GROUP_NAME));
		layout.add(ndl);

		layout.add(new LabelLayout2(EDGE_DECORATORS));
		//		layout.add(new LabelLayout2(AGGR_DECORATORS));
		layout.add(new RepaintAction());

		final GraphDistanceFilter filter = new GraphDistanceFilter("graph", 25000000);
		ActionList draw = new ActionList();
		draw.add(filter);

		//		m_vis.putAction("draw", draw);
		m_vis.putAction("color", color);
		m_vis.putAction("layout", layout);

		setSize(820, 350); // set display size
		pan(360, 250);
		setHighQuality(true);
		//		addControlListener(new DragControl());
		//		addControlListener(new PanControl());
		//		addControlListener(new ZoomControl());

		addControlListener(new DragControl());
		addControlListener(new PanControl());
		addControlListener(new ZoomControl());

		WheelZoomControl zoomControl;
		zoomControl = new WheelZoomControl(true, true);
		zoomControl.setMinScale(0.5);
		addControlListener(zoomControl);
		addControlListener(new ToolTipControl(PNConstants.NODE_LABEL_KEY));

		//

		//		m_vis.run("draw");
		m_vis.run("color");
		m_vis.run("layout");

	}

	public static JPanel getLogView(XLog log) {
		LogView ex = new LogView(log);

		JPanel panel = new JPanel();
		panel.add(ex);
		return panel;
	}

	public static void main(String[] argv) {
		LogView ex = new LogView(null);
		JFrame frame = new JFrame("prefuse example");
		frame.getContentPane().add(ex);
		frame.pack(); // layout components in window
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true); // show the window
	}

	class LabelLayout2 extends Layout {
		public LabelLayout2(String group) {
			super(group);
		}

		public void run(double frac) {
			Iterator iter = m_vis.items(m_group);
			while (iter.hasNext()) {
				DecoratorItem decorator = (DecoratorItem) iter.next();
				VisualItem decoratedItem = decorator.getDecoratedItem();
				Rectangle2D bounds = decoratedItem.getBounds();
				double x = bounds.getCenterX();
				double y = bounds.getCenterY();
				/* modification to move edge labels more to the arrow head */
				//				double x2 = 0, y2 = 0;
				//				if (decoratedItem instanceof EdgeItem) {
				//					VisualItem dest = ((EdgeItem) decoratedItem).getTargetItem();
				//					x2 = dest.getX();
				//					y2 = dest.getY();
				//					x = (x + x2) / 2;
				//					y = (y + y2) / 2;
				//				}

				setX(decorator, null, x);
				setY(decorator, null, y);
			}
		}
	}

	class EdgeRenderer2 extends EdgeRenderer {
		public EdgeRenderer2(int edgeTypeCurve, int edgeArrowForward) {
			super(edgeTypeCurve, edgeArrowForward);
		}

		protected Shape getRawShape(VisualItem item) {
			EdgeItem edge = (EdgeItem) item;
			VisualItem item1 = edge.getSourceItem();
			VisualItem item2 = edge.getTargetItem();

			int type = 0;
			if (item1.equals(item2))
				type = 1;

			getAlignedPoint(m_tmpPoints[0], item1.getBounds(), m_xAlign1, m_yAlign1);
			getAlignedPoint(m_tmpPoints[1], item2.getBounds(), m_xAlign2, m_yAlign2);
			m_curWidth = (float) (m_width * getLineWidth(item));

			// create the arrow head, if needed
			EdgeItem e = (EdgeItem) item;
			if (e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE) {
				// get starting and ending edge endpoints
				boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
				Point2D start = null, end = null;
				start = m_tmpPoints[forward ? 0 : 1];
				end = m_tmpPoints[forward ? 1 : 0];

				// compute the intersection with the target bounding box
				VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
				int i = GraphicsLib.intersectLineRectangle(start, end, dest.getBounds(), m_isctPoints);
				if (i > 0)
					end = m_isctPoints[0];

				// create the arrow head shape
				AffineTransform at = getArrowTrans(start, end, m_curWidth);
				m_curArrow = at.createTransformedShape(m_arrowHead);

				// update the endpoints for the edge shape
				// need to bias this by arrow head size
				Point2D lineEnd = m_tmpPoints[forward ? 1 : 0];
				lineEnd.setLocation(0, -m_arrowHeight);
				at.transform(lineEnd, lineEnd);
			} else {
				m_curArrow = null;
			}

			// create the edge shape
			Shape shape = null;
			double n1x = m_tmpPoints[0].getX();
			double n1y = m_tmpPoints[0].getY();
			double n2x = m_tmpPoints[1].getX();
			double n2y = m_tmpPoints[1].getY();
			switch (type) {
				case Constants.EDGE_TYPE_LINE :
					m_line.setLine(n1x, n1y, n2x, n2y);
					shape = m_line;
					break;
				case Constants.EDGE_TYPE_CURVE :
					getCurveControlPoints(edge, m_ctrlPoints, n1x, n1y, n2x, n2y);
					m_cubic.setCurve(n1x, n1y, n1x, n1y + 100, n2x, n2y + 20, n2x, n2y);
					shape = m_cubic;
					break;
				default :
					throw new IllegalStateException("Unknown edge type");
			}

			// return the edge shape
			return shape;
		}
	}

}