package org.processmining.prabhakardixit.alignments.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.Action;
import prefuse.action.ActionList;
import prefuse.action.ItemAction;
import prefuse.action.RepaintAction;
import prefuse.action.animate.ColorAnimator;
import prefuse.action.animate.LocationAnimator;
import prefuse.action.animate.QualityControlAnimator;
import prefuse.action.animate.VisibilityAnimator;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.FontAction;
import prefuse.action.filter.FisheyeTreeFilter;
import prefuse.action.filter.GraphDistanceFilter;
import prefuse.action.layout.CollapsedSubtreeLayout;
import prefuse.action.layout.graph.ForceDirectedLayout;
import prefuse.action.layout.graph.NodeLinkTreeLayout;
import prefuse.activity.Activity;
import prefuse.activity.SlowInSlowOutPacer;
import prefuse.controls.DragControl;
import prefuse.controls.FocusControl;
import prefuse.controls.NeighborHighlightControl;
import prefuse.controls.PanControl;
import prefuse.controls.WheelZoomControl;
import prefuse.controls.ZoomControl;
import prefuse.controls.ZoomToFitControl;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Table;
import prefuse.data.Tree;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.search.PrefixSearchTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.AbstractShapeRenderer;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.GraphicsLib;
import prefuse.util.force.ForceSimulator;
import prefuse.util.ui.JForcePanel;
import prefuse.util.ui.JValueSlider;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;
import prefuse.visual.sort.TreeDepthItemSorter;

public class LogView2 extends Display {

	public class OrientAction extends AbstractAction {
		private int orientation;

		public OrientAction(int orientation) {
			this.orientation = orientation;
		}

		public void actionPerformed(ActionEvent evt) {
			setOrientation(orientation);
			getVisualization().cancel("orient");
			getVisualization().run("treeLayout");
			getVisualization().run("orient");
		}
	}

	public class AutoPanAction extends Action {
		private Point2D m_start = new Point2D.Double();
		private Point2D m_end = new Point2D.Double();
		private Point2D m_cur = new Point2D.Double();
		private int m_bias = 150;

		public void run(double frac) {
			TupleSet ts = m_vis.getFocusGroup(Visualization.FOCUS_ITEMS);
			if (ts.getTupleCount() == 0)
				return;

			if (frac == 0.0) {
				int xbias = 0, ybias = 0;
				switch (m_orientation) {
					case Constants.ORIENT_LEFT_RIGHT :
						xbias = m_bias;
						break;
					case Constants.ORIENT_RIGHT_LEFT :
						xbias = -m_bias;
						break;
					case Constants.ORIENT_TOP_BOTTOM :
						ybias = m_bias;
						break;
					case Constants.ORIENT_BOTTOM_TOP :
						ybias = -m_bias;
						break;
				}

				VisualItem vi = (VisualItem) ts.tuples().next();
				m_cur.setLocation(getWidth() / 2, getHeight() / 2);
				getAbsoluteCoordinate(m_cur, m_start);
				m_end.setLocation(vi.getX() + xbias, vi.getY() + ybias);
			} else {
				m_cur.setLocation(m_start.getX() + frac * (m_end.getX() - m_start.getX()), m_start.getY() + frac
						* (m_end.getY() - m_start.getY()));
				panToAbs(m_cur);
			}
		}
	}

	public static class NodeColorAction extends ColorAction {

		public NodeColorAction(String group) {
			super(group, VisualItem.FILLCOLOR);
		}

		public int getColor(VisualItem item) {
			if (m_vis.isInGroup(item, Visualization.SEARCH_ITEMS))
				return ColorLib.rgb(255, 190, 190);
			else if (m_vis.isInGroup(item, Visualization.FOCUS_ITEMS))
				return ColorLib.rgb(198, 229, 229);
			else if (item.getDOI() > -1)
				return ColorLib.rgb(164, 193, 193);
			else
				return ColorLib.rgba(255, 255, 255, 0);
		}

	} // end of inner class TreeMapColorAction
	
	private String LABEL_DECORATOR_GROUP_NAME = "labelgroup";
	private static final String tree = "tree";
	private static final String treeNodes = "tree.nodes";
	private static final String treeEdges = "tree.edges";
	private LabelRenderer m_nodeRenderer;
	private EdgeRenderer m_edgeRenderer;

	private String m_label = "name";
	private int m_orientation = Constants.ORIENT_LEFT_RIGHT;

	private static Graph makeGraph() {

		
		
		// Create tables for node and edge data, and configure their columns.
		Table nodeData = new Table();
		Table edgeData = new Table(0, 1);
		nodeData.addColumn("flag", boolean.class);
		nodeData.addColumn("colVal", int.class);
		nodeData.addColumn("label", String.class);
		edgeData.addColumn(Graph.DEFAULT_SOURCE_KEY, int.class);
		edgeData.addColumn(Graph.DEFAULT_TARGET_KEY, int.class);
		edgeData.addColumn("label", String.class);
		// Need more data in your nodes or edges?  Just add more
		// columns.

		Graph g = new Graph(nodeData, edgeData, true);

		Node root = g.addNode();
		Node n1 = g.addNode();
		Node n2 = g.addNode();
		Node n3 = g.addNode();
		Node n4 = g.addNode();
		n1.setBoolean("flag", false);
		n2.setBoolean("flag", true);
		n3.setBoolean("flag", true);
		n1.set("label", "a");
		n2.set("label", "b");
		n3.set("label", "c");
		n4.set("label", "b");
		n1.set("colVal", 4);
		n2.set("colVal", 2);
		n3.set("colVal", 3);
		n4.set("colVal", 2);
		root.set("colVal", 1);
		Edge e1 = g.addEdge(root, n1);
		Edge e2 = g.addEdge(root, n2);
		Edge e3 = g.addEdge(root, n3);
		Edge e4 = g.addEdge(n1, n4);
		Edge e5 = g.addEdge(n4, n4);
		e1.setString("label", "a");
		e2.setString("label", "a");
		e3.setString("label", "a");
		//		}
		//		Edge e4 = g.getEdge(g.addEdge(0, 3));
		//		Edge e5 = g.getEdge(g.addEdge(3, 6));
		//		Edge e6 = g.getEdge(g.addEdge(6, 0));
		//		e4.setString("label", "b");
		//		e5.setString("label", "b");
		//		e6.setString("label", "b");

		return g;
	}

	private Tree makeGraphTree() {

		// Create tables for node and edge data, and configure their columns.
		Table nodeData = new Table();
		Table edgeData = new Table(0, 1);
		nodeData.addColumn("flag", boolean.class);
		nodeData.addColumn("label", String.class);
		edgeData.addColumn(Graph.DEFAULT_SOURCE_KEY, int.class);
		edgeData.addColumn(Graph.DEFAULT_TARGET_KEY, int.class);
		edgeData.addColumn("label", String.class);
		// Need more data in your nodes or edges?  Just add more
		// columns.

		// Create Graph backed by those tables.  Note that I'm
		// creating a directed graph here also.
		//		Graph g = new Graph(nodeData, edgeData, true);

		// Create some nodes and edges, each carrying some data.
		// There are surely prettier ways to do this, but for the
		// example it gets the job done.
		Tree t = new Tree();
		t.addColumn("flag", boolean.class);
		t.addColumn("label", String.class);
		t.addColumn("colVal", int.class);

		Node r = t.addRoot();
		Node n1 = t.addChild(r);
		Node n2 = t.addChild(r);
		Node n3 = t.addChild(r);

		n1.setBoolean("flag", false);
		n2.setBoolean("flag", true);
		n3.setBoolean("flag", true);
		n1.set("label", "a");
		n2.set("label", "b");
		n3.set("label", "c");
		n1.set("colVal", 4);
		n2.set("colVal", 2);
		n3.set("colVal", 3);
		r.set("colVal", 2);

		Edge e1 = t.addEdge(r, n1);
		Edge e2 = t.addEdge(r, n2);
		Edge e3 = t.addEdge(r, n3);
		e1.setString("label", "a");
		e2.setString("label", "a");
		e3.setString("label", "a");

		//		Edge e4 = g.getEdge(g.addEdge(0, 3));
		//		Edge e5 = g.getEdge(g.addEdge(3, 6));
		//		Edge e6 = g.getEdge(g.addEdge(6, 0));
		//		e4.setString("label", "b");
		//		e5.setString("label", "b");
		//		e6.setString("label", "b");
		return t;
	}

	public void setOrientation(int orientation) {
		NodeLinkTreeLayout rtl = (NodeLinkTreeLayout) m_vis.getAction("treeLayout");
		
		CollapsedSubtreeLayout stl = (CollapsedSubtreeLayout) m_vis.getAction("subLayout");
		switch (orientation) {
			case Constants.ORIENT_LEFT_RIGHT :
				m_nodeRenderer.setHorizontalAlignment(Constants.LEFT);
				m_edgeRenderer.setHorizontalAlignment1(Constants.RIGHT);
				m_edgeRenderer.setHorizontalAlignment2(Constants.LEFT);
				m_edgeRenderer.setVerticalAlignment1(Constants.CENTER);
				m_edgeRenderer.setVerticalAlignment2(Constants.CENTER);
				break;
			case Constants.ORIENT_RIGHT_LEFT :
				m_nodeRenderer.setHorizontalAlignment(Constants.RIGHT);
				m_edgeRenderer.setHorizontalAlignment1(Constants.LEFT);
				m_edgeRenderer.setHorizontalAlignment2(Constants.RIGHT);
				m_edgeRenderer.setVerticalAlignment1(Constants.CENTER);
				m_edgeRenderer.setVerticalAlignment2(Constants.CENTER);
				break;
			case Constants.ORIENT_TOP_BOTTOM :
				m_nodeRenderer.setHorizontalAlignment(Constants.CENTER);
				m_edgeRenderer.setHorizontalAlignment1(Constants.CENTER);
				m_edgeRenderer.setHorizontalAlignment2(Constants.CENTER);
				m_edgeRenderer.setVerticalAlignment1(Constants.BOTTOM);
				m_edgeRenderer.setVerticalAlignment2(Constants.TOP);
				break;
			case Constants.ORIENT_BOTTOM_TOP :
				m_nodeRenderer.setHorizontalAlignment(Constants.CENTER);
				m_edgeRenderer.setHorizontalAlignment1(Constants.CENTER);
				m_edgeRenderer.setHorizontalAlignment2(Constants.CENTER);
				m_edgeRenderer.setVerticalAlignment1(Constants.TOP);
				m_edgeRenderer.setVerticalAlignment2(Constants.BOTTOM);
				break;
			default :
				throw new IllegalArgumentException("Unrecognized orientation value: " + orientation);
		}
		m_orientation = orientation;
		rtl.setOrientation(orientation);
		stl.setOrientation(orientation);
	}

	public int getOrientation() {
		return m_orientation;
	}
	public LogView2() {

		super(new Visualization());
		Tree t = makeGraphTree();
		m_vis.add(tree, t);

		m_nodeRenderer = new LabelRenderer(m_label);
		m_nodeRenderer.setRenderType(AbstractShapeRenderer.RENDER_TYPE_FILL);
		m_nodeRenderer.setHorizontalAlignment(Constants.LEFT);
		m_nodeRenderer.setRoundedCorner(8, 8);
		m_edgeRenderer = new EdgeRenderer(Constants.EDGE_TYPE_CURVE);

		DefaultRendererFactory rf = new DefaultRendererFactory(m_nodeRenderer);
		rf.add(new InGroupPredicate(treeEdges), m_edgeRenderer);
		m_vis.setRendererFactory(rf);

		// colors
		ItemAction nodeColor = new NodeColorAction(treeNodes);
		ItemAction textColor = new ColorAction(treeNodes, VisualItem.TEXTCOLOR, ColorLib.rgb(0, 0, 0));
		m_vis.putAction("textColor", textColor);

		ItemAction edgeColor = new ColorAction(treeEdges, VisualItem.STROKECOLOR, ColorLib.rgb(200, 200, 200));

		// quick repaint
		ActionList repaint = new ActionList();
		repaint.add(nodeColor);
		repaint.add(new RepaintAction());
		m_vis.putAction("repaint", repaint);

		// full paint
		ActionList fullPaint = new ActionList();
		fullPaint.add(nodeColor);
		m_vis.putAction("fullPaint", fullPaint);

		// animate paint change
		ActionList animatePaint = new ActionList(400);
		animatePaint.add(new ColorAnimator(treeNodes));
		animatePaint.add(new RepaintAction());
		m_vis.putAction("animatePaint", animatePaint);

		// create the tree layout action
		NodeLinkTreeLayout treeLayout = new NodeLinkTreeLayout(tree, m_orientation, 50, 0, 8);
		treeLayout.setLayoutAnchor(new Point2D.Double(25, 300));
		m_vis.putAction("treeLayout", treeLayout);

		CollapsedSubtreeLayout subLayout = new CollapsedSubtreeLayout(tree, m_orientation);
		m_vis.putAction("subLayout", subLayout);

		AutoPanAction autoPan = new AutoPanAction();

		// create the filtering and layout
		ActionList filter = new ActionList();
		filter.add(new FisheyeTreeFilter(tree, 2));
		filter.add(new FontAction(treeNodes, FontLib.getFont("Tahoma", 16)));
		filter.add(treeLayout);
		filter.add(subLayout);
		filter.add(textColor);
		filter.add(nodeColor);
		filter.add(edgeColor);
		m_vis.putAction("filter", filter);

		// animated transition
		ActionList animate = new ActionList(1000);
		animate.setPacingFunction(new SlowInSlowOutPacer());
		animate.add(autoPan);
		animate.add(new QualityControlAnimator());
		animate.add(new VisibilityAnimator(tree));
		animate.add(new LocationAnimator(treeNodes));
		animate.add(new ColorAnimator(treeNodes));
		animate.add(new RepaintAction());
		m_vis.putAction("animate", animate);
		m_vis.alwaysRunAfter("filter", "animate");

		// create animator for orientation changes
		ActionList orient = new ActionList(2000);
		orient.setPacingFunction(new SlowInSlowOutPacer());
		orient.add(autoPan);
		orient.add(new QualityControlAnimator());
		orient.add(new LocationAnimator(treeNodes));
		orient.add(new RepaintAction());
		m_vis.putAction("orient", orient);

		// ------------------------------------------------

		// initialize the display
		setSize(700, 600);
		setItemSorter(new TreeDepthItemSorter());
		addControlListener(new ZoomToFitControl());
		addControlListener(new ZoomControl());
		addControlListener(new WheelZoomControl());
		addControlListener(new PanControl());
		addControlListener(new FocusControl(1, "filter"));

		registerKeyboardAction(new OrientAction(Constants.ORIENT_LEFT_RIGHT), "left-to-right",
				KeyStroke.getKeyStroke("ctrl 1"), WHEN_FOCUSED);
		registerKeyboardAction(new OrientAction(Constants.ORIENT_TOP_BOTTOM), "top-to-bottom",
				KeyStroke.getKeyStroke("ctrl 2"), WHEN_FOCUSED);
		registerKeyboardAction(new OrientAction(Constants.ORIENT_RIGHT_LEFT), "right-to-left",
				KeyStroke.getKeyStroke("ctrl 3"), WHEN_FOCUSED);
		registerKeyboardAction(new OrientAction(Constants.ORIENT_BOTTOM_TOP), "bottom-to-top",
				KeyStroke.getKeyStroke("ctrl 4"), WHEN_FOCUSED);

		// ------------------------------------------------

		// filter graph and perform layout
		setOrientation(m_orientation);
		m_vis.run("filter");

		TupleSet search = new PrefixSearchTupleSet();
		m_vis.addFocusGroup(Visualization.SEARCH_ITEMS, search);
		search.addTupleSetListener(new TupleSetListener() {
			public void tupleSetChanged(TupleSet t, Tuple[] add, Tuple[] rem) {
				m_vis.cancel("animatePaint");
				m_vis.run("fullPaint");
				m_vis.run("animatePaint");
			}
		});

	}

	public static JPanel getLogView() {
		LogView2 ex = new LogView2();
		JPanel panel = new JPanel();
		panel.add(ex);
		return panel;
	}

	public static void main(String[] argv) {
		LogView2 ex = new LogView2();
		Graph g = makeGraph();
		JPanel panel = new JPanel();
		panel.add(new GraphPanel(g));
			        panel.add(ex);
		JFrame frame = new JFrame("prefuse example");
		frame.getContentPane().add(panel);
		frame.pack(); // layout components in window
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true); // show the window
	}

	public final static class GraphPanel extends JPanel {

		/** for serialization. */
		private static final long serialVersionUID = 5943939093143764654L;

		/** the constant for "graph". */
		private final static String GRAPH = "graph";

		/** the constant for "graph.nodes". */
		private final static String GRAPH_NODES = "graph.nodes";

		/** the constant for "graph.edges". */
		private final static String GRAPH_EDGES = "graph.edges";

		/** the constant for "label". */
		private final static String LABEL = "label";

		/** the constant for "draw". */
		private final static String DRAW = "draw";

		/** the constant for "layout". */
		private final static String LAYOUT = "layout";

		/** for visualizing the graph. */
		private Visualization m_vis;

		/**
		 * Initializes the panel.
		 * 
		 * @param graph
		 *            the graph to display
		 */
		/**
		 * Initializes the panel.
		 * 
		 * @param graph
		 *            the graph to display
		 */
		private GraphPanel(Graph graph) {
			super(new BorderLayout());

			m_vis = new Visualization();

			// --------------------------------------------------------------------
			// set up the renderers

			LabelRenderer tr = new LabelRenderer();
			tr.setRoundedCorner(8, 8);
			m_vis.setRendererFactory(new DefaultRendererFactory(tr));

			// --------------------------------------------------------------------
			// register the data with a visualization

			// adds graph to visualization and sets renderer label field
			DefaultRendererFactory drf = (DefaultRendererFactory) m_vis.getRendererFactory();
			((LabelRenderer) drf.getDefaultRenderer()).setTextField(LABEL);

			// --------------------------------------------------------------------
			// create actions to process the visual data

			int hops = 30;
			final GraphDistanceFilter filter = new GraphDistanceFilter(GRAPH, hops);

			ColorAction fill = new ColorAction(GRAPH_NODES, VisualItem.FILLCOLOR, ColorLib.rgb(200, 200, 255));
			fill.add(VisualItem.FIXED, ColorLib.rgb(255, 100, 100));
			fill.add(VisualItem.HIGHLIGHT, ColorLib.rgb(255, 200, 125));

			ActionList draw = new ActionList();
			draw.add(filter);
			draw.add(fill);
			draw.add(new ColorAction(GRAPH_NODES, VisualItem.STROKECOLOR, 0));
			draw.add(new ColorAction(GRAPH_NODES, VisualItem.TEXTCOLOR, ColorLib.rgb(0, 0, 0)));
			draw.add(new ColorAction(GRAPH_EDGES, VisualItem.FILLCOLOR, ColorLib.gray(200)));
			draw.add(new ColorAction(GRAPH_EDGES, VisualItem.STROKECOLOR, ColorLib.gray(200)));

			ActionList animate = new ActionList(Activity.INFINITY);
			animate.setStepTime(ABORT);
			animate.add(new ForceDirectedLayout(GRAPH));
			animate.add(fill);
			animate.add(new RepaintAction());

			// finally, we register our ActionList with the Visualization.
			// we can later execute our Actions by invoking a method on our
			// Visualization, using the name we've chosen below.
			m_vis.putAction(DRAW, draw);
			m_vis.putAction(LAYOUT, animate);

			m_vis.runAfter(DRAW, LAYOUT);

			// --------------------------------------------------------------------
			// set up a display to show the visualization

			Display display = new Display(m_vis);
			display.setSize(700, 700);
			display.pan(350, 350);
			display.setForeground(Color.GRAY);
			display.setBackground(Color.WHITE);

			// main display controls
			display.addControlListener(new FocusControl(1));
			display.addControlListener(new DragControl());
			display.addControlListener(new PanControl());
			display.addControlListener(new ZoomControl());
			display.addControlListener(new WheelZoomControl());
			display.addControlListener(new ZoomToFitControl());
			display.addControlListener(new NeighborHighlightControl());

			display.setForeground(Color.GRAY);
			display.setBackground(Color.WHITE);

			// --------------------------------------------------------------------        
			// launch the visualization

			// create a panel for editing force values
			ForceSimulator fsim = ((ForceDirectedLayout) animate.get(0)).getForceSimulator();
			JForcePanel fpanel = new JForcePanel(fsim);

			final JValueSlider slider = new JValueSlider("Distance", 0, hops, hops);
			slider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					filter.setDistance(slider.getValue().intValue());
					m_vis.run(DRAW);
				}
			});
			slider.setBackground(Color.WHITE);
			slider.setPreferredSize(new Dimension(300, 30));
			slider.setMaximumSize(new Dimension(300, 30));

			Box cf = new Box(BoxLayout.Y_AXIS);
			cf.add(slider);
			cf.setBorder(BorderFactory.createTitledBorder("Connectivity Filter"));
			fpanel.add(cf);

			//fpanel.add(opanel);

			fpanel.add(Box.createVerticalGlue());

			// create a new JSplitPane to present the interface
			JSplitPane split = new JSplitPane();
			split.setLeftComponent(display);
			split.setRightComponent(fpanel);
			split.setOneTouchExpandable(true);
			split.setContinuousLayout(false);
			split.setDividerLocation(700);

			add(split, BorderLayout.CENTER);

			// update graph
			m_vis.removeGroup(GRAPH);
			VisualGraph vg = m_vis.addGraph(GRAPH, graph);
			m_vis.setValue(GRAPH_EDGES, null, VisualItem.INTERACTIVE, Boolean.FALSE);
			VisualItem f = (VisualItem) vg.getNode(0);
			m_vis.getGroup(Visualization.FOCUS_ITEMS).setTuple(f);
			f.setFixed(false);

			// fix selected focus nodes
			TupleSet focusGroup = m_vis.getGroup(Visualization.FOCUS_ITEMS);
			focusGroup.addTupleSetListener(new TupleSetListener() {
				public void tupleSetChanged(TupleSet ts, Tuple[] add, Tuple[] rem) {
					for (int i = 0; i < rem.length; ++i)
						((VisualItem) rem[i]).setFixed(false);
					for (int i = 0; i < add.length; ++i) {
						((VisualItem) add[i]).setFixed(false);
						((VisualItem) add[i]).setFixed(true);
					}
					if (ts.getTupleCount() == 0) {
						ts.addTuple(rem[0]);
						((VisualItem) rem[0]).setFixed(false);
					}
					m_vis.run(DRAW);
				}
			});

			m_vis.run(DRAW);
		}

	}

	public class RainbowEdgeRenderer extends EdgeRenderer {
		protected Polygon m_arrowHead2; //we split the head into two parts 
		protected Shape m_curArrow2;
		protected Color color = Color.RED;
		protected Color color2 = Color.BLUE;

		public RainbowEdgeRenderer(int edgeTypeCurve, int edgeArrowForward) {
			super(edgeTypeCurve, edgeArrowForward);
		}

		//			public void render(Graphics2D g, VisualItem item) {
		//				//render the edge line 
		//				Shape shape = getShape(item);
		//				if (shape != null) {
		//					double x1, x2, y1, y2, xx1, xx2, yy1, yy2, c, radAngle, degAngle;
		//					//a = length of x-axis, b = length of y-axis 
		//					int a, b;
		//					//get the coordinates of the original shape 
		//					Line2D tmpline = (Line2D) shape;
		//					x1 = tmpline.getX1();
		//					x2 = tmpline.getX2();
		//					y1 = tmpline.getY1();
		//					y2 = tmpline.getY2();
		//					a = (int) ((x1 - x2));
		//					b = (int) ((y1 - y2));
		//					c = Math.sqrt((a * a) + (b * b));
		//					radAngle = Math.acos(((Math.abs(a)) / c));
		//					degAngle = radAngle * (180 / Math.PI);
		//					xx1 = x1;
		//					xx2 = x2;
		//					yy1 = y1;
		//					yy2 = y2;
		//					//a <= 0 -> target -----> source, or vertical if a == 0 
		//					if (a <= 0) {
		//						//b < 0 -> source is above target, or horizontal if b == 0 
		//						if (b <= 0) {
		//							if (degAngle > 45) {
		//								xx1 -= 0.5;
		//								xx2 -= 0.5;
		//								x1 += 0.5;
		//								x2 += 0.5;
		//							} else {
		//								yy1 += 0.5;
		//								yy2 += 0.5;
		//								y1 -= 0.5;
		//								y2 -= 0.5;
		//							}
		//						} else {
		//							if (degAngle > 45) {
		//								xx1 += 0.5;
		//								xx2 += 0.5;
		//								x1 -= 0.5;
		//								x2 -= 0.5;
		//							} else {
		//								yy1 += 0.5;
		//								yy2 += 0.5;
		//								y1 -= 0.5;
		//								y2 -= 0.5;
		//							}
		//						}
		//					} else {
		//						//b < 0 -> source is above target, or horizontal if b == 0 
		//						if (b <= 0) {
		//							if (degAngle > 45) {
		//								xx1 -= 0.5;
		//								xx2 -= 0.5;
		//								x1 += 0.5;
		//								x2 += 0.5;
		//							} else {
		//								yy1 -= 0.5;
		//								yy2 -= 0.5;
		//								y1 += 0.5;
		//								y2 += 0.5;
		//							}
		//						} else {
		//							if (degAngle > 45) {
		//								xx1 += 0.5;
		//								xx2 += 0.5;
		//								x1 -= 0.5;
		//								x2 -= 0.5;
		//							} else {
		//								yy1 -= 0.5;
		//								yy2 -= 0.5;
		//								y1 += 0.5;
		//								y2 += 0.5;
		//							}
		//						}
		//					}
		//					Line2D line = new Line2D.Double(x1, y1, x2, y2);
		//					Line2D line2 = new Line2D.Double(xx1, yy1, xx2, yy2);
		//					g.setColor(color2);
		//					g.draw(line2);
		//					g.setColor(color);
		//					g.draw(line);
		//				}
		//				//render the edge arrow head, if appropriate 
		//				if (m_curArrow != null) {
		//					g.setPaint(color2);
		//					g.fill(m_curArrow2);
		//					g.setPaint(color);
		//					g.fill(m_curArrow);
		//				}
		//			}

		//			protected Polygon updateArrowHead(int w, int h) {
		//				if (m_arrowHead == null) {
		//					m_arrowHead = new Polygon();
		//					m_arrowHead2 = new Polygon();
		//				} else {
		//					m_arrowHead.reset();
		//					m_arrowHead2.reset();
		//				}
		//				m_arrowHead.addPoint(0, 0);
		//				m_arrowHead.addPoint(0, -h);
		//				m_arrowHead.addPoint(w / 2, -h);
		//				m_arrowHead.addPoint(0, 0);
		//				m_arrowHead2.addPoint(0, 0);
		//				m_arrowHead2.addPoint(0, -h);
		//				m_arrowHead2.addPoint(-w / 2, -h);
		//				m_arrowHead2.addPoint(0, 0);
		//				return m_arrowHead;
		//			}

		/*
		 * unfortunately I had to overwrite this method in order to include
		 * affine transformation of the second half of the arrow head. Only two
		 * lines had to be added to the copy of the original method() from the
		 * super class
		 */
		protected Shape getRawShape(VisualItem item) {
			EdgeItem edge = (EdgeItem) item;
			VisualItem item1 = edge.getSourceItem();
			VisualItem item2 = edge.getTargetItem();
			int type = 0;
			if (item1.equals(item2))
				type = 1;
			getAlignedPoint(m_tmpPoints[0], item1.getBounds(), m_xAlign1, m_yAlign1);
			getAlignedPoint(m_tmpPoints[1], item2.getBounds(), m_xAlign2, m_yAlign2);
			m_curWidth = (float) (m_width * getLineWidth(item));
			//create the arrow head, if needed
			EdgeItem e = (EdgeItem) item;
			if (e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE) {
				//get starting and ending edge endpoints
				boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
				Point2D start = null, end = null;
				start = m_tmpPoints[forward ? 0 : 1];
				end = m_tmpPoints[forward ? 1 : 0];
				//compute the intersection with the target bounding box
				VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
				int i = GraphicsLib.intersectLineRectangle(start, end, dest.getBounds(), m_isctPoints);
				if (i > 0)
					end = m_isctPoints[0];
				//create the arrow head shape
				AffineTransform at = getArrowTrans(start, end, m_curWidth);
				m_curArrow = at.createTransformedShape(m_arrowHead);
				m_curArrow2 = at.createTransformedShape(m_arrowHead2); //---------added line
				//update the endpoints for the edge shape
				//need to bias this by arrow head size
				Point2D lineEnd = m_tmpPoints[forward ? 1 : 0];
				lineEnd.setLocation(0, -m_arrowHeight);
				at.transform(lineEnd, lineEnd);
			} else {
				m_curArrow = null;
				m_curArrow2 = null; //--------------------------------------added line
			}
			//create the edge shape
			Shape shape = null;
			double n1x = m_tmpPoints[0].getX();
			double n1y = m_tmpPoints[0].getY();
			double n2x = m_tmpPoints[1].getX();
			double n2y = m_tmpPoints[1].getY();
			switch (type) {
				case Constants.EDGE_TYPE_LINE :
					m_line.setLine(n1x, n1y, n2x, n2y);
					shape = m_line;
					break;
				case Constants.EDGE_TYPE_CURVE :
					getCurveControlPoints(edge, m_ctrlPoints, n1x, n1y, n2x, n2y);
					m_cubic.setCurve(n1x, n1y, m_ctrlPoints[0].getX(), m_ctrlPoints[0].getY() + 50,
							m_ctrlPoints[0].getX() + m_ctrlPoints[0].getX() / 2, m_ctrlPoints[0].getY()
									+ m_ctrlPoints[0].getY() / 2, n2x, n2y);
					shape = m_cubic;
					break;
				default :
					throw new IllegalStateException("Unknown edge type");
			}
			//return the edge shape
			return shape;
		}
	}

}