package org.processmining.prabhakardixit.alignments.visualization;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import javax.swing.JFrame;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.layout.graph.NodeLinkTreeLayout;
import prefuse.activity.Activity;
import prefuse.controls.DragControl;
import prefuse.controls.PanControl;
import prefuse.controls.ZoomControl;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.GraphicsLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualItem;

public class temp extends Display {
	public static final String GRAPH = "graph";
	public static final String NODES = "graph.nodes";
	public static final String EDGES = "graph.edges";

	public temp() {
		//initialize display and data
		super(new Visualization());
		initDataGroups();
		LabelRenderer nodeRenderer = new LabelRenderer("label");
		EdgeRenderer edgeRenderer = new RainbowEdgeRenderer();
		DefaultRendererFactory drf = new DefaultRendererFactory();
		drf.setDefaultRenderer(nodeRenderer);
		drf.setDefaultEdgeRenderer(edgeRenderer);
		m_vis.setRendererFactory(drf);
		//set up the visual operators
		//first set up all the color actions
		ColorAction nText = new ColorAction(NODES, VisualItem.TEXTCOLOR);
		nText.setDefaultColor(ColorLib.rgb(100, 0, 0));
		ColorAction nStroke = new ColorAction(NODES, VisualItem.STROKECOLOR);
		nStroke.setDefaultColor(ColorLib.gray(100));
		ColorAction nFill = new ColorAction(NODES, VisualItem.FILLCOLOR);
		nFill.setDefaultColor(ColorLib.gray(255));
		ColorAction nEdges = new ColorAction(EDGES, VisualItem.STROKECOLOR);
		nEdges.setDefaultColor(ColorLib.gray(100));
		//bundle the color actions
		ActionList draw = new ActionList();
		draw.add(nText);
		draw.add(nStroke);
		draw.add(nFill);
		draw.add(nEdges);
		m_vis.putAction("draw", draw);
		//now create the main animate routine
		ActionList animate = new ActionList(Activity.INFINITY);
		NodeLinkTreeLayout ndl = new NodeLinkTreeLayout(GRAPH);
		ndl.setBreadthSpacing(500);
		ndl.setDepthSpacing(100);
		animate.add(ndl);
		animate.add(new RepaintAction());
		m_vis.putAction("animate", animate);
		m_vis.runAfter("draw", "animate");
		//set up the display
		setSize(500, 500);
		pan(250, 250);
		setHighQuality(true);
		addControlListener(new ZoomControl());
		addControlListener(new PanControl());
		addControlListener(new DragControl());
		zoom(new Point(getWidth() / 2, getHeight() / 2), 2);
		//set things running
		m_vis.run("draw");
	}

	private void initDataGroups() {
		Graph g = new Graph(true);
		g.addColumn("label", String.class);
		Node n1 = g.addNode();
		Node n2 = g.addNode();
		Node n3 = g.addNode();
		Node n4 = g.addNode();
		Node n5 = g.addNode();
		n1.setString("label", "Do");
		n2.setString("label", "you");
		n3.setString("label", "love");
		n4.setString("label", "rainbows");
		n5.setString("label", "too?");
		g.addEdge(n1, n2);
		g.addEdge(n2, n3);
		g.addEdge(n3, n4);
		g.addEdge(n4, n5);
		m_vis.addGraph(GRAPH, g);
	}

	public static void main(String[] argv) {
		JFrame frame = demo();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}

	public static JFrame demo() {
		temp sag = new temp();
		JFrame frame = new JFrame("rainbows4all");
		frame.getContentPane().add(sag);
		frame.pack();
		return frame;
	}

	public class RainbowEdgeRenderer extends EdgeRenderer {
		protected Polygon m_arrowHead2; //we split the head into two parts 
		protected Shape m_curArrow2;
		protected Color color = Color.RED;
		protected Color color2 = Color.BLUE;

		public void render(Graphics2D g, VisualItem item) {
			//render the edge line 
			Shape shape = getShape(item);
			if (shape != null) {
				double x1, x2, y1, y2, xx1, xx2, yy1, yy2, c, radAngle, degAngle;
				//a = length of x-axis, b = length of y-axis 
				int a, b;
				//get the coordinates of the original shape 
				Line2D tmpline = (Line2D) shape;
				x1 = tmpline.getX1();
				x2 = tmpline.getX2();
				y1 = tmpline.getY1();
				y2 = tmpline.getY2();
				a = (int) ((x1 - x2));
				b = (int) ((y1 - y2));
				c = Math.sqrt((a * a) + (b * b));
				radAngle = Math.acos(((Math.abs(a)) / c));
				degAngle = radAngle * (180 / Math.PI);
				xx1 = x1;
				xx2 = x2;
				yy1 = y1;
				yy2 = y2;
				//a <= 0 -> target -----> source, or vertical if a == 0 
				if (a <= 0) {
					//b < 0 -> source is above target, or horizontal if b == 0 
					if (b <= 0) {
						if (degAngle > 45) {
							xx1 -= 0.5;
							xx2 -= 0.5;
							x1 += 0.5;
							x2 += 0.5;
						} else {
							yy1 += 0.5;
							yy2 += 0.5;
							y1 -= 0.5;
							y2 -= 0.5;
						}
					} else {
						if (degAngle > 45) {
							xx1 += 0.5;
							xx2 += 0.5;
							x1 -= 0.5;
							x2 -= 0.5;
						} else {
							yy1 += 0.5;
							yy2 += 0.5;
							y1 -= 0.5;
							y2 -= 0.5;
						}
					}
				} else {
					//b < 0 -> source is above target, or horizontal if b == 0 
					if (b <= 0) {
						if (degAngle > 45) {
							xx1 -= 0.5;
							xx2 -= 0.5;
							x1 += 0.5;
							x2 += 0.5;
						} else {
							yy1 -= 0.5;
							yy2 -= 0.5;
							y1 += 0.5;
							y2 += 0.5;
						}
					} else {
						if (degAngle > 45) {
							xx1 += 0.5;
							xx2 += 0.5;
							x1 -= 0.5;
							x2 -= 0.5;
						} else {
							yy1 -= 0.5;
							yy2 -= 0.5;
							y1 += 0.5;
							y2 += 0.5;
						}
					}
				}
				Line2D line = new Line2D.Double(x1, y1, x2, y2);
				Line2D line2 = new Line2D.Double(xx1, yy1, xx2, yy2);
				g.setColor(color2);
				g.draw(line2);
				g.setColor(color);
				g.draw(line);
			}
			//render the edge arrow head, if appropriate 
			if (m_curArrow != null) {
				g.setPaint(color2);
				g.fill(m_curArrow2);
				g.setPaint(color);
				g.fill(m_curArrow);
			}
		}

		protected Polygon updateArrowHead(int w, int h) {
			if (m_arrowHead == null) {
				m_arrowHead = new Polygon();
				m_arrowHead2 = new Polygon();
			} else {
				m_arrowHead.reset();
				m_arrowHead2.reset();
			}
			m_arrowHead.addPoint(0, 0);
			m_arrowHead.addPoint(0, -h);
			m_arrowHead.addPoint(w / 2, -h);
			m_arrowHead.addPoint(0, 0);
			m_arrowHead2.addPoint(0, 0);
			m_arrowHead2.addPoint(0, -h);
			m_arrowHead2.addPoint(-w / 2, -h);
			m_arrowHead2.addPoint(0, 0);
			return m_arrowHead;
		}

		/*
		 * unfortunately I had to overwrite this method in order to include
		 * affine transformation of the second half of the arrow head. Only two
		 * lines had to be added to the copy of the original method() from the
		 * super class
		 */
		protected Shape getRawShape(VisualItem item) {
			EdgeItem edge = (EdgeItem) item;
			VisualItem item1 = edge.getSourceItem();
			VisualItem item2 = edge.getTargetItem();
			int type = m_edgeType;
			getAlignedPoint(m_tmpPoints[0], item1.getBounds(), m_xAlign1, m_yAlign1);
			getAlignedPoint(m_tmpPoints[1], item2.getBounds(), m_xAlign2, m_yAlign2);
			m_curWidth = (float) (m_width * getLineWidth(item));
			//create the arrow head, if needed
			EdgeItem e = (EdgeItem) item;
			if (e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE) {
				//get starting and ending edge endpoints
				boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
				Point2D start = null, end = null;
				start = m_tmpPoints[forward ? 0 : 1];
				end = m_tmpPoints[forward ? 1 : 0];
				//compute the intersection with the target bounding box
				VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
				int i = GraphicsLib.intersectLineRectangle(start, end, dest.getBounds(), m_isctPoints);
				if (i > 0)
					end = m_isctPoints[0];
				//create the arrow head shape
				AffineTransform at = getArrowTrans(start, end, m_curWidth);
				m_curArrow = at.createTransformedShape(m_arrowHead);
				m_curArrow2 = at.createTransformedShape(m_arrowHead2); //---------added line
				//update the endpoints for the edge shape
				//need to bias this by arrow head size
				Point2D lineEnd = m_tmpPoints[forward ? 1 : 0];
				lineEnd.setLocation(0, -m_arrowHeight);
				at.transform(lineEnd, lineEnd);
			} else {
				m_curArrow = null;
				m_curArrow2 = null; //--------------------------------------added line
			}
			//create the edge shape
			Shape shape = null;
			double n1x = m_tmpPoints[0].getX();
			double n1y = m_tmpPoints[0].getY();
			double n2x = m_tmpPoints[1].getX();
			double n2y = m_tmpPoints[1].getY();
			switch (type) {
				case Constants.EDGE_TYPE_LINE :
					m_line.setLine(n1x, n1y, n2x, n2y);
					shape = m_line;
					break;
				case Constants.EDGE_TYPE_CURVE :
					getCurveControlPoints(edge, m_ctrlPoints, n1x, n1y, n2x, n2y);
					m_cubic.setCurve(n1x, n1y, m_ctrlPoints[0].getX(), m_ctrlPoints[0].getY(), m_ctrlPoints[1].getX(),
							m_ctrlPoints[1].getY(), n2x, n2y);
					shape = m_cubic;
					break;
				default :
					throw new IllegalStateException("Unknown edge type");
			}
			//return the edge shape
			return shape;
		}
	}
}
