package org.processmining.prabhakardixit.fcnets.expansion;

import java.util.HashSet;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.alignments.UpdateProjectedAveragedAlignments;
import org.processmining.prabhakardixit.fcnets.matrix.MatrixUtils;
import org.processmining.prabhakardixit.fcnets.models.InteractiveMatrixNet;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;

public class ChoiceTransition {
	public static void addChoiceTransition(Place pStart, Place pEnd, String transitionName, InteractivePetriNet ipn,
			int newTranID, boolean isSilent) {
		Set<Place> pStartSet = new HashSet<Place>();
		Set<Place> pEndSet = new HashSet<Place>();
		pStartSet.add(pStart);
		pEndSet.add(pEnd);
		addChoiceTransition(pStartSet, pEndSet, transitionName, ipn, newTranID, isSilent);
	}

	public static void addChoiceTransition(Set<Place> pStartSet, Set<Place> pEndSet, String transitionName,
			InteractivePetriNet ipn, int newTranID, boolean isSilent) {
		//		int tNewID = ipn.getAllTranId().size();
		PetriNetMap pnmap = ipn.getselectedPetriNetMap();
		Transition tNew = pnmap.getPetriNet().addTransition(transitionName);
		if (isSilent)
			tNew.setInvisible(true);
		ipn.setTranId(tNew, newTranID);
		Set<Transition> fcSet = new HashSet<Transition>();
		Transition tempTran = null;
		for (Place pStart : pStartSet) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnmap.getPetriNet().getOutEdges(
					pStart)) {
				//add every start trans as freeChoice
				fcSet.add((Transition) e.getTarget());
				tempTran = (Transition) e.getTarget();
			}
			pnmap.getPetriNet().addArc(pStart, tNew);
		}
		//PMD 0306 changed to outgoing instead of incoming
		for (Place pEnd : pEndSet) {
			pnmap.getPetriNet().addArc(tNew, pEnd);
		}
		if (ipn.isKeepHistory())
			updateLDSets(pStartSet, pEndSet, ipn.getInteractiveMatNet(), tNew);
		//		if (VizToPnetMapper.log)
		long msecs = System.currentTimeMillis();
		if (ipn.isCalculateAlignments()) //ALIGNMENT CALCULATIONS
			UpdateProjectedAveragedAlignments.addChoiceAlignmentCal(ipn, tNew);

		msecs += -System.currentTimeMillis();

		//DQ is data quality related? then update activities
		if (ipn.isDataQuality()) {
			ipn.getDqModel().updateRepairActivities();
		}
		//		System.out.println("Time taken for Choice alignment : " + msecs);
		//		System.out.println("Tran  ids post choice : " + ipn.getAllTranId());
		//		System.out.println("Place ids post choice : " + ipn.getAllPlaceId());
	}

	public static void updateLDSets(Set<Place> placeStart, Set<Place> placeEnd, InteractiveMatrixNet imn,
			Transition tNew) {
		Integer tNewID = imn.getTranId(tNew);

		//update TranLD

		MatrixUtils.updatePlaceLDForNewTran(imn, imn.getselectedPetriNetMap().getPetriNet(), tNewID);

		//get the in between transitions (for non-sc net only) and update the choice tran map
		//		Transition t = imn.getTranBasedOnID(tNewID);
		//		Map<Transition, Integer> tranInciMap = new HashMap<Transition, Integer>();
		//		Map<Place, Integer> placeInciMap = new HashMap<Place, Integer>();
		//		SimpleMatrix inciMat = MatrixUtils.createInciMap(tranInciMap, placeInciMap, imn.getselectedPetriNetMap()
		//				.getPetriNet());
		//		MatrixUtils.printMatrix(inciMat);
		//		SimpleMatrix augMat3 = inciMat.combine(0, inciMat.numCols(), new SimpleMatrix(inciMat.numRows(), 1));
		//		MatrixUtils.printMatrix(augMat3);
		//		SimpleMatrix inciMatWithoutNewLD = new SimpleMatrix(augMat3.numRows(), augMat3.numCols() - 1);
		//		MatrixUtils.printMatrix(inciMatWithoutNewLD);
		//		SimpleMatrix newLDCol = new SimpleMatrix(augMat3.numRows(), 1);
		//		MatrixUtils.printMatrix(newLDCol);
		//		MatrixUtils.splitInciMat(augMat3, inciMatWithoutNewLD, newLDCol, tranInciMap.get(t));
		//		Set<Integer> allChoiceTransWrtTNewInciMat = getNodesInBetween(inciMatWithoutNewLD, newLDCol, tranInciMap.get(t));
		//		Set<Integer> orgIds = new HashSet<Integer>();
		//		for (Transition t2 : tranInciMap.keySet())
		//			if (allChoiceTransWrtTNewInciMat.contains(tranInciMap.get(t2)))
		//				orgIds.add(imn.getTranId(t2));
		//		addMapLDTrans(imn,orgIds, tNewID);
	}

	//	public static Set<Integer> getNodesInBetween(SimpleMatrix inciMatrixWithoutLastCol, SimpleMatrix linDepMatrix,
	//			int indexOfNewNodeInInciMat) {
	//		MatrixUtils.printMatrix(inciMatrixWithoutLastCol);
	//		MatrixUtils.printMatrix(linDepMatrix);
	//		
	//		Set<Integer> ldCols = new HashSet<Integer>();
	//		double[] coeff = new double[inciMatrixWithoutLastCol.numCols()];
	//		LinearObjectiveFunction f = new LinearObjectiveFunction(coeff, 0);
	//		Collection<LinearConstraint> constraints = new ArrayList<LinearConstraint>();
	//		Collection<LinearConstraint> constraints_Loop = new ArrayList<LinearConstraint>();
	//		for (int r = 0; r < inciMatrixWithoutLastCol.numRows(); r++) {
	//			double[] inciMat = new double[inciMatrixWithoutLastCol.numCols()];
	//			for (int c = 0; c < inciMatrixWithoutLastCol.numCols(); c++) {
	//				inciMat[c] = inciMatrixWithoutLastCol.get(r, c);
	//			}
	//			constraints.add(new LinearConstraint(inciMat, Relationship.EQ, linDepMatrix.get(r, 0)));
	//			constraints_Loop.add(new LinearConstraint(inciMat, Relationship.EQ, -linDepMatrix.get(r, 0)));
	//		}
	//		SimplexSolver solver = new SimplexSolver();
	//		PointValuePair solution = null;
	//
	//		try {
	//			solution = solver.optimize(new MaxIter(1000), f, new LinearConstraintSet(constraints), GoalType.MINIMIZE,
	//					new NonNegativeConstraint(true));
	//		} catch (Exception e) {//loop without sc
	//			solution = solver.optimize(new MaxIter(1000), f, new LinearConstraintSet(constraints_Loop),
	//					GoalType.MINIMIZE, new NonNegativeConstraint(true));
	//		} finally {
	//			for (int c = 0; c < solution.getKey().length; c++) {
	//				double val = solution.getKey()[c];
	//				if (val != 0.)
	//					if (val != 1.)
	//						System.out.println("Error!!");
	//					else {
	//						if (c >= indexOfNewNodeInInciMat) {
	//							ldCols.add(c + 1); //since ld col is removed from position 'indexOfNewNodeInInciMat', add 1 to compensate 
	//						} else {
	//							ldCols.add(c);
	//						}
	//					}
	//			}
	//		}
	//		return ldCols;
	//	}

	//	private static void addMapLDTrans(InteractiveMatrixNet imn, Set<Integer> ldTrans, int tNew) {
	//		Set<Integer> tNewSet = new HashSet<Integer>();
	//		tNewSet.add(tNew);
	//		imn.getMapLDTrans().put(tNewSet, new HashSet<Integer>(ldTrans));
	//
	//	}
}
