package org.processmining.prabhakardixit.fcnets.synthesis;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.dataquality.DetectionAndRepairModel;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class CreateIPN {
	
	public static InteractivePetriNet createIPN(XLog log,UIPluginContext context, boolean alignmentPlugin, boolean predictorPlugin,boolean keepHistory, boolean dqPlugin) {
		InteractivePetriNet ipn= createIPN(log, context, alignmentPlugin, predictorPlugin, keepHistory);
		ipn.setDataQuality(dqPlugin);

		//create DQ model obj here
		DetectionAndRepairModel drm = new DetectionAndRepairModel(ipn);

		ipn.setDqModel(drm);
		return ipn;
	}
	
	/**
	 * Create ipn and set alignmentMode, predictor mode etc
	 * @param log
	 * @param context
	 * @param alignmentPlugin
	 * @param predictorPlugin
	 * @return
	 */
	public static InteractivePetriNet createIPN(XLog log,UIPluginContext context, boolean alignmentPlugin, boolean predictorPlugin,boolean keepHistory) {
		Petrinet pnet = new PetrinetImpl("ipn");
		Place p1 = pnet.addPlace("start");
		Transition t1 = pnet.addTransition("DUMMY");
		Place p2 = pnet.addPlace("mid");
		Transition t2 = pnet.addTransition("DUMMY");
		Place p3 = pnet.addPlace("end");
		t1.setInvisible(true);
		t2.setInvisible(true);
		pnet.addArc(p1, t1);
		pnet.addArc(t1, p2);
		pnet.addArc(p2, t2);
		pnet.addArc(t2, p3);

		PetriNetMap pn = new PetriNetMap(pnet);
		for (Place p : pn.getPetriNet().getPlaces())
			if (!PetrinetUtils.initialOrFinalPlace(p, pn.getPetriNet())) {
				Place pI = PetrinetUtils.getInitialPlace(pn.getPetriNet());
				Place pO = PetrinetUtils.getFinalPlace(pn.getPetriNet());
			}
		Transition tNew1 = null;
		Transition tNew2 = null;
		for (Transition t : pn.getPetriNet().getTransitions()) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getPetriNet().getInEdges(t)) {
				if (e.getSource().equals(PetrinetUtils.getInitialPlace(pn.getPetriNet())))
					tNew1 = t;
				else
					tNew2 = t;
			}
		}
		InteractivePetriNet ipn = new InteractivePetriNet(pn, log,context);
		ipn.setAlignmentMode(alignmentPlugin);
		ipn.setPredictorMode(predictorPlugin);
		ipn.setKeepHistory(keepHistory);
		ipn.setCalculateAlignments(false);
	
		return ipn;

	}
}
