package org.processmining.prabhakardixit.fcnets.synthesis;

import java.util.HashSet;
import java.util.Set;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.expansion.AddNewTranBwTranAndPlace;
import org.processmining.prabhakardixit.fcnets.expansion.ChoiceTransition;
import org.processmining.prabhakardixit.fcnets.expansion.ParallelPlace;
import org.processmining.prabhakardixit.fcnets.models.InteractivePetriNet;
import org.processmining.reduction.ReductionTracker;
import org.processmining.reduction.ReductionTracker.Rule;

public class VizToPnetMapper {
	public static boolean log = false;

	public static void print() {
		System.out.println("x - 1- 1");

	}

	public static void addNewTransition(int node1ID, int node2ID, String newTransStr, String newPlaceStr,
			InteractivePetriNet ipn, boolean placeToTran, int newPlaceID, int newTranID, boolean isSilent) {
		Set<Integer> source = new HashSet<Integer>();
		Set<Integer> target = new HashSet<Integer>();
		source.add(node1ID);
		target.add(node2ID);

		if (!placeToTran)
			addNewTransition(source, target, newTransStr, newPlaceStr, ipn, placeToTran, newPlaceID, newTranID,
					isSilent);

	}

	public static void addParallelPlace(Set<Integer> selectedInput, Set<Integer> outputNodeID, String newPlaceStr,
			InteractivePetriNet ipn, int newPlaceID) {
		ipn.updateInteractiveMatNet(new ReductionTracker(Rule.LDPLACE, selectedInput, outputNodeID, "", newPlaceStr,
				false, newPlaceID, 0));
		Set<Transition> inputNodes = new HashSet<Transition>();
		Set<Transition> ouputNodes = new HashSet<Transition>();
		for (Integer input : selectedInput)
			inputNodes.add((Transition) ipn.getTranBasedOnID(input));
		for (Integer output : outputNodeID)
			ouputNodes.add((Transition) ipn.getTranBasedOnID(output));
		if (log) {
			System.out.println("start.clear(); end.clear();");
			for (Integer i : selectedInput)
				System.out.println("start.add(" + i + ");");
			for (Integer i : outputNodeID)
				System.out.println("end.add(" + i + ");");
			System.out.println("newPlaceStr=\"" + newPlaceStr + "\"; newPlaceID=" + newPlaceID + "; ");
			System.out.println("VizToPnetMapper.addParallelPlace(start, end, newPlaceStr, ipn, newPlaceID);");

		}
		ParallelPlace.addParallelPlace(inputNodes, ouputNodes, newPlaceStr, ipn, newPlaceID);
	}

	public static void addChoiceTransition(Set<Integer> inputNodeID, Set<Integer> outputNodeID, String activity,
			InteractivePetriNet ipn, int newTranID, boolean isSilent) {
		ipn.updateInteractiveMatNet(new ReductionTracker(Rule.LDTRAN, inputNodeID, outputNodeID, activity, "",
				isSilent, 0, newTranID));

		Set<Place> inputNodes = new HashSet<Place>();
		Set<Place> outputNodes = new HashSet<Place>();
		for (Integer input : inputNodeID)
			inputNodes.add((Place) ipn.getPlaceBasedOnID(input));
		for (Integer output : outputNodeID)
			outputNodes.add((Place) ipn.getPlaceBasedOnID(output));
		if (log) {
			System.out.println("start.clear(); end.clear();");
			for (Integer i : inputNodeID)
				System.out.println("start.add(" + i + ");");
			for (Integer i : outputNodeID)
				System.out.println("end.add(" + i + ");");
			System.out
					.println("activity=\"" + activity + "\"; newTranID=" + newTranID + "; isSilent=" + isSilent + ";");
			System.out.println("VizToPnetMapper.addChoiceTransition(start, end, activity, ipn,newTranID, isSilent);");

		}
		ChoiceTransition.addChoiceTransition(inputNodes, outputNodes, activity, ipn, newTranID, isSilent);
		//		for(TransPairAlignment tp : ipn.getTranPairs()){
		//			System.out.println("Size of tranpairs : " + tp.getActivities());
		//		}
	}

	public static void addNewTransition(Set<Integer> sourceNodes, Set<Integer> targetNodes, String newTransStr,
			String newPlaceStr, InteractivePetriNet ipn, boolean placeToTran, int newPlaceID, int newTranID,
			boolean isSilent) {
		log = false;
		ipn.updateInteractiveMatNet(new ReductionTracker(Rule.ABSTRACTION, sourceNodes, targetNodes, newTransStr,
				newPlaceStr, false, newPlaceID, newTranID));
		Set<Place> placeSet = new HashSet<Place>();
		Set<Transition> tranSet = new HashSet<Transition>();
		boolean isSourcePlace = false;
		for (Integer id : sourceNodes) {
			if (placeToTran) {
				Place place = ipn.getPlaceBasedOnID(id);
				placeSet.add(place);
			} else {
				Transition transition = ipn.getTranBasedOnID(id);
				tranSet.add(transition);
			}
		}
		for (Integer id : targetNodes) {
			if (placeToTran) {
				Transition transition = ipn.getTranBasedOnID(id);
				tranSet.add(transition);
			} else {
				Place place = ipn.getPlaceBasedOnID(id);
				placeSet.add(place);
			}
		}
		if (log) {
			System.out.println("start.clear(); end.clear();");
			for (Integer i : sourceNodes)
				System.out.println("start.add(" + i + ");");
			for (Integer i : targetNodes)
				System.out.println("end.add(" + i + ");");
			System.out.println("newTransStr=\"" + newTransStr + "\"; placeToTran=" + placeToTran + " ; newPlaceStr=\""
					+ newPlaceStr + "\"; newPlaceID=" + newPlaceID + "; newTranID=" + newTranID + "; isSilent="
					+ isSilent + ";");
			System.out
					.println("VizToPnetMapper.addNewTransition(start, end, newTransStr,newPlaceStr, ipn,placeToTran,newPlaceID,newTranID, isSilent);");
		}

		if (!placeToTran)
			//			AddNewTranBwPlaceAndTran_1.addTranBwPlaceAndTransition(placeSet, tranSet, newTransStr, ipn);
			//		else
			AddNewTranBwTranAndPlace.addTranBwTransitionAndPlace(placeSet, tranSet, newTransStr, newPlaceStr, ipn,
					newPlaceID, newTranID, isSilent);

	}

	public static void changeActivityName(Integer id, InteractivePetriNet ipn, boolean isSilent) {
		Transition transition = ipn.getTranBasedOnID(id);
		transition.getAttributeMap().put(AttributeMap.LABEL, ipn.getSelectedActivity());
		transition.setInvisible(isSilent);
	}

	//	public static void conformanceAnalysis(Set<Integer> sourceNodes, PNetVisualizerController controller) {
	//		InteractivePetriNet ipn = controller.getIPN();
	//		Set<Transition> tranSet = new HashSet<Transition>();
	//		for (Integer id : sourceNodes) {
	//			Transition transition = ipn.getTranBasedOnID(id);
	//			tranSet.add(transition);
	//		}
	//	}
}
