package org.processmining.prabhakardixit.fcnets.utils;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ejml.simple.SimpleMatrix;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.prabhakardixit.fcnets.models.InteractiveMatrixNet;
import org.processmining.prabhakardixit.fcnets.models.PetriNetMap;
import org.processmining.prabhakardixit.pnets.PetrinetUtils;

public class fcnetsUtils {

	//verify if pn reaches final marking starting with Pi, without marking the newly added place
	public static boolean isPlaceSiphon(Petrinet pn2, Set<Transition> startTranSet, Set<Transition> endTranSet) {
		PetriNetMap pnmap = new PetriNetMap(pn2);
		Petrinet pn = pnmap.getPetriNet();
		Place pNew = pn.addPlace("p");
		//update TranSets according to new pn start
		Set<Transition> tranSet = new HashSet<Transition>(startTranSet);
		//		System.out.println(startTranSet.toString() + endTranSet.toString());
		for (Transition t : tranSet)
			startTranSet.add(pnmap.getMapTransitions().get(t));
		startTranSet.removeAll(tranSet);
		tranSet = new HashSet<Transition>(endTranSet);
		for (Transition t : tranSet)
			endTranSet.add(pnmap.getMapTransitions().get(t));
		endTranSet.removeAll(tranSet);
		//update TranSets according to new pn ends
		//		System.out.println(startTranSet.toString() + endTranSet.toString());
		for (Transition t : pn.getTransitions()) {
			//			System.out.println(t);
			if (startTranSet.contains(t))
				pn.addArc(t, pNew);
			if (endTranSet.contains(t))
				pn.addArc(pNew, t);
		}
		Place pi = PetrinetUtils.getInitialPlace(pn);
		Set<Place> piSet = new HashSet<Place>();
		piSet.add(pi);
		Set<Transition> execTranSet = new HashSet<Transition>();
		recurseMarkings(pn, piSet, execTranSet);

		if (execTranSet.containsAll(startTranSet) && execTranSet.containsAll(endTranSet)) //if start and end not exec - its a siphon
			return false;
		else
			return true;
	}

	//returns false if final marking reached
	public static void recurseMarkings(Petrinet pn, Set<Place> marking, Set<Transition> executedTrans) {
		Set<Transition> enabledTransitions = PetrinetUtils.getEnabledTransitions(pn, marking);
		if (enabledTransitions.isEmpty() || executedTrans.containsAll(enabledTransitions)) //no tran enabled or all trans already exec'd
			return;
		for (Transition enabledTran : enabledTransitions) {
			if (executedTrans.contains(enabledTran))
				continue;
			executedTrans.add(enabledTran);
			//			Set<Place> newMarking = new HashSet<Place>(marking);
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pn.getInEdges(enabledTran))
				marking.remove(edge.getSource());
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pn.getOutEdges(enabledTran))
				marking.add((Place) edge.getTarget());
			recurseMarkings(pn, marking, executedTrans);
		}
		return;
	}

	public static boolean checkFCForLDPlaces(Petrinet pn, Set<Transition> tStart, Set<Transition> tEnd) {
		boolean fcStartTrans = true;
		//get all incoming places for this tran, and compare pairwise with every other
		Set<PetrinetNode> allPrePlaces = new HashSet<PetrinetNode>();
		boolean firstLoop = true;
		for (Transition t : tEnd) {
			if (firstLoop) {
				allPrePlaces = getPreNodes(t, pn);
				firstLoop = false;
			} else {
				Set<PetrinetNode> allPrePlaces2 = getPreNodes(t, pn);
				if (!allPrePlaces2.containsAll(allPrePlaces) || !allPrePlaces.containsAll(allPrePlaces2)) //if not same -1's for all trans then not fc
					fcStartTrans = false;
			}
		}

		//next, check to see if all the post trans of prev Places is equal to tEnd
		Set<PetrinetNode> allPostTransOfPrePlaces = new HashSet<PetrinetNode>();
		for (PetrinetNode p : allPrePlaces) {
			allPostTransOfPrePlaces.addAll(getPostNodes(p, pn));
		}

		if (!allPostTransOfPrePlaces.containsAll(tEnd) || !tEnd.containsAll(allPostTransOfPrePlaces))
			fcStartTrans = false;
		//add choice and check here
		return fcStartTrans;
	}

	public static boolean checkFCForLDTrans(Petrinet pn, Set<Place> pStart) {
		boolean fcStartTrans = true;
		//get all outgoing trans for start place, and compare pairwise with every other
		Set<PetrinetNode> allPostTrans = new HashSet<PetrinetNode>();
		boolean firstLoop = true;
		for (Place p : pStart) {
			if (firstLoop) {
				allPostTrans = getPostNodes(p, pn);
				firstLoop = false;
			} else {
				Set<PetrinetNode> allPostTrans2 = getPostNodes(p, pn);
				if (!allPostTrans2.containsAll(allPostTrans) || !allPostTrans.containsAll(allPostTrans2)) //if not same -1's for all trans then not fc
					fcStartTrans = false;
			}
		}

		//next, check to see if all the pre places of post trans is equal to pStart
		Set<PetrinetNode> allPrePlacesOfPostTrans = new HashSet<PetrinetNode>();
		for (PetrinetNode t : allPostTrans) {
			allPrePlacesOfPostTrans.addAll(getPreNodes(t, pn));
		}

		if (!allPrePlacesOfPostTrans.containsAll(pStart) || !pStart.containsAll(allPrePlacesOfPostTrans))
			fcStartTrans = false;
		//add choice and check here
		return fcStartTrans;
	}

	private static Set<PetrinetNode> getPostNodes(PetrinetNode n, Petrinet petriNet) {
		Set<PetrinetNode> postNodes = new HashSet<PetrinetNode>();
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = petriNet.getOutEdges(n);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : outEdges) {
			postNodes.add(e.getTarget());
		}
		return postNodes;
	}

	private static Set<PetrinetNode> getPreNodes(PetrinetNode n, Petrinet petriNet) {
		Set<PetrinetNode> preNodes = new HashSet<PetrinetNode>();
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = petriNet.getInEdges(n);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : inEdges) {
			preNodes.add(e.getSource());
		}
		return preNodes;
	}

	public static Set<Integer> getSameRows(Integer colId, SimpleMatrix placeLDAll, Integer val) {
		Set<Integer> allOneRows = new HashSet<Integer>();
		for (int row = 0; row < placeLDAll.numRows(); row++)
			if (placeLDAll.get(row, colId) == val)
				allOneRows.add(row);
		return allOneRows;
	}

	public static Set<Integer> getSameCols(Integer rowId, SimpleMatrix placeLDAll, Integer val) {
		Set<Integer> allOneCols = new HashSet<Integer>();
		for (int col = 0; col < placeLDAll.numCols(); col++)
			if (placeLDAll.get(rowId, col) == val)
				allOneCols.add(col);
		return allOneCols;
	}

	public static Set<Place> getPathToStart(Petrinet pn, Place place) {
		Set<Place> allPrevPlaces = new HashSet<Place>();
		Place placeInitial = PetrinetUtils.getInitialPlace(pn);
		getPathToStart(pn, place, allPrevPlaces, placeInitial);
		return allPrevPlaces;
	}

	public static Set<Place> getPathToStart(Petrinet pn, Place place, Set<Place> allPrevPlaces, Place placeInitial) {
		Set<Place> originalPath = new HashSet<Place>(allPrevPlaces);
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = pn.getInEdges(place);
		//if place has no incoming edges - return path
		if (edges == null) {
			return originalPath;
		}
		//else explore the path
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : edges) {
			Transition t = (Transition) e.getSource();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e2 : pn.getInEdges(t)) {
				if (allPrevPlaces.contains(placeInitial))
					return allPrevPlaces;
				else if (allPrevPlaces.add((Place) e2.getSource()))
					return getPathToStart(pn, (Place) e2.getSource(), allPrevPlaces, placeInitial);

			}
		}
		return originalPath;
	}

	public static Set<Integer> getAllPostPlacesAfterTranSplit(Petrinet petriNet, InteractiveMatrixNet imn) {
		Set<Integer> allPostPlacesAfterTranSplit = new HashSet<Integer>();
		for (Transition t : petriNet.getTransitions()) {
			Set<PetrinetNode> postPlaces = getPostNodes(t, petriNet);
			if (postPlaces.size() > 1)
				for (PetrinetNode p : postPlaces)
					allPostPlacesAfterTranSplit.add(imn.getPlaceId((Place) p));

		}

		return allPostPlacesAfterTranSplit;
	}

	//get post transitions of first place: 
	public static boolean checkSelfLoopLD(Set<Place> startPlaces, Petrinet petriNet) {
		boolean selfLoop = true;
		//get post transitions of first (any random) place from start places
		Set<PetrinetNode> postTransitionsOfFirstPlace = getPostNodes((Place) startPlaces.toArray()[0], petriNet); //get all output trans of first place 
		Set<PetrinetNode> preNodesOfPostTran;
		for (PetrinetNode t : postTransitionsOfFirstPlace) {
			preNodesOfPostTran = getPreNodes(t, petriNet);
			if (preNodesOfPostTran.size() > 1) {
				selfLoop = false;
			}
		}
		return selfLoop;
	}

	public static boolean isFCNet(Petrinet pn) {
		Map<Place, Set<Transition>> placeMap = new HashMap<Place, Set<Transition>>();

		for (Place p : pn.getPlaces()) {
			Set<Transition> trans = new HashSet<Transition>();
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pn.getOutEdges(p)) {
				trans.add((Transition) e.getTarget());
			}
			placeMap.put(p, trans);
		}

		for (Place p : pn.getPlaces()) {
			Set<Transition> trans = placeMap.get(p);
			for (Place p2 : pn.getPlaces()) {
				if (p2 != p) {
					Set<Transition> trans2 = placeMap.get(p2);
					if ((!Collections.disjoint(trans, trans2) && !trans2.equals(trans)))
						return false;
				}
			}
		}
		return true;
	}
}
