package org.processmining.prabhakardixit.featureSelection;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.JPanel;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.experiments.CSVImport;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.processanalytics.models.ClassificationTypes;

import weka.attributeSelection.AttributeSelection;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.SparseInstance;

public class SelectFeatures {
	// - nominal
	static ArrayList<Attribute> outputAttVals = new ArrayList<Attribute>();
	protected static Map<String, String> outputs = new HashMap<String, String>();
	static Evaluators eval;

	protected static int numOfAtts = 3;

	protected static int maxNumAttVals = 15;
	protected static double fitnessThreshold = 1;

	public static void scoreFeatures(Set<Transition> tranSet, PNRepResult replayResult, XLog log, JPanel panel,
			CalculateLogHeuristics lh) {
		scoreTraceFitness(replayResult, log, panel, lh);
		//		scoreActivityFitness(replayResult, log, ipn);

	}

	private static void scoreTraceFitness(PNRepResult replayResult, XLog log, JPanel panel, CalculateLogHeuristics lh) {
		ConfigureOutputs.initiate(ClassificationTypes.TRACEFITNESS); //Configure inputs first!
		//expt starts
		Map<String, String> caseIdAwareModMap = null;
		try {
			caseIdAwareModMap = CSVImport.getAwareOfModuleMap();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (ParseException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		//expt ends

		ArrayList<Attribute> attInfo = SelectFeaturesUtils.getAttInfo(lh.getAttrValuesMap());
		Instances wekaInstances = new Instances("Test", attInfo, log.size());
		String[][] dataMatrix = new String[log.size()][attInfo.size()];
		int dataRow = 0;
		//attrname, outputName (eg output1, output2...), AttrVal, Count
		Map<String, Map<String, Map<String, Double>>> attrMapPerOutput = new HashMap<String, Map<String, Map<String, Double>>>();
		SelectFeaturesUtils.initializeAttMaps(attrMapPerOutput, attInfo);
		for (SyncReplayResult sync : replayResult) {
			//first get the conformance score corresponding to the trace
			//			String output = ConfigureOutputs.getOutputVals(sync.getInfo().get(PNRepResult.TRACEFITNESS));

			for (int index : sync.getTraceIndex()) {

				int dataCol = 0;
				XTrace trace = log.get(index);
				XEvent evt = trace.get(0);
				String str = "";
				if (caseIdAwareModMap.containsKey(trace.getAttributes().get("concept:name").toString()))
					str = caseIdAwareModMap.get(trace.getAttributes().get("concept:name").toString());
				if(str.contains("ja"))
					System.out.println("con");
				String output = ConfigureOutputs.getExptOutputVals(str);

				Instance instance = new SparseInstance(attInfo.size());
				instance.setDataset(wekaInstances);
				for (int i = 0; i < attInfo.size(); i++) {
					String attr = attInfo.get(i).name();
					String attrVal = "";

					if (evt.getAttributes().containsKey(attr)) {
						attrVal = evt.getAttributes().get(attr).toString();

					} else if (trace.getAttributes().containsKey(attr)) {
						attrVal = trace.getAttributes().get(attr).toString();
					} else
						continue;

					for (int outputIndex = 0; outputIndex < outputAttVals.size(); outputIndex++) {
						Map<String, Double> attrMapVal = attrMapPerOutput.get(attr).get("output" + outputIndex);
						if (!attrMapVal.containsKey(attrVal))
							attrMapVal.put(attrVal, (double) 0);
						if (output.equals(outputs.get("output" + outputIndex)))
							attrMapVal.put(attrVal, attrMapVal.get(attrVal) + 1);
					}
					//					if(attrVal.contains("#"))
					//						attrVal= "NaN";
					int attrIndex = SelectFeaturesUtils.getAttIndexByStr(attr, attInfo);
					//					instance.setValue(attInfo.get(attrIndex), attInfo.get(attrIndex).indexOfValue(attrVal));
					instance.setValue(attInfo.get(attrIndex), attInfo.get(attrIndex).indexOfValue(attrVal));
					dataMatrix[dataRow][dataCol] = attrVal;
					dataCol++;
				}
				int attrIndex = SelectFeaturesUtils.getAttIndexByStr("output", attInfo);
				instance.setValue(attInfo.get(attrIndex), outputAttVals.indexOf(output));
				//				System.out.println(instance);

				wekaInstances.add(instance);
				dataRow++;
			}
		}
		try {

			AttributeSelection attrSelection = FilterEvaluatorFunctions.filterEvaluator(wekaInstances, attInfo, eval);
			Set<String> rankAttributes = SelectFeaturesUtils.rankAttributesString(attrSelection.rankedAttributes(),
					attInfo);
			PlotGraphs.updateData(panel, rankAttributes, attrMapPerOutput, outputAttVals.size(), outputs);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static double getFitnessThreshold() {
		return fitnessThreshold;
	}

	public static void setFitnessThreshold(double fitnessThreshold) {
		SelectFeatures.fitnessThreshold = fitnessThreshold;
	}

	public static int getNumOfAtts() {
		return numOfAtts;
	}

	public static void setNumOfAtts(int numOfAtts) {
		SelectFeatures.numOfAtts = numOfAtts;
	}

	public static Evaluators getEvaluator() {
		return eval;
	}

	public static void setEvaluator(Evaluators eval) {
		SelectFeatures.eval = eval;
	}

}
