package org.processmining.prabhakardixit.featureSelection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.NumericToNominal;

public class SelectFeaturesUtils extends SelectFeatures {
	static int getAttIndexByStr(String str, ArrayList<Attribute> attList) {
		for (Attribute att : attList)
			if (att.name().equals(str)) {
				return attList.indexOf(att);
			}
		return -1;
	}

	public static void printMatrix(double[][] matrix, ArrayList<Attribute> attInfo) {
		System.out.println("---");
		for (int r = 0; r < matrix.length; r++) {
			System.out.println(attInfo.get((int) matrix[r][0]) + " : " + matrix[r][1]);
		}
	}

	public static Instances getNumeriNominalInstances(Instances trainingData, String options2) {
		NumericToNominal convert = new NumericToNominal();
		String[] options = new String[2];
		options[0] = "-R";
		options[1] = options2; //range of variables to make numeric

		try {
			convert.setOptions(options);
			convert.setInputFormat(trainingData);
			Instances newData = Filter.useFilter(trainingData, convert);
			System.out.println("Before");
			for (int i = 0; i < trainingData.numAttributes(); i++) {
				System.out.println(i + newData.attribute(i).name() + "Nominal? "
						+ trainingData.attribute(i).isNominal());
			}

			System.out.println("After");
			for (int i = 0; i < trainingData.numAttributes(); i++) {
				System.out.println(i + newData.attribute(i).name() + " Nominal? " + newData.attribute(i).isNominal());
			}
			return newData;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	//returns all the attributes (both case level and the first event level)
	public static List<String> getAllAttributes(ArrayList<Attribute> attInfo) {
		List<String> attList = new ArrayList<String>();
		for (Attribute att : attInfo) {
			attList.add(att.name());
		}
		return attList;
	}

	public static void initializeAttMaps(Map<String, Map<String, Map<String, Double>>> attrMapPerOutput,
			List<Attribute> allAttributes) {
		for (int j = 0; j < allAttributes.size(); j++) {
			String attr = allAttributes.get(j).name();
			Map<String, Map<String, Double>> outputTypes = new HashMap<String, Map<String, Double>>();
			for (int i = 0; i < outputAttVals.size(); i++)
				outputTypes.put("output" + i, new HashMap<String, Double>());
			attrMapPerOutput.put(attr, outputTypes);
		}
	}

	//get the top-k attribute rank names
	public static Set<String> rankAttributesString(double[][] rankedAttributes, ArrayList<Attribute> attInfo) {
		Set<String> rankAttributes = new HashSet<String>();
		for (Attribute att : attInfo) {
			for (int r = 0; r < numOfAtts; r++) {
				if (rankedAttributes[r][0] == attInfo.indexOf(att))
					rankAttributes.add(att.name());
			}
		}
		return rankAttributes;
	}

	public static ArrayList<Attribute> getAttInfo(Map<String, Set<String>> attrValMap) {
		ArrayList<Attribute> attList = new ArrayList<Attribute>();

		//		// - numeric
		//		for (String key : map) {
		//			attList.add(new Attribute(key));
		//		}

		// - nominal 
		for (String key : attrValMap.keySet()) {
			if (attrValMap.get(key).size() > maxNumAttVals)
				continue;
			FastVector nomVals = new FastVector();
			nomVals.addAll(attrValMap.get(key));
			attList.add(new Attribute(key, nomVals));
		}

//		attList.add(new Attribute("output", outputAttVals));
		return attList;
	}
}
