package org.processmining.prabhakardixit.pluginsPN;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.alignments.AlignmentCalculations;

@Plugin(name = "Generate event log from alignments", parameterLabels = { " log ", " Net " }, returnLabels = { "Alignment Event log" }, returnTypes = { XLog.class })
public class AlignmentEventLog {

	static Set<String> events = new HashSet<String>();

	public XLog alignmentEventLog(PluginContext context, XLog log, Petrinet net, Map<Transition, String> tranEvtName) {

		//calculate replay starts
		PNRepResult replay = AlignmentCalculations.calculateNAlignment1(context, log, net, 5, false, null);
		//calculate replay ends

		//		Map<Transition, String> tranEvtName = new HashMap<Transition, String>();
		Map<String, Integer> tranEvtStrName = new HashMap<String, Integer>();
		for (Transition t : net.getTransitions()) {
			if (!t.isInvisible()) {
				tranEvtStrName.put(t.getLabel(),
						tranEvtStrName.containsKey(t.getLabel()) ? tranEvtStrName.get(t.getLabel()) + 1 : 0);
				tranEvtName.put(t, t.getLabel() + tranEvtStrName.get(t.getLabel()));
			}
		}

		XFactory factory = XFactoryRegistry.instance().currentDefault();

		XLog newLog = factory.createLog((XAttributeMap) log.getAttributes().clone());
		//
		//		for (XTrace t : log) {
		//			XTrace newTrace = factory.createTrace();
		//
		//			XEvent newEvent = factory.createEvent();
		//			newTrace.add(newEvent);
		//			XConceptExtension.instance().assignName(newEvent, "");
		//
		//			newLog.add(newTrace);
		//		}

		for (SyncReplayResult sync : replay) {
			for (int i : sync.getTraceIndex()) {
				XTrace newTrace = factory.createTrace();
				long time = 1;
				for (int actIndex = 0; actIndex < sync.getStepTypes().size(); actIndex++) {
					StepTypes type = sync.getStepTypes().get(actIndex);
					Object stepObj = sync.getNodeInstance().get(actIndex);
					switch (type) {
						case MREAL :
							XEvent newEvent = factory.createEvent();
							newTrace.add(newEvent);
							XConceptExtension.instance().assignName(newEvent, tranEvtName.get(stepObj));
							break;
						case LMGOOD :
							newEvent = factory.createEvent();
							newTrace.add(newEvent);
							XConceptExtension.instance().assignName(newEvent, tranEvtName.get(stepObj));
							XTimeExtension.instance().assignTimestamp(newEvent, time++);
							break;
						case L :
						case MINVI :
							break;
						case LMNOGOOD :
						case LMREPLACED :
						case LMSWAPPED :
							break;
					}
				}
				newLog.add(newTrace);
			}
		}

		return newLog;
	}

	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get
	 * the configuration.
	 * 
	 * @param context
	 *            The context to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @return The output.
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	@UITopiaVariant(affiliation = "TU/e", author = "P.M. Dixit", email = "p.m.dixit@tue.nl")
	@PluginVariant(variantLabel = "Petri Net generati11on", requiredParameterLabels = { 0, 1 })
	public XLog yourDefaultPlugin(UIPluginContext context, XLog log, Petrinet net) throws IllegalAccessException,
			InstantiationException {
		return alignmentEventLog(context, log, net, new HashMap<Transition, String>());

	}
}
