package org.processmining.prabhakardixit.pnets;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class PetrinetCompare {
	public static boolean comparePetriNets(Petrinet petrinet1, Petrinet petrinet2) {
		boolean samePetriNet = true;
		Map<String, Set<String>> follows1 = follows(petrinet1);
		Map<String, Set<String>> follows2 = follows(petrinet2);

		samePetriNet = comparePetriNets(follows1, follows2); //check follows
		if (samePetriNet)
			samePetriNet = comparePetriNets(precedes(petrinet1), precedes(petrinet2)); //check precedes
		return samePetriNet;
	}

	private static boolean comparePetriNets(Map<String, Set<String>> set1, Map<String, Set<String>> set2) {
		if (set1.size() != set2.size())
			return false;
		for (String key : set1.keySet()) {
			if (set2.containsKey(key)) {
				for (String s : set1.get(key)) {
					if (!set2.get(key).contains(s))
						return false;
				}
			} else
				return false;
		}
		return true;

	}

	public static Set<String> transitionFollows(Transition t, Petrinet petrinet, String preString) {
		Set<String> follows = new HashSet<String>();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : petrinet.getOutEdges(t)) {
			Place p = (Place) e.getTarget();
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = petrinet.getOutEdges(p);
			if (edges.size() > 1) {
				//recurse to get all followed by - marked as XOR
				follows.addAll(placeFollows(p, petrinet));
			} else {
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
					Transition transition = (Transition) edge.getTarget();
					if (selfLoop(t, petrinet))
						follows.add("selfloop+" + t.toString());
					else if (!(transition.isInvisible())) {
						follows.add(preString + "seq+" + transition.toString());
					} else {
						follows.addAll(transitionFollows(transition, petrinet, preString));
					}

				}
			}
		}
		return follows;
	}

	public static Set<String> transitionPrecedes(Transition t, Petrinet petrinet, String preString) {
		Set<String> precedes = new HashSet<String>();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : petrinet.getInEdges(t)) {
			Place p = (Place) e.getSource();
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = petrinet.getInEdges(p);
			if (edges.size() > 1) {
				//recurse to get all preceded by - marked as XOR
				precedes.addAll(placePrecedes(p, petrinet));
			} else {
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
					Transition transition = (Transition) edge.getSource();
					if (selfLoop(t, petrinet))
						precedes.add("selfloop+" + t.toString());
					else if (!(transition.isInvisible())) {
						precedes.add(preString + "seq+" + transition.toString());
					} else {
						precedes.addAll(transitionPrecedes(transition, petrinet, preString));
					}
				}
			}
		}
		return precedes;
	}

	private static Set<String> placePrecedes(Place p, Petrinet petrinet) {
		Set<String> precedes = new HashSet<String>();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : petrinet.getInEdges(p)) {
			Transition t = (Transition) e.getSource();
			if (selfLoop(t, petrinet)) {
				precedes.add("selfloop+" + t.toString());
				continue;
			}
			if (!t.isInvisible())
				precedes.add("xor+" + t.toString());
			else
				//if invisible, then check the add prevtransition as precedes
				precedes.addAll(transitionPrecedes(t, petrinet, "xor+")); //set choice indicator as true
		}
		return precedes;
	}

	public static Map<String, Set<String>> follows(Petrinet petrinet) {
		Map<String, Set<String>> followsMap = new HashMap<String, Set<String>>();
		for (Transition t : petrinet.getTransitions()) {
			if (t.isInvisible())
				continue;
			Set<String> follows = new HashSet<String>();
			follows = transitionFollows(t, petrinet, "");
			followsMap.put(t.getLabel(), follows);
		}
		return followsMap;
	}

	public static Map<String, Set<String>> precedes(Petrinet petrinet) {
		Map<String, Set<String>> precedesMap = new HashMap<String, Set<String>>();
		for (Transition t : petrinet.getTransitions()) {
			if (t.isInvisible())
				continue;
			Set<String> precedes = new HashSet<String>();
			precedes = transitionPrecedes(t, petrinet, "");
			precedesMap.put(t.getLabel(), precedes);
		}
		return precedesMap;
	}

	private static Set<String> placeFollows(Place p, Petrinet petrinet) {
		Set<String> follows = new HashSet<String>();
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : petrinet.getOutEdges(p)) {
			Transition t = (Transition) e.getTarget();
			if (selfLoop(t, petrinet)) {
				follows.add("selfloop+" + t.toString());
				continue;
			}
			if (!t.isInvisible())
				follows.add("xor+" + t.toString());
			else
				//if invisible, then check the add next transition as follows
				follows.addAll(transitionFollows(t, petrinet, "xor+")); //set pre string as xor+ to indicate presence of choice
		}
		return follows;
	}

	private static boolean selfLoop(Transition t, Petrinet petrinet) {
		boolean selfLoop = false;
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eOut : petrinet.getOutEdges(t)) {
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> eIn : petrinet.getInEdges(t)) {
				if (eOut.getTarget().equals(eIn.getSource()))
					selfLoop = true;
			}
		}
		return selfLoop;
	}

}
