package org.processmining.prabhakardixit.processanalytics;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.processanalytics.models.ActivityStartOptions;
import org.processmining.prabhakardixit.processanalytics.models.ProcessView;
import org.processmining.prabhakardixit.processanalytics.models.TimeView;

public class ProcessAnalyticsUtils {
	public static Set<String> getTransitionNames(Set<Transition> tranSet) {
		Set<String> tranNames = new HashSet<String>();
		for (Transition transition : tranSet) {
			if (!transition.isInvisible()) {
				tranNames.add(transition.getLabel());
			}
		}
		return tranNames;
	}

	public static void execTran(Transition enabledTran, Petrinet pn, Set<Place> marking) {
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = pn.getInEdges(enabledTran);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : edges) {
			marking.remove(e.getSource());
		}
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges2 = pn.getOutEdges(enabledTran);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : edges2) {
			marking.add((Place) e.getTarget());
		}
	}

	public static void printLogMoves(Map<Integer, Map<XEventClass, Set<Place>>> syncLogMovePlaces,
			PNRepResult replayResult, Collection<Place> places) {
		Map<Place, Map<XEventClass, Double>> placeMaps = new HashMap<Place, Map<XEventClass, Double>>();
		for (Place place : places)
			placeMaps.put(place, new HashMap<XEventClass, Double>());

		for (SyncReplayResult sync : replayResult) {
			for (XEventClass eventClass : syncLogMovePlaces.get(sync.hashCode()).keySet()) {
				for (Place place : syncLogMovePlaces.get(sync.hashCode()).get(eventClass)) {
					double numOfPlaces = 1. / (syncLogMovePlaces.get(sync.hashCode()).get(eventClass).size());
					if (placeMaps.get(place).containsKey(eventClass)) {
						placeMaps.get(place).put(eventClass, placeMaps.get(place).get(eventClass) + numOfPlaces);
					} else {
						placeMaps.get(place).put(eventClass, numOfPlaces);
					}
				}
			}
		}

		System.out.println(" -- -- -- Places and log moves : ");

		for (Place place : placeMaps.keySet()) {
			System.out.println(place + " : " + placeMaps.get(place));
		}
	}

	public static long getSelctedTimeGranularity(Date d1, Date d2, TimeView timeView) {
//		Date d1 = XTimeExtension.instance().extractTimestamp(d1);
//		Date d2 = XTimeExtension.instance().extractTimestamp(d2);

		long time = 0;
		timeView = TimeView.HOURS;
		switch (timeView) {
			case DAYS :
				time = getDateDiff(d1, d2, TimeUnit.DAYS);
				break;
			case HOURS :
				time = getDateDiff(d1, d2, TimeUnit.HOURS);
				break;
			case MINUTES :
				time = getDateDiff(d1, d2, TimeUnit.MINUTES);
				break;
			case MONTHS :
				time = (int) (getDateDiff(d1, d2, TimeUnit.DAYS) / 30);
				break;
			case WEEKS :
				time = (int) (getDateDiff(d1, d2, TimeUnit.DAYS) / 7);
				break;
			case YEARS :
				time = (int) (getDateDiff(d1, d2, TimeUnit.DAYS) / 365);
				break;
			default :
				break;
		}
		return time;
	}

	public static void initializeCounter(Map<String, Map<Double, Double>> synchornousCount, Set<Transition> tranSet) {
		Set<String> activities = getTransitionNames(tranSet);
		synchornousCount.clear();
		for (String activity : activities)
			synchornousCount.put(activity, new HashMap<Double, Double>());
	}

	public static long getDateDiff(Date date1, Date date2, TimeUnit timeUnit) {
		long diffInMillies = date2.getTime() - date1.getTime();
		return timeUnit.convert(diffInMillies, TimeUnit.MILLISECONDS);
	}

	public static Set<Place> getCurrentMarkingPlaceKey(Map<Set<Place>, Map<String, Float>> logMovesFrequency,
			Set<Place> marking) {
		for (Set<Place> places : logMovesFrequency.keySet())
			if (places.equals(marking))
				return places;
		return null;
	}

	public static void printLogMoves(Map<Transition, Float> syncMovesFrequency,
			Map<Set<Place>, Map<String, Float>> logMovesFrequency, Map<Transition, Float> modelMovesFrequency) {
		System.out.println("Sync moves : ");
		for (Transition act : syncMovesFrequency.keySet()) {
			System.out.println(act + " : " + syncMovesFrequency.get(act));
		}
		System.out.println("Model moves : ");
		for (Transition act : modelMovesFrequency.keySet()) {
			System.out.println(act + " : " + modelMovesFrequency.get(act));
		}

		for (Set<Place> placeKeys : logMovesFrequency.keySet()) {
			for (String act : logMovesFrequency.get(placeKeys).keySet()) {
				System.out.println("Place keys : " + placeKeys + " Activity: " + act + " Value : "
						+ logMovesFrequency.get(placeKeys).get(act));
			}
		}
	}

	private static boolean finalPlace(Place p, Petrinet pn) {
		if (p.getLabel().equals("o"))
			return true;
		else {
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> e = pn.getOutEdges(p);
			if (e.isEmpty())
				return true;
			else
				return false;
		}
	}

	public static Place getInitialPlace(Petrinet pn) {
		for (Place p : pn.getPlaces())
			if (initialPlace(p, pn))
				return p;
		return null;
	}

	public static Place getFinalPlace(Petrinet pn) {
		for (Place p : pn.getPlaces())
			if (finalPlace(p, pn))
				return p;
		return null;
	}

	private static boolean initialPlace(Place p, Petrinet pn) {
		if (p.getLabel().equals("i"))
			return true;
		else {
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> e = pn.getInEdges(p);
			if (e.isEmpty())
				return true;
			else
				return false;
		}
	}

	//returns if the value should be added or not depending on the settings chosen
	public static boolean addToCounter(Map<String, Double> actValue, Map<String, Integer> actCount,
			ProcessView processView, ActivityStartOptions activityOption, String activity, double timeDiff) {

		switch (processView) {
			case ActivityStart :
				return true;
//				break;
//				switch (activityOption) {
//					case AvgSyncMove : //update the local counters 
//						if (actValue.containsKey(activity)) {
//							actCount.put(activity, actCount.get(activity) + 1);
//							actValue.put(activity, actValue.get(activity) + 1);
//						} else {
//
//							actCount.put(activity, 1);
//							actValue.put(activity, timeDiff);
//						}
//						return false;
//					case allSyncMoves :
//						return true;
//					case firstSyncMove :
//						if (actCount.containsKey(activity))
//							return false;
//						else {
//							actCount.put(activity, 1);
//							return true;
//						}
//					default :
//						break;
//
//				}
			case CaseStart :
				return true;
			case Normal :
				return true;
			default :
				break;

		}
		return false;
	}

	public static Set<Integer> getAllSyncMoveCodes(Transition selectedTran, Set<CaseLevelInfo> caseLevelInfo) {
		Set<Integer> syncMoveCodes = new HashSet<Integer>();
		for(CaseLevelInfo cli: caseLevelInfo){
			if(cli.getSyncTrans().contains(selectedTran))
				syncMoveCodes.add(cli.getSyncId());
		}
		return syncMoveCodes;
	}

	
}
